/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * Base resource object.
 *
 * @member {string} [id] ID of the resource.
 * @member {string} [name] Name of the resource.
 * @member {string} [type] Type of Resource.
 * @member {object} [tags] Custom tags for the resource.
 * @member {string} [etag] The entity tag used for optimistic concurency when
 * modifying the resource.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  tags?: { [propertyName: string]: string };
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the Display class.
 * @constructor
 * Contains the localized display information for this particular operation or
 * action.
 *
 * @member {string} [provider] The localized, friendly version of the resource
 * provider name.
 * @member {string} [resource] The localized, friendly version of the resource
 * type related to this action or operation; the resource type should match the
 * public documentation for the resource provider.
 * @member {string} [operation] The localized, friendly name for the operation.
 * Use the name as it will displayed to the user.
 * @member {string} [description] The localized, friendly description for the
 * operation. The description will be displayed to the user. It should be
 * thorough and concise for used in both tooltips and detailed views.
 */
export interface Display {
  provider?: string;
  resource?: string;
  operation?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Describes the supported REST operation.
 *
 * @member {string} [name] The name of the operation being performed on this
 * particular object.
 * @member {object} [display] Contains the localized display information for
 * this particular operation or action.
 * @member {string} [display.provider] The localized, friendly version of the
 * resource provider name.
 * @member {string} [display.resource] The localized, friendly version of the
 * resource type related to this action or operation; the resource type should
 * match the public documentation for the resource provider.
 * @member {string} [display.operation] The localized, friendly name for the
 * operation. Use the name as it will displayed to the user.
 * @member {string} [display.description] The localized, friendly description
 * for the operation. The description will be displayed to the user. It should
 * be thorough and concise for used in both tooltips and detailed views.
 * @member {string} [origin] The intended executor of the operation.
 */
export interface Operation {
  name?: string;
  display?: Display;
  origin?: string;
}

/**
 * @class
 * Initializes a new instance of the OsDiskImage class.
 * @constructor
 * OS disk image.
 *
 * @member {string} [operatingSystem] OS operating system type. Possible values
 * include: 'None', 'Windows', 'Linux'
 * @member {string} [sourceBlobSasUri] SAS key for source blob.
 */
export interface OsDiskImage {
  readonly operatingSystem?: string;
  readonly sourceBlobSasUri?: string;
}

/**
 * @class
 * Initializes a new instance of the DataDiskImage class.
 * @constructor
 * Data disk image.
 *
 * @member {number} [lun] The LUN.
 * @member {string} [sourceBlobSasUri] SAS key for source blob.
 */
export interface DataDiskImage {
  readonly lun?: number;
  readonly sourceBlobSasUri?: string;
}

/**
 * @class
 * Initializes a new instance of the ExtendedProduct class.
 * @constructor
 * Extended description about the product required for installing it into Azure
 * Stack.
 *
 * @member {string} [galleryPackageBlobSasUri] The URI to the .azpkg file that
 * provides information required for showing product in the gallery.
 * @member {string} [productKind] Specifies the kind of the product
 * (virtualMachine or virtualMachineExtension).
 * @member {string} [computeRole] Specifies kind of compute role inclided in
 * the package. Possible values include: 'None', 'IaaS', 'PaaS'
 * @member {boolean} [isSystemExtension] Specifies if product is a Virtual
 * Machine Extension.
 * @member {string} [uri] The URI.
 * @member {boolean} [supportMultipleExtensions] Indicates if specified product
 * supports multiple extensions.
 * @member {string} [version] Specifies product version.
 * @member {string} [vmOsType] Specifies operating system used by the product.
 * Possible values include: 'None', 'Windows', 'Linux'
 * @member {boolean} [vmScaleSetEnabled] Indicates if virtual machine Scale Set
 * is enabled in the specified product.
 * @member {object} [osDiskImage] OS disk image used by product.
 * @member {string} [osDiskImage.operatingSystem] OS operating system type.
 * Possible values include: 'None', 'Windows', 'Linux'
 * @member {string} [osDiskImage.sourceBlobSasUri] SAS key for source blob.
 * @member {array} [dataDiskImages] List of attached data disks.
 */
export interface ExtendedProduct {
  readonly galleryPackageBlobSasUri?: string;
  readonly productKind?: string;
  readonly computeRole?: string;
  readonly isSystemExtension?: boolean;
  readonly uri?: string;
  readonly supportMultipleExtensions?: boolean;
  readonly version?: string;
  readonly vmOsType?: string;
  readonly vmScaleSetEnabled?: boolean;
  readonly osDiskImage?: OsDiskImage;
  readonly dataDiskImages?: DataDiskImage[];
}

/**
 * @class
 * Initializes a new instance of the VirtualMachineExtensionProductProperties class.
 * @constructor
 * Product information.
 *
 * @member {string} [computeRole] Specifies kind of compute role inclided in
 * the package. Possible values include: 'None', 'IaaS', 'PaaS'
 * @member {boolean} [isSystemExtension] Specifies if product is a Virtual
 * Machine Extension.
 * @member {string} [uri] The URI.
 * @member {boolean} [supportMultipleExtensions] Indicates if specified product
 * supports multiple extensions.
 * @member {string} [version] Specifies product version.
 * @member {string} [vmOsType] Specifies operating system used by the product.
 * Possible values include: 'None', 'Windows', 'Linux'
 * @member {boolean} [vmScaleSetEnabled] Indicates if virtual machine Scale Set
 * is enabled in the specified product.
 */
export interface VirtualMachineExtensionProductProperties {
  readonly computeRole?: string;
  readonly isSystemExtension?: boolean;
  readonly uri?: string;
  readonly supportMultipleExtensions?: boolean;
  readonly version?: string;
  readonly vmOsType?: string;
  readonly vmScaleSetEnabled?: boolean;
}

/**
 * @class
 * Initializes a new instance of the VirtualMachineProductProperties class.
 * @constructor
 * Product information.
 *
 * @member {string} [version] Specifies product version.
 * @member {object} [osDiskImage] OS disk image used by product.
 * @member {string} [osDiskImage.operatingSystem] OS operating system type.
 * Possible values include: 'None', 'Windows', 'Linux'
 * @member {string} [osDiskImage.sourceBlobSasUri] SAS key for source blob.
 * @member {array} [dataDiskImages] List of attached data disks.
 */
export interface VirtualMachineProductProperties {
  readonly version?: string;
  readonly osDiskImage?: OsDiskImage;
  readonly dataDiskImages?: DataDiskImage[];
}

/**
 * @class
 * Initializes a new instance of the IconUris class.
 * @constructor
 * Links to product icons.
 *
 * @member {string} [large] URI to large icon.
 * @member {string} [wide] URI to wide icon.
 * @member {string} [medium] URI to medium icon.
 * @member {string} [small] URI to small icon.
 * @member {string} [hero] URI to hero icon.
 */
export interface IconUris {
  large?: string;
  wide?: string;
  medium?: string;
  small?: string;
  hero?: string;
}

/**
 * @class
 * Initializes a new instance of the ProductLink class.
 * @constructor
 * Link with additional information about a product.
 *
 * @member {string} [displayName] The description of the link.
 * @member {string} [uri] The URI corresponding to the link.
 */
export interface ProductLink {
  displayName?: string;
  uri?: string;
}

/**
 * @class
 * Initializes a new instance of the ProductProperties class.
 * @constructor
 * Additional properties of the product
 *
 * @member {string} [version] The version.
 */
export interface ProductProperties {
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the Product class.
 * @constructor
 * Product information.
 *
 * @member {string} [displayName] The display name of the product.
 * @member {string} [description] The description of the product.
 * @member {string} [publisherDisplayName] The user-friendly name of the
 * product publisher.
 * @member {string} [publisherIdentifier] Publisher identifier.
 * @member {string} [offer] The offer representing the product.
 * @member {string} [offerVersion] The version of the product offer.
 * @member {string} [sku] The product SKU.
 * @member {string} [billingPartNumber] The part number used for billing
 * purposes.
 * @member {string} [vmExtensionType] The type of the Virtual Machine
 * Extension.
 * @member {string} [galleryItemIdentity] The identifier of the gallery item
 * corresponding to the product.
 * @member {object} [iconUris] Additional links available for this product.
 * @member {string} [iconUris.large] URI to large icon.
 * @member {string} [iconUris.wide] URI to wide icon.
 * @member {string} [iconUris.medium] URI to medium icon.
 * @member {string} [iconUris.small] URI to small icon.
 * @member {string} [iconUris.hero] URI to hero icon.
 * @member {array} [links] Additional links available for this product.
 * @member {string} [legalTerms] The legal terms.
 * @member {string} [privacyPolicy] The privacy policy.
 * @member {number} [payloadLength] The length of product content.
 * @member {string} [productKind] The kind of the product (virtualMachine or
 * virtualMachineExtension)
 * @member {object} [productProperties] Additional properties for the product.
 * @member {string} [productProperties.version] The version.
 */
export interface Product extends Resource {
  displayName?: string;
  description?: string;
  publisherDisplayName?: string;
  publisherIdentifier?: string;
  offer?: string;
  offerVersion?: string;
  sku?: string;
  billingPartNumber?: string;
  vmExtensionType?: string;
  galleryItemIdentity?: string;
  iconUris?: IconUris;
  links?: ProductLink[];
  legalTerms?: string;
  privacyPolicy?: string;
  payloadLength?: number;
  productKind?: string;
  productProperties?: ProductProperties;
}

/**
 * @class
 * Initializes a new instance of the Registration class.
 * @constructor
 * Registration information.
 *
 * @member {string} [objectId] The object identifier associated with the Azure
 * Stack connecting to Azure.
 * @member {string} [cloudId] The identifier of the registered Azure Stack.
 * @member {string} [billingModel] Specifies the billing mode for the Azure
 * Stack registration.
 */
export interface Registration extends Resource {
  objectId?: string;
  cloudId?: string;
  billingModel?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivationKeyResult class.
 * @constructor
 * The resource containing the Azure Stack activation key.
 *
 * @member {string} [activationKey] Azure Stack activation key.
 */
export interface ActivationKeyResult {
  activationKey?: string;
}

/**
 * @class
 * Initializes a new instance of the RegistrationParameter class.
 * @constructor
 * Registration resource
 *
 * @member {string} registrationToken The token identifying registered Azure
 * Stack
 */
export interface RegistrationParameter extends Resource {
  registrationToken: string;
}

/**
 * @class
 * Initializes a new instance of the CustomerSubscription class.
 * @constructor
 * Customer subcription.
 *
 * @member {string} [tenantId] Tenant Id.
 */
export interface CustomerSubscription extends Resource {
  tenantId?: string;
}


/**
 * @class
 * Initializes a new instance of the OperationList class.
 * @constructor
 * List of Operations
 *
 * @member {string} [nextLink] URI to the next page of operations.
 */
export interface OperationList extends Array<Operation> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ProductList class.
 * @constructor
 * Pageable list of products.
 *
 * @member {string} [nextLink] URI to the next page.
 */
export interface ProductList extends Array<Product> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RegistrationList class.
 * @constructor
 * Pageable list of registrations.
 *
 * @member {string} [nextLink] URI to the next page.
 */
export interface RegistrationList extends Array<Registration> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the CustomerSubscriptionList class.
 * @constructor
 * Pageable list of customer subscriptions.
 *
 * @member {string} [nextLink] URI to the next page.
 */
export interface CustomerSubscriptionList extends Array<CustomerSubscription> {
  nextLink?: string;
}
