/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary Statistics related to resource consumption by Compute Nodes in a
 * Pool.
  *
 */
class ResourceStatistics {
  /**
   * Create a ResourceStatistics.
   * @property {date} startTime The start time of the time range covered by the
   * statistics.
   * @property {date} lastUpdateTime The time at which the statistics were last
   * updated. All statistics are limited to the range between startTime and
   * lastUpdateTime.
   * @property {number} avgCPUPercentage The average CPU usage across all
   * Compute Nodes in the Pool (percentage per node).
   * @property {number} avgMemoryGiB The average memory usage in GiB across all
   * Compute Nodes in the Pool.
   * @property {number} peakMemoryGiB The peak memory usage in GiB across all
   * Compute Nodes in the Pool.
   * @property {number} avgDiskGiB The average used disk space in GiB across
   * all Compute Nodes in the Pool.
   * @property {number} peakDiskGiB The peak used disk space in GiB across all
   * Compute Nodes in the Pool.
   * @property {number} diskReadIOps The total number of disk read operations
   * across all Compute Nodes in the Pool.
   * @property {number} diskWriteIOps The total number of disk write operations
   * across all Compute Nodes in the Pool.
   * @property {number} diskReadGiB The total amount of data in GiB of disk
   * reads across all Compute Nodes in the Pool.
   * @property {number} diskWriteGiB The total amount of data in GiB of disk
   * writes across all Compute Nodes in the Pool.
   * @property {number} networkReadGiB The total amount of data in GiB of
   * network reads across all Compute Nodes in the Pool.
   * @property {number} networkWriteGiB The total amount of data in GiB of
   * network writes across all Compute Nodes in the Pool.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ResourceStatistics
   *
   * @returns {object} metadata of ResourceStatistics
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ResourceStatistics',
      type: {
        name: 'Composite',
        className: 'ResourceStatistics',
        modelProperties: {
          startTime: {
            required: true,
            serializedName: 'startTime',
            type: {
              name: 'DateTime'
            }
          },
          lastUpdateTime: {
            required: true,
            serializedName: 'lastUpdateTime',
            type: {
              name: 'DateTime'
            }
          },
          avgCPUPercentage: {
            required: true,
            serializedName: 'avgCPUPercentage',
            type: {
              name: 'Number'
            }
          },
          avgMemoryGiB: {
            required: true,
            serializedName: 'avgMemoryGiB',
            type: {
              name: 'Number'
            }
          },
          peakMemoryGiB: {
            required: true,
            serializedName: 'peakMemoryGiB',
            type: {
              name: 'Number'
            }
          },
          avgDiskGiB: {
            required: true,
            serializedName: 'avgDiskGiB',
            type: {
              name: 'Number'
            }
          },
          peakDiskGiB: {
            required: true,
            serializedName: 'peakDiskGiB',
            type: {
              name: 'Number'
            }
          },
          diskReadIOps: {
            required: true,
            serializedName: 'diskReadIOps',
            type: {
              name: 'Number'
            }
          },
          diskWriteIOps: {
            required: true,
            serializedName: 'diskWriteIOps',
            type: {
              name: 'Number'
            }
          },
          diskReadGiB: {
            required: true,
            serializedName: 'diskReadGiB',
            type: {
              name: 'Number'
            }
          },
          diskWriteGiB: {
            required: true,
            serializedName: 'diskWriteGiB',
            type: {
              name: 'Number'
            }
          },
          networkReadGiB: {
            required: true,
            serializedName: 'networkReadGiB',
            type: {
              name: 'Number'
            }
          },
          networkWriteGiB: {
            required: true,
            serializedName: 'networkWriteGiB',
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = ResourceStatistics;
