/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Request body for identify face operation.
 *
 */
class IdentifyRequest {
  /**
   * Create a IdentifyRequest.
   * @property {array} faceIds Array of query faces faceIds, created by the
   * Face - Detect. Each of the faces are identified independently. The valid
   * number of faceIds is between [1, 10].
   * @property {string} [personGroupId] PersonGroupId of the target person
   * group, created by PersonGroup - Create. Parameter personGroupId and
   * largePersonGroupId should not be provided at the same time.
   * @property {string} [largePersonGroupId] LargePersonGroupId of the target
   * large person group, created by LargePersonGroup - Create. Parameter
   * personGroupId and largePersonGroupId should not be provided at the same
   * time.
   * @property {number} [maxNumOfCandidatesReturned] The range of
   * maxNumOfCandidatesReturned is between 1 and 5 (default is 1). Default
   * value: 1 .
   * @property {number} [confidenceThreshold] Confidence threshold of
   * identification, used to judge whether one face belong to one person. The
   * range of confidenceThreshold is [0, 1] (default specified by algorithm).
   */
  constructor() {
  }

  /**
   * Defines the metadata of IdentifyRequest
   *
   * @returns {object} metadata of IdentifyRequest
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'IdentifyRequest',
      type: {
        name: 'Composite',
        className: 'IdentifyRequest',
        modelProperties: {
          faceIds: {
            required: true,
            serializedName: 'faceIds',
            constraints: {
              MaxItems: 10
            },
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'UuidElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          personGroupId: {
            required: false,
            serializedName: 'personGroupId',
            constraints: {
              MaxLength: 64,
              Pattern: /^[a-z0-9-_]+$/
            },
            type: {
              name: 'String'
            }
          },
          largePersonGroupId: {
            required: false,
            serializedName: 'largePersonGroupId',
            constraints: {
              MaxLength: 64,
              Pattern: /^[a-z0-9-_]+$/
            },
            type: {
              name: 'String'
            }
          },
          maxNumOfCandidatesReturned: {
            required: false,
            serializedName: 'maxNumOfCandidatesReturned',
            defaultValue: 1,
            constraints: {
              InclusiveMaximum: 5,
              InclusiveMinimum: 1
            },
            type: {
              name: 'Number'
            }
          },
          confidenceThreshold: {
            required: false,
            serializedName: 'confidenceThreshold',
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = IdentifyRequest;
