/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Defines a postal address.
 *
 * @extends models['StructuredValue']
 */
class PostalAddress extends models['StructuredValue'] {
  /**
   * Create a PostalAddress.
   * @property {string} [streetAddress]
   * @property {string} [addressLocality] The city where the street address is
   * located. For example, Seattle.
   * @property {string} [addressSubregion]
   * @property {string} [addressRegion] The state or province code where the
   * street address is located. This could be the two-letter code. For example,
   * WA, or the full name , Washington.
   * @property {string} [postalCode] The zip code or postal code where the
   * street address is located. For example, 98052.
   * @property {string} [postOfficeBoxNumber]
   * @property {string} [addressCountry] The country/region where the street
   * address is located. This could be the two-letter ISO code. For example,
   * US, or the full name, United States.
   * @property {string} [countryIso] The two letter ISO code of this country.
   * For example, US.
   * @property {string} [neighborhood] The neighborhood where the street
   * address is located. For example, Westlake.
   * @property {string} [addressRegionAbbreviation] Region Abbreviation. For
   * example, WA.
   * @property {string} [text] The complete address. For example, 2100 Westlake
   * Ave N, Bellevue, WA 98052.
   * @property {string} [houseNumber]
   * @property {string} [streetName]
   * @property {string} [formattingRuleId]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of PostalAddress
   *
   * @returns {object} metadata of PostalAddress
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'PostalAddress',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '_type',
          clientName: '_type'
        },
        uberParent: 'ResponseBase',
        className: 'PostalAddress',
        modelProperties: {
          _type: {
            required: true,
            serializedName: '_type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          readLink: {
            required: false,
            readOnly: true,
            serializedName: 'readLink',
            type: {
              name: 'String'
            }
          },
          webSearchUrl: {
            required: false,
            readOnly: true,
            serializedName: 'webSearchUrl',
            type: {
              name: 'String'
            }
          },
          potentialAction: {
            required: false,
            readOnly: true,
            serializedName: 'potentialAction',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ActionElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Action'
                  }
              }
            }
          },
          immediateAction: {
            required: false,
            readOnly: true,
            serializedName: 'immediateAction',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ActionElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Action'
                  }
              }
            }
          },
          preferredClickthroughUrl: {
            required: false,
            readOnly: true,
            serializedName: 'preferredClickthroughUrl',
            type: {
              name: 'String'
            }
          },
          adaptiveCard: {
            required: false,
            readOnly: true,
            serializedName: 'adaptiveCard',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          url: {
            required: false,
            readOnly: true,
            serializedName: 'url',
            type: {
              name: 'String'
            }
          },
          entityPresentationInfo: {
            required: false,
            readOnly: true,
            serializedName: 'entityPresentationInfo',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'EntitiesEntityPresentationInfo',
              className: 'EntitiesEntityPresentationInfo'
            }
          },
          streetAddress: {
            required: false,
            readOnly: true,
            serializedName: 'streetAddress',
            type: {
              name: 'String'
            }
          },
          addressLocality: {
            required: false,
            readOnly: true,
            serializedName: 'addressLocality',
            type: {
              name: 'String'
            }
          },
          addressSubregion: {
            required: false,
            readOnly: true,
            serializedName: 'addressSubregion',
            type: {
              name: 'String'
            }
          },
          addressRegion: {
            required: false,
            readOnly: true,
            serializedName: 'addressRegion',
            type: {
              name: 'String'
            }
          },
          postalCode: {
            required: false,
            readOnly: true,
            serializedName: 'postalCode',
            type: {
              name: 'String'
            }
          },
          postOfficeBoxNumber: {
            required: false,
            readOnly: true,
            serializedName: 'postOfficeBoxNumber',
            type: {
              name: 'String'
            }
          },
          addressCountry: {
            required: false,
            readOnly: true,
            serializedName: 'addressCountry',
            type: {
              name: 'String'
            }
          },
          countryIso: {
            required: false,
            readOnly: true,
            serializedName: 'countryIso',
            type: {
              name: 'String'
            }
          },
          neighborhood: {
            required: false,
            readOnly: true,
            serializedName: 'neighborhood',
            type: {
              name: 'String'
            }
          },
          addressRegionAbbreviation: {
            required: false,
            readOnly: true,
            serializedName: 'addressRegionAbbreviation',
            type: {
              name: 'String'
            }
          },
          text: {
            required: false,
            readOnly: true,
            serializedName: 'text',
            type: {
              name: 'String'
            }
          },
          houseNumber: {
            required: false,
            readOnly: true,
            serializedName: 'houseNumber',
            type: {
              name: 'String'
            }
          },
          streetName: {
            required: false,
            readOnly: true,
            serializedName: 'streetName',
            type: {
              name: 'String'
            }
          },
          formattingRuleId: {
            required: false,
            readOnly: true,
            serializedName: 'formattingRuleId',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = PostalAddress;
