/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * reservation detail resource.
 *
 * @extends models['Resource']
 */
class ReservationDetail extends models['Resource'] {
  /**
   * Create a ReservationDetail.
   * @member {string} [reservationOrderId] The reservation order ID is the
   * identifier for a reservation purchase. Each reservation order ID
   * represents a single purchase transaction. A reservation order contains
   * reservations. The reservation order specifies the VM size and region for
   * the reservations.
   * @member {string} [reservationId] The reservation ID is the identifier of a
   * reservation within a reservation order. Each reservation is the grouping
   * for applying the benefit scope and also specifies the number of instances
   * to which the reservation benefit can be applied to.
   * @member {string} [skuName] This is the ARM Sku name. It can be used to
   * join with the servicetype field in additoinalinfo in usage records.
   * @member {number} [reservedHours] This is the total hours reserved for the
   * day. E.g. if reservation for 1 instance was made on 1 PM, this will be 11
   * hours for that day and 24 hours from subsequent days.
   * @member {date} [usageDate] The date on which consumption occurred.
   * @member {number} [usedHours] This is the total hours used by the instance.
   * @member {string} [instanceId] This identifier is the name of the resource
   * or the fully qualified Resource ID.
   * @member {number} [totalReservedQuantity] This is the total count of
   * instances that are reserved for the reservationid.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ReservationDetail
   *
   * @returns {object} metadata of ReservationDetail
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ReservationDetail',
      type: {
        name: 'Composite',
        className: 'ReservationDetail',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            readOnly: true,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          reservationOrderId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.reservationOrderId',
            type: {
              name: 'String'
            }
          },
          reservationId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.reservationId',
            type: {
              name: 'String'
            }
          },
          skuName: {
            required: false,
            readOnly: true,
            serializedName: 'properties.skuName',
            type: {
              name: 'String'
            }
          },
          reservedHours: {
            required: false,
            readOnly: true,
            serializedName: 'properties.reservedHours',
            type: {
              name: 'Number'
            }
          },
          usageDate: {
            required: false,
            readOnly: true,
            serializedName: 'properties.usageDate',
            type: {
              name: 'DateTime'
            }
          },
          usedHours: {
            required: false,
            readOnly: true,
            serializedName: 'properties.usedHours',
            type: {
              name: 'Number'
            }
          },
          instanceId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.instanceId',
            type: {
              name: 'String'
            }
          },
          totalReservedQuantity: {
            required: false,
            readOnly: true,
            serializedName: 'properties.totalReservedQuantity',
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = ReservationDetail;
