/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The queue build request parameters for a quick build.
 *
 * @extends models['QueueBuildRequest']
 */
class QuickBuildRequest extends models['QueueBuildRequest'] {
  /**
   * Create a QuickBuildRequest.
   * @member {array} [imageNames] The fully qualified image names including the
   * repository and tag.
   * @member {string} sourceLocation The URL(absolute or relative) of the
   * source that needs to be built. For Docker build, it can be an URL to a tar
   * or github repoistory as supported by Docker.
   * If it is relative URL, the relative path should be obtained from calling
   * getSourceUploadUrl API.
   * @member {array} [buildArguments] The collection of build arguments to be
   * used.
   * @member {boolean} [isPushEnabled] The value of this property indicates
   * whether the image built should be pushed to the registry or not. Default
   * value: true .
   * @member {boolean} [noCache] The value of this property indicates whether
   * the image cache is enabled or not. Default value: false .
   * @member {number} [timeout] Build timeout in seconds. Default value: 3600 .
   * @member {object} platform The platform properties against which the build
   * will happen.
   * @member {string} [platform.osType] The operating system type required for
   * the build. Possible values include: 'Windows', 'Linux'
   * @member {number} [platform.cpu] The CPU configuration in terms of number
   * of cores required for the build.
   * @member {string} dockerFilePath The Docker file path relative to the
   * source location.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of QuickBuildRequest
   *
   * @returns {object} metadata of QuickBuildRequest
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'QuickBuild',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'QueueBuildRequest',
        className: 'QuickBuildRequest',
        modelProperties: {
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          imageNames: {
            required: false,
            serializedName: 'imageNames',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          sourceLocation: {
            required: true,
            serializedName: 'sourceLocation',
            type: {
              name: 'String'
            }
          },
          buildArguments: {
            required: false,
            serializedName: 'buildArguments',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'BuildArgumentElementType',
                  type: {
                    name: 'Composite',
                    className: 'BuildArgument'
                  }
              }
            }
          },
          isPushEnabled: {
            required: false,
            serializedName: 'isPushEnabled',
            defaultValue: true,
            type: {
              name: 'Boolean'
            }
          },
          noCache: {
            required: false,
            serializedName: 'noCache',
            defaultValue: false,
            type: {
              name: 'Boolean'
            }
          },
          timeout: {
            required: false,
            serializedName: 'timeout',
            defaultValue: 3600,
            constraints: {
              InclusiveMaximum: 28800,
              InclusiveMinimum: 300
            },
            type: {
              name: 'Number'
            }
          },
          platform: {
            required: true,
            serializedName: 'platform',
            type: {
              name: 'Composite',
              className: 'PlatformProperties'
            }
          },
          dockerFilePath: {
            required: true,
            serializedName: 'dockerFilePath',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = QuickBuildRequest;
