/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Disk Job Details.
 *
 * @extends models['JobDetails']
 */
class DiskJobDetails extends models['JobDetails'] {
  /**
   * Create a DiskJobDetails.
   * @member {object} [preferredDisks] User preference on what size disks are
   * needed for the job. The map is from the disk size in TB to the count. Eg.
   * {2,5} means 5 disks of 2 TB size. Key is string but will be checked
   * against an int.
   * @member {array} [copyProgress] Copy progress per disk.
   * @member {object} [disksAndSizeDetails] Contains the map of disk serial
   * number to the disk size being used for the job. Is returned only after the
   * disks are shipped to the customer.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DiskJobDetails
   *
   * @returns {object} metadata of DiskJobDetails
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Disk',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'jobDetailsType',
          clientName: 'jobDetailsType'
        },
        uberParent: 'JobDetails',
        className: 'DiskJobDetails',
        modelProperties: {
          expectedDataSizeInTeraBytes: {
            required: false,
            serializedName: 'expectedDataSizeInTeraBytes',
            type: {
              name: 'Number'
            }
          },
          jobStages: {
            required: false,
            serializedName: 'jobStages',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'JobStagesElementType',
                  type: {
                    name: 'Composite',
                    className: 'JobStages'
                  }
              }
            }
          },
          contactDetails: {
            required: true,
            serializedName: 'contactDetails',
            type: {
              name: 'Composite',
              className: 'ContactDetails'
            }
          },
          shippingAddress: {
            required: true,
            serializedName: 'shippingAddress',
            type: {
              name: 'Composite',
              className: 'ShippingAddress'
            }
          },
          errorDetails: {
            required: false,
            serializedName: 'errorDetails',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'JobErrorDetailsElementType',
                  type: {
                    name: 'Composite',
                    className: 'JobErrorDetails'
                  }
              }
            }
          },
          jobDetailsType: {
            required: true,
            serializedName: 'jobDetailsType',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          preferredDisks: {
            required: false,
            serializedName: 'preferredDisks',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'NumberElementType',
                  type: {
                    name: 'Number'
                  }
              }
            }
          },
          copyProgress: {
            required: false,
            serializedName: 'copyProgress',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'DiskCopyProgressElementType',
                  type: {
                    name: 'Composite',
                    className: 'DiskCopyProgress'
                  }
              }
            }
          },
          disksAndSizeDetails: {
            required: false,
            serializedName: 'disksAndSizeDetails',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'NumberElementType',
                  type: {
                    name: 'Number'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = DiskJobDetails;
