/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the WorkspaceProviderAuthorization class.
 * @constructor
 * The workspace provider authorization.
 *
 * @member {uuid} principalId The provider's principal identifier. This is the
 * identity that the provider will use to call ARM to manage the workspace
 * resources.
 * @member {uuid} roleDefinitionId The provider's role definition identifier.
 * This role will define all the permissions that the provider must have on the
 * workspace's container resource group. This role definition cannot have
 * permission to delete the resource group.
 */
export interface WorkspaceProviderAuthorization {
  principalId: string;
  roleDefinitionId: string;
}

/**
 * @class
 * Initializes a new instance of the Sku class.
 * @constructor
 * SKU for the resource.
 *
 * @member {string} name The SKU name.
 * @member {string} [tier] The SKU tier.
 */
export interface Sku {
  name: string;
  tier?: string;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * The core properties of ARM resources
 *
 * @member {string} [id] Fully qualified resource Id for the resource. Ex -
 * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}
 * @member {string} [name] The name of the resource
 * @member {string} [type] The type of the resource. Ex-
 * Microsoft.Compute/virtualMachines or Microsoft.Storage/storageAccounts.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the TrackedResource class.
 * @constructor
 * The resource model definition for a ARM tracked top level resource
 *
 * @member {object} [tags] Resource tags.
 * @member {string} location The geo-location where the resource lives
 */
export interface TrackedResource extends Resource {
  tags?: { [propertyName: string]: string };
  location: string;
}

/**
 * @class
 * Initializes a new instance of the Workspace class.
 * @constructor
 * Information about workspace.
 *
 * @member {string} managedResourceGroupId The managed resource group Id.
 * @member {object} [parameters] Name and value pairs that define the workspace
 * parameters.
 * @member {string} [provisioningState] The workspace provisioning state.
 * Possible values include: 'Accepted', 'Running', 'Ready', 'Creating',
 * 'Created', 'Deleting', 'Deleted', 'Canceled', 'Failed', 'Succeeded',
 * 'Updating'
 * @member {string} [uiDefinitionUri] The blob URI where the UI definition file
 * is located.
 * @member {array} [authorizations] The workspace provider authorizations.
 * @member {object} [sku] The SKU of the resource.
 * @member {string} [sku.name] The SKU name.
 * @member {string} [sku.tier] The SKU tier.
 */
export interface Workspace extends TrackedResource {
  managedResourceGroupId: string;
  parameters?: any;
  readonly provisioningState?: string;
  uiDefinitionUri?: string;
  authorizations?: WorkspaceProviderAuthorization[];
  sku?: Sku;
}

/**
 * @class
 * Initializes a new instance of the WorkspaceUpdate class.
 * @constructor
 * An update to a workspace.
 *
 * @member {object} [tags] Resource tags.
 */
export interface WorkspaceUpdate {
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ErrorDetail class.
 * @constructor
 * @summary Error details.
 *
 * @member {string} code The error's code.
 * @member {string} message A human readable error message.
 * @member {string} [target] Indicates which property in the request is
 * responsible for the error.
 */
export interface ErrorDetail {
  code: string;
  message: string;
  target?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorInfo class.
 * @constructor
 * @summary The code and message for an error.
 *
 * @member {string} code A machine readable error code.
 * @member {string} message A human readable error message.
 * @member {array} [details] error details.
 * @member {string} [innererror] Inner error details if they exist.
 */
export interface ErrorInfo {
  code: string;
  message: string;
  details?: ErrorDetail[];
  innererror?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * @summary Error response.
 *
 * Contains details when the response code indicates an error.
 *
 * @member {object} error The error details.
 * @member {string} [error.code] A machine readable error code.
 * @member {string} [error.message] A human readable error message.
 * @member {array} [error.details] error details.
 * @member {string} [error.innererror] Inner error details if they exist.
 */
export interface ErrorResponse {
  error: ErrorInfo;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * The object that represents the operation.
 *
 * @member {string} [provider] Service provider: Microsoft.ResourceProvider
 * @member {string} [resource] Resource on which the operation is performed.
 * @member {string} [operation] Operation type: Read, write, delete, etc.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * REST API operation
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] The object that represents the operation.
 * @member {string} [display.provider] Service provider:
 * Microsoft.ResourceProvider
 * @member {string} [display.resource] Resource on which the operation is
 * performed.
 * @member {string} [display.operation] Operation type: Read, write, delete,
 * etc.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}


/**
 * @class
 * Initializes a new instance of the WorkspaceListResult class.
 * @constructor
 * List of workspaces.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface WorkspaceListResult extends Array<Workspace> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result of the request to list Resource Provider operations. It contains a
 * list of operations and a URL link to get the next set of results.
 *
 * @member {string} [nextLink] URL to get the next set of operation list
 * results if there are any.
 */
export interface OperationListResult extends Array<Operation> {
  nextLink?: string;
}
