/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Azure Data Lake Analytics linked service.
 *
 * @extends models['LinkedService']
 */
class AzureDataLakeAnalyticsLinkedService extends models['LinkedService'] {
  /**
   * Create a AzureDataLakeAnalyticsLinkedService.
   * @member {object} accountName The Azure Data Lake Analytics account name.
   * Type: string (or Expression with resultType string).
   * @member {object} [servicePrincipalId] The ID of the application used to
   * authenticate against the Azure Data Lake Analytics account. Type: string
   * (or Expression with resultType string).
   * @member {object} [servicePrincipalKey] The Key of the application used to
   * authenticate against the Azure Data Lake Analytics account.
   * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
   * @member {object} tenant The name or ID of the tenant to which the service
   * principal belongs. Type: string (or Expression with resultType string).
   * @member {object} [subscriptionId] Data Lake Analytics account subscription
   * ID (if different from Data Factory account). Type: string (or Expression
   * with resultType string).
   * @member {object} [resourceGroupName] Data Lake Analytics account resource
   * group name (if different from Data Factory account). Type: string (or
   * Expression with resultType string).
   * @member {object} [dataLakeAnalyticsUri] Azure Data Lake Analytics URI
   * Type: string (or Expression with resultType string).
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AzureDataLakeAnalyticsLinkedService
   *
   * @returns {object} metadata of AzureDataLakeAnalyticsLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AzureDataLakeAnalytics',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'AzureDataLakeAnalyticsLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          accountName: {
            required: true,
            serializedName: 'typeProperties.accountName',
            type: {
              name: 'Object'
            }
          },
          servicePrincipalId: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalId',
            type: {
              name: 'Object'
            }
          },
          servicePrincipalKey: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalKey',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          tenant: {
            required: true,
            serializedName: 'typeProperties.tenant',
            type: {
              name: 'Object'
            }
          },
          subscriptionId: {
            required: false,
            serializedName: 'typeProperties.subscriptionId',
            type: {
              name: 'Object'
            }
          },
          resourceGroupName: {
            required: false,
            serializedName: 'typeProperties.resourceGroupName',
            type: {
              name: 'Object'
            }
          },
          dataLakeAnalyticsUri: {
            required: false,
            serializedName: 'typeProperties.dataLakeAnalyticsUri',
            type: {
              name: 'Object'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = AzureDataLakeAnalyticsLinkedService;
