/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * HDInsight linked service.
 *
 * @extends models['LinkedService']
 */
class HDInsightLinkedService extends models['LinkedService'] {
  /**
   * Create a HDInsightLinkedService.
   * @member {object} clusterUri HDInsight cluster URI. Type: string (or
   * Expression with resultType string).
   * @member {object} [userName] HDInsight cluster user name. Type: string (or
   * Expression with resultType string).
   * @member {object} [password] HDInsight cluster password.
   * @member {string} [password.type] Polymorphic Discriminator
   * @member {object} [linkedServiceName] The Azure Storage linked service
   * reference.
   * @member {string} [linkedServiceName.referenceName] Reference LinkedService
   * name.
   * @member {object} [linkedServiceName.parameters] Arguments for
   * LinkedService.
   * @member {object} [hcatalogLinkedServiceName] A reference to the Azure SQL
   * linked service that points to the HCatalog database.
   * @member {string} [hcatalogLinkedServiceName.referenceName] Reference
   * LinkedService name.
   * @member {object} [hcatalogLinkedServiceName.parameters] Arguments for
   * LinkedService.
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of HDInsightLinkedService
   *
   * @returns {object} metadata of HDInsightLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'HDInsight',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'HDInsightLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          clusterUri: {
            required: true,
            serializedName: 'typeProperties.clusterUri',
            type: {
              name: 'Object'
            }
          },
          userName: {
            required: false,
            serializedName: 'typeProperties.userName',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          linkedServiceName: {
            required: false,
            serializedName: 'typeProperties.linkedServiceName',
            type: {
              name: 'Composite',
              className: 'LinkedServiceReference'
            }
          },
          hcatalogLinkedServiceName: {
            required: false,
            serializedName: 'typeProperties.hcatalogLinkedServiceName',
            type: {
              name: 'Composite',
              className: 'LinkedServiceReference'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = HDInsightLinkedService;
