/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Class representing a CreateDataLakeStoreAccountParameters.
 */
class CreateDataLakeStoreAccountParameters {
  /**
   * Create a CreateDataLakeStoreAccountParameters.
   * @property {string} location The resource location.
   * @property {object} [tags] The resource tags.
   * @property {object} [identity] The Key Vault encryption identity, if any.
   * @property {uuid} [identity.principalId] The principal identifier
   * associated with the encryption.
   * @property {uuid} [identity.tenantId] The tenant identifier associated with
   * the encryption.
   * @property {string} [defaultGroup] The default owner group for all new
   * folders and files created in the Data Lake Store account.
   * @property {object} [encryptionConfig] The Key Vault encryption
   * configuration.
   * @property {string} [encryptionConfig.type] The type of encryption
   * configuration being used. Currently the only supported types are
   * 'UserManaged' and 'ServiceManaged'. Possible values include:
   * 'UserManaged', 'ServiceManaged'
   * @property {object} [encryptionConfig.keyVaultMetaInfo] The Key Vault
   * information for connecting to user managed encryption keys.
   * @property {string} [encryptionConfig.keyVaultMetaInfo.keyVaultResourceId]
   * The resource identifier for the user managed Key Vault being used to
   * encrypt.
   * @property {string} [encryptionConfig.keyVaultMetaInfo.encryptionKeyName]
   * The name of the user managed encryption key.
   * @property {string}
   * [encryptionConfig.keyVaultMetaInfo.encryptionKeyVersion] The version of
   * the user managed encryption key.
   * @property {string} [encryptionState] The current state of encryption for
   * this Data Lake Store account. Possible values include: 'Enabled',
   * 'Disabled'
   * @property {array} [firewallRules] The list of firewall rules associated
   * with this Data Lake Store account.
   * @property {array} [virtualNetworkRules] The list of virtual network rules
   * associated with this Data Lake Store account.
   * @property {string} [firewallState] The current state of the IP address
   * firewall for this Data Lake Store account. Possible values include:
   * 'Enabled', 'Disabled'
   * @property {string} [firewallAllowAzureIps] The current state of allowing
   * or disallowing IPs originating within Azure through the firewall. If the
   * firewall is disabled, this is not enforced. Possible values include:
   * 'Enabled', 'Disabled'
   * @property {array} [trustedIdProviders] The list of trusted identity
   * providers associated with this Data Lake Store account.
   * @property {string} [trustedIdProviderState] The current state of the
   * trusted identity provider feature for this Data Lake Store account.
   * Possible values include: 'Enabled', 'Disabled'
   * @property {string} [newTier] The commitment tier to use for next month.
   * Possible values include: 'Consumption', 'Commitment_1TB',
   * 'Commitment_10TB', 'Commitment_100TB', 'Commitment_500TB',
   * 'Commitment_1PB', 'Commitment_5PB'
   */
  constructor() {
  }

  /**
   * Defines the metadata of CreateDataLakeStoreAccountParameters
   *
   * @returns {object} metadata of CreateDataLakeStoreAccountParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'CreateDataLakeStoreAccountParameters',
      type: {
        name: 'Composite',
        className: 'CreateDataLakeStoreAccountParameters',
        modelProperties: {
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          identity: {
            required: false,
            serializedName: 'identity',
            type: {
              name: 'Composite',
              className: 'EncryptionIdentity'
            }
          },
          defaultGroup: {
            required: false,
            serializedName: 'properties.defaultGroup',
            type: {
              name: 'String'
            }
          },
          encryptionConfig: {
            required: false,
            serializedName: 'properties.encryptionConfig',
            type: {
              name: 'Composite',
              className: 'EncryptionConfig'
            }
          },
          encryptionState: {
            required: false,
            serializedName: 'properties.encryptionState',
            type: {
              name: 'Enum',
              allowedValues: [ 'Enabled', 'Disabled' ]
            }
          },
          firewallRules: {
            required: false,
            serializedName: 'properties.firewallRules',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'CreateFirewallRuleWithAccountParametersElementType',
                  type: {
                    name: 'Composite',
                    className: 'CreateFirewallRuleWithAccountParameters'
                  }
              }
            }
          },
          virtualNetworkRules: {
            required: false,
            serializedName: 'properties.virtualNetworkRules',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'CreateVirtualNetworkRuleWithAccountParametersElementType',
                  type: {
                    name: 'Composite',
                    className: 'CreateVirtualNetworkRuleWithAccountParameters'
                  }
              }
            }
          },
          firewallState: {
            required: false,
            serializedName: 'properties.firewallState',
            type: {
              name: 'Enum',
              allowedValues: [ 'Enabled', 'Disabled' ]
            }
          },
          firewallAllowAzureIps: {
            required: false,
            serializedName: 'properties.firewallAllowAzureIps',
            type: {
              name: 'Enum',
              allowedValues: [ 'Enabled', 'Disabled' ]
            }
          },
          trustedIdProviders: {
            required: false,
            serializedName: 'properties.trustedIdProviders',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'CreateTrustedIdProviderWithAccountParametersElementType',
                  type: {
                    name: 'Composite',
                    className: 'CreateTrustedIdProviderWithAccountParameters'
                  }
              }
            }
          },
          trustedIdProviderState: {
            required: false,
            serializedName: 'properties.trustedIdProviderState',
            type: {
              name: 'Enum',
              allowedValues: [ 'Enabled', 'Disabled' ]
            }
          },
          newTier: {
            required: false,
            serializedName: 'properties.newTier',
            type: {
              name: 'Enum',
              allowedValues: [ 'Consumption', 'Commitment_1TB', 'Commitment_10TB', 'Commitment_100TB', 'Commitment_500TB', 'Commitment_1PB', 'Commitment_5PB' ]
            }
          }
        }
      }
    };
  }
}

module.exports = CreateDataLakeStoreAccountParameters;
