/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Controllers
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevSpacesManagementClient.
 */
export interface Controllers {


    /**
     * @summary Gets an Azure Dev Spaces Controller.
     *
     * Gets the properties for an Azure Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Controller>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Controller>>;

    /**
     * @summary Gets an Azure Dev Spaces Controller.
     *
     * Gets the properties for an Azure Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Controller} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Controller} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Controller} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Controller>;
    get(resourceGroupName: string, name: string, callback: ServiceCallback<models.Controller>): void;
    get(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Controller>): void;


    /**
     * @summary Creates an Azure Dev Spaces Controller.
     *
     * Creates an Azure Dev Spaces Controller with the specified create parameters.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} controller Controller create parameters.
     *
     * @param {string} controller.hostSuffix DNS suffix for public endpoints
     * running in the Azure Dev Spaces Controller.
     *
     * @param {string} controller.targetContainerHostResourceId Resource ID of the
     * target container host
     *
     * @param {string} controller.targetContainerHostCredentialsBase64 Credentials
     * of the target container host (base64).
     *
     * @param {object} controller.sku
     *
     * @param {string} [controller.sku.tier] The tier of the SKU for Azure Dev
     * Spaces Controller. Possible values include: 'Standard'
     *
     * @param {object} [controller.tags] Tags for the Azure resource.
     *
     * @param {string} [controller.location] Region where the Azure resource is
     * located.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Controller>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, name: string, controller: models.Controller, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Controller>>;

    /**
     * @summary Creates an Azure Dev Spaces Controller.
     *
     * Creates an Azure Dev Spaces Controller with the specified create parameters.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} controller Controller create parameters.
     *
     * @param {string} controller.hostSuffix DNS suffix for public endpoints
     * running in the Azure Dev Spaces Controller.
     *
     * @param {string} controller.targetContainerHostResourceId Resource ID of the
     * target container host
     *
     * @param {string} controller.targetContainerHostCredentialsBase64 Credentials
     * of the target container host (base64).
     *
     * @param {object} controller.sku
     *
     * @param {string} [controller.sku.tier] The tier of the SKU for Azure Dev
     * Spaces Controller. Possible values include: 'Standard'
     *
     * @param {object} [controller.tags] Tags for the Azure resource.
     *
     * @param {string} [controller.location] Region where the Azure resource is
     * located.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Controller} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Controller} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Controller} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, name: string, controller: models.Controller, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Controller>;
    create(resourceGroupName: string, name: string, controller: models.Controller, callback: ServiceCallback<models.Controller>): void;
    create(resourceGroupName: string, name: string, controller: models.Controller, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Controller>): void;


    /**
     * @summary Deletes an Azure Dev Spaces Controller.
     *
     * Deletes an existing Azure Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Deletes an Azure Dev Spaces Controller.
     *
     * Deletes an existing Azure Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Updates an Azure Dev Spaces Controller.
     *
     * Updates the properties of an existing Azure Dev Spaces Controller with the
     * specified update parameters.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} controllerUpdateParameters
     *
     * @param {object} [controllerUpdateParameters.tags] Tags for the Azure Dev
     * Spaces Controller.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Controller>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, name: string, controllerUpdateParameters: models.ControllerUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Controller>>;

    /**
     * @summary Updates an Azure Dev Spaces Controller.
     *
     * Updates the properties of an existing Azure Dev Spaces Controller with the
     * specified update parameters.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} controllerUpdateParameters
     *
     * @param {object} [controllerUpdateParameters.tags] Tags for the Azure Dev
     * Spaces Controller.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Controller} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Controller} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Controller} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, name: string, controllerUpdateParameters: models.ControllerUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Controller>;
    update(resourceGroupName: string, name: string, controllerUpdateParameters: models.ControllerUpdateParameters, callback: ServiceCallback<models.Controller>): void;
    update(resourceGroupName: string, name: string, controllerUpdateParameters: models.ControllerUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Controller>): void;


    /**
     * @summary Lists the Azure Dev Spaces Controllers in a resource group.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * specified resource group and subscription.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ControllerList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ControllerList>>;

    /**
     * @summary Lists the Azure Dev Spaces Controllers in a resource group.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * specified resource group and subscription.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ControllerList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ControllerList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ControllerList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ControllerList>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.ControllerList>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ControllerList>): void;


    /**
     * @summary Lists the Azure Dev Spaces Controllers in a subscription.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ControllerList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ControllerList>>;

    /**
     * @summary Lists the Azure Dev Spaces Controllers in a subscription.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ControllerList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ControllerList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ControllerList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ControllerList>;
    list(callback: ServiceCallback<models.ControllerList>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ControllerList>): void;


    /**
     * @summary Lists connection details for an Azure Dev Spaces Controller.
     *
     * Lists connection details for the underlying container resources of an Azure
     * Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ControllerConnectionDetailsList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listConnectionDetailsWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ControllerConnectionDetailsList>>;

    /**
     * @summary Lists connection details for an Azure Dev Spaces Controller.
     *
     * Lists connection details for the underlying container resources of an Azure
     * Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ControllerConnectionDetailsList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ControllerConnectionDetailsList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ControllerConnectionDetailsList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listConnectionDetails(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ControllerConnectionDetailsList>;
    listConnectionDetails(resourceGroupName: string, name: string, callback: ServiceCallback<models.ControllerConnectionDetailsList>): void;
    listConnectionDetails(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ControllerConnectionDetailsList>): void;


    /**
     * @summary Creates an Azure Dev Spaces Controller.
     *
     * Creates an Azure Dev Spaces Controller with the specified create parameters.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} controller Controller create parameters.
     *
     * @param {string} controller.hostSuffix DNS suffix for public endpoints
     * running in the Azure Dev Spaces Controller.
     *
     * @param {string} controller.targetContainerHostResourceId Resource ID of the
     * target container host
     *
     * @param {string} controller.targetContainerHostCredentialsBase64 Credentials
     * of the target container host (base64).
     *
     * @param {object} controller.sku
     *
     * @param {string} [controller.sku.tier] The tier of the SKU for Azure Dev
     * Spaces Controller. Possible values include: 'Standard'
     *
     * @param {object} [controller.tags] Tags for the Azure resource.
     *
     * @param {string} [controller.location] Region where the Azure resource is
     * located.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Controller>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, name: string, controller: models.Controller, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Controller>>;

    /**
     * @summary Creates an Azure Dev Spaces Controller.
     *
     * Creates an Azure Dev Spaces Controller with the specified create parameters.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} controller Controller create parameters.
     *
     * @param {string} controller.hostSuffix DNS suffix for public endpoints
     * running in the Azure Dev Spaces Controller.
     *
     * @param {string} controller.targetContainerHostResourceId Resource ID of the
     * target container host
     *
     * @param {string} controller.targetContainerHostCredentialsBase64 Credentials
     * of the target container host (base64).
     *
     * @param {object} controller.sku
     *
     * @param {string} [controller.sku.tier] The tier of the SKU for Azure Dev
     * Spaces Controller. Possible values include: 'Standard'
     *
     * @param {object} [controller.tags] Tags for the Azure resource.
     *
     * @param {string} [controller.location] Region where the Azure resource is
     * located.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Controller} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Controller} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Controller} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, name: string, controller: models.Controller, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Controller>;
    beginCreate(resourceGroupName: string, name: string, controller: models.Controller, callback: ServiceCallback<models.Controller>): void;
    beginCreate(resourceGroupName: string, name: string, controller: models.Controller, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Controller>): void;


    /**
     * @summary Deletes an Azure Dev Spaces Controller.
     *
     * Deletes an existing Azure Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Deletes an Azure Dev Spaces Controller.
     *
     * Deletes an existing Azure Dev Spaces Controller.
     *
     * @param {string} resourceGroupName Resource group to which the resource
     * belongs.
     *
     * @param {string} name Name of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, name: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Lists the Azure Dev Spaces Controllers in a resource group.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * specified resource group and subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ControllerList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ControllerList>>;

    /**
     * @summary Lists the Azure Dev Spaces Controllers in a resource group.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * specified resource group and subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ControllerList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ControllerList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ControllerList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ControllerList>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.ControllerList>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ControllerList>): void;


    /**
     * @summary Lists the Azure Dev Spaces Controllers in a subscription.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ControllerList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ControllerList>>;

    /**
     * @summary Lists the Azure Dev Spaces Controllers in a subscription.
     *
     * Lists all the Azure Dev Spaces Controllers with their properties in the
     * subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ControllerList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ControllerList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ControllerList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ControllerList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ControllerList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ControllerList>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DevSpacesManagementClient.
 */
export interface Operations {


    /**
     * @summary Lists operations for the resource provider.
     *
     * Lists all the supported operations by the Microsoft.DevSpaces resource
     * provider along with their description.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ResourceProviderOperationList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ResourceProviderOperationList>>;

    /**
     * @summary Lists operations for the resource provider.
     *
     * Lists all the supported operations by the Microsoft.DevSpaces resource
     * provider along with their description.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ResourceProviderOperationList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ResourceProviderOperationList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ResourceProviderOperationList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ResourceProviderOperationList>;
    list(callback: ServiceCallback<models.ResourceProviderOperationList>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ResourceProviderOperationList>): void;


    /**
     * @summary Lists operations for the resource provider.
     *
     * Lists all the supported operations by the Microsoft.DevSpaces resource
     * provider along with their description.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ResourceProviderOperationList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ResourceProviderOperationList>>;

    /**
     * @summary Lists operations for the resource provider.
     *
     * Lists all the supported operations by the Microsoft.DevSpaces resource
     * provider along with their description.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ResourceProviderOperationList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ResourceProviderOperationList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ResourceProviderOperationList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ResourceProviderOperationList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ResourceProviderOperationList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ResourceProviderOperationList>): void;
}
