/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Update Properties of the Domain Service.
 *
 */
class DomainServicePatchProperties {
  /**
   * Create a DomainServicePatchProperties.
   * @member {object} [ldapsSettings] Secure LDAP Settings
   * @member {string} [ldapsSettings.ldaps] A flag to determine whether or not
   * Secure LDAP is enabled or disabled. Possible values include: 'Enabled',
   * 'Disabled'
   * @member {string} [ldapsSettings.pfxCertificate] The certificate required
   * to configure Secure LDAP. The parameter passed here should be a
   * base64encoded representation of the certificate pfx file.
   * @member {string} [ldapsSettings.pfxCertificatePassword] The password to
   * decrypt the provided Secure LDAP certificate pfx file.
   * @member {string} [ldapsSettings.publicCertificate] Public certificate used
   * to configure secure ldap.
   * @member {string} [ldapsSettings.certificateThumbprint] Thumbprint of
   * configure ldaps certificate.
   * @member {date} [ldapsSettings.certificateNotAfter] NotAfter DateTime of
   * configure ldaps certificate.
   * @member {string} [ldapsSettings.externalAccess] A flag to determine
   * whether or not Secure LDAP access over the internet is enabled or
   * disabled. Possible values include: 'Enabled', 'Disabled'
   * @member {string} [ldapsSettings.externalAccessIpAddress] External access
   * ip address.
   * @member {object} [notificationSettings] Notification Settings
   * @member {string} [notificationSettings.notifyGlobalAdmins] Should global
   * admins be notified. Possible values include: 'Enabled', 'Disabled'
   * @member {string} [notificationSettings.notifyDcAdmins] Should domain
   * controller admins be notified. Possible values include: 'Enabled',
   * 'Disabled'
   * @member {array} [notificationSettings.additionalRecipients] The list of
   * additional recipients
   * @member {object} [domainSecuritySettings] DomainSecurity Settings
   * @member {string} [domainSecuritySettings.ntlmV1] A flag to determine
   * whether or not NtlmV1 is enabled or disabled. Possible values include:
   * 'Enabled', 'Disabled'
   * @member {string} [domainSecuritySettings.tlsV1] A flag to determine
   * whether or not TlsV1 is enabled or disabled. Possible values include:
   * 'Enabled', 'Disabled'
   * @member {string} [domainSecuritySettings.syncNtlmPasswords] A flag to
   * determine whether or not SyncNtlmPasswords is enabled or disabled.
   * Possible values include: 'Enabled', 'Disabled'
   * @member {string} [filteredSync] Enabled or Disabled flag to turn on
   * Group-based filtered sync. Possible values include: 'Enabled', 'Disabled'
   */
  constructor() {
  }

  /**
   * Defines the metadata of DomainServicePatchProperties
   *
   * @returns {object} metadata of DomainServicePatchProperties
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'DomainServicePatchProperties',
      type: {
        name: 'Composite',
        className: 'DomainServicePatchProperties',
        modelProperties: {
          ldapsSettings: {
            required: false,
            serializedName: 'ldapsSettings',
            type: {
              name: 'Composite',
              className: 'LdapsSettings'
            }
          },
          notificationSettings: {
            required: false,
            serializedName: 'notificationSettings',
            type: {
              name: 'Composite',
              className: 'NotificationSettings'
            }
          },
          domainSecuritySettings: {
            required: false,
            serializedName: 'domainSecuritySettings',
            type: {
              name: 'Composite',
              className: 'DomainSecuritySettings'
            }
          },
          filteredSync: {
            required: false,
            serializedName: 'filteredSync',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = DomainServicePatchProperties;
