/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the StorageBlobCreatedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for an
 * Microsoft.Storage.BlobCreated event.
 *
 * @member {string} [api] The name of the API/operation that triggered this
 * event.
 * @member {string} [clientRequestId] A request id provided by the client of
 * the storage API operation that triggered this event.
 * @member {string} [requestId] The request id generated by the Storage service
 * for the storage API operation that triggered this event.
 * @member {string} [eTag] The etag of the object at the time this event was
 * triggered.
 * @member {string} [contentType] The content type of the blob. This is the
 * same as what would be returned in the Content-Type header from the blob.
 * @member {number} [contentLength] The size of the blob in bytes. This is the
 * same as what would be returned in the Content-Length header from the blob.
 * @member {string} [blobType] The type of blob.
 * @member {string} [url] The path to the blob.
 * @member {string} [sequencer] An opaque string value representing the logical
 * sequence of events for any particular blob name. Users can use standard
 * string comparison to understand the relative sequence of two events on the
 * same blob name.
 * @member {object} [storageDiagnostics] For service use only. Diagnostic data
 * occasionally included by the Azure Storage service. This property should be
 * ignored by event consumers.
 */
export interface StorageBlobCreatedEventData {
  api?: string;
  clientRequestId?: string;
  requestId?: string;
  eTag?: string;
  contentType?: string;
  contentLength?: number;
  blobType?: string;
  url?: string;
  sequencer?: string;
  storageDiagnostics?: any;
}

/**
 * @class
 * Initializes a new instance of the StorageBlobDeletedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for an
 * Microsoft.Storage.BlobDeleted event.
 *
 * @member {string} [api] The name of the API/operation that triggered this
 * event.
 * @member {string} [clientRequestId] A request id provided by the client of
 * the storage API operation that triggered this event.
 * @member {string} [requestId] The request id generated by the Storage service
 * for the storage API operation that triggered this event.
 * @member {string} [contentType] The content type of the blob. This is the
 * same as what would be returned in the Content-Type header from the blob.
 * @member {string} [blobType] The type of blob.
 * @member {string} [url] The path to the blob.
 * @member {string} [sequencer] An opaque string value representing the logical
 * sequence of events for any particular blob name. Users can use standard
 * string comparison to understand the relative sequence of two events on the
 * same blob name.
 * @member {object} [storageDiagnostics] For service use only. Diagnostic data
 * occasionally included by the Azure Storage service. This property should be
 * ignored by event consumers.
 */
export interface StorageBlobDeletedEventData {
  api?: string;
  clientRequestId?: string;
  requestId?: string;
  contentType?: string;
  blobType?: string;
  url?: string;
  sequencer?: string;
  storageDiagnostics?: any;
}

/**
 * @class
 * Initializes a new instance of the EventHubCaptureFileCreatedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for an
 * Microsoft.EventHub.CaptureFileCreated event.
 *
 * @member {string} [fileurl] The path to the capture file.
 * @member {string} [fileType] The file type of the capture file.
 * @member {string} [partitionId] The shard ID.
 * @member {number} [sizeInBytes] The file size.
 * @member {number} [eventCount] The number of events in the file.
 * @member {number} [firstSequenceNumber] The smallest sequence number from the
 * queue.
 * @member {number} [lastSequenceNumber] The last sequence number from the
 * queue.
 * @member {date} [firstEnqueueTime] The first time from the queue.
 * @member {date} [lastEnqueueTime] The last time from the queue.
 */
export interface EventHubCaptureFileCreatedEventData {
  fileurl?: string;
  fileType?: string;
  partitionId?: string;
  sizeInBytes?: number;
  eventCount?: number;
  firstSequenceNumber?: number;
  lastSequenceNumber?: number;
  firstEnqueueTime?: Date;
  lastEnqueueTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the ResourceWriteSuccessData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Resources.ResourceWriteSuccess event. This is raised when a
 * resource create or update operation succeeds.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceWriteSuccessData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceWriteFailureData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Resources.ResourceWriteFailure event. This is raised when a
 * resource create or update operation fails.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceWriteFailureData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceWriteCancelData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Resources.ResourceWriteCancel event. This is raised when a
 * resource create or update operation is canceled.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceWriteCancelData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceDeleteSuccessData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Resources.ResourceDeleteSuccess event. This is raised when a
 * resource delete operation succeeds.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceDeleteSuccessData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceDeleteFailureData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Resources.ResourceDeleteFailure event. This is raised when a
 * resource delete operation fails.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceDeleteFailureData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceDeleteCancelData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for an
 * Microsoft.Resources.ResourceDeleteCancel event. This is raised when a
 * resource delete operation is canceled.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceDeleteCancelData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceActionSuccessData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Resources.ResourceActionSuccess event. This is raised when a
 * resource action operation succeeds.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceActionSuccessData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceActionFailureData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Resources.ResourceActionFailure event. This is raised when a
 * resource action operation fails.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceActionFailureData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceActionCancelData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for an
 * Microsoft.Resources.ResourceActionCancel event. This is raised when a
 * resource action operation is canceled.
 *
 * @member {string} [tenantId] The tenant ID of the resource.
 * @member {string} [subscriptionId] The subscription ID of the resource.
 * @member {string} [resourceGroup] The resource group of the resource.
 * @member {string} [resourceProvider] The resource provider performing the
 * operation.
 * @member {string} [resourceUri] The URI of the resource in the operation.
 * @member {string} [operationName] The operation that was performed.
 * @member {string} [status] The status of the operation.
 * @member {string} [authorization] The requested authorization for the
 * operation.
 * @member {string} [claims] The properties of the claims.
 * @member {string} [correlationId] An operation ID used for troubleshooting.
 * @member {string} [httpRequest] The details of the operation.
 */
export interface ResourceActionCancelData {
  tenantId?: string;
  subscriptionId?: string;
  resourceGroup?: string;
  resourceProvider?: string;
  resourceUri?: string;
  operationName?: string;
  status?: string;
  authorization?: string;
  claims?: string;
  correlationId?: string;
  httpRequest?: string;
}

/**
 * @class
 * Initializes a new instance of the EventGridEvent class.
 * @constructor
 * Properties of an event published to an Event Grid topic.
 *
 * @member {string} id An unique identifier for the event.
 * @member {string} [topic] The resource path of the event source.
 * @member {string} subject A resource path relative to the topic path.
 * @member {object} data Event data specific to the event type.
 * @member {string} eventType The type of the event that occurred.
 * @member {date} eventTime The time (in UTC) the event was generated.
 * @member {string} [metadataVersion] The schema version of the event metadata.
 * @member {string} dataVersion The schema version of the data object.
 */
export interface EventGridEvent {
  id: string;
  topic?: string;
  subject: string;
  data: any;
  eventType: string;
  eventTime: Date;
  readonly metadataVersion?: string;
  dataVersion: string;
}

/**
 * @class
 * Initializes a new instance of the SubscriptionValidationEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.EventGrid.SubscriptionValidationEvent.
 *
 * @member {string} [validationCode] The validation code sent by Azure Event
 * Grid to validate an event subscription. To complete the validation
 * handshake, the subscriber must either respond with this validation code as
 * part of the validation response, or perform a GET request on the
 * validationUrl (available starting version 2018-05-01-preview).
 * @member {string} [validationUrl] The validation URL sent by Azure Event Grid
 * (available starting version 2018-05-01-preview). To complete the validation
 * handshake, the subscriber must either respond with the validationCode as
 * part of the validation response, or perform a GET request on the
 * validationUrl (available starting version 2018-05-01-preview).
 */
export interface SubscriptionValidationEventData {
  readonly validationCode?: string;
  readonly validationUrl?: string;
}

/**
 * @class
 * Initializes a new instance of the SubscriptionValidationResponse class.
 * @constructor
 * To complete an event subscription validation handshake, a subscriber can use
 * either the validationCode or the validationUrl received in a
 * SubscriptionValidationEvent. When the validationCode is used, the
 * SubscriptionValidationResponse can be used to build the response.
 *
 * @member {string} [validationResponse] The validation response sent by the
 * subscriber to Azure Event Grid to complete the validation of an event
 * subscription.
 */
export interface SubscriptionValidationResponse {
  validationResponse?: string;
}

/**
 * @class
 * Initializes a new instance of the SubscriptionDeletedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.EventGrid.SubscriptionDeletedEvent.
 *
 * @member {string} [eventSubscriptionId] The Azure resource ID of the deleted
 * event subscription.
 */
export interface SubscriptionDeletedEventData {
  readonly eventSubscriptionId?: string;
}

/**
 * @class
 * Initializes a new instance of the DeviceLifeCycleEventProperties class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a device life cycle
 * event (DeviceCreated, DeviceDeleted).
 *
 * @member {string} [deviceId] The unique identifier of the device. This
 * case-sensitive string can be up to 128 characters long, and supports ASCII
 * 7-bit alphanumeric characters plus the following special characters: - : . +
 * % _ &#35; * ? ! ( ) , = @ ; $ '.
 * @member {string} [hubName] Name of the IoT Hub where the device was created
 * or deleted.
 * @member {object} [twin] Information about the device twin, which is the
 * cloud representation of application device metadata.
 * @member {string} [twin.authenticationType] Authentication type used for this
 * device: either SAS, SelfSigned, or CertificateAuthority.
 * @member {number} [twin.cloudToDeviceMessageCount] Count of cloud to device
 * messages sent to this device.
 * @member {string} [twin.connectionState] Whether the device is connected or
 * disconnected.
 * @member {string} [twin.deviceId] The unique identifier of the device twin.
 * @member {string} [twin.etag] A piece of information that describes the
 * content of the device twin. Each etag is guaranteed to be unique per device
 * twin.
 * @member {string} [twin.lastActivityTime] The ISO8601 timestamp of the last
 * activity.
 * @member {object} [twin.properties] Properties JSON element.
 * @member {object} [twin.properties.desired] A portion of the properties that
 * can be written only by the application back-end, and read by the device.
 * @member {object} [twin.properties.desired.metadata] Metadata information for
 * the properties JSON document.
 * @member {string} [twin.properties.desired.metadata.lastUpdated] The ISO8601
 * timestamp of the last time the properties were updated.
 * @member {number} [twin.properties.desired.version] Version of device twin
 * properties.
 * @member {object} [twin.properties.reported] A portion of the properties that
 * can be written only by the device, and read by the application back-end.
 * @member {object} [twin.properties.reported.metadata] Metadata information
 * for the properties JSON document.
 * @member {string} [twin.properties.reported.metadata.lastUpdated] The ISO8601
 * timestamp of the last time the properties were updated.
 * @member {number} [twin.properties.reported.version] Version of device twin
 * properties.
 * @member {string} [twin.status] Whether the device twin is enabled or
 * disabled.
 * @member {string} [twin.statusUpdateTime] The ISO8601 timestamp of the last
 * device twin status update.
 * @member {number} [twin.version] An integer that is incremented by one each
 * time the device twin is updated.
 * @member {object} [twin.x509Thumbprint] The thumbprint is a unique value for
 * the x509 certificate, commonly used to find a particular certificate in a
 * certificate store. The thumbprint is dynamically generated using the SHA1
 * algorithm, and does not physically exist in the certificate.
 * @member {string} [twin.x509Thumbprint.primaryThumbprint] Primary thumbprint
 * for the x509 certificate.
 * @member {string} [twin.x509Thumbprint.secondaryThumbprint] Secondary
 * thumbprint for the x509 certificate.
 */
export interface DeviceLifeCycleEventProperties {
  deviceId?: string;
  hubName?: string;
  twin?: DeviceTwinInfo;
}

/**
 * @class
 * Initializes a new instance of the IotHubDeviceCreatedEventData class.
 * @constructor
 * Event data for Microsoft.Devices.DeviceCreated event.
 *
 */
export interface IotHubDeviceCreatedEventData extends DeviceLifeCycleEventProperties {
}

/**
 * @class
 * Initializes a new instance of the IotHubDeviceDeletedEventData class.
 * @constructor
 * Event data for Microsoft.Devices.DeviceDeleted event.
 *
 */
export interface IotHubDeviceDeletedEventData extends DeviceLifeCycleEventProperties {
}

/**
 * @class
 * Initializes a new instance of the DeviceConnectionStateEventProperties class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a device connection
 * state event (DeviceConnected, DeviceDisconnected).
 *
 * @member {string} [deviceId] The unique identifier of the device. This
 * case-sensitive string can be up to 128 characters long, and supports ASCII
 * 7-bit alphanumeric characters plus the following special characters: - : . +
 * % _ &#35; * ? ! ( ) , = @ ; $ '.
 * @member {string} [moduleId] The unique identifier of the module. This
 * case-sensitive string can be up to 128 characters long, and supports ASCII
 * 7-bit alphanumeric characters plus the following special characters: - : . +
 * % _ &#35; * ? ! ( ) , = @ ; $ '.
 * @member {string} [hubName] Name of the IoT Hub where the device was created
 * or deleted.
 * @member {object} [deviceConnectionStateEventInfo] Information about the
 * device connection state event.
 * @member {string} [deviceConnectionStateEventInfo.sequenceNumber] Sequence
 * number is string representation of a hexadecimal number. string compare can
 * be used to identify the larger number because both in ASCII and HEX numbers
 * come after alphabets. If you are converting the string to hex, then the
 * number is a 256 bit number.
 */
export interface DeviceConnectionStateEventProperties {
  deviceId?: string;
  moduleId?: string;
  hubName?: string;
  deviceConnectionStateEventInfo?: DeviceConnectionStateEventInfo;
}

/**
 * @class
 * Initializes a new instance of the IotHubDeviceConnectedEventData class.
 * @constructor
 * Event data for Microsoft.Devices.DeviceConnected event.
 *
 */
export interface IotHubDeviceConnectedEventData extends DeviceConnectionStateEventProperties {
}

/**
 * @class
 * Initializes a new instance of the IotHubDeviceDisconnectedEventData class.
 * @constructor
 * Event data for Microsoft.Devices.DeviceDisconnected event.
 *
 */
export interface IotHubDeviceDisconnectedEventData extends DeviceConnectionStateEventProperties {
}

/**
 * @class
 * Initializes a new instance of the DeviceTelemetryEventProperties class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a device telemetry
 * event (DeviceTelemetry).
 *
 * @member {object} [body] The content of the message from the device.
 * @member {object} [properties] Application properties are user-defined
 * strings that can be added to the message. These fields are optional.
 * @member {object} [systemProperties] System properties help identify contents
 * and source of the messages.
 */
export interface DeviceTelemetryEventProperties {
  body?: any;
  properties?: { [propertyName: string]: string };
  systemProperties?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the IotHubDeviceTelemetryEventData class.
 * @constructor
 * Event data for Microsoft.Devices.DeviceTelemetry event.
 *
 */
export interface IotHubDeviceTelemetryEventData extends DeviceTelemetryEventProperties {
}

/**
 * @class
 * Initializes a new instance of the DeviceTwinMetadata class.
 * @constructor
 * Metadata information for the properties JSON document.
 *
 * @member {string} [lastUpdated] The ISO8601 timestamp of the last time the
 * properties were updated.
 */
export interface DeviceTwinMetadata {
  lastUpdated?: string;
}

/**
 * @class
 * Initializes a new instance of the DeviceTwinProperties class.
 * @constructor
 * A portion of the properties that can be written only by the application
 * back-end, and read by the device.
 *
 * @member {object} [metadata] Metadata information for the properties JSON
 * document.
 * @member {string} [metadata.lastUpdated] The ISO8601 timestamp of the last
 * time the properties were updated.
 * @member {number} [version] Version of device twin properties.
 */
export interface DeviceTwinProperties {
  metadata?: DeviceTwinMetadata;
  version?: number;
}

/**
 * @class
 * Initializes a new instance of the DeviceTwinInfoProperties class.
 * @constructor
 * Properties JSON element.
 *
 * @member {object} [desired] A portion of the properties that can be written
 * only by the application back-end, and read by the device.
 * @member {object} [desired.metadata] Metadata information for the properties
 * JSON document.
 * @member {string} [desired.metadata.lastUpdated] The ISO8601 timestamp of the
 * last time the properties were updated.
 * @member {number} [desired.version] Version of device twin properties.
 * @member {object} [reported] A portion of the properties that can be written
 * only by the device, and read by the application back-end.
 * @member {object} [reported.metadata] Metadata information for the properties
 * JSON document.
 * @member {string} [reported.metadata.lastUpdated] The ISO8601 timestamp of
 * the last time the properties were updated.
 * @member {number} [reported.version] Version of device twin properties.
 */
export interface DeviceTwinInfoProperties {
  desired?: DeviceTwinProperties;
  reported?: DeviceTwinProperties;
}

/**
 * @class
 * Initializes a new instance of the DeviceTwinInfoX509Thumbprint class.
 * @constructor
 * The thumbprint is a unique value for the x509 certificate, commonly used to
 * find a particular certificate in a certificate store. The thumbprint is
 * dynamically generated using the SHA1 algorithm, and does not physically
 * exist in the certificate.
 *
 * @member {string} [primaryThumbprint] Primary thumbprint for the x509
 * certificate.
 * @member {string} [secondaryThumbprint] Secondary thumbprint for the x509
 * certificate.
 */
export interface DeviceTwinInfoX509Thumbprint {
  primaryThumbprint?: string;
  secondaryThumbprint?: string;
}

/**
 * @class
 * Initializes a new instance of the DeviceTwinInfo class.
 * @constructor
 * Information about the device twin, which is the cloud representation of
 * application device metadata.
 *
 * @member {string} [authenticationType] Authentication type used for this
 * device: either SAS, SelfSigned, or CertificateAuthority.
 * @member {number} [cloudToDeviceMessageCount] Count of cloud to device
 * messages sent to this device.
 * @member {string} [connectionState] Whether the device is connected or
 * disconnected.
 * @member {string} [deviceId] The unique identifier of the device twin.
 * @member {string} [etag] A piece of information that describes the content of
 * the device twin. Each etag is guaranteed to be unique per device twin.
 * @member {string} [lastActivityTime] The ISO8601 timestamp of the last
 * activity.
 * @member {object} [properties] Properties JSON element.
 * @member {object} [properties.desired] A portion of the properties that can
 * be written only by the application back-end, and read by the device.
 * @member {object} [properties.desired.metadata] Metadata information for the
 * properties JSON document.
 * @member {string} [properties.desired.metadata.lastUpdated] The ISO8601
 * timestamp of the last time the properties were updated.
 * @member {number} [properties.desired.version] Version of device twin
 * properties.
 * @member {object} [properties.reported] A portion of the properties that can
 * be written only by the device, and read by the application back-end.
 * @member {object} [properties.reported.metadata] Metadata information for the
 * properties JSON document.
 * @member {string} [properties.reported.metadata.lastUpdated] The ISO8601
 * timestamp of the last time the properties were updated.
 * @member {number} [properties.reported.version] Version of device twin
 * properties.
 * @member {string} [status] Whether the device twin is enabled or disabled.
 * @member {string} [statusUpdateTime] The ISO8601 timestamp of the last device
 * twin status update.
 * @member {number} [version] An integer that is incremented by one each time
 * the device twin is updated.
 * @member {object} [x509Thumbprint] The thumbprint is a unique value for the
 * x509 certificate, commonly used to find a particular certificate in a
 * certificate store. The thumbprint is dynamically generated using the SHA1
 * algorithm, and does not physically exist in the certificate.
 * @member {string} [x509Thumbprint.primaryThumbprint] Primary thumbprint for
 * the x509 certificate.
 * @member {string} [x509Thumbprint.secondaryThumbprint] Secondary thumbprint
 * for the x509 certificate.
 */
export interface DeviceTwinInfo {
  authenticationType?: string;
  cloudToDeviceMessageCount?: number;
  connectionState?: string;
  deviceId?: string;
  etag?: string;
  lastActivityTime?: string;
  properties?: DeviceTwinInfoProperties;
  status?: string;
  statusUpdateTime?: string;
  version?: number;
  x509Thumbprint?: DeviceTwinInfoX509Thumbprint;
}

/**
 * @class
 * Initializes a new instance of the DeviceConnectionStateEventInfo class.
 * @constructor
 * Information about the device connection state event.
 *
 * @member {string} [sequenceNumber] Sequence number is string representation
 * of a hexadecimal number. string compare can be used to identify the larger
 * number because both in ASCII and HEX numbers come after alphabets. If you
 * are converting the string to hex, then the number is a 256 bit number.
 */
export interface DeviceConnectionStateEventInfo {
  sequenceNumber?: string;
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryEventData class.
 * @constructor
 * The content of the event request message.
 *
 * @member {string} [id] The event ID.
 * @member {date} [timestamp] The time at which the event occurred.
 * @member {string} [action] The action that encompasses the provided event.
 * @member {object} [target] The target of the event.
 * @member {string} [target.mediaType] The MIME type of the referenced object.
 * @member {number} [target.size] The number of bytes of the content. Same as
 * Length field.
 * @member {string} [target.digest] The digest of the content, as defined by
 * the Registry V2 HTTP API Specification.
 * @member {number} [target.length] The number of bytes of the content. Same as
 * Size field.
 * @member {string} [target.repository] The repository name.
 * @member {string} [target.url] The direct URL to the content.
 * @member {string} [target.tag] The tag name.
 * @member {object} [request] The request that generated the event.
 * @member {string} [request.id] The ID of the request that initiated the
 * event.
 * @member {string} [request.addr] The IP or hostname and possibly port of the
 * client connection that initiated the event. This is the RemoteAddr from the
 * standard http request.
 * @member {string} [request.host] The externally accessible hostname of the
 * registry instance, as specified by the http host header on incoming
 * requests.
 * @member {string} [request.method] The request method that generated the
 * event.
 * @member {string} [request.useragent] The user agent header of the request.
 * @member {object} [actor] The agent that initiated the event. For most
 * situations, this could be from the authorization context of the request.
 * @member {string} [actor.name] The subject or username associated with the
 * request context that generated the event.
 * @member {object} [source] The registry node that generated the event. Put
 * differently, while the actor initiates the event, the source generates it.
 * @member {string} [source.addr] The IP or hostname and the port of the
 * registry node that generated the event. Generally, this will be resolved by
 * os.Hostname() along with the running port.
 * @member {string} [source.instanceID] The running instance of an application.
 * Changes after each restart.
 */
export interface ContainerRegistryEventData {
  id?: string;
  timestamp?: Date;
  action?: string;
  target?: ContainerRegistryEventTarget;
  request?: ContainerRegistryEventRequest;
  actor?: ContainerRegistryEventActor;
  source?: ContainerRegistryEventSource;
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryImagePushedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.ContainerRegistry.ImagePushed event.
 *
 */
export interface ContainerRegistryImagePushedEventData extends ContainerRegistryEventData {
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryImageDeletedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.ContainerRegistry.ImageDeleted event.
 *
 */
export interface ContainerRegistryImageDeletedEventData extends ContainerRegistryEventData {
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryArtifactEventData class.
 * @constructor
 * The content of the event request message.
 *
 * @member {string} [id] The event ID.
 * @member {date} [timestamp] The time at which the event occurred.
 * @member {string} [action] The action that encompasses the provided event.
 * @member {object} [target] The target of the event.
 * @member {string} [target.mediaType] The MIME type of the artifact.
 * @member {number} [target.size] The size in bytes of the artifact.
 * @member {string} [target.digest] The digest of the artifact.
 * @member {string} [target.repository] The repository name of the artifact.
 * @member {string} [target.tag] The tag of the artifact.
 * @member {string} [target.name] The name of the artifact.
 * @member {string} [target.version] The version of the artifact.
 */
export interface ContainerRegistryArtifactEventData {
  id?: string;
  timestamp?: Date;
  action?: string;
  target?: ContainerRegistryArtifactEventTarget;
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryChartPushedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.ContainerRegistry.ChartPushed event.
 *
 */
export interface ContainerRegistryChartPushedEventData extends ContainerRegistryArtifactEventData {
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryChartDeletedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.ContainerRegistry.ChartDeleted event.
 *
 */
export interface ContainerRegistryChartDeletedEventData extends ContainerRegistryArtifactEventData {
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryEventTarget class.
 * @constructor
 * The target of the event.
 *
 * @member {string} [mediaType] The MIME type of the referenced object.
 * @member {number} [size] The number of bytes of the content. Same as Length
 * field.
 * @member {string} [digest] The digest of the content, as defined by the
 * Registry V2 HTTP API Specification.
 * @member {number} [length] The number of bytes of the content. Same as Size
 * field.
 * @member {string} [repository] The repository name.
 * @member {string} [url] The direct URL to the content.
 * @member {string} [tag] The tag name.
 */
export interface ContainerRegistryEventTarget {
  mediaType?: string;
  size?: number;
  digest?: string;
  length?: number;
  repository?: string;
  url?: string;
  tag?: string;
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryEventRequest class.
 * @constructor
 * The request that generated the event.
 *
 * @member {string} [id] The ID of the request that initiated the event.
 * @member {string} [addr] The IP or hostname and possibly port of the client
 * connection that initiated the event. This is the RemoteAddr from the
 * standard http request.
 * @member {string} [host] The externally accessible hostname of the registry
 * instance, as specified by the http host header on incoming requests.
 * @member {string} [method] The request method that generated the event.
 * @member {string} [useragent] The user agent header of the request.
 */
export interface ContainerRegistryEventRequest {
  id?: string;
  addr?: string;
  host?: string;
  method?: string;
  useragent?: string;
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryEventActor class.
 * @constructor
 * The agent that initiated the event. For most situations, this could be from
 * the authorization context of the request.
 *
 * @member {string} [name] The subject or username associated with the request
 * context that generated the event.
 */
export interface ContainerRegistryEventActor {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryEventSource class.
 * @constructor
 * The registry node that generated the event. Put differently, while the actor
 * initiates the event, the source generates it.
 *
 * @member {string} [addr] The IP or hostname and the port of the registry node
 * that generated the event. Generally, this will be resolved by os.Hostname()
 * along with the running port.
 * @member {string} [instanceID] The running instance of an application.
 * Changes after each restart.
 */
export interface ContainerRegistryEventSource {
  addr?: string;
  instanceID?: string;
}

/**
 * @class
 * Initializes a new instance of the ContainerRegistryArtifactEventTarget class.
 * @constructor
 * The target of the event.
 *
 * @member {string} [mediaType] The MIME type of the artifact.
 * @member {number} [size] The size in bytes of the artifact.
 * @member {string} [digest] The digest of the artifact.
 * @member {string} [repository] The repository name of the artifact.
 * @member {string} [tag] The tag of the artifact.
 * @member {string} [name] The name of the artifact.
 * @member {string} [version] The version of the artifact.
 */
export interface ContainerRegistryArtifactEventTarget {
  mediaType?: string;
  size?: number;
  digest?: string;
  repository?: string;
  tag?: string;
  name?: string;
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the ServiceBusActiveMessagesAvailableWithNoListenersEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.ServiceBus.ActiveMessagesAvailableWithNoListeners event.
 *
 * @member {string} [namespaceName] The namespace name of the
 * Microsoft.ServiceBus resource.
 * @member {string} [requestUri] The endpoint of the Microsoft.ServiceBus
 * resource.
 * @member {string} [entityType] The entity type of the Microsoft.ServiceBus
 * resource. Could be one of 'queue' or 'subscriber'.
 * @member {string} [queueName] The name of the Microsoft.ServiceBus queue. If
 * the entity type is of type 'subscriber', then this value will be null.
 * @member {string} [topicName] The name of the Microsoft.ServiceBus topic. If
 * the entity type is of type 'queue', then this value will be null.
 * @member {string} [subscriptionName] The name of the Microsoft.ServiceBus
 * topic's subscription. If the entity type is of type 'queue', then this value
 * will be null.
 */
export interface ServiceBusActiveMessagesAvailableWithNoListenersEventData {
  namespaceName?: string;
  requestUri?: string;
  entityType?: string;
  queueName?: string;
  topicName?: string;
  subscriptionName?: string;
}

/**
 * @class
 * Initializes a new instance of the ServiceBusDeadletterMessagesAvailableWithNoListenersEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.ServiceBus.DeadletterMessagesAvailableWithNoListenersEvent event.
 *
 * @member {string} [namespaceName] The namespace name of the
 * Microsoft.ServiceBus resource.
 * @member {string} [requestUri] The endpoint of the Microsoft.ServiceBus
 * resource.
 * @member {string} [entityType] The entity type of the Microsoft.ServiceBus
 * resource. Could be one of 'queue' or 'subscriber'.
 * @member {string} [queueName] The name of the Microsoft.ServiceBus queue. If
 * the entity type is of type 'subscriber', then this value will be null.
 * @member {string} [topicName] The name of the Microsoft.ServiceBus topic. If
 * the entity type is of type 'queue', then this value will be null.
 * @member {string} [subscriptionName] The name of the Microsoft.ServiceBus
 * topic's subscription. If the entity type is of type 'queue', then this value
 * will be null.
 */
export interface ServiceBusDeadletterMessagesAvailableWithNoListenersEventData {
  namespaceName?: string;
  requestUri?: string;
  entityType?: string;
  queueName?: string;
  topicName?: string;
  subscriptionName?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaJobStateChangeEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Media.JobStateChange event.
 *
 * @member {string} [previousState] The previous state of the Job. Possible
 * values include: 'Canceled', 'Canceling', 'Error', 'Finished', 'Processing',
 * 'Queued', 'Scheduled'
 * @member {string} [state] The new state of the Job. Possible values include:
 * 'Canceled', 'Canceling', 'Error', 'Finished', 'Processing', 'Queued',
 * 'Scheduled'
 * @member {object} [correlationData] Gets the Job correlation data.
 */
export interface MediaJobStateChangeEventData {
  readonly previousState?: string;
  readonly state?: string;
  correlationData?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the MediaJobErrorDetail class.
 * @constructor
 * Details of JobOutput errors.
 *
 * @member {string} [code] Code describing the error detail.
 * @member {string} [message] A human-readable representation of the error.
 */
export interface MediaJobErrorDetail {
  readonly code?: string;
  readonly message?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaJobError class.
 * @constructor
 * Details of JobOutput errors.
 *
 * @member {string} [code] Error code describing the error. Possible values
 * include: 'ServiceError', 'ServiceTransientError', 'DownloadNotAccessible',
 * 'DownloadTransientError', 'UploadNotAccessible', 'UploadTransientError',
 * 'ConfigurationUnsupported', 'ContentMalformed', 'ContentUnsupported'
 * @member {string} [message] A human-readable language-dependent
 * representation of the error.
 * @member {string} [category] Helps with categorization of errors. Possible
 * values include: 'Service', 'Download', 'Upload', 'Configuration', 'Content'
 * @member {string} [retry] Indicates that it may be possible to retry the Job.
 * If retry is unsuccessful, please contact Azure support via Azure Portal.
 * Possible values include: 'DoNotRetry', 'MayRetry'
 * @member {array} [details] An array of details about specific errors that led
 * to this reported error.
 */
export interface MediaJobError {
  readonly code?: string;
  readonly message?: string;
  readonly category?: string;
  readonly retry?: string;
  readonly details?: MediaJobErrorDetail[];
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutput class.
 * @constructor
 * The event data for a Job output.
 *
 * @member {object} [error] Gets the Job output error.
 * @member {string} [error.code] Error code describing the error. Possible
 * values include: 'ServiceError', 'ServiceTransientError',
 * 'DownloadNotAccessible', 'DownloadTransientError', 'UploadNotAccessible',
 * 'UploadTransientError', 'ConfigurationUnsupported', 'ContentMalformed',
 * 'ContentUnsupported'
 * @member {string} [error.message] A human-readable language-dependent
 * representation of the error.
 * @member {string} [error.category] Helps with categorization of errors.
 * Possible values include: 'Service', 'Download', 'Upload', 'Configuration',
 * 'Content'
 * @member {string} [error.retry] Indicates that it may be possible to retry
 * the Job. If retry is unsuccessful, please contact Azure support via Azure
 * Portal. Possible values include: 'DoNotRetry', 'MayRetry'
 * @member {array} [error.details] An array of details about specific errors
 * that led to this reported error.
 * @member {string} [label] Gets the Job output label.
 * @member {number} progress Gets the Job output progress.
 * @member {string} state Gets the Job output state. Possible values include:
 * 'Canceled', 'Canceling', 'Error', 'Finished', 'Processing', 'Queued',
 * 'Scheduled'
 * @member {string} odatatype Polymorphic Discriminator
 */
export interface MediaJobOutput {
  error?: MediaJobError;
  label?: string;
  progress: number;
  state: string;
  odatatype: string;
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputAsset class.
 * @constructor
 * The event data for a Job output asset.
 *
 * @member {string} [assetName] Gets the Job output asset name.
 */
export interface MediaJobOutputAsset extends MediaJobOutput {
  assetName?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputProgressEventData class.
 * @constructor
 * Job Output Progress Event Data.
 *
 * @member {string} [label] Gets the Job output label.
 * @member {number} [progress] Gets the Job output progress.
 * @member {object} [jobCorrelationData] Gets the Job correlation data.
 */
export interface MediaJobOutputProgressEventData {
  label?: string;
  progress?: number;
  jobCorrelationData?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputStateChangeEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Media.JobOutputStateChange event.
 *
 * @member {string} [previousState] The previous state of the Job. Possible
 * values include: 'Canceled', 'Canceling', 'Error', 'Finished', 'Processing',
 * 'Queued', 'Scheduled'
 * @member {object} [output] Gets the output.
 * @member {object} [output.error] Gets the Job output error.
 * @member {string} [output.error.code] Error code describing the error.
 * Possible values include: 'ServiceError', 'ServiceTransientError',
 * 'DownloadNotAccessible', 'DownloadTransientError', 'UploadNotAccessible',
 * 'UploadTransientError', 'ConfigurationUnsupported', 'ContentMalformed',
 * 'ContentUnsupported'
 * @member {string} [output.error.message] A human-readable language-dependent
 * representation of the error.
 * @member {string} [output.error.category] Helps with categorization of
 * errors. Possible values include: 'Service', 'Download', 'Upload',
 * 'Configuration', 'Content'
 * @member {string} [output.error.retry] Indicates that it may be possible to
 * retry the Job. If retry is unsuccessful, please contact Azure support via
 * Azure Portal. Possible values include: 'DoNotRetry', 'MayRetry'
 * @member {array} [output.error.details] An array of details about specific
 * errors that led to this reported error.
 * @member {string} [output.label] Gets the Job output label.
 * @member {number} [output.progress] Gets the Job output progress.
 * @member {string} [output.state] Gets the Job output state. Possible values
 * include: 'Canceled', 'Canceling', 'Error', 'Finished', 'Processing',
 * 'Queued', 'Scheduled'
 * @member {string} [output.odatatype] Polymorphic Discriminator
 * @member {object} [jobCorrelationData] Gets the Job correlation data.
 */
export interface MediaJobOutputStateChangeEventData {
  readonly previousState?: string;
  output?: MediaJobOutput;
  jobCorrelationData?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the MediaJobScheduledEventData class.
 * @constructor
 * Job scheduled event data
 *
 */
export interface MediaJobScheduledEventData extends MediaJobStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobProcessingEventData class.
 * @constructor
 * Job processing event data
 *
 */
export interface MediaJobProcessingEventData extends MediaJobStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobCancelingEventData class.
 * @constructor
 * Job canceling event data
 *
 */
export interface MediaJobCancelingEventData extends MediaJobStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobFinishedEventData class.
 * @constructor
 * Job finished event data
 *
 * @member {array} [outputs] Gets the Job outputs.
 */
export interface MediaJobFinishedEventData extends MediaJobStateChangeEventData {
  outputs?: MediaJobOutput[];
}

/**
 * @class
 * Initializes a new instance of the MediaJobCanceledEventData class.
 * @constructor
 * Job canceled event data
 *
 * @member {array} [outputs] Gets the Job outputs.
 */
export interface MediaJobCanceledEventData extends MediaJobStateChangeEventData {
  outputs?: MediaJobOutput[];
}

/**
 * @class
 * Initializes a new instance of the MediaJobErroredEventData class.
 * @constructor
 * Job error state event data
 *
 * @member {array} [outputs] Gets the Job outputs.
 */
export interface MediaJobErroredEventData extends MediaJobStateChangeEventData {
  outputs?: MediaJobOutput[];
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputCanceledEventData class.
 * @constructor
 * Job output canceled event data
 *
 */
export interface MediaJobOutputCanceledEventData extends MediaJobOutputStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputCancelingEventData class.
 * @constructor
 * Job output canceling event data
 *
 */
export interface MediaJobOutputCancelingEventData extends MediaJobOutputStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputErroredEventData class.
 * @constructor
 * Job output error event data
 *
 */
export interface MediaJobOutputErroredEventData extends MediaJobOutputStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputFinishedEventData class.
 * @constructor
 * Job output finished event data
 *
 */
export interface MediaJobOutputFinishedEventData extends MediaJobOutputStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputProcessingEventData class.
 * @constructor
 * Job output processing event data
 *
 */
export interface MediaJobOutputProcessingEventData extends MediaJobOutputStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaJobOutputScheduledEventData class.
 * @constructor
 * Job output scheduled event data
 *
 */
export interface MediaJobOutputScheduledEventData extends MediaJobOutputStateChangeEventData {
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventEncoderConnectedEventData class.
 * @constructor
 * Encoder connect event data.
 *
 * @member {string} [ingestUrl] Gets the ingest URL provided by the live event.
 * @member {string} [streamId] Gets the stream Id.
 * @member {string} [encoderIp] Gets the remote IP.
 * @member {string} [encoderPort] Gets the remote port.
 */
export interface MediaLiveEventEncoderConnectedEventData {
  readonly ingestUrl?: string;
  readonly streamId?: string;
  readonly encoderIp?: string;
  readonly encoderPort?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventConnectionRejectedEventData class.
 * @constructor
 * Encoder connection rejected event data.
 *
 * @member {string} [ingestUrl] Gets the ingest URL provided by the live event.
 * @member {string} [streamId] Gets the stream Id.
 * @member {string} [encoderIp] Gets the remote IP.
 * @member {string} [encoderPort] Gets the remote port.
 * @member {string} [resultCode] Gets the result code.
 */
export interface MediaLiveEventConnectionRejectedEventData {
  readonly ingestUrl?: string;
  readonly streamId?: string;
  readonly encoderIp?: string;
  readonly encoderPort?: string;
  readonly resultCode?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventEncoderDisconnectedEventData class.
 * @constructor
 * Encoder disconnected event data.
 *
 * @member {string} [ingestUrl] Gets the ingest URL provided by the live event.
 * @member {string} [streamId] Gets the stream Id.
 * @member {string} [encoderIp] Gets the remote IP.
 * @member {string} [encoderPort] Gets the remote port.
 * @member {string} [resultCode] Gets the result code.
 */
export interface MediaLiveEventEncoderDisconnectedEventData {
  readonly ingestUrl?: string;
  readonly streamId?: string;
  readonly encoderIp?: string;
  readonly encoderPort?: string;
  readonly resultCode?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventIncomingStreamReceivedEventData class.
 * @constructor
 * Encoder connect event data.
 *
 * @member {string} [ingestUrl] Gets the ingest URL provided by the live event.
 * @member {string} [trackType] Gets the type of the track (Audio / Video).
 * @member {string} [trackName] Gets the track name.
 * @member {number} [bitrate] Gets the bitrate of the track.
 * @member {string} [encoderIp] Gets the remote IP.
 * @member {string} [encoderPort] Gets the remote port.
 * @member {string} [timestamp] Gets the first timestamp of the data chunk
 * received.
 * @member {string} [duration] Gets the duration of the first data chunk.
 * @member {string} [timescale] Gets the timescale in which timestamp is
 * represented.
 */
export interface MediaLiveEventIncomingStreamReceivedEventData {
  readonly ingestUrl?: string;
  readonly trackType?: string;
  readonly trackName?: string;
  readonly bitrate?: number;
  readonly encoderIp?: string;
  readonly encoderPort?: string;
  readonly timestamp?: string;
  readonly duration?: string;
  readonly timescale?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventIncomingStreamsOutOfSyncEventData class.
 * @constructor
 * Incoming streams out of sync event data.
 *
 * @member {string} [minLastTimestamp] Gets the minimum last timestamp
 * received.
 * @member {string} [typeOfStreamWithMinLastTimestamp] Gets the type of stream
 * with minimum last timestamp.
 * @member {string} [maxLastTimestamp] Gets the maximum timestamp among all the
 * tracks (audio or video).
 * @member {string} [typeOfStreamWithMaxLastTimestamp] Gets the type of stream
 * with maximum last timestamp.
 * @member {string} [timescaleOfMinLastTimestamp] Gets the timescale in which
 * "MinLastTimestamp" is represented.
 * @member {string} [timescaleOfMaxLastTimestamp] Gets the timescale in which
 * "MaxLastTimestamp" is represented.
 */
export interface MediaLiveEventIncomingStreamsOutOfSyncEventData {
  readonly minLastTimestamp?: string;
  readonly typeOfStreamWithMinLastTimestamp?: string;
  readonly maxLastTimestamp?: string;
  readonly typeOfStreamWithMaxLastTimestamp?: string;
  readonly timescaleOfMinLastTimestamp?: string;
  readonly timescaleOfMaxLastTimestamp?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventIncomingVideoStreamsOutOfSyncEventData class.
 * @constructor
 * Incoming video stream out of synch event data.
 *
 * @member {string} [firstTimestamp] Gets the first timestamp received for one
 * of the quality levels.
 * @member {string} [firstDuration] Gets the duration of the data chunk with
 * first timestamp.
 * @member {string} [secondTimestamp] Gets the timestamp received for some
 * other quality levels.
 * @member {string} [secondDuration] Gets the duration of the data chunk with
 * second timestamp.
 * @member {string} [timescale] Gets the timescale in which both the timestamps
 * and durations are represented.
 */
export interface MediaLiveEventIncomingVideoStreamsOutOfSyncEventData {
  readonly firstTimestamp?: string;
  readonly firstDuration?: string;
  readonly secondTimestamp?: string;
  readonly secondDuration?: string;
  readonly timescale?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventIncomingDataChunkDroppedEventData class.
 * @constructor
 * Ingest fragment dropped event data.
 *
 * @member {string} [timestamp] Gets the timestamp of the data chunk dropped.
 * @member {string} [trackType] Gets the type of the track (Audio / Video).
 * @member {number} [bitrate] Gets the bitrate of the track.
 * @member {string} [timescale] Gets the timescale of the Timestamp.
 * @member {string} [resultCode] Gets the result code for fragment drop
 * operation.
 * @member {string} [trackName] Gets the name of the track for which fragment
 * is dropped.
 */
export interface MediaLiveEventIncomingDataChunkDroppedEventData {
  readonly timestamp?: string;
  readonly trackType?: string;
  readonly bitrate?: number;
  readonly timescale?: string;
  readonly resultCode?: string;
  readonly trackName?: string;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventIngestHeartbeatEventData class.
 * @constructor
 * Ingest fragment dropped event data.
 *
 * @member {string} [trackType] Gets the type of the track (Audio / Video).
 * @member {string} [trackName] Gets the track name.
 * @member {number} [bitrate] Gets the bitrate of the track.
 * @member {number} [incomingBitrate] Gets the incoming bitrate.
 * @member {string} [lastTimestamp] Gets the last timestamp.
 * @member {string} [timescale] Gets the timescale of the last timestamp.
 * @member {number} [overlapCount] Gets the fragment Overlap count.
 * @member {number} [discontinuityCount] Gets the fragment Discontinuity count.
 * @member {number} [nonincreasingCount] Gets Non increasing count.
 * @member {boolean} [unexpectedBitrate] Gets a value indicating whether
 * unexpected bitrate is present or not.
 * @member {string} [state] Gets the state of the live event.
 * @member {boolean} [healthy] Gets a value indicating whether preview is
 * healthy or not.
 */
export interface MediaLiveEventIngestHeartbeatEventData {
  readonly trackType?: string;
  readonly trackName?: string;
  readonly bitrate?: number;
  readonly incomingBitrate?: number;
  readonly lastTimestamp?: string;
  readonly timescale?: string;
  readonly overlapCount?: number;
  readonly discontinuityCount?: number;
  readonly nonincreasingCount?: number;
  readonly unexpectedBitrate?: boolean;
  readonly state?: string;
  readonly healthy?: boolean;
}

/**
 * @class
 * Initializes a new instance of the MediaLiveEventTrackDiscontinuityDetectedEventData class.
 * @constructor
 * Ingest track discontinuity detected event data.
 *
 * @member {string} [trackType] Gets the type of the track (Audio / Video).
 * @member {string} [trackName] Gets the track name.
 * @member {number} [bitrate] Gets the bitrate.
 * @member {string} [previousTimestamp] Gets the timestamp of the previous
 * fragment.
 * @member {string} [newTimestamp] Gets the timestamp of the current fragment.
 * @member {string} [timescale] Gets the timescale in which both timestamps and
 * discontinuity gap are represented.
 * @member {string} [discontinuityGap] Gets the discontinuity gap between
 * PreviousTimestamp and NewTimestamp.
 */
export interface MediaLiveEventTrackDiscontinuityDetectedEventData {
  readonly trackType?: string;
  readonly trackName?: string;
  readonly bitrate?: number;
  readonly previousTimestamp?: string;
  readonly newTimestamp?: string;
  readonly timescale?: string;
  readonly discontinuityGap?: string;
}

/**
 * @class
 * Initializes a new instance of the MapsGeofenceEventProperties class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a Geofence event
 * (GeofenceEntered, GeofenceExited, GeofenceResult).
 *
 * @member {array} [expiredGeofenceGeometryId] Lists of the geometry ID of the
 * geofence which is expired relative to the user time in the request.
 * @member {array} [geometries] Lists the fence geometries that either fully
 * contain the coordinate position or have an overlap with the searchBuffer
 * around the fence.
 * @member {array} [invalidPeriodGeofenceGeometryId] Lists of the geometry ID
 * of the geofence which is in invalid period relative to the user time in the
 * request.
 * @member {boolean} [isEventPublished] True if at least one event is published
 * to the Azure Maps event subscriber, false if no event is published to the
 * Azure Maps event subscriber.
 */
export interface MapsGeofenceEventProperties {
  expiredGeofenceGeometryId?: string[];
  geometries?: MapsGeofenceGeometry[];
  invalidPeriodGeofenceGeometryId?: string[];
  isEventPublished?: boolean;
}

/**
 * @class
 * Initializes a new instance of the MapsGeofenceEnteredEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Maps.GeofenceEntered event.
 *
 */
export interface MapsGeofenceEnteredEventData extends MapsGeofenceEventProperties {
}

/**
 * @class
 * Initializes a new instance of the MapsGeofenceExitedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Maps.GeofenceExited event.
 *
 */
export interface MapsGeofenceExitedEventData extends MapsGeofenceEventProperties {
}

/**
 * @class
 * Initializes a new instance of the MapsGeofenceResultEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Maps.GeofenceResult event.
 *
 */
export interface MapsGeofenceResultEventData extends MapsGeofenceEventProperties {
}

/**
 * @class
 * Initializes a new instance of the MapsGeofenceGeometry class.
 * @constructor
 * The geofence geometry.
 *
 * @member {string} [deviceId] ID of the device.
 * @member {number} [distance] Distance from the coordinate to the closest
 * border of the geofence. Positive means the coordinate is outside of the
 * geofence. If the coordinate is outside of the geofence, but more than the
 * value of searchBuffer away from the closest geofence border, then the value
 * is 999. Negative means the coordinate is inside of the geofence. If the
 * coordinate is inside the polygon, but more than the value of searchBuffer
 * away from the closest geofencing border,then the value is -999. A value of
 * 999 means that there is great confidence the coordinate is well outside the
 * geofence. A value of -999 means that there is great confidence the
 * coordinate is well within the geofence.
 * @member {string} [geometryId] The unique ID for the geofence geometry.
 * @member {number} [nearestLat] Latitude of the nearest point of the geometry.
 * @member {number} [nearestLon] Longitude of the nearest point of the
 * geometry.
 * @member {string} [udId] The unique id returned from user upload service when
 * uploading a geofence. Will not be included in geofencing post API.
 */
export interface MapsGeofenceGeometry {
  deviceId?: string;
  distance?: number;
  geometryId?: string;
  nearestLat?: number;
  nearestLon?: number;
  udId?: string;
}

/**
 * @class
 * Initializes a new instance of the AppConfigurationKeyValueModifiedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for an
 * Microsoft.AppConfiguration.KeyValueModified event.
 *
 * @member {string} [key] The key used to identify the key-value that was
 * modified.
 * @member {string} [label] The label, if any, used to identify the key-value
 * that was modified.
 * @member {string} [etag] The etag representing the new state of the
 * key-value.
 */
export interface AppConfigurationKeyValueModifiedEventData {
  key?: string;
  label?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the AppConfigurationKeyValueDeletedEventData class.
 * @constructor
 * Schema of the Data property of an EventGridEvent for an
 * Microsoft.AppConfiguration.KeyValueDeleted event.
 *
 * @member {string} [key] The key used to identify the key-value that was
 * deleted.
 * @member {string} [label] The label, if any, used to identify the key-value
 * that was deleted.
 * @member {string} [etag] The etag representing the key-value that was
 * deleted.
 */
export interface AppConfigurationKeyValueDeletedEventData {
  key?: string;
  label?: string;
  etag?: string;
}

