/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents a lab account.
 *
 * @extends models['Resource']
 */
class LabAccount extends models['Resource'] {
  /**
   * Create a LabAccount.
   * @property {object} [sizeConfiguration] Represents the size configuration
   * under the lab account
   * @property {array} [sizeConfiguration.environmentSizes] Represents a list
   * of size categories supported by this Lab Account (Small, Medium, Large)
   * @property {boolean} [enabledRegionSelection] Represents if region
   * selection is enabled
   * @property {string} [provisioningState] The provisioning status of the
   * resource.
   * @property {string} [uniqueIdentifier] The unique immutable identifier of a
   * resource (Guid).
   * @property {object} [latestOperationResult] The details of the latest
   * operation. ex: status, error
   * @property {string} [latestOperationResult.status] The current status of
   * the operation.
   * @property {string} [latestOperationResult.errorCode] Error code on
   * failure.
   * @property {string} [latestOperationResult.errorMessage] The error message.
   * @property {string} [latestOperationResult.requestUri] Request URI of the
   * operation.
   * @property {string} [latestOperationResult.httpMethod] The HttpMethod -
   * PUT/POST/DELETE for the operation.
   * @property {string} [latestOperationResult.operationUrl] The URL to use to
   * check long-running operation status
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of LabAccount
   *
   * @returns {object} metadata of LabAccount
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'LabAccount',
      type: {
        name: 'Composite',
        className: 'LabAccount',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          sizeConfiguration: {
            required: false,
            readOnly: true,
            serializedName: 'properties.sizeConfiguration',
            type: {
              name: 'Composite',
              className: 'SizeConfigurationProperties'
            }
          },
          enabledRegionSelection: {
            required: false,
            serializedName: 'properties.enabledRegionSelection',
            type: {
              name: 'Boolean'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          uniqueIdentifier: {
            required: false,
            serializedName: 'properties.uniqueIdentifier',
            type: {
              name: 'String'
            }
          },
          latestOperationResult: {
            required: false,
            readOnly: true,
            serializedName: 'properties.latestOperationResult',
            type: {
              name: 'Composite',
              className: 'LatestOperationResult'
            }
          }
        }
      }
    };
  }
}

module.exports = LabAccount;
