/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * An Azure resource.
 *
 * @member {string} [id] The resource ID.
 * @member {string} [name] The name of the resource.
 * @member {string} [type] The type of the resource.
 * @member {string} location The location of the resource. This cannot be
 * changed after the resource is created.
 * @member {object} [tags] The tags of the resource.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the Workspace class.
 * @constructor
 * An object that represents a machine learning team account workspace.
 *
 * @member {string} [description] The description of this workspace.
 * @member {string} [accountId] The immutable id of the team account which
 * contains this workspace.
 * @member {string} [workspaceId] The immutable id of this workspace.
 * @member {string} friendlyName The friendly name for this workspace. This
 * will be the workspace name in the arm id when the workspace object gets
 * created
 * @member {date} [creationDate] The creation date of the machine learning
 * workspace in ISO8601 format.
 * @member {string} [provisioningState] The current deployment state of team
 * account workspace resource. The provisioningState is to indicate states for
 * resource provisioning. Possible values include: 'Creating', 'Succeeded',
 * 'Updating', 'Deleting', 'Failed'
 */
export interface Workspace extends Resource {
  description?: string;
  readonly accountId?: string;
  readonly workspaceId?: string;
  friendlyName: string;
  readonly creationDate?: Date;
  readonly provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the WorkspaceUpdateParameters class.
 * @constructor
 * The parameters for updating a machine learning team account workspace.
 *
 * @member {object} [tags] The resource tags for the machine learning team
 * account workspace.
 * @member {string} [friendlyName] Friendly name of this workspace.
 * @member {string} [description] Description for this workspace.
 */
export interface WorkspaceUpdateParameters {
  tags?: { [propertyName: string]: string };
  friendlyName?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Project class.
 * @constructor
 * An object that represents a machine learning project.
 *
 * @member {string} [description] The description of this project.
 * @member {string} [accountId] The immutable id of the team account which
 * contains this project.
 * @member {string} [workspaceId] The immutable id of the workspace which
 * contains this project.
 * @member {string} [projectId] The immutable id of this project.
 * @member {string} [gitrepo] The reference to git repo for this project.
 * @member {string} friendlyName The friendly name for this project.
 * @member {date} [creationDate] The creation date of the project in ISO8601
 * format.
 * @member {string} [provisioningState] The current deployment state of project
 * resource. The provisioningState is to indicate states for resource
 * provisioning. Possible values include: 'Creating', 'Succeeded', 'Updating',
 * 'Deleting', 'Failed'
 */
export interface Project extends Resource {
  description?: string;
  readonly accountId?: string;
  readonly workspaceId?: string;
  readonly projectId?: string;
  gitrepo?: string;
  friendlyName: string;
  readonly creationDate?: Date;
  readonly provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the ProjectUpdateParameters class.
 * @constructor
 * The parameters for updating a machine learning project.
 *
 * @member {object} [tags] The resource tags for the machine learning project.
 * @member {string} [friendlyName] The friendly name for this project.
 * @member {string} [description] The description of this project.
 * @member {string} [gitrepo] The reference to git repo for this project.
 */
export interface ProjectUpdateParameters {
  tags?: { [propertyName: string]: string };
  friendlyName?: string;
  description?: string;
  gitrepo?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * The error response send when an operation fails.
 *
 * @member {string} code error code
 * @member {string} message error message
 */
export interface ErrorResponse {
  code: string;
  message: string;
}

/**
 * @class
 * Initializes a new instance of the StorageAccountProperties class.
 * @constructor
 * The properties of a storage account for a machine learning team account.
 *
 * @member {string} storageAccountId The fully qualified arm Id of the storage
 * account.
 * @member {string} accessKey The access key to the storage account.
 */
export interface StorageAccountProperties {
  storageAccountId: string;
  accessKey: string;
}

/**
 * @class
 * Initializes a new instance of the Account class.
 * @constructor
 * An object that represents a machine learning team account.
 *
 * @member {string} vsoAccountId The fully qualified arm id of the vso account
 * to be used for this team account.
 * @member {string} [accountId] The immutable id associated with this team
 * account.
 * @member {string} [description] The description of this workspace.
 * @member {string} [friendlyName] The friendly name for this workspace. This
 * will be the workspace name in the arm id when the workspace object gets
 * created
 * @member {string} keyVaultId The fully qualified arm id of the user key
 * vault.
 * @member {string} [seats] The no of users/seats who can access this team
 * account. This property defines the charge on the team account.
 * @member {string} [discoveryUri] The uri for this machine learning team
 * account.
 * @member {date} [creationDate] The creation date of the machine learning team
 * account in ISO8601 format.
 * @member {object} storageAccount The properties of the storage account for
 * the machine learning team account.
 * @member {string} [storageAccount.storageAccountId] The fully qualified arm
 * Id of the storage account.
 * @member {string} [storageAccount.accessKey] The access key to the storage
 * account.
 * @member {string} [provisioningState] The current deployment state of team
 * account resource. The provisioningState is to indicate states for resource
 * provisioning. Possible values include: 'Creating', 'Succeeded', 'Updating',
 * 'Deleting', 'Failed'
 */
export interface Account extends Resource {
  vsoAccountId: string;
  readonly accountId?: string;
  description?: string;
  friendlyName?: string;
  keyVaultId: string;
  seats?: string;
  readonly discoveryUri?: string;
  readonly creationDate?: Date;
  storageAccount: StorageAccountProperties;
  readonly provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the AccountUpdateParameters class.
 * @constructor
 * The parameters for updating a machine learning team account.
 *
 * @member {object} [tags] The resource tags for the machine learning team
 * account.
 * @member {string} [description] The description of this workspace.
 * @member {string} [friendlyName] The friendly name for this workspace. This
 * will be the workspace name in the arm id when the workspace object gets
 * created
 * @member {string} [seats] The no of users/seats who can access this team
 * account. This property defines the charge on the team account.
 * @member {string} [storageAccountKey] The key for storage account associated
 * with this team account
 */
export interface AccountUpdateParameters {
  tags?: { [propertyName: string]: string };
  description?: string;
  friendlyName?: string;
  seats?: string;
  storageAccountKey?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Display name of operation
 *
 * @member {string} [provider] The resource provider name:
 * Microsoft.MachineLearningExperimentation
 * @member {string} [resource] The resource on which the operation is
 * performed.
 * @member {string} [operation] The operation that users can perform.
 * @member {string} [description] The description for the operation.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Azure Machine Learning team account REST API operation
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] Display name of operation
 * @member {string} [display.provider] The resource provider name:
 * Microsoft.MachineLearningExperimentation
 * @member {string} [display.resource] The resource on which the operation is
 * performed.
 * @member {string} [display.operation] The operation that users can perform.
 * @member {string} [display.description] The description for the operation.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}


/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * An array of operations supported by the resource provider.
 *
 */
export interface OperationListResult extends Array<Operation> {
}

/**
 * @class
 * Initializes a new instance of the AccountListResult class.
 * @constructor
 * The result of a request to list machine learning team accounts.
 *
 * @member {string} [nextLink] The URI that can be used to request the next
 * list of machine learning team accounts.
 */
export interface AccountListResult extends Array<Account> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the WorkspaceListResult class.
 * @constructor
 * The result of a request to list machine learning team account workspaces.
 *
 * @member {string} [nextLink] The URI that can be used to request the next
 * list of machine learning workspaces.
 */
export interface WorkspaceListResult extends Array<Workspace> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ProjectListResult class.
 * @constructor
 * The result of a request to list projects.
 *
 * @member {string} [nextLink] The URI that can be used to request the next
 * list of projects.
 */
export interface ProjectListResult extends Array<Project> {
  nextLink?: string;
}
