/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Display name of operation
 *
 * @member {string} [provider] The resource provider name:
 * Microsoft.MachineLearningExperimentation
 * @member {string} [resource] The resource on which the operation is
 * performed.
 * @member {string} [operation] The operation that users can perform.
 * @member {string} [description] The description for the operation.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Azure Machine Learning workspace REST API operation
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] Display name of operation
 * @member {string} [display.provider] The resource provider name:
 * Microsoft.MachineLearningExperimentation
 * @member {string} [display.resource] The resource on which the operation is
 * performed.
 * @member {string} [display.operation] The operation that users can perform.
 * @member {string} [display.description] The description for the operation.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * Azure Resource Manager resource envelope.
 *
 * @member {string} [id] Specifies the resource ID.
 * @member {string} [name] Specifies the name of the resource.
 * @member {object} [identity] The identity of the resource.
 * @member {string} [identity.principalId] The principal ID of resource
 * identity.
 * @member {string} [identity.tenantId] The tenant ID of resource.
 * @member {string} [identity.type] The identity type. Possible values include:
 * 'SystemAssigned'
 * @member {string} [location] Specifies the location of the resource.
 * @member {string} [type] Specifies the type of the resource.
 * @member {object} [tags] Contains resource tags defined as key/value pairs.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly identity?: Identity;
  location?: string;
  readonly type?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the Workspace class.
 * @constructor
 * An object that represents a machine learning workspace.
 *
 * @member {string} [workspaceId] The immutable id associated with this
 * workspace.
 * @member {string} [description] The description of this workspace.
 * @member {string} [friendlyName] The friendly name for this workspace. This
 * name in mutable
 * @member {date} [creationTime] The creation time of the machine learning
 * workspace in ISO8601 format.
 * @member {string} [batchaiWorkspace] ARM id of the Batch AI workspace
 * associated with this workspace. This cannot be changed once the workspace
 * has been created
 * @member {string} [keyVault] ARM id of the key vault associated with this
 * workspace. This cannot be changed once the workspace has been created
 * @member {string} [applicationInsights] ARM id of the application insights
 * associated with this workspace. This cannot be changed once the workspace
 * has been created
 * @member {string} [containerRegistry] ARM id of the container registry
 * associated with this workspace. This cannot be changed once the workspace
 * has been created
 * @member {string} [storageAccount] ARM id of the storage account associated
 * with this workspace. This cannot be changed once the workspace has been
 * created
 * @member {string} [discoveryUrl] Url for the discovery service to identify
 * regional endpoints for machine learning experimentation services
 * @member {string} [provisioningState] The current deployment state of
 * workspace resource. The provisioningState is to indicate states for resource
 * provisioning. Possible values include: 'Unknown', 'Updating', 'Creating',
 * 'Deleting', 'Succeeded', 'Failed', 'Canceled'
 */
export interface Workspace extends Resource {
  readonly workspaceId?: string;
  description?: string;
  friendlyName?: string;
  readonly creationTime?: Date;
  batchaiWorkspace?: string;
  keyVault?: string;
  applicationInsights?: string;
  containerRegistry?: string;
  storageAccount?: string;
  discoveryUrl?: string;
  readonly provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the WorkspaceUpdateParameters class.
 * @constructor
 * The parameters for updating a machine learning workspace.
 *
 * @member {object} [tags] The resource tags for the machine learning
 * workspace.
 * @member {string} [description] The description of this workspace.
 * @member {string} [friendlyName] The friendly name for this workspace.
 */
export interface WorkspaceUpdateParameters {
  tags?: { [propertyName: string]: string };
  description?: string;
  friendlyName?: string;
}

/**
 * @class
 * Initializes a new instance of the Identity class.
 * @constructor
 * Identity for the resource.
 *
 * @member {string} [principalId] The principal ID of resource identity.
 * @member {string} [tenantId] The tenant ID of resource.
 * @member {string} [type] The identity type. Possible values include:
 * 'SystemAssigned'
 */
export interface Identity {
  readonly principalId?: string;
  readonly tenantId?: string;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the Password class.
 * @constructor
 * @member {string} [name]
 * @member {string} [value]
 */
export interface Password {
  readonly name?: string;
  readonly value?: string;
}

/**
 * @class
 * Initializes a new instance of the RegistryListCredentialsResult class.
 * @constructor
 * @member {string} [location]
 * @member {string} [username]
 * @member {array} [passwords]
 */
export interface RegistryListCredentialsResult {
  readonly location?: string;
  readonly username?: string;
  passwords?: Password[];
}

/**
 * @class
 * Initializes a new instance of the ListWorkspaceKeysResult class.
 * @constructor
 * @member {string} [userStorageKey]
 * @member {string} [userStorageResourceId]
 * @member {string} [appInsightsInstrumentationKey]
 * @member {object} [containerRegistryCredentials]
 * @member {string} [containerRegistryCredentials.location]
 * @member {string} [containerRegistryCredentials.username]
 * @member {array} [containerRegistryCredentials.passwords]
 */
export interface ListWorkspaceKeysResult {
  readonly userStorageKey?: string;
  readonly userStorageResourceId?: string;
  readonly appInsightsInstrumentationKey?: string;
  readonly containerRegistryCredentials?: RegistryListCredentialsResult;
}

/**
 * @class
 * Initializes a new instance of the ErrorDetail class.
 * @constructor
 * Error detail information.
 *
 * @member {string} code Error code.
 * @member {string} message Error message.
 */
export interface ErrorDetail {
  code: string;
  message: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * Error response information.
 *
 * @member {string} code Error code.
 * @member {string} message Error message.
 * @member {array} [details] An array of error detail objects.
 */
export interface ErrorResponse {
  code: string;
  message: string;
  details?: ErrorDetail[];
}

/**
 * @class
 * Initializes a new instance of the MachineLearningServiceError class.
 * @constructor
 * Wrapper for error response to follow ARM guidelines.
 *
 * @member {object} [error] The error response.
 * @member {string} [error.code] Error code.
 * @member {string} [error.message] Error message.
 * @member {array} [error.details] An array of error detail objects.
 */
export interface MachineLearningServiceError {
  error?: ErrorResponse;
}

/**
 * @class
 * Initializes a new instance of the Compute class.
 * @constructor
 * Machine Learning compute object.
 *
 * @member {string} [computeLocation] Location for the underlying compute
 * @member {string} [provisioningState] The provision state of the cluster.
 * Valid values are Unknown, Updating, Provisioning, Succeeded, and Failed.
 * Possible values include: 'Unknown', 'Updating', 'Creating', 'Deleting',
 * 'Succeeded', 'Failed', 'Canceled'
 * @member {string} [description] The description of the Machine Learning
 * compute.
 * @member {date} [createdOn] The date and time when the compute was created.
 * @member {date} [modifiedOn] The date and time when the compute was last
 * modified.
 * @member {string} [resourceId] ARM resource id of the compute
 * @member {array} [provisioningErrors] Errors during provisioning
 * @member {string} computeType Polymorphic Discriminator
 */
export interface Compute {
  computeLocation?: string;
  readonly provisioningState?: string;
  description?: string;
  readonly createdOn?: Date;
  readonly modifiedOn?: Date;
  resourceId?: string;
  readonly provisioningErrors?: MachineLearningServiceError[];
  computeType: string;
}

/**
 * @class
 * Initializes a new instance of the ComputeResource class.
 * @constructor
 * Machine Learning compute object wrapped into ARM resource envelope.
 *
 * @member {object} [properties] Compute properties
 * @member {string} [properties.computeLocation] Location for the underlying
 * compute
 * @member {string} [properties.provisioningState] The provision state of the
 * cluster. Valid values are Unknown, Updating, Provisioning, Succeeded, and
 * Failed. Possible values include: 'Unknown', 'Updating', 'Creating',
 * 'Deleting', 'Succeeded', 'Failed', 'Canceled'
 * @member {string} [properties.description] The description of the Machine
 * Learning compute.
 * @member {date} [properties.createdOn] The date and time when the compute was
 * created.
 * @member {date} [properties.modifiedOn] The date and time when the compute
 * was last modified.
 * @member {string} [properties.resourceId] ARM resource id of the compute
 * @member {array} [properties.provisioningErrors] Errors during provisioning
 * @member {string} [properties.computeType] Polymorphic Discriminator
 */
export interface ComputeResource extends Resource {
  properties?: Compute;
}

/**
 * @class
 * Initializes a new instance of the SystemService class.
 * @constructor
 * A system service running on a compute.
 *
 * @member {string} [systemServiceType] The type of this system service.
 * @member {string} [publicIpAddress] Public IP address
 * @member {string} [version] The version for this type.
 */
export interface SystemService {
  readonly systemServiceType?: string;
  readonly publicIpAddress?: string;
  readonly version?: string;
}

/**
 * @class
 * Initializes a new instance of the SslConfiguration class.
 * @constructor
 * The ssl configugation for scoring
 *
 * @member {string} [status] Enable or disable ssl for scoring. Possible values
 * include: 'Disabled', 'Enabled'
 * @member {string} [cert] Cert data
 * @member {string} [key] Key data
 * @member {string} [cname] CNAME of the cert
 */
export interface SslConfiguration {
  status?: string;
  cert?: string;
  key?: string;
  cname?: string;
}

/**
 * @class
 * Initializes a new instance of the AKSProperties class.
 * @constructor
 * AKS properties
 *
 * @member {string} [clusterFqdn] Cluster full qualified domain name
 * @member {array} [systemServices] System services
 * @member {number} [agentCount] Number of agents
 * @member {string} [agentVMSize] Agent virtual machine size
 * @member {object} [sslConfiguration] SSL configuration
 * @member {string} [sslConfiguration.status] Enable or disable ssl for
 * scoring. Possible values include: 'Disabled', 'Enabled'
 * @member {string} [sslConfiguration.cert] Cert data
 * @member {string} [sslConfiguration.key] Key data
 * @member {string} [sslConfiguration.cname] CNAME of the cert
 */
export interface AKSProperties {
  clusterFqdn?: string;
  systemServices?: SystemService[];
  agentCount?: number;
  agentVMSize?: string;
  sslConfiguration?: SslConfiguration;
}

/**
 * @class
 * Initializes a new instance of the AKS class.
 * @constructor
 * A Machine Learning compute based on AKS.
 *
 * @member {object} [properties] AKS properties
 * @member {string} [properties.clusterFqdn] Cluster full qualified domain name
 * @member {array} [properties.systemServices] System services
 * @member {number} [properties.agentCount] Number of agents
 * @member {string} [properties.agentVMSize] Agent virtual machine size
 * @member {object} [properties.sslConfiguration] SSL configuration
 * @member {string} [properties.sslConfiguration.status] Enable or disable ssl
 * for scoring. Possible values include: 'Disabled', 'Enabled'
 * @member {string} [properties.sslConfiguration.cert] Cert data
 * @member {string} [properties.sslConfiguration.key] Key data
 * @member {string} [properties.sslConfiguration.cname] CNAME of the cert
 */
export interface AKS extends Compute {
  properties?: AKSProperties;
}

/**
 * @class
 * Initializes a new instance of the ScaleSettings class.
 * @constructor
 * scale settings for BatchAI Compute
 *
 * @member {number} [maxNodeCount] Max number of nodes to use
 * @member {number} [minNodeCount] Min number of nodes to use
 * @member {boolean} [autoScaleEnabled] Enable or disable auto scale
 */
export interface ScaleSettings {
  maxNodeCount?: number;
  minNodeCount?: number;
  autoScaleEnabled?: boolean;
}

/**
 * @class
 * Initializes a new instance of the BatchAIProperties class.
 * @constructor
 * BatchAI properties
 *
 * @member {string} [vmSize] Virtual Machine Size
 * @member {string} [vmPriority] Virtual Machine priority
 * @member {object} [scaleSettings] Scale settings for BatchAI
 * @member {number} [scaleSettings.maxNodeCount] Max number of nodes to use
 * @member {number} [scaleSettings.minNodeCount] Min number of nodes to use
 * @member {boolean} [scaleSettings.autoScaleEnabled] Enable or disable auto
 * scale
 */
export interface BatchAIProperties {
  vmSize?: string;
  vmPriority?: string;
  scaleSettings?: ScaleSettings;
}

/**
 * @class
 * Initializes a new instance of the BatchAI class.
 * @constructor
 * A Machine Learning compute based on Azure BatchAI.
 *
 * @member {object} [properties] BatchAI properties
 * @member {string} [properties.vmSize] Virtual Machine Size
 * @member {string} [properties.vmPriority] Virtual Machine priority
 * @member {object} [properties.scaleSettings] Scale settings for BatchAI
 * @member {number} [properties.scaleSettings.maxNodeCount] Max number of nodes
 * to use
 * @member {number} [properties.scaleSettings.minNodeCount] Min number of nodes
 * to use
 * @member {boolean} [properties.scaleSettings.autoScaleEnabled] Enable or
 * disable auto scale
 */
export interface BatchAI extends Compute {
  properties?: BatchAIProperties;
}

/**
 * @class
 * Initializes a new instance of the VirtualMachineSshCredentials class.
 * @constructor
 * Admin credentials for virtual machine
 *
 * @member {string} [username] Username of admin account
 * @member {string} [password] Password of admin account
 * @member {string} [publicKeyData] Public key data
 * @member {string} [privateKeyData] Private key data
 */
export interface VirtualMachineSshCredentials {
  username?: string;
  password?: string;
  publicKeyData?: string;
  privateKeyData?: string;
}

/**
 * @class
 * Initializes a new instance of the VirtualMachineProperties class.
 * @constructor
 * @member {string} [virtualMachineSize] Virtual Machine size
 * @member {number} [sshPort] Port open for ssh connections.
 * @member {string} [address] Public IP address of the virtual machine.
 * @member {object} [administratorAccount] Admin credentials for virtual
 * machine
 * @member {string} [administratorAccount.username] Username of admin account
 * @member {string} [administratorAccount.password] Password of admin account
 * @member {string} [administratorAccount.publicKeyData] Public key data
 * @member {string} [administratorAccount.privateKeyData] Private key data
 */
export interface VirtualMachineProperties {
  virtualMachineSize?: string;
  sshPort?: number;
  address?: string;
  administratorAccount?: VirtualMachineSshCredentials;
}

/**
 * @class
 * Initializes a new instance of the VirtualMachine class.
 * @constructor
 * A Machine Learning compute based on Azure Virtual Machines.
 *
 * @member {object} [properties]
 * @member {string} [properties.virtualMachineSize] Virtual Machine size
 * @member {number} [properties.sshPort] Port open for ssh connections.
 * @member {string} [properties.address] Public IP address of the virtual
 * machine.
 * @member {object} [properties.administratorAccount] Admin credentials for
 * virtual machine
 * @member {string} [properties.administratorAccount.username] Username of
 * admin account
 * @member {string} [properties.administratorAccount.password] Password of
 * admin account
 * @member {string} [properties.administratorAccount.publicKeyData] Public key
 * data
 * @member {string} [properties.administratorAccount.privateKeyData] Private
 * key data
 */
export interface VirtualMachine extends Compute {
  properties?: VirtualMachineProperties;
}

/**
 * @class
 * Initializes a new instance of the HDInsightProperties class.
 * @constructor
 * @member {number} [sshPort] Port open for ssh connections on the master node
 * of the cluster.
 * @member {string} [address] Public IP address of the master node of the
 * cluster.
 * @member {object} [administratorAccount] Admin credentials for master node of
 * the cluster
 * @member {string} [administratorAccount.username] Username of admin account
 * @member {string} [administratorAccount.password] Password of admin account
 * @member {string} [administratorAccount.publicKeyData] Public key data
 * @member {string} [administratorAccount.privateKeyData] Private key data
 */
export interface HDInsightProperties {
  sshPort?: number;
  address?: string;
  administratorAccount?: VirtualMachineSshCredentials;
}

/**
 * @class
 * Initializes a new instance of the HDInsight class.
 * @constructor
 * A HDInsight compute.
 *
 * @member {object} [properties]
 * @member {number} [properties.sshPort] Port open for ssh connections on the
 * master node of the cluster.
 * @member {string} [properties.address] Public IP address of the master node
 * of the cluster.
 * @member {object} [properties.administratorAccount] Admin credentials for
 * master node of the cluster
 * @member {string} [properties.administratorAccount.username] Username of
 * admin account
 * @member {string} [properties.administratorAccount.password] Password of
 * admin account
 * @member {string} [properties.administratorAccount.publicKeyData] Public key
 * data
 * @member {string} [properties.administratorAccount.privateKeyData] Private
 * key data
 */
export interface HDInsight extends Compute {
  properties?: HDInsightProperties;
}

/**
 * @class
 * Initializes a new instance of the DataFactory class.
 * @constructor
 * A DataFactory compute.
 *
 */
export interface DataFactory extends Compute {
}

/**
 * @class
 * Initializes a new instance of the ServicePrincipalCredentials class.
 * @constructor
 * Service principal credentials.
 *
 * @member {string} clientId Client Id
 * @member {string} clientSecret Client secret
 */
export interface ServicePrincipalCredentials {
  clientId: string;
  clientSecret: string;
}

/**
 * @class
 * Initializes a new instance of the ComputeSecrets class.
 * @constructor
 * Secrets related to a Machine Learning compute. Might differ for every type
 * of compute.
 *
 * @member {string} computeType Polymorphic Discriminator
 */
export interface ComputeSecrets {
  computeType: string;
}

/**
 * @class
 * Initializes a new instance of the AksComputeSecrets class.
 * @constructor
 * Secrets related to a Machine Learning compute based on AKS.
 *
 * @member {string} [userKubeConfig] Content of kubeconfig file that can be
 * used to connect to the Kubernetes cluster.
 * @member {string} [adminKubeConfig] Content of kubeconfig file that can be
 * used to connect to the Kubernetes cluster.
 * @member {string} [imagePullSecretName] Image registry pull secret.
 */
export interface AksComputeSecrets extends ComputeSecrets {
  userKubeConfig?: string;
  adminKubeConfig?: string;
  imagePullSecretName?: string;
}

/**
 * @class
 * Initializes a new instance of the VirtualMachineSecrets class.
 * @constructor
 * Secrets related to a Machine Learning compute based on AKS.
 *
 * @member {object} [administratorAccount] Admin creadentials for virtual
 * machine.
 * @member {string} [administratorAccount.username] Username of admin account
 * @member {string} [administratorAccount.password] Password of admin account
 * @member {string} [administratorAccount.publicKeyData] Public key data
 * @member {string} [administratorAccount.privateKeyData] Private key data
 */
export interface VirtualMachineSecrets extends ComputeSecrets {
  administratorAccount?: VirtualMachineSshCredentials;
}


/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * An array of operations supported by the resource provider.
 *
 */
export interface OperationListResult extends Array<Operation> {
}

/**
 * @class
 * Initializes a new instance of the WorkspaceListResult class.
 * @constructor
 * The result of a request to list machine learning workspaces.
 *
 * @member {string} [nextLink] The URI that can be used to request the next
 * list of machine learning workspaces.
 */
export interface WorkspaceListResult extends Array<Workspace> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the PaginatedComputeResourcesList class.
 * @constructor
 * Paginated list of Machine Learning compute objects wrapped in ARM resource
 * envelope.
 *
 * @member {string} [nextLink] A continuation link (absolute URI) to the next
 * page of results in the list.
 */
export interface PaginatedComputeResourcesList extends Array<ComputeResource> {
  nextLink?: string;
}
