/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the ErrorDetails class.
 * @constructor
 * The details of the error.
 *
 * @member {string} [code] One of a server-defined set of error codes.
 * @member {string} [message] A human-readable representation of the error.
 */
export interface ErrorDetails {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * The error object.
 *
 * @member {object} [error] Error.
 * @member {string} [error.code] One of a server-defined set of error codes.
 * @member {string} [error.message] A human-readable representation of the
 * error.
 */
export interface ErrorResponse {
  error?: ErrorDetails;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * The object that represents the operation.
 *
 * @member {string} [provider] The name of the provider.
 * @member {string} [resource] The resource on which the operation is
 * performed.
 * @member {string} [operation] The operation that can be performed.
 * @member {string} [description] Operation description.
 */
export interface OperationDisplay {
  readonly provider?: string;
  readonly resource?: string;
  readonly operation?: string;
  readonly description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Operation supported by the Microsoft.Management resource provider.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}.
 * @member {object} [display] The object that represents the operation.
 * @member {string} [display.provider] The name of the provider.
 * @member {string} [display.resource] The resource on which the operation is
 * performed.
 * @member {string} [display.operation] The operation that can be performed.
 * @member {string} [display.description] Operation description.
 */
export interface Operation {
  readonly name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupInfo class.
 * @constructor
 * The management group resource.
 *
 * @member {string} [id] The fully qualified ID for the management group.  For
 * example,
 * /providers/Microsoft.Management/managementGroups/0000000-0000-0000-0000-000000000000
 * @member {string} [type] The type of the resource. For example,
 * /providers/Microsoft.Management/managementGroups
 * @member {string} [name] The name of the management group. For example,
 * 00000000-0000-0000-0000-000000000000
 * @member {string} [tenantId] The AAD Tenant ID associated with the management
 * group. For example, 00000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the management group.
 */
export interface ManagementGroupInfo {
  readonly id?: string;
  readonly type?: string;
  readonly name?: string;
  tenantId?: string;
  displayName?: string;
}

/**
 * @class
 * Initializes a new instance of the ParentGroupInfo class.
 * @constructor
 * (Optional) The ID of the parent management group.
 *
 * @member {string} [parentId] The fully qualified ID for the parent management
 * group.  For example,
 * /providers/Microsoft.Management/managementGroups/0000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the parent management
 * group.
 */
export interface ParentGroupInfo {
  parentId?: string;
  displayName?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupDetails class.
 * @constructor
 * The details of a management group.
 *
 * @member {number} [version] The version number of the object.
 * @member {date} [updatedTime] The date and time when this object was last
 * updated.
 * @member {string} [updatedBy] The identity of the principal or process that
 * updated the object.
 * @member {object} [parent] Parent.
 * @member {string} [parent.parentId] The fully qualified ID for the parent
 * management group.  For example,
 * /providers/Microsoft.Management/managementGroups/0000000-0000-0000-0000-000000000000
 * @member {string} [parent.displayName] The friendly name of the parent
 * management group.
 */
export interface ManagementGroupDetails {
  version?: number;
  updatedTime?: Date;
  updatedBy?: string;
  parent?: ParentGroupInfo;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupChildInfo class.
 * @constructor
 * The child information of a management group.
 *
 * @member {string} [childType] The type of child resource. Possible values
 * include: 'ManagementGroup', 'Subscription'
 * @member {string} [childId] The fully qualified ID for the child resource
 * (management group or subscription).  For example,
 * /providers/Microsoft.Management/managementGroups/0000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the child resource.
 * @member {array} [children] The list of children.
 */
export interface ManagementGroupChildInfo {
  childType?: string;
  childId?: string;
  displayName?: string;
  children?: ManagementGroupChildInfo[];
}

/**
 * @class
 * Initializes a new instance of the ManagementGroup class.
 * @constructor
 * The management group details.
 *
 * @member {string} [id] The fully qualified ID for the management group.  For
 * example,
 * /providers/Microsoft.Management/managementGroups/0000000-0000-0000-0000-000000000000
 * @member {string} [type] The type of the resource.  For example,
 * /providers/Microsoft.Management/managementGroups
 * @member {string} [name] The name of the management group. For example,
 * 00000000-0000-0000-0000-000000000000
 * @member {string} [tenantId] The AAD Tenant ID associated with the management
 * group. For example, 00000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the management group.
 * @member {object} [details] Details.
 * @member {number} [details.version] The version number of the object.
 * @member {date} [details.updatedTime] The date and time when this object was
 * last updated.
 * @member {string} [details.updatedBy] The identity of the principal or
 * process that updated the object.
 * @member {object} [details.parent]
 * @member {string} [details.parent.parentId] The fully qualified ID for the
 * parent management group.  For example,
 * /providers/Microsoft.Management/managementGroups/0000000-0000-0000-0000-000000000000
 * @member {string} [details.parent.displayName] The friendly name of the
 * parent management group.
 * @member {array} [children] The list of children.
 */
export interface ManagementGroup extends BaseResource {
  readonly id?: string;
  readonly type?: string;
  readonly name?: string;
  tenantId?: string;
  displayName?: string;
  details?: ManagementGroupDetails;
  children?: ManagementGroupChildInfo[];
}

/**
 * @class
 * Initializes a new instance of the CreateManagementGroupRequest class.
 * @constructor
 * Management group creation parameters.
 *
 * @member {string} [displayName] The friendly name of the management group.
 * @member {string} [parentId] (Optional) The fully qualified ID for the parent
 * management group.  For example,
 * /providers/Microsoft.Management/managementGroups/0000000-0000-0000-0000-000000000000
 */
export interface CreateManagementGroupRequest {
  displayName?: string;
  parentId?: string;
}


/**
 * @class
 * Initializes a new instance of the ManagementGroupListResult class.
 * @constructor
 * Describes the result of the request to list management groups.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface ManagementGroupListResult extends Array<ManagementGroupInfo> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Describes the result of the request to list Microsoft.Management operations.
 *
 * @member {string} [nextLink] URL to get the next set of operation list
 * results if there are any.
 */
export interface OperationListResult extends Array<Operation> {
  readonly nextLink?: string;
}
