/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes all the properties for encoding a video with the H.264 codec.
 *
 * @extends models['Video']
 */
class H264Video extends models['Video'] {
  /**
   * Create a H264Video.
   * @property {boolean} [sceneChangeDetection] Whether or not the encoder
   * should insert key frames at scene changes. If not specified, the default
   * is false. This flag should be set to true only when the encoder is being
   * configured to produce a single output video.
   * @property {string} [complexity] Tells the encoder how to choose its
   * encoding settings. The default value is Balanced. Possible values include:
   * 'Speed', 'Balanced', 'Quality'
   * @property {array} [layers] The collection of output H.264 layers to be
   * produced by the encoder.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of H264Video
   *
   * @returns {object} metadata of H264Video
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: '#Microsoft.Media.H264Video',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '@odata.type',
          clientName: 'odatatype'
        },
        uberParent: 'Codec',
        className: 'H264Video',
        modelProperties: {
          label: {
            required: false,
            serializedName: 'label',
            type: {
              name: 'String'
            }
          },
          odatatype: {
            required: true,
            serializedName: '@odata\\.type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          keyFrameInterval: {
            required: false,
            serializedName: 'keyFrameInterval',
            type: {
              name: 'TimeSpan'
            }
          },
          stretchMode: {
            required: false,
            serializedName: 'stretchMode',
            type: {
              name: 'String'
            }
          },
          sceneChangeDetection: {
            required: false,
            serializedName: 'sceneChangeDetection',
            type: {
              name: 'Boolean'
            }
          },
          complexity: {
            required: false,
            serializedName: 'complexity',
            type: {
              name: 'String'
            }
          },
          layers: {
            required: false,
            serializedName: 'layers',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'H264LayerElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '@odata.type',
                      clientName: 'odatatype'
                    },
                    uberParent: 'Layer',
                    className: 'H264Layer'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = H264Video;
