/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import { BaseResource, CloudError } from "ms-rest-azure";
import * as moment from "moment";

export {

  BaseResource,
  CloudError
};

/**
 * The presentation time range, this is asset related and not recommended for Account Filter.
 */
export interface PresentationTimeRange {
  /**
   * The absolute start time boundary.
   */
  startTimestamp?: number;
  /**
   * The absolute end time boundary.
   */
  endTimestamp?: number;
  /**
   * The relative to end sliding window.
   */
  presentationWindowDuration?: number;
  /**
   * The relative to end right edge.
   */
  liveBackoffDuration?: number;
  /**
   * The time scale of time stamps.
   */
  timescale?: number;
  /**
   * The indicator of forcing exsiting of end time stamp.
   */
  forceEndTimestamp?: boolean;
}

/**
 * The class to specify one track property condition.
 */
export interface FilterTrackPropertyCondition {
  /**
   * The track property type. Possible values include: 'Unknown', 'Type', 'Name', 'Language',
   * 'FourCC', 'Bitrate'
   */
  property: string;
  /**
   * The track proprty value.
   */
  value: string;
  /**
   * The track property condition operation. Possible values include: 'Equal', 'NotEqual'
   */
  operation: string;
}

/**
 * Filter First Quality
 */
export interface FirstQuality {
  /**
   * The first quality bitrate.
   */
  bitrate: number;
}

/**
 * Representing a list of FilterTrackPropertyConditions to select a track.  The filters are
 * combined using a logical AND operation.
 */
export interface FilterTrackSelection {
  /**
   * The track selections.
   */
  trackSelections: FilterTrackPropertyCondition[];
}

/**
 * The core properties of ARM resources.
 */
export interface Resource extends BaseResource {
  /**
   * Fully qualified resource ID for the resource.
   */
  readonly id?: string;
  /**
   * The name of the resource.
   */
  readonly name?: string;
  /**
   * The type of the resource.
   */
  readonly type?: string;
}

/**
 * The resource model definition for a ARM proxy resource.
 */
export interface ProxyResource extends Resource {
}

/**
 * An Account Filter.
 */
export interface AccountFilter extends ProxyResource {
  /**
   * The presentation time range.
   */
  presentationTimeRange?: PresentationTimeRange;
  /**
   * The first quality.
   */
  firstQuality?: FirstQuality;
  /**
   * The tracks selection conditions.
   */
  tracks?: FilterTrackSelection[];
}

/**
 * Information about an error.
 */
export interface ODataError {
  /**
   * A language-independent error name.
   */
  code?: string;
  /**
   * The error message.
   */
  message?: string;
  /**
   * The target of the error (for example, the name of the property in error).
   */
  target?: string;
  /**
   * The error details.
   */
  details?: ODataError[];
}

/**
 * The API error.
 */
export interface ApiError {
  /**
   * @summary ApiError
   * @description The error properties.
   */
  error?: ODataError;
}

/**
 * The resource model definition for a ARM tracked resource.
 */
export interface TrackedResource extends Resource {
  /**
   * Resource tags.
   */
  tags?: { [propertyName: string]: string };
  /**
   * The Azure Region of the resource.
   */
  location?: string;
}

/**
 * A resource provider.
 */
export interface Provider {
  /**
   * The provider name.
   */
  providerName: string;
}

/**
 * Operation details.
 */
export interface OperationDisplay {
  /**
   * The service provider.
   */
  provider?: string;
  /**
   * Resource on which the operation is performed.
   */
  resource?: string;
  /**
   * The operation type.
   */
  operation?: string;
  /**
   * The operation description.
   */
  description?: string;
}

/**
 * A metric dimension.
 */
export interface MetricDimension {
  /**
   * The metric dimension name.
   */
  readonly name?: string;
  /**
   * The display name for the dimension.
   */
  readonly displayName?: string;
  /**
   * Whether to export metric to shoebox.
   */
  readonly toBeExportedForShoebox?: boolean;
}

/**
 * A metric emitted by service.
 */
export interface Metric {
  /**
   * The metric name.
   */
  readonly name?: string;
  /**
   * The metric display name.
   */
  readonly displayName?: string;
  /**
   * The metric display description.
   */
  readonly displayDescription?: string;
  /**
   * The metric unit. Possible values include: 'Bytes', 'Count', 'Milliseconds'
   */
  readonly unit?: string;
  /**
   * The metric aggregation type. Possible values include: 'Average', 'Count', 'Total'
   */
  readonly aggregationType?: string;
  /**
   * The metric dimensions.
   */
  readonly dimensions?: MetricDimension[];
}

/**
 * The service metric specifications.
 */
export interface ServiceSpecification {
  /**
   * List of metric specifications.
   */
  readonly metricSpecifications?: Metric[];
}

/**
 * Metric properties.
 */
export interface MetricProperties {
  /**
   * The service specifications.
   */
  readonly serviceSpecification?: ServiceSpecification;
}

/**
 * An operation.
 */
export interface Operation {
  /**
   * The operation name.
   */
  name: string;
  /**
   * The operation display name.
   */
  display?: OperationDisplay;
  /**
   * Origin of the operation.
   */
  origin?: string;
  /**
   * Operation properties format.
   */
  properties?: MetricProperties;
}

export interface Location {
  name: string;
}

/**
 * The response from the check name availability request.
*/
export interface EntityNameAvailabilityCheckOutput {
  /**
   * Specifies if the name is available.
  */
  nameAvailable: boolean;
  /**
   * Specifies the reason if the name is not available.
  */
  reason?: string;
  /**
   * Specifies the detailed reason if the name is not available.
  */
  message?: string;
}

/**
 * The storage account details.
*/
export interface StorageAccount {
  /**
   * The ID of the storage account resource. Media Services relies on tables and queues as well as
   * blobs, so the primary storage account must be a Standard Storage account (either
   * Microsoft.ClassicStorage or Microsoft.Storage). Blob only storage accounts can be added as
   * secondary storage accounts.
  */
  id?: string;
  /**
   * The type of the storage account. Possible values include: 'Primary', 'Secondary'
  */
  type: string;
}

/**
 * The input to the sync storage keys request.
*/
export interface SyncStorageKeysInput {
  /**
   * The ID of the storage account resource.
  */
  id?: string;
}

/**
 * A Media Services account.
*/
export interface MediaService extends TrackedResource {
  /**
   * The Media Services account ID.
  */
  readonly mediaServiceId?: string;
  /**
   * The storage accounts for this resource.
  */
  storageAccounts?: StorageAccount[];
}

/**
 * A Media Services account.
*/
export interface SubscriptionMediaService extends TrackedResource {
  /**
   * The Media Services account ID.
  */
  readonly mediaServiceId?: string;
  /**
   * The storage accounts for this resource.
  */
  storageAccounts?: StorageAccount[];
}

/**
 * The input to the check name availability request.
*/
export interface CheckNameAvailabilityInput {
  /**
   * The account name.
  */
  name?: string;
  /**
   * The account type. For a Media Services account, this should be 'MediaServices'.
  */
  type?: string;
}

/**
 * The Asset Storage container SAS URLs.
*/
export interface AssetContainerSas {
  /**
   * The list of Asset container SAS URLs.
  */
  assetContainerSasUrls?: string[];
}

/**
 * The Asset File Storage encryption metadata.
*/
export interface AssetFileEncryptionMetadata {
  /**
   * The Asset File initialization vector.
  */
  initializationVector?: string;
  /**
   * The Asset File name.
  */
  assetFileName?: string;
  /**
   * The Asset File Id.
  */
  assetFileId: string;
}

/**
 * Data needed to decrypt asset files encrypted with legacy storage encryption.
*/
export interface StorageEncryptedAssetDecryptionData {
  /**
   * The Asset File storage encryption key.
  */
  key?: Buffer;
  /**
   * Asset File encryption metadata.
  */
  assetFileEncryptionMetadata?: AssetFileEncryptionMetadata[];
}

/**
 * Properties of the Streaming Locator.
*/
export interface AssetStreamingLocator {
  /**
   * Streaming Locator name.
  */
  readonly name?: string;
  /**
   * Asset Name.
  */
  readonly assetName?: string;
  /**
   * The creation time of the Streaming Locator.
  */
  readonly created?: Date;
  /**
   * The start time of the Streaming Locator.
  */
  readonly startTime?: Date;
  /**
   * The end time of the Streaming Locator.
  */
  readonly endTime?: Date;
  /**
   * StreamingLocatorId of the Streaming Locator.
  */
  readonly streamingLocatorId?: string;
  /**
   * Name of the Streaming Policy used by this Streaming Locator.
  */
  readonly streamingPolicyName?: string;
  /**
   * Name of the default ContentKeyPolicy used by this Streaming Locator.
  */
  readonly defaultContentKeyPolicyName?: string;
}

/**
 * The Streaming Locators associated with this Asset.
*/
export interface ListStreamingLocatorsResponse {
  /**
   * The list of Streaming Locators.
  */
  readonly streamingLocators?: AssetStreamingLocator[];
}

/**
 * An Asset.
*/
export interface Asset extends ProxyResource {
  /**
   * The Asset ID.
  */
  readonly assetId?: string;
  /**
   * The creation date of the Asset.
  */
  readonly created?: Date;
  /**
   * The last modified date of the Asset.
  */
  readonly lastModified?: Date;
  /**
   * The alternate ID of the Asset.
  */
  alternateId?: string;
  /**
   * The Asset description.
  */
  description?: string;
  /**
   * The name of the asset blob container.
  */
  container?: string;
  /**
   * The name of the storage account.
  */
  storageAccountName?: string;
  /**
   * The Asset encryption format. One of None or MediaStorageEncryption. Possible values include:
   * 'None', 'MediaStorageClientEncryption'
  */
  readonly storageEncryptionFormat?: string;
}

/**
 * An Asset Filter.
*/
export interface AssetFilter extends ProxyResource {
  /**
   * The presentation time range.
  */
  presentationTimeRange?: PresentationTimeRange;
  /**
   * The first quality.
  */
  firstQuality?: FirstQuality;
  /**
   * The tracks selection conditions.
  */
  tracks?: FilterTrackSelection[];
}

/**
 * The parameters to the list SAS request.
*/
export interface ListContainerSasInput {
  /**
   * The permissions to set on the SAS URL. Possible values include: 'Read', 'ReadWrite',
   * 'ReadWriteDelete'
  */
  permissions?: string;
  /**
   * The SAS URL expiration time.  This must be less than 24 hours from the current time.
  */
  expiryTime?: Date;
}

/**
 * Configures the Explicit Analog Television Output Restriction control bits. For further details
 * see the PlayReady Compliance Rules.
*/
export interface ContentKeyPolicyPlayReadyExplicitAnalogTelevisionRestriction {
  /**
   * Indicates whether this restriction is enforced on a Best Effort basis.
  */
  bestEffort: boolean;
  /**
   * Configures the restriction control bits. Must be between 0 and 3 inclusive.
  */
  configurationData: number;
}

/**
 * Base class for content key ID location. A derived class must be used to represent the location.
*/
export interface ContentKeyPolicyPlayReadyContentKeyLocation {
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Specifies that the content key ID is in the PlayReady header.
*/
export interface ContentKeyPolicyPlayReadyContentEncryptionKeyFromHeader extends
ContentKeyPolicyPlayReadyContentKeyLocation {
}

/**
 * Specifies that the content key ID is specified in the PlayReady configuration.
*/
export interface ContentKeyPolicyPlayReadyContentEncryptionKeyFromKeyIdentifier extends
ContentKeyPolicyPlayReadyContentKeyLocation {
  /**
   * The content key ID.
  */
  keyId: string;
}

/**
 * Configures the Play Right in the PlayReady license.
*/
export interface ContentKeyPolicyPlayReadyPlayRight {
  /**
   * The amount of time that the license is valid after the license is first used to play content.
  */
  firstPlayExpiration?: moment.Duration;
  /**
   * Configures the Serial Copy Management System (SCMS) in the license. Must be between 0 and 3
   * inclusive.
  */
  scmsRestriction?: number;
  /**
   * Configures Automatic Gain Control (AGC) and Color Stripe in the license. Must be between 0 and
   * 3 inclusive.
  */
  agcAndColorStripeRestriction?: number;
  /**
   * Configures the Explicit Analog Television Output Restriction in the license. Configuration
   * data must be between 0 and 3 inclusive.
  */
  explicitAnalogTelevisionOutputRestriction?:
  ContentKeyPolicyPlayReadyExplicitAnalogTelevisionRestriction;
  /**
   * Enables the Image Constraint For Analog Component Video Restriction in the license.
  */
  digitalVideoOnlyContentRestriction: boolean;
  /**
   * Enables the Image Constraint For Analog Component Video Restriction in the license.
  */
  imageConstraintForAnalogComponentVideoRestriction: boolean;
  /**
   * Enables the Image Constraint For Analog Component Video Restriction in the license.
  */
  imageConstraintForAnalogComputerMonitorRestriction: boolean;
  /**
   * Configures Unknown output handling settings of the license. Possible values include:
   * 'Unknown', 'NotAllowed', 'Allowed', 'AllowedWithVideoConstriction'
  */
  allowPassingVideoContentToUnknownOutput: string;
  /**
   * Specifies the output protection level for uncompressed digital video.
  */
  uncompressedDigitalVideoOpl?: number;
  /**
   * Specifies the output protection level for compressed digital video.
  */
  compressedDigitalVideoOpl?: number;
  /**
   * Specifies the output protection level for compressed digital audio.
  */
  analogVideoOpl?: number;
  /**
   * Specifies the output protection level for compressed digital audio.
  */
  compressedDigitalAudioOpl?: number;
  /**
   * Specifies the output protection level for uncompressed digital audio.
  */
  uncompressedDigitalAudioOpl?: number;
}

/**
 * Represents a token claim.
*/
export interface ContentKeyPolicyTokenClaim {
  /**
   * Token claim type.
  */
  claimType?: string;
  /**
   * Token claim value.
  */
  claimValue?: string;
}

/**
 * The PlayReady license
*/
export interface ContentKeyPolicyPlayReadyLicense {
  /**
   * A flag indicating whether test devices can use the license.
  */
  allowTestDevices: boolean;
  /**
   * The begin date of license
  */
  beginDate?: Date;
  /**
   * The expiration date of license.
  */
  expirationDate?: Date;
  /**
   * The relative begin date of license.
  */
  relativeBeginDate?: moment.Duration;
  /**
   * The relative expiration date of license.
  */
  relativeExpirationDate?: moment.Duration;
  /**
   * The grace period of license.
  */
  gracePeriod?: moment.Duration;
  /**
   * The license PlayRight
  */
  playRight?: ContentKeyPolicyPlayReadyPlayRight;
  /**
   * The license type. Possible values include: 'Unknown', 'NonPersistent', 'Persistent'
  */
  licenseType: string;
  /**
   * The content key location.
  */
  contentKeyLocation: ContentKeyPolicyPlayReadyContentKeyLocation;
  /**
   * The PlayReady content type. Possible values include: 'Unknown', 'Unspecified',
   * 'UltraVioletDownload', 'UltraVioletStreaming'
  */
  contentType: string;
}

/**
 * Base class for Content Key Policy restrictions. A derived class must be used to create a
 * restriction.
*/
export interface ContentKeyPolicyRestriction {
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Represents an open restriction. License or key will be delivered on every request.
*/
export interface ContentKeyPolicyOpenRestriction extends ContentKeyPolicyRestriction {
}

/**
 * Represents a ContentKeyPolicyRestriction that is unavailable in the current API version.
*/
export interface ContentKeyPolicyUnknownRestriction extends ContentKeyPolicyRestriction {
}

/**
 * Base class for Content Key Policy configuration. A derived class must be used to create a
 * configuration.
*/
export interface ContentKeyPolicyConfiguration {
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Base class for Content Key Policy key for token validation. A derived class must be used to
 * create a token key.
*/
export interface ContentKeyPolicyRestrictionTokenKey {
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Specifies a symmetric key for token validation.
*/
export interface ContentKeyPolicySymmetricTokenKey extends ContentKeyPolicyRestrictionTokenKey {
  /**
   * The key value of the key
  */
  keyValue: Buffer;
}

/**
 * Specifies a RSA key for token validation
*/
export interface ContentKeyPolicyRsaTokenKey extends ContentKeyPolicyRestrictionTokenKey {
  /**
   * The RSA Parameter exponent
  */
  exponent: Buffer;
  /**
   * The RSA Parameter modulus
  */
  modulus: Buffer;
}

/**
 * Specifies a certificate for token validation.
*/
export interface ContentKeyPolicyX509CertificateTokenKey extends
ContentKeyPolicyRestrictionTokenKey {
  /**
   * The raw data field of a certificate in PKCS 12 format (X509Certificate2 in .NET)
  */
  rawBody: Buffer;
}

/**
 * Represents a token restriction. Provided token must match these requirements for successful
 * license or key delivery.
*/
export interface ContentKeyPolicyTokenRestriction extends ContentKeyPolicyRestriction {
  /**
   * The token issuer.
  */
  issuer: string;
  /**
   * The audience for the token.
  */
  audience: string;
  /**
   * The primary verification key.
  */
  primaryVerificationKey: ContentKeyPolicyRestrictionTokenKey;
  /**
   * A list of alternative verification keys.
  */
  alternateVerificationKeys?: ContentKeyPolicyRestrictionTokenKey[];
  /**
   * A list of required token claims.
  */
  requiredClaims?: ContentKeyPolicyTokenClaim[];
  /**
   * The type of token. Possible values include: 'Unknown', 'Swt', 'Jwt'
  */
  restrictionTokenType: string;
  /**
   * The OpenID connect discovery document.
  */
  openIdConnectDiscoveryDocument?: string;
}

/**
 * Represents a configuration for non-DRM keys.
*/
export interface ContentKeyPolicyClearKeyConfiguration extends ContentKeyPolicyConfiguration {
}

/**
 * Represents a ContentKeyPolicyConfiguration that is unavailable in the current API version.
*/
export interface ContentKeyPolicyUnknownConfiguration extends ContentKeyPolicyConfiguration {
}

/**
 * Specifies a configuration for Widevine licenses.
*/
export interface ContentKeyPolicyWidevineConfiguration extends ContentKeyPolicyConfiguration {
  /**
   * The Widevine template.
  */
  widevineTemplate: string;
}

/**
 * Specifies a configuration for PlayReady licenses.
*/
export interface ContentKeyPolicyPlayReadyConfiguration extends ContentKeyPolicyConfiguration {
  /**
   * The PlayReady licenses.
  */
  licenses: ContentKeyPolicyPlayReadyLicense[];
  /**
   * The custom response data.
  */
  responseCustomData?: string;
}

/**
 * Specifies a configuration for FairPlay licenses.
*/
export interface ContentKeyPolicyFairPlayConfiguration extends ContentKeyPolicyConfiguration {
  /**
   * The key that must be used as FairPlay Application Secret key.
  */
  ask: Buffer;
  /**
   * The password encrypting FairPlay certificate in PKCS 12 (pfx) format.
  */
  fairPlayPfxPassword: string;
  /**
   * The Base64 representation of FairPlay certificate in PKCS 12 (pfx) format (including private
   * key).
  */
  fairPlayPfx: string;
  /**
   * The rental and lease key type. Possible values include: 'Unknown', 'Undefined',
   * 'PersistentUnlimited', 'PersistentLimited'
  */
  rentalAndLeaseKeyType: string;
  /**
   * The rental duration. Must be greater than or equal to 0.
  */
  rentalDuration: number;
}

/**
 * Represents a policy option.
*/
export interface ContentKeyPolicyOption {
  /**
   * The legacy Policy Option ID.
  */
  readonly policyOptionId?: string;
  /**
   * The Policy Option description.
  */
  name?: string;
  /**
   * The key delivery configuration.
  */
  configuration: ContentKeyPolicyConfiguration;
  /**
   * The requirements that must be met to deliver keys with this configuration
  */
  restriction: ContentKeyPolicyRestriction;
}

/**
 * The properties of the Content Key Policy.
*/
export interface ContentKeyPolicyProperties {
  /**
   * The legacy Policy ID.
  */
  readonly policyId?: string;
  /**
   * The creation date of the Policy
  */
  readonly created?: Date;
  /**
   * The last modified date of the Policy
  */
  readonly lastModified?: Date;
  /**
   * A description for the Policy.
  */
  description?: string;
  /**
   * The Key Policy options.
  */
  options: ContentKeyPolicyOption[];
}

/**
 * A Content Key Policy resource.
*/
export interface ContentKeyPolicy extends ProxyResource {
  /**
   * The legacy Policy ID.
  */
  readonly policyId?: string;
  /**
   * The creation date of the Policy
  */
  readonly created?: Date;
  /**
   * The last modified date of the Policy
  */
  readonly lastModified?: Date;
  /**
   * A description for the Policy.
  */
  description?: string;
  /**
   * The Key Policy options.
  */
  options: ContentKeyPolicyOption[];
}

/**
 * Base type for all Presets, which define the recipe or instructions on how the input media files
 * should be processed.
*/
export interface Preset {
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Describes the basic properties of all codecs.
*/
export interface Codec {
  /**
   * An optional label for the codec. The label can be used to control muxing behavior.
  */
  label?: string;
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Defines the common properties for all audio codecs.
*/
export interface Audio extends Codec {
  /**
   * The number of channels in the audio.
  */
  channels?: number;
  /**
   * The sampling rate to use for encoding in hertz.
  */
  samplingRate?: number;
  /**
   * The bitrate, in bits per second, of the output encoded audio.
  */
  bitrate?: number;
}

/**
 * Describes Advanced Audio Codec (AAC) audio encoding settings.
*/
export interface AacAudio extends Audio {
  /**
   * The encoding profile to be used when encoding audio with AAC. Possible values include:
   * 'AacLc', 'HeAacV1', 'HeAacV2'
  */
  profile?: string;
}

/**
 * Describes all the settings to be used when analyzing a video in order to detect all the faces
 * present.
*/
export interface FaceDetectorPreset extends Preset {
  /**
   * Specifies the maximum resolution at which your video is analyzed. The default behavior is
   * "SourceResolution," which will keep the input video at its original resolution when analyzed.
   * Using "StandardDefinition" will resize input videos to standard definition while preserving
   * the appropriate aspect ratio. It will only resize if the video is of higher resolution. For
   * example, a 1920x1080 input would be scaled to 640x360 before processing. Switching to
   * "StandardDefinition" will reduce the time it takes to process high resolution video. It may
   * also reduce the cost of using this component (see
   * https://azure.microsoft.com/en-us/pricing/details/media-services/#analytics for details).
   * However, faces that end up being too small in the resized video may not be detected. Possible
   * values include: 'SourceResolution', 'StandardDefinition'
  */
  resolution?: string;
}

/**
 * The Audio Analyzer preset applies a pre-defined set of AI-based analysis operations, including
 * speech transcription. Currently, the preset supports processing of content with a single audio
 * track.
*/
export interface AudioAnalyzerPreset extends Preset {
  /**
   * The language for the audio payload in the input using the BCP-47 format of 'language
   * tag-region' (e.g: 'en-US').  The list of supported languages are English ('en-US' and
   * 'en-GB'), Spanish ('es-ES' and 'es-MX'), French ('fr-FR'), Italian ('it-IT'), Japanese
   * ('ja-JP'), Portuguese ('pt-BR'), Chinese ('zh-CN'), German ('de-DE'), Arabic ('ar-EG' and
   * 'ar-SY'), Russian ('ru-RU'), Hindi ('hi-IN'), and Korean ('ko-KR'). If you know the language
   * of your content, it is recommended that you specify it. If the language isn't specified or set
   * to null, automatic language detection will choose the first language detected and process with
   * the selected language for the duration of the file. This language detection feature currently
   * supports English, Chinese, French, German, Italian, Japanese, Spanish, Russian, and
   * Portuguese. It does not currently support dynamically switching between languages after the
   * first language is detected. The automatic detection works best with audio recordings with
   * clearly discernable speech. If automatic detection fails to find the language, transcription
   * would fallback to 'en-US'."
  */
  audioLanguage?: string;
}

/**
 * Base type for all overlays - image, audio or video.
*/
export interface Overlay {
  /**
   * The label of the job input which is to be used as an overlay. The Input must specify exactly
   * one file. You can specify an image file in JPG or PNG formats, or an audio file (such as a
   * WAV, MP3, WMA or M4A file), or a video file. See https://aka.ms/mesformats for the complete
   * list of supported audio and video file formats.
  */
  inputLabel: string;
  /**
   * The start position, with reference to the input video, at which the overlay starts. The value
   * should be in ISO 8601 format. For example, PT05S to start the overlay at 5 seconds in to the
   * input video. If not specified the overlay starts from the beginning of the input video.
  */
  start?: moment.Duration;
  /**
   * The position in the input video at which the overlay ends. The value should be in ISO 8601
   * duration format. For example, PT30S to end the overlay at 30 seconds in to the input video. If
   * not specified the overlay will be applied until the end of the input video if inputLoop is
   * true. Else, if inputLoop is false, then overlay will last as long as the duration of the
   * overlay media.
  */
  end?: moment.Duration;
  /**
   * The duration over which the overlay fades in onto the input video. The value should be in ISO
   * 8601 duration format. If not specified the default behavior is to have no fade in (same as
   * PT0S).
  */
  fadeInDuration?: moment.Duration;
  /**
   * The duration over which the overlay fades out of the input video. The value should be in ISO
   * 8601 duration format. If not specified the default behavior is to have no fade out (same as
   * PT0S).
  */
  fadeOutDuration?: moment.Duration;
  /**
   * The gain level of audio in the overlay. The value should be in the range [0, 1.0]. The default
   * is 1.0.
  */
  audioGainLevel?: number;
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Describes the properties of an audio overlay.
*/
export interface AudioOverlay extends Overlay {
}

/**
 * A codec flag, which tells the encoder to copy the input video bitstream without re-encoding.
*/
export interface CopyVideo extends Codec {
}

/**
 * Describes the basic properties for encoding the input video.
*/
export interface Video extends Codec {
  /**
   * The distance between two key frames, thereby defining a group of pictures (GOP). The value
   * should be a non-zero integer in the range [1, 30] seconds, specified in ISO 8601 format. The
   * default is 2 seconds (PT2S).
  */
  keyFrameInterval?: moment.Duration;
  /**
   * The resizing mode - how the input video will be resized to fit the desired output
   * resolution(s). Default is AutoSize. Possible values include: 'None', 'AutoSize', 'AutoFit'
  */
  stretchMode?: string;
}

/**
 * Describes the basic properties for generating thumbnails from the input video
*/
export interface Image extends Video {
  /**
   * The position in the input video from where to start generating thumbnails. The value can be in
   * absolute timestamp (ISO 8601, e.g: PT05S), or a frame count (For example, 10 for the 10th
   * frame), or a relative value (For example, 1%). Also supports a macro {Best}, which tells the
   * encoder to select the best thumbnail from the first few seconds of the video.
  */
  start: string;
  /**
   * The intervals at which thumbnails are generated. The value can be in absolute timestamp (ISO
   * 8601, e.g: PT05S for one image every 5 seconds), or a frame count (For example, 30 for every
   * 30 frames), or a relative value (For example, 1%).
  */
  step?: string;
  /**
   * The position in the input video at which to stop generating thumbnails. The value can be in
   * absolute timestamp (ISO 8601, e.g: PT5M30S to stop at 5 minutes and 30 seconds), or a frame
   * count (For example, 300 to stop at the 300th frame), or a relative value (For example, 100%).
  */
  range?: string;
}

/**
 * Base class for output.
*/
export interface Format {
  /**
   * The pattern of the file names for the generated output files. The following macros are
   * supported in the file name: {Basename} - The base name of the input video {Extension} - The
   * appropriate extension for this format. {Label} - The label assigned to the codec/layer.
   * {Index} - A unique index for thumbnails. Only applicable to thumbnails. {Bitrate} - The
   * audio/video bitrate. Not applicable to thumbnails. {Codec} - The type of the audio/video
   * codec. Any unsubstituted macros will be collapsed and removed from the filename.
  */
  filenamePattern: string;
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Describes the properties for an output image file.
*/
export interface ImageFormat extends Format {
}

/**
 * Describes the settings for producing JPEG thumbnails.
*/
export interface JpgFormat extends ImageFormat {
}

/**
 * Describes the settings for producing PNG thumbnails.
*/
export interface PngFormat extends ImageFormat {
}

/**
 * A codec flag, which tells the encoder to copy the input audio bitstream.
*/
export interface CopyAudio extends Codec {
}

/**
 * Describes the de-interlacing settings.
*/
export interface Deinterlace {
  /**
   * The field parity for de-interlacing, defaults to Auto. Possible values include: 'Auto',
   * 'TopFieldFirst', 'BottomFieldFirst'
  */
  parity?: string;
  /**
   * The deinterlacing mode. Defaults to AutoPixelAdaptive. Possible values include: 'Off',
   * 'AutoPixelAdaptive'
  */
  mode?: string;
}

/**
 * Describes the properties of a rectangular window applied to the input media before processing
 * it.
*/
export interface Rectangle {
  /**
   * The number of pixels from the left-margin. This can be absolute pixel value (e.g 100), or
   * relative to the size of the video (For example, 50%).
  */
  left?: string;
  /**
   * The number of pixels from the top-margin. This can be absolute pixel value (e.g 100), or
   * relative to the size of the video (For example, 50%).
  */
  top?: string;
  /**
   * The width of the rectangular region in pixels. This can be absolute pixel value (e.g 100), or
   * relative to the size of the video (For example, 50%).
  */
  width?: string;
  /**
   * The height of the rectangular region in pixels. This can be absolute pixel value (e.g 100), or
   * relative to the size of the video (For example, 50%).
  */
  height?: string;
}

/**
 * Describes all the filtering operations, such as de-interlacing, rotation etc. that are to be
 * applied to the input media before encoding.
*/
export interface Filters {
  /**
   * The de-interlacing settings.
  */
  deinterlace?: Deinterlace;
  /**
   * The rotation, if any, to be applied to the input video, before it is encoded. Default is Auto.
   * Possible values include: 'Auto', 'None', 'Rotate0', 'Rotate90', 'Rotate180', 'Rotate270'
  */
  rotation?: string;
  /**
   * The parameters for the rectangular window with which to crop the input video.
  */
  crop?: Rectangle;
  /**
   * The properties of overlays to be applied to the input video. These could be audio, image or
   * video overlays.
  */
  overlays?: Overlay[];
}

/**
 * The encoder can be configured to produce video and/or images (thumbnails) at different
 * resolutions, by specifying a layer for each desired resolution. A layer represents the
 * properties for the video or image at a resolution.
*/
export interface Layer {
  /**
   * The width of the output video for this layer. The value can be absolute (in pixels) or
   * relative (in percentage). For example 50% means the output video has half as many pixels in
   * width as the input.
  */
  width?: string;
  /**
   * The height of the output video for this layer. The value can be absolute (in pixels) or
   * relative (in percentage). For example 50% means the output video has half as many pixels in
   * height as the input.
  */
  height?: string;
  /**
   * The alphanumeric label for this layer, which can be used in multiplexing different video and
   * audio layers, or in naming the output file.
  */
  label?: string;
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Describes the settings to be used when encoding the input video into a desired output bitrate
 * layer.
*/
export interface VideoLayer extends Layer {
  /**
   * The average bitrate in bits per second at which to encode the input video when generating this
   * layer. This is a required field.
  */
  bitrate: number;
  /**
   * The maximum bitrate (in bits per second), at which the VBV buffer should be assumed to refill.
   * If not specified, defaults to the same value as bitrate.
  */
  maxBitrate?: number;
  /**
   * The number of B-frames to be used when encoding this layer.  If not specified, the encoder
   * chooses an appropriate number based on the video profile and level.
  */
  bFrames?: number;
  /**
   * The frame rate (in frames per second) at which to encode this layer. The value can be in the
   * form of M/N where M and N are integers (For example, 30000/1001), or in the form of a number
   * (For example, 30, or 29.97). The encoder enforces constraints on allowed frame rates based on
   * the profile and level. If it is not specified, the encoder will use the same frame rate as the
   * input video.
  */
  frameRate?: string;
  /**
   * The number of slices to be used when encoding this layer. If not specified, default is zero,
   * which means that encoder will use a single slice for each frame.
  */
  slices?: number;
  /**
   * Whether or not adaptive B-frames are to be used when encoding this layer. If not specified,
   * the encoder will turn it on whenever the video profile permits its use.
  */
  adaptiveBFrame?: boolean;
}

/**
 * Describes the settings to be used when encoding the input video into a desired output bitrate
 * layer with the H.264 video codec.
*/
export interface H264Layer extends VideoLayer {
  /**
   * We currently support Baseline, Main, High, High422, High444. Default is Auto. Possible values
   * include: 'Auto', 'Baseline', 'Main', 'High', 'High422', 'High444'
  */
  profile?: string;
  /**
   * We currently support Level up to 6.2. The value can be Auto, or a number that matches the
   * H.264 profile. If not specified, the default is Auto, which lets the encoder choose the Level
   * that is appropriate for this layer.
  */
  level?: string;
  /**
   * The VBV buffer window length. The value should be in ISO 8601 format. The value should be in
   * the range [0.1-100] seconds. The default is 5 seconds (for example, PT5S).
  */
  bufferWindow?: moment.Duration;
  /**
   * The number of reference frames to be used when encoding this layer. If not specified, the
   * encoder determines an appropriate number based on the encoder complexity setting.
  */
  referenceFrames?: number;
  /**
   * The entropy mode to be used for this layer. If not specified, the encoder chooses the mode
   * that is appropriate for the profile and level. Possible values include: 'Cabac', 'Cavlc'
  */
  entropyMode?: string;
}

/**
 * Describes all the properties for encoding a video with the H.264 codec.
*/
export interface H264Video extends Video {
  /**
   * Whether or not the encoder should insert key frames at scene changes. If not specified, the
   * default is false. This flag should be set to true only when the encoder is being configured to
   * produce a single output video.
  */
  sceneChangeDetection?: boolean;
  /**
   * Tells the encoder how to choose its encoding settings. The default value is Balanced. Possible
   * values include: 'Speed', 'Balanced', 'Quality'
  */
  complexity?: string;
  /**
   * The collection of output H.264 layers to be produced by the encoder.
  */
  layers?: H264Layer[];
}

/**
 * Describes the settings to produce a JPEG image from the input video.
*/
export interface JpgLayer extends Layer {
  /**
   * The compression quality of the JPEG output. Range is from 0-100 and the default is 70.
  */
  quality?: number;
}

/**
 * Describes the properties for producing a series of JPEG images from the input video.
*/
export interface JpgImage extends Image {
  /**
   * A collection of output JPEG image layers to be produced by the encoder.
  */
  layers?: JpgLayer[];
}

/**
 * Represents an output file produced.
*/
export interface OutputFile {
  /**
   * The list of labels that describe how the encoder should multiplex video and audio into an
   * output file. For example, if the encoder is producing two video layers with labels v1 and v2,
   * and one audio layer with label a1, then an array like '[v1, a1]' tells the encoder to produce
   * an output file with the video track represented by v1 and the audio track represented by a1.
  */
  labels: string[];
}

/**
 * Describes the properties for producing a collection of GOP aligned multi-bitrate files. The
 * default behavior is to produce one output file for each video layer which is muxed together with
 * all the audios. The exact output files produced can be controlled by specifying the outputFiles
 * collection.
*/
export interface MultiBitrateFormat extends Format {
  /**
   * The list of output files to produce.  Each entry in the list is a set of audio and video layer
   * labels to be muxed together .
  */
  outputFiles?: OutputFile[];
}

/**
 * Describes the properties for an output ISO MP4 file.
*/
export interface Mp4Format extends MultiBitrateFormat {
}

/**
 * Describes the settings to produce a PNG image from the input video.
*/
export interface PngLayer extends Layer {
}

/**
 * Describes the properties for producing a series of PNG images from the input video.
*/
export interface PngImage extends Image {
  /**
   * A collection of output PNG image layers to be produced by the encoder.
  */
  layers?: PngLayer[];
}

/**
 * Describes a built-in preset for encoding the input video with the Standard Encoder.
*/
export interface BuiltInStandardEncoderPreset extends Preset {
  /**
   * The built-in preset to be used for encoding videos. Possible values include:
   * 'H264SingleBitrateSD', 'H264SingleBitrate720p', 'H264SingleBitrate1080p', 'AdaptiveStreaming',
   * 'AACGoodQualityAudio', 'ContentAwareEncodingExperimental', 'H264MultipleBitrate1080p',
   * 'H264MultipleBitrate720p', 'H264MultipleBitrateSD'
  */
  presetName: string;
}

/**
 * Describes all the settings to be used when encoding the input video with the Standard Encoder.
*/
export interface StandardEncoderPreset extends Preset {
  /**
   * One or more filtering operations that are applied to the input media before encoding.
  */
  filters?: Filters;
  /**
   * The list of codecs to be used when encoding the input video.
  */
  codecs: Codec[];
  /**
   * The list of outputs to be produced by the encoder.
  */
  formats: Format[];
}

/**
 * A video analyzer preset that extracts insights (rich metadata) from both audio and video, and
 * outputs a JSON format file.
*/
export interface VideoAnalyzerPreset extends AudioAnalyzerPreset {
  /**
   * The type of insights to be extracted. If not set then based on the content the type will
   * selected.  If the content is audio only then only audio insights are extracted and if it is
   * video only. Possible values include: 'AudioInsightsOnly', 'VideoInsightsOnly', 'AllInsights'
  */
  insightsToExtract?: string;
}

/**
 * Describes the properties for generating an MPEG-2 Transport Stream (ISO/IEC 13818-1) output
 * video file(s).
*/
export interface TransportStreamFormat extends MultiBitrateFormat {
}

/**
 * Describes the properties of a video overlay.
*/
export interface VideoOverlay extends Overlay {
  /**
   * The location in the input video where the overlay is applied.
  */
  position?: Rectangle;
  /**
   * The opacity of the overlay. This is a value in the range [0 - 1.0]. Default is 1.0 which mean
   * the overlay is opaque.
  */
  opacity?: number;
  /**
   * An optional rectangular window used to crop the overlay image or video.
  */
  cropRectangle?: Rectangle;
}

/**
 * Describes the properties of a TransformOutput, which are the rules to be applied while
 * generating the desired output.
*/
export interface TransformOutput {
  /**
   * A Transform can define more than one outputs. This property defines what the service should do
   * when one output fails - either continue to produce other outputs, or, stop the other outputs.
   * The overall Job state will not reflect failures of outputs that are specified with
   * 'ContinueJob'. The default is 'StopProcessingJob'. Possible values include:
   * 'StopProcessingJob', 'ContinueJob'
  */
  onError?: string;
  /**
   * Sets the relative priority of the TransformOutputs within a Transform. This sets the priority
   * that the service uses for processing TransformOutputs. The default priority is Normal.
   * Possible values include: 'Low', 'Normal', 'High'
  */
  relativePriority?: string;
  /**
   * Preset that describes the operations that will be used to modify, transcode, or extract
   * insights from the source file to generate the output.
  */
  preset: Preset;
}

/**
 * A Transform encapsulates the rules or instructions for generating desired outputs from input
 * media, such as by transcoding or by extracting insights. After the Transform is created, it can
 * be applied to input media by creating Jobs.
*/
export interface Transform extends ProxyResource {
  /**
   * The UTC date and time when the Transform was created, in 'YYYY-MM-DDThh:mm:ssZ' format.
  */
  readonly created?: Date;
  /**
   * An optional verbose description of the Transform.
  */
  description?: string;
  /**
   * The UTC date and time when the Transform was last updated, in 'YYYY-MM-DDThh:mm:ssZ' format.
  */
  readonly lastModified?: Date;
  /**
   * An array of one or more TransformOutputs that the Transform should generate.
  */
  outputs: TransformOutput[];
}

/**
 * Base class for inputs to a Job.
*/
export interface JobInput {
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Represents input files for a Job.
*/
export interface JobInputClip extends JobInput {
  /**
   * List of files. Required for JobInputHttp. Maximum of 4000 characters each.
  */
  files?: string[];
  /**
   * A label that is assigned to a JobInputClip, that is used to satisfy a reference used in the
   * Transform. For example, a Transform can be authored so as to take an image file with the label
   * 'xyz' and apply it as an overlay onto the input video before it is encoded. When submitting a
   * Job, exactly one of the JobInputs should be the image file, and it should have the label
   * 'xyz'.
  */
  label?: string;
}

/**
 * Describes a list of inputs to a Job.
*/
export interface JobInputs extends JobInput {
  /**
   * List of inputs to a Job.
  */
  inputs?: JobInput[];
}

/**
 * Represents an Asset for input into a Job.
*/
export interface JobInputAsset extends JobInputClip {
  /**
   * The name of the input Asset.
  */
  assetName: string;
}

/**
 * Represents HTTPS job input.
*/
export interface JobInputHttp extends JobInputClip {
  /**
   * Base URI for HTTPS job input. It will be concatenated with provided file names. If no base uri
   * is given, then the provided file list is assumed to be fully qualified uris. Maximum length of
   * 4000 characters.
  */
  baseUri?: string;
}

/**
 * Details of JobOutput errors.
*/
export interface JobErrorDetail {
  /**
   * Code describing the error detail.
  */
  readonly code?: string;
  /**
   * A human-readable representation of the error.
  */
  readonly message?: string;
}

/**
 * Details of JobOutput errors.
*/
export interface JobError {
  /**
   * Error code describing the error. Possible values include: 'ServiceError',
   * 'ServiceTransientError', 'DownloadNotAccessible', 'DownloadTransientError',
   * 'UploadNotAccessible', 'UploadTransientError', 'ConfigurationUnsupported', 'ContentMalformed',
   * 'ContentUnsupported'
  */
  readonly code?: string;
  /**
   * A human-readable language-dependent representation of the error.
  */
  readonly message?: string;
  /**
   * Helps with categorization of errors. Possible values include: 'Service', 'Download', 'Upload',
   * 'Configuration', 'Content'
  */
  readonly category?: string;
  /**
   * Indicates that it may be possible to retry the Job. If retry is unsuccessful, please contact
   * Azure support via Azure Portal. Possible values include: 'DoNotRetry', 'MayRetry'
  */
  readonly retry?: string;
  /**
   * An array of details about specific errors that led to this reported error.
  */
  readonly details?: JobErrorDetail[];
}

/**
 * Describes all the properties of a JobOutput.
*/
export interface JobOutput {
  /**
   * If the JobOutput is in the Error state, it contains the details of the error.
  */
  readonly error?: JobError;
  /**
   * Describes the state of the JobOutput. Possible values include: 'Canceled', 'Canceling',
   * 'Error', 'Finished', 'Processing', 'Queued', 'Scheduled'
  */
  readonly state?: string;
  /**
   * If the JobOutput is in a Processing state, this contains the Job completion percentage. The
   * value is an estimate and not intended to be used to predict Job completion times. To determine
   * if the JobOutput is complete, use the State property.
  */
  readonly progress?: number;
  /**
   * A label that is assigned to a JobOutput in order to help uniquely identify it. This is useful
   * when your Transform has more than one TransformOutput, whereby your Job has more than one
   * JobOutput. In such cases, when you submit the Job, you will add two or more JobOutputs, in the
   * same order as TransformOutputs in the Transform. Subsequently, when you retrieve the Job,
   * either through events or on a GET request, you can use the label to easily identify the
   * JobOutput. If a label is not provided, a default value of '{presetName}_{outputIndex}' will be
   * used, where the preset name is the name of the preset in the corresponding TransformOutput and
   * the output index is the relative index of the this JobOutput within the Job. Note that this
   * index is the same as the relative index of the corresponding TransformOutput within its
   * Transform.
  */
  label?: string;
  /**
   * Polymorphic Discriminator
  */
  odatatype: string;
}

/**
 * Represents an Asset used as a JobOutput.
*/
export interface JobOutputAsset extends JobOutput {
  /**
   * The name of the output Asset.
  */
  assetName: string;
}

/**
 * A Job resource type. The progress and state can be obtained by polling a Job or subscribing to
 * events using EventGrid.
*/
export interface Job extends ProxyResource {
  /**
   * The UTC date and time when the Job was created, in 'YYYY-MM-DDThh:mm:ssZ' format.
  */
  readonly created?: Date;
  /**
   * The current state of the job. Possible values include: 'Canceled', 'Canceling', 'Error',
   * 'Finished', 'Processing', 'Queued', 'Scheduled'
  */
  readonly state?: string;
  /**
   * Optional customer supplied description of the Job.
  */
  description?: string;
  /**
   * The inputs for the Job.
  */
  input: JobInput;
  /**
   * The UTC date and time when the Job was last updated, in 'YYYY-MM-DDThh:mm:ssZ' format.
  */
  readonly lastModified?: Date;
  /**
   * The outputs for the Job.
  */
  outputs: JobOutput[];
  /**
   * Priority with which the job should be processed. Higher priority jobs are processed before
   * lower priority jobs. If not set, the default is normal. Possible values include: 'Low',
   * 'Normal', 'High'
  */
  priority?: string;
  /**
   * Customer provided key, value pairs that will be returned in Job and JobOutput state events.
  */
  correlationData?: { [propertyName: string]: string };
}

/**
 * Class to specify one track property condition
*/
export interface TrackPropertyCondition {
  /**
   * Track property type. Possible values include: 'Unknown', 'FourCC'
  */
  property: string;
  /**
   * Track property condition operation. Possible values include: 'Unknown', 'Equal'
  */
  operation: string;
  /**
   * Track property value
  */
  value?: string;
}

/**
 * Class to select a track
*/
export interface TrackSelection {
  /**
   * TrackSelections is a track property condition list which can specify track(s)
  */
  trackSelections?: TrackPropertyCondition[];
}

/**
 * Class to specify properties of default content key for each encryption scheme
*/
export interface DefaultKey {
  /**
   * Label can be used to specify Content Key when creating a Streaming Locator
  */
  label?: string;
  /**
   * Policy used by Default Key
  */
  policyName?: string;
}

/**
 * Class to specify properties of content key
*/
export interface StreamingPolicyContentKey {
  /**
   * Label can be used to specify Content Key when creating a Streaming Locator
  */
  label?: string;
  /**
   * Policy used by Content Key
  */
  policyName?: string;
  /**
   * Tracks which use this content key
  */
  tracks?: TrackSelection[];
}

/**
 * Class to specify properties of all content keys in Streaming Policy
*/
export interface StreamingPolicyContentKeys {
  /**
   * Default content key for an encryption scheme
  */
  defaultKey?: DefaultKey;
  /**
   * Representing tracks needs separate content key
  */
  keyToTrackMappings?: StreamingPolicyContentKey[];
}

/**
 * Class to specify configurations of PlayReady in Streaming Policy
*/
export interface StreamingPolicyPlayReadyConfiguration {
  /**
   * Template for the URL of the custom service delivering licenses to end user players.  Not
   * required when using Azure Media Services for issuing licenses.  The template supports
   * replaceable tokens that the service will update at runtime with the value specific to the
   * request.  The currently supported token values are {AlternativeMediaId}, which is replaced
   * with the value of StreamingLocatorId.AlternativeMediaId, and {ContentKeyId}, which is replaced
   * with the value of identifier of the key being requested.
  */
  customLicenseAcquisitionUrlTemplate?: string;
  /**
   * Custom attributes for PlayReady
  */
  playReadyCustomAttributes?: string;
}

/**
 * Class to specify configurations of Widevine in Streaming Policy
*/
export interface StreamingPolicyWidevineConfiguration {
  /**
   * Template for the URL of the custom service delivering licenses to end user players.  Not
   * required when using Azure Media Services for issuing licenses.  The template supports
   * replaceable tokens that the service will update at runtime with the value specific to the
   * request.  The currently supported token values are {AlternativeMediaId}, which is replaced
   * with the value of StreamingLocatorId.AlternativeMediaId, and {ContentKeyId}, which is replaced
   * with the value of identifier of the key being requested.
  */
  customLicenseAcquisitionUrlTemplate?: string;
}

/**
 * Class to specify configurations of FairPlay in Streaming Policy
*/
export interface StreamingPolicyFairPlayConfiguration {
  /**
   * Template for the URL of the custom service delivering licenses to end user players.  Not
   * required when using Azure Media Services for issuing licenses.  The template supports
   * replaceable tokens that the service will update at runtime with the value specific to the
   * request.  The currently supported token values are {AlternativeMediaId}, which is replaced
   * with the value of StreamingLocatorId.AlternativeMediaId, and {ContentKeyId}, which is replaced
   * with the value of identifier of the key being requested.
  */
  customLicenseAcquisitionUrlTemplate?: string;
  /**
   * All license to be persistent or not
  */
  allowPersistentLicense: boolean;
}

/**
 * Class to specify DRM configurations of CommonEncryptionCbcs scheme in Streaming Policy
*/
export interface CbcsDrmConfiguration {
  /**
   * FairPlay configurations
  */
  fairPlay?: StreamingPolicyFairPlayConfiguration;
  /**
   * PlayReady configurations
  */
  playReady?: StreamingPolicyPlayReadyConfiguration;
  /**
   * Widevine configurations
  */
  widevine?: StreamingPolicyWidevineConfiguration;
}

/**
 * Class to specify DRM configurations of CommonEncryptionCenc scheme in Streaming Policy
*/
export interface CencDrmConfiguration {
  /**
   * PlayReady configurations
  */
  playReady?: StreamingPolicyPlayReadyConfiguration;
  /**
   * Widevine configurations
  */
  widevine?: StreamingPolicyWidevineConfiguration;
}

/**
 * Class to specify which protocols are enabled
*/
export interface EnabledProtocols {
  /**
   * Enable Download protocol or not
  */
  download: boolean;
  /**
   * Enable DASH protocol or not
  */
  dash: boolean;
  /**
   * Enable HLS protocol or not
  */
  hls: boolean;
  /**
   * Enable SmoothStreaming protocol or not
  */
  smoothStreaming: boolean;
}

/**
 * Class for NoEncryption scheme
*/
export interface NoEncryption {
  /**
   * Representing supported protocols
  */
  enabledProtocols?: EnabledProtocols;
}

/**
 * Class for EnvelopeEncryption encryption scheme
*/
export interface EnvelopeEncryption {
  /**
   * Representing supported protocols
  */
  enabledProtocols?: EnabledProtocols;
  /**
   * Representing which tracks should not be encrypted
  */
  clearTracks?: TrackSelection[];
  /**
   * Representing default content key for each encryption scheme and separate content keys for
   * specific tracks
  */
  contentKeys?: StreamingPolicyContentKeys;
  /**
   * Template for the URL of the custom service delivering keys to end user players.  Not required
   * when using Azure Media Services for issuing keys.  The template supports replaceable tokens
   * that the service will update at runtime with the value specific to the request.  The currently
   * supported token values are {AlternativeMediaId}, which is replaced with the value of
   * StreamingLocatorId.AlternativeMediaId, and {ContentKeyId}, which is replaced with the value of
   * identifier of the key being requested.
  */
  customKeyAcquisitionUrlTemplate?: string;
}

/**
 * Class for envelope encryption scheme
*/
export interface CommonEncryptionCenc {
  /**
   * Representing supported protocols
  */
  enabledProtocols?: EnabledProtocols;
  /**
   * Representing which tracks should not be encrypted
  */
  clearTracks?: TrackSelection[];
  /**
   * Representing default content key for each encryption scheme and separate content keys for
   * specific tracks
  */
  contentKeys?: StreamingPolicyContentKeys;
  /**
   * Configuration of DRMs for CommonEncryptionCenc encryption scheme
  */
  drm?: CencDrmConfiguration;
}

/**
 * Class for CommonEncryptionCbcs encryption scheme
*/
export interface CommonEncryptionCbcs {
  /**
   * Representing supported protocols
  */
  enabledProtocols?: EnabledProtocols;
  /**
   * Representing which tracks should not be encrypted
  */
  clearTracks?: TrackSelection[];
  /**
   * Representing default content key for each encryption scheme and separate content keys for
   * specific tracks
  */
  contentKeys?: StreamingPolicyContentKeys;
  /**
   * Configuration of DRMs for current encryption scheme
  */
  drm?: CbcsDrmConfiguration;
}

/**
 * A Streaming Policy resource
*/
export interface StreamingPolicy extends ProxyResource {
  /**
   * Creation time of Streaming Policy
  */
  readonly created?: Date;
  /**
   * Default ContentKey used by current Streaming Policy
  */
  defaultContentKeyPolicyName?: string;
  /**
   * Configuration of EnvelopeEncryption
  */
  envelopeEncryption?: EnvelopeEncryption;
  /**
   * Configuration of CommonEncryptionCenc
  */
  commonEncryptionCenc?: CommonEncryptionCenc;
  /**
   * Configuration of CommonEncryptionCbcs
  */
  commonEncryptionCbcs?: CommonEncryptionCbcs;
  /**
   * Configurations of NoEncryption
  */
  noEncryption?: NoEncryption;
}

/**
 * Class for content key in Streaming Locator
*/
export interface StreamingLocatorContentKey {
  /**
   * ID of Content Key
  */
  id: string;
  /**
   * Encryption type of Content Key. Possible values include: 'CommonEncryptionCenc',
   * 'CommonEncryptionCbcs', 'EnvelopeEncryption'
  */
  readonly type?: string;
  /**
   * Label of Content Key as specified in the Streaming Policy
  */
  labelReferenceInStreamingPolicy?: string;
  /**
   * Value of Content Key
  */
  value?: string;
  /**
   * ContentKeyPolicy used by Content Key
  */
  readonly policyName?: string;
  /**
   * Tracks which use this Content Key
  */
  readonly tracks?: TrackSelection[];
}

/**
 * Class of paths for streaming
*/
export interface StreamingPath {
  /**
   * Streaming protocol. Possible values include: 'Hls', 'Dash', 'SmoothStreaming', 'Download'
  */
  streamingProtocol: string;
  /**
   * Encryption scheme. Possible values include: 'NoEncryption', 'EnvelopeEncryption',
   * 'CommonEncryptionCenc', 'CommonEncryptionCbcs'
  */
  encryptionScheme: string;
  /**
   * Streaming paths for each protocol and encryptionScheme pair
  */
  paths?: string[];
}

/**
 * Class of response for listContentKeys action
*/
export interface ListContentKeysResponse {
  /**
   * ContentKeys used by current Streaming Locator
  */
  contentKeys?: StreamingLocatorContentKey[];
}

/**
 * Class of response for listPaths action
*/
export interface ListPathsResponse {
  /**
   * Streaming Paths supported by current Streaming Locator
  */
  streamingPaths?: StreamingPath[];
  /**
   * Download Paths supported by current Streaming Locator
  */
  downloadPaths?: string[];
}

/**
 * A Streaming Locator resource
*/
export interface StreamingLocator extends ProxyResource {
  /**
   * Asset Name
  */
  assetName: string;
  /**
   * The creation time of the Streaming Locator.
  */
  readonly created?: Date;
  /**
   * The start time of the Streaming Locator.
  */
  startTime?: Date;
  /**
   * The end time of the Streaming Locator.
  */
  endTime?: Date;
  /**
   * The StreamingLocatorId of the Streaming Locator.
  */
  streamingLocatorId?: string;
  /**
   * Name of the Streaming Policy used by this Streaming Locator. Either specify the name of
   * Streaming Policy you created or use one of the predefined Streaming Policies. The predefined
   * Streaming Policies available are: 'Predefined_DownloadOnly', 'Predefined_ClearStreamingOnly',
   * 'Predefined_DownloadAndClearStreaming', 'Predefined_ClearKey',
   * 'Predefined_MultiDrmCencStreaming' and 'Predefined_MultiDrmStreaming'
  */
  streamingPolicyName: string;
  /**
   * Name of the default ContentKeyPolicy used by this Streaming Locator.
  */
  defaultContentKeyPolicyName?: string;
  /**
   * The ContentKeys used by this Streaming Locator.
  */
  contentKeys?: StreamingLocatorContentKey[];
  /**
   * Alternative Media ID of this Streaming Locator
  */
  alternativeMediaId?: string;
  /**
   * A list of asset or account filters which apply to this streaming locator
  */
  filters?: string[];
}

/**
 * The HLS configuration.
*/
export interface Hls {
  /**
   * The amount of fragments per HTTP Live Streaming (HLS) segment.
  */
  fragmentsPerTsSegment?: number;
}

/**
 * The Live Output.
*/
export interface LiveOutput extends ProxyResource {
  /**
   * The description of the Live Output.
  */
  description?: string;
  /**
   * The asset name.
  */
  assetName: string;
  /**
   * ISO 8601 timespan duration of the archive window length. This is duration that customer want
   * to retain the recorded content.
  */
  archiveWindowLength: moment.Duration;
  /**
   * The manifest file name.  If not provided, the service will generate one automatically.
  */
  manifestName?: string;
  /**
   * The HLS configuration.
  */
  hls?: Hls;
  /**
   * The output snapshot time.
  */
  outputSnapTime?: number;
  /**
   * The exact time the Live Output was created.
  */
  readonly created?: Date;
  /**
   * The exact time the Live Output was last modified.
  */
  readonly lastModified?: Date;
  /**
   * The provisioning state of the Live Output.
  */
  readonly provisioningState?: string;
  /**
   * The resource state of the Live Output. Possible values include: 'Creating', 'Running',
   * 'Deleting'
  */
  readonly resourceState?: string;
}

/**
 * The Live Event endpoint.
*/
export interface LiveEventEndpoint {
  /**
   * The endpoint protocol.
  */
  protocol?: string;
  /**
   * The endpoint URL.
  */
  url?: string;
}

/**
 * The IP address range in the CIDR scheme.
*/
export interface IPRange {
  /**
   * The friendly name for the IP address range.
  */
  name?: string;
  /**
   * The IP address.
  */
  address?: string;
  /**
   * The subnet mask prefix length (see CIDR notation).
  */
  subnetPrefixLength?: number;
}

/**
 * The IP access control.
*/
export interface IPAccessControl {
  /**
   * The IP allow list.
  */
  allow?: IPRange[];
}

/**
 * The IP access control for Live Event Input.
*/
export interface LiveEventInputAccessControl {
  /**
   * The IP access control properties.
  */
  ip?: IPAccessControl;
}

/**
 * The Live Event input.
*/
export interface LiveEventInput {
  /**
   * The streaming protocol for the Live Event.  This is specified at creation time and cannot be
   * updated. Possible values include: 'FragmentedMP4', 'RTMP'
  */
  streamingProtocol: string;
  /**
   * The access control for LiveEvent Input.
  */
  accessControl?: LiveEventInputAccessControl;
  /**
   * ISO 8601 timespan duration of the key frame interval duration.
  */
  keyFrameIntervalDuration?: string;
  /**
   * A unique identifier for a stream.  This can be specified at creation time but cannot be
   * updated.  If omitted, the service will generate a unique value.
  */
  accessToken?: string;
  /**
   * The input endpoints for the Live Event.
  */
  endpoints?: LiveEventEndpoint[];
}

/**
 * The IP access control for Live Event preview.
*/
export interface LiveEventPreviewAccessControl {
  /**
   * The IP access control properties.
  */
  ip?: IPAccessControl;
}

/**
 * The Live Event preview.
*/
export interface LiveEventPreview {
  /**
   * The endpoints for preview.
  */
  endpoints?: LiveEventEndpoint[];
  /**
   * The access control for LiveEvent preview.
  */
  accessControl?: LiveEventPreviewAccessControl;
  /**
   * The identifier of the preview locator in Guid format.  Specifying this at creation time allows
   * the caller to know the preview locator url before the event is created.  If omitted, the
   * service will generate a random identifier.  This value cannot be updated once the live event
   * is created.
  */
  previewLocator?: string;
  /**
   * The name of streaming policy used for the LiveEvent preview.  This value is specified at
   * creation time and cannot be updated.
  */
  streamingPolicyName?: string;
  /**
   * An Alternative Media Identifier associated with the StreamingLocator created for the preview.
   * This value is specified at creation time and cannot be updated.  The identifier can be used in
   * the CustomLicenseAcquisitionUrlTemplate or the CustomKeyAcquisitionUrlTemplate of the
   * StreamingPolicy specified in the StreamingPolicyName field.
  */
  alternativeMediaId?: string;
}

/**
 * The Live Event encoding.
*/
export interface LiveEventEncoding {
  /**
   * The encoding type for Live Event.  This value is specified at creation time and cannot be
   * updated. Possible values include: 'None', 'Basic', 'Standard'
  */
  encodingType?: string;
  /**
   * The encoding preset name.  This value is specified at creation time and cannot be updated.
  */
  presetName?: string;
}

/**
 * The client access policy.
*/
export interface CrossSiteAccessPolicies {
  /**
   * The content of clientaccesspolicy.xml used by Silverlight.
  */
  clientAccessPolicy?: string;
  /**
   * The content of crossdomain.xml used by Silverlight.
  */
  crossDomainPolicy?: string;
}

/**
 * The LiveEvent action input parameter definition.
*/
export interface LiveEventActionInput {
  /**
   * The flag indicates if remove LiveOutputs on Stop.
  */
  removeOutputsOnStop?: boolean;
}

/**
 * The Live Event.
*/
export interface LiveEvent extends TrackedResource {
  /**
   * The Live Event description.
  */
  description?: string;
  /**
   * The Live Event input.
  */
  input: LiveEventInput;
  /**
   * The Live Event preview.
  */
  preview?: LiveEventPreview;
  /**
   * The Live Event encoding.
  */
  encoding?: LiveEventEncoding;
  /**
   * The provisioning state of the Live Event.
  */
  readonly provisioningState?: string;
  /**
   * The resource state of the Live Event. Possible values include: 'Stopped', 'Starting',
   * 'Running', 'Stopping', 'Deleting'
  */
  readonly resourceState?: string;
  /**
   * The Live Event access policies.
  */
  crossSiteAccessPolicies?: CrossSiteAccessPolicies;
  /**
   * Specifies whether to use a vanity url with the Live Event.  This value is specified at
   * creation time and cannot be updated.
  */
  vanityUrl?: boolean;
  /**
   * The options to use for the LiveEvent.  This value is specified at creation time and cannot be
   * updated.
  */
  streamOptions?: string[];
  /**
   * The exact time the Live Event was created.
  */
  readonly created?: Date;
  /**
   * The exact time the Live Event was last modified.
  */
  readonly lastModified?: Date;
}

/**
 * Akamai Signature Header authentication key.
*/
export interface AkamaiSignatureHeaderAuthenticationKey {
  /**
   * identifier of the key
  */
  identifier?: string;
  /**
   * authentication key
  */
  base64Key?: string;
  /**
   * The expiration time of the authentication key.
  */
  expiration?: Date;
}

/**
 * Akamai access control
*/
export interface AkamaiAccessControl {
  /**
   * authentication key list
  */
  akamaiSignatureHeaderAuthenticationKeyList?: AkamaiSignatureHeaderAuthenticationKey[];
}

/**
 * StreamingEndpoint access control definition.
*/
export interface StreamingEndpointAccessControl {
  /**
   * The access control of Akamai
  */
  akamai?: AkamaiAccessControl;
  /**
   * The IP access control of the StreamingEndpoint.
  */
  ip?: IPAccessControl;
}

/**
 * scale units definition
*/
export interface StreamingEntityScaleUnit {
  /**
   * The scale unit number of the StreamingEndpoint.
  */
  scaleUnit?: number;
}

/**
 * The StreamingEndpoint.
*/
export interface StreamingEndpoint extends TrackedResource {
  /**
   * The StreamingEndpoint description.
  */
  description?: string;
  /**
   * The number of scale units.  Use the Scale operation to adjust this value.
  */
  scaleUnits: number;
  /**
   * The name of the AvailabilitySet used with this StreamingEndpoint for high availability
   * streaming.  This value can only be set at creation time.
  */
  availabilitySetName?: string;
  /**
   * The access control definition of the StreamingEndpoint.
  */
  accessControl?: StreamingEndpointAccessControl;
  /**
   * Max cache age
  */
  maxCacheAge?: number;
  /**
   * The custom host names of the StreamingEndpoint
  */
  customHostNames?: string[];
  /**
   * The StreamingEndpoint host name.
  */
  readonly hostName?: string;
  /**
   * The CDN enabled flag.
  */
  cdnEnabled?: boolean;
  /**
   * The CDN provider name.
  */
  cdnProvider?: string;
  /**
   * The CDN profile name.
  */
  cdnProfile?: string;
  /**
   * The provisioning state of the StreamingEndpoint.
  */
  readonly provisioningState?: string;
  /**
   * The resource state of the StreamingEndpoint. Possible values include: 'Stopped', 'Starting',
   * 'Running', 'Stopping', 'Deleting', 'Scaling'
  */
  readonly resourceState?: string;
  /**
   * The StreamingEndpoint access policies.
  */
  crossSiteAccessPolicies?: CrossSiteAccessPolicies;
  /**
   * The free trial expiration time.
  */
  readonly freeTrialEndTime?: Date;
  /**
   * The exact time the StreamingEndpoint was created.
  */
  readonly created?: Date;
  /**
   * The exact time the StreamingEndpoint was last modified.
  */
  readonly lastModified?: Date;
}

/**
 * A collection of AccountFilter items.
*/
export interface AccountFilterCollection extends Array<AccountFilter> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of Operation items.
*/
export interface OperationCollection extends Array<Operation> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of MediaService items.
*/
export interface MediaServiceCollection extends Array<MediaService> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of SubscriptionMediaService items.
*/
export interface SubscriptionMediaServiceCollection extends Array<SubscriptionMediaService> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of Asset items.
*/
export interface AssetCollection extends Array<Asset> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of AssetFilter items.
*/
export interface AssetFilterCollection extends Array<AssetFilter> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of ContentKeyPolicy items.
*/
export interface ContentKeyPolicyCollection extends Array<ContentKeyPolicy> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of Transform items.
*/
export interface TransformCollection extends Array<Transform> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of Job items.
*/
export interface JobCollection extends Array<Job> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of StreamingPolicy items.
*/
export interface StreamingPolicyCollection extends Array<StreamingPolicy> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * A collection of StreamingLocator items.
*/
export interface StreamingLocatorCollection extends Array<StreamingLocator> {
  /**
   * A link to the next page of the collection (when the collection contains too many results to
   * return in one response).
  */
  odatanextLink?: string;
}

/**
 * @summary LiveEventListResult
 * @description The LiveEvent list result.
*/
export interface LiveEventListResult extends Array<LiveEvent> {
  /**
   * Th link to the next set of results. Not empty if value contains incomplete list of Live
   * Outputs.
  */
  odatanextLink?: string;
}

/**
 * @summary LiveOutputListResult
 * @description The LiveOutput list result.
*/
export interface LiveOutputListResult extends Array<LiveOutput> {
  /**
   * Th link to the next set of results. Not empty if value contains incomplete list of Live
   * Outputs.
  */
  odatanextLink?: string;
}

/**
 * @summary StreamingEndpointListResult
 * @description The StreamingEndpoint list result.
*/
export interface StreamingEndpointListResult extends Array<StreamingEndpoint> {
  /**
   * Th link to the next set of results. Not empty if value contains incomplete list of
   * StreamingEndpoints.
  */
  odatanextLink?: string;
}
