/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Display metadata associated with the operation.
 *
 * @member {string} [provider] Service provider: Microsoft
 * OperationsManagement.
 * @member {string} [resource] Resource on which the operation is performed
 * etc.
 * @member {string} [operation] Type of operation: get, read, delete, etc.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Supported operation of OperationsManagement resource provider.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] Display metadata associated with the operation.
 * @member {string} [display.provider] Service provider: Microsoft
 * OperationsManagement.
 * @member {string} [display.resource] Resource on which the operation is
 * performed etc.
 * @member {string} [display.operation] Type of operation: get, read, delete,
 * etc.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the SolutionProperties class.
 * @constructor
 * Solution properties supported by the OperationsManagement resource provider.
 *
 * @member {string} workspaceResourceId The azure resourceId for the workspace
 * where the solution will be deployed/enabled.
 * @member {string} [provisioningState] The provisioning state for the
 * solution.
 * @member {array} [containedResources] The azure resources that will be
 * contained within the solutions. They will be locked and gets deleted
 * automatically when the solution is deleted.
 * @member {array} [referencedResources] The resources that will be referenced
 * from this solution. Deleting any of those solution out of band will break
 * the solution.
 */
export interface SolutionProperties extends BaseResource {
  workspaceResourceId: string;
  readonly provisioningState?: string;
  containedResources?: string[];
  referencedResources?: string[];
}

/**
 * @class
 * Initializes a new instance of the ManagementAssociationProperties class.
 * @constructor
 * ManagementAssociation properties supported by the OperationsManagement
 * resource provider.
 *
 * @member {string} applicationId The applicationId of the appliance for this
 * association.
 */
export interface ManagementAssociationProperties extends BaseResource {
  applicationId: string;
}

/**
 * @class
 * Initializes a new instance of the ArmTemplateParameter class.
 * @constructor
 * Parameter to pass to ARM template
 *
 * @member {string} [name] name of the parameter.
 * @member {string} [value] value for the parameter. In Jtoken
 */
export interface ArmTemplateParameter {
  name?: string;
  value?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagementConfigurationProperties class.
 * @constructor
 * ManagementConfiguration properties supported by the OperationsManagement
 * resource provider.
 *
 * @member {string} [applicationId] The applicationId of the appliance for this
 * Management.
 * @member {string} parentResourceType The type of the parent resource.
 * @member {array} parameters Parameters to run the ARM template
 * @member {string} [provisioningState] The provisioning state for the
 * ManagementConfiguration.
 * @member {object} template The Json object containing the ARM template to
 * deploy
 */
export interface ManagementConfigurationProperties extends BaseResource {
  applicationId?: string;
  parentResourceType: string;
  parameters: ArmTemplateParameter[];
  readonly provisioningState?: string;
  template: any;
}

/**
 * @class
 * Initializes a new instance of the SolutionPlan class.
 * @constructor
 * Plan for solution object supported by the OperationsManagement resource
 * provider.
 *
 * @member {string} [name] name of the solution to be created. For Microsoft
 * published solution it should be in the format of
 * solutionType(workspaceName). SolutionType part is case sensitive. For third
 * party solution, it can be anything.
 * @member {string} [publisher] Publisher name. For gallery solution, it is
 * Microsoft.
 * @member {string} [promotionCode] promotionCode, Not really used now, can you
 * left as empty
 * @member {string} [product] name of the solution to enabled/add. For
 * Microsoft published gallery solution it should be in the format of
 * OMSGallery/<solutionType>. This is case sensitive
 */
export interface SolutionPlan {
  name?: string;
  publisher?: string;
  promotionCode?: string;
  product?: string;
}

/**
 * @class
 * Initializes a new instance of the Solution class.
 * @constructor
 * The container for solution.
 *
 * @member {string} [id] Resource ID.
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 * @member {string} [location] Resource location
 * @member {object} [plan] Plan for solution object supported by the
 * OperationsManagement resource provider.
 * @member {string} [plan.name] name of the solution to be created. For
 * Microsoft published solution it should be in the format of
 * solutionType(workspaceName). SolutionType part is case sensitive. For third
 * party solution, it can be anything.
 * @member {string} [plan.publisher] Publisher name. For gallery solution, it
 * is Microsoft.
 * @member {string} [plan.promotionCode] promotionCode, Not really used now,
 * can you left as empty
 * @member {string} [plan.product] name of the solution to enabled/add. For
 * Microsoft published gallery solution it should be in the format of
 * OMSGallery/<solutionType>. This is case sensitive
 * @member {object} [properties] Properties for solution object supported by
 * the OperationsManagement resource provider.
 * @member {string} [properties.workspaceResourceId] The azure resourceId for
 * the workspace where the solution will be deployed/enabled.
 * @member {string} [properties.provisioningState] The provisioning state for
 * the solution.
 * @member {array} [properties.containedResources] The azure resources that
 * will be contained within the solutions. They will be locked and gets deleted
 * automatically when the solution is deleted.
 * @member {array} [properties.referencedResources] The resources that will be
 * referenced from this solution. Deleting any of those solution out of band
 * will break the solution.
 */
export interface Solution extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  plan?: SolutionPlan;
  properties?: SolutionProperties;
}

/**
 * @class
 * Initializes a new instance of the SolutionPropertiesList class.
 * @constructor
 * the list of solution response
 *
 * @member {array} [value] List of solution properites within the subscription.
 */
export interface SolutionPropertiesList {
  value?: Solution[];
}

/**
 * @class
 * Initializes a new instance of the ManagementAssociation class.
 * @constructor
 * The container for solution.
 *
 * @member {string} [id] Resource ID.
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 * @member {string} [location] Resource location
 * @member {object} [properties] Properties for ManagementAssociation object
 * supported by the OperationsManagement resource provider.
 * @member {string} [properties.applicationId] The applicationId of the
 * appliance for this association.
 */
export interface ManagementAssociation extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  properties?: ManagementAssociationProperties;
}

/**
 * @class
 * Initializes a new instance of the ManagementAssociationPropertiesList class.
 * @constructor
 * the list of ManagementAssociation response
 *
 * @member {array} [value] List of Management Association properites within the
 * subscription.
 */
export interface ManagementAssociationPropertiesList {
  value?: ManagementAssociation[];
}

/**
 * @class
 * Initializes a new instance of the ManagementConfiguration class.
 * @constructor
 * The container for solution.
 *
 * @member {string} [id] Resource ID.
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 * @member {string} [location] Resource location
 * @member {object} [properties] Properties for ManagementConfiguration object
 * supported by the OperationsManagement resource provider.
 * @member {string} [properties.applicationId] The applicationId of the
 * appliance for this Management.
 * @member {string} [properties.parentResourceType] The type of the parent
 * resource.
 * @member {array} [properties.parameters] Parameters to run the ARM template
 * @member {string} [properties.provisioningState] The provisioning state for
 * the ManagementConfiguration.
 * @member {object} [properties.template] The Json object containing the ARM
 * template to deploy
 */
export interface ManagementConfiguration extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  properties?: ManagementConfigurationProperties;
}

/**
 * @class
 * Initializes a new instance of the ManagementConfigurationPropertiesList class.
 * @constructor
 * the list of ManagementConfiguration response
 *
 * @member {array} [value] List of Management Configuration properites within
 * the subscription.
 */
export interface ManagementConfigurationPropertiesList {
  value?: ManagementConfiguration[];
}

/**
 * @class
 * Initializes a new instance of the CodeMessageErrorError class.
 * @constructor
 * The error details for a failed request.
 *
 * @member {string} [code] The error type.
 * @member {string} [message] The error message.
 */
export interface CodeMessageErrorError {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the CodeMessageError class.
 * @constructor
 * The error body contract.
 *
 * @member {object} [error] The error details for a failed request.
 * @member {string} [error.code] The error type.
 * @member {string} [error.message] The error message.
 */
export interface CodeMessageError {
  error?: CodeMessageErrorError;
}


/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result of the request to list solution operations.
 *
 */
export interface OperationListResult extends Array<Operation> {
}
