/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import { BaseResource, CloudError } from "ms-rest-azure";
import * as moment from "moment";

export {

  BaseResource,
  CloudError
};

/**
 * Resource properties.
 */
export interface ProxyResource extends BaseResource {
  /**
   * Resource ID
   */
  readonly id?: string;
  /**
   * Resource name.
   */
  readonly name?: string;
  /**
   * Resource type.
   */
  readonly type?: string;
}

/**
 * Resource properties including location and tags for track resources.
 */
export interface TrackedResource extends ProxyResource {
  /**
   * The location the resource resides in.
   */
  location: string;
  /**
   * Application-specific metadata in the form of key-value pairs.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * Storage Profile properties of a server
 */
export interface StorageProfile {
  /**
   * Backup retention days for the server.
   */
  backupRetentionDays?: number;
  /**
   * Enable Geo-redundant or not for server backup. Possible values include: 'Enabled', 'Disabled'
   */
  geoRedundantBackup?: string;
  /**
   * Max storage allowed for a server.
   */
  storageMB?: number;
}

/**
 * The properties used to create a new server.
 */
export interface ServerPropertiesForCreate {
  /**
   * Server version. Possible values include: '9.5', '9.6', '10', '10.0', '10.2'
   */
  version?: string;
  /**
   * Enable ssl enforcement or not when connect to server. Possible values include: 'Enabled',
   * 'Disabled'
   */
  sslEnforcement?: string;
  /**
   * Storage profile of a server.
   */
  storageProfile?: StorageProfile;
  /**
   * Polymorphic Discriminator
   */
  createMode: string;
}

/**
 * The properties used to create a new server.
 */
export interface ServerPropertiesForDefaultCreate extends ServerPropertiesForCreate {
  /**
   * The administrator's login name of a server. Can only be specified when the server is being
   * created (and is required for creation).
   */
  administratorLogin: string;
  /**
   * The password of the administrator login.
   */
  administratorLoginPassword: string;
}

/**
 * The properties used to create a new server by restoring from a backup.
 */
export interface ServerPropertiesForRestore extends ServerPropertiesForCreate {
  /**
   * The source server id to restore from.
   */
  sourceServerId: string;
  /**
   * Restore point creation time (ISO8601 format), specifying the time to restore from.
   */
  restorePointInTime: Date;
}

/**
 * The properties used to create a new server by restoring to a different region from a geo
 * replicated backup.
 */
export interface ServerPropertiesForGeoRestore extends ServerPropertiesForCreate {
  /**
   * The source server id to restore from.
   */
  sourceServerId: string;
}

/**
 * The properties to create a new replica.
 */
export interface ServerPropertiesForReplica extends ServerPropertiesForCreate {
  /**
   * The master server id to create replica from.
   */
  sourceServerId: string;
}

/**
 * Billing information related properties of a server.
 */
export interface Sku {
  /**
   * The name of the sku, typically, tier + family + cores, e.g. B_Gen4_1, GP_Gen5_8.
   */
  name?: string;
  /**
   * The tier of the particular SKU, e.g. Basic. Possible values include: 'Basic',
   * 'GeneralPurpose', 'MemoryOptimized'
   */
  tier?: string;
  /**
   * The scale up/out capacity, representing server's compute units.
   */
  capacity?: number;
  /**
   * The size code, to be interpreted by resource as appropriate.
   */
  size?: string;
  /**
   * The family of hardware.
   */
  family?: string;
}

/**
 * Represents a server.
 */
export interface Server extends TrackedResource {
  /**
   * The SKU (pricing tier) of the server.
   */
  sku?: Sku;
  /**
   * The administrator's login name of a server. Can only be specified when the server is being
   * created (and is required for creation).
   */
  administratorLogin?: string;
  /**
   * Server version. Possible values include: '9.5', '9.6', '10', '10.0', '10.2'
   */
  version?: string;
  /**
   * Enable ssl enforcement or not when connect to server. Possible values include: 'Enabled',
   * 'Disabled'
   */
  sslEnforcement?: string;
  /**
   * A state of a server that is visible to user. Possible values include: 'Ready', 'Dropping',
   * 'Disabled'
   */
  userVisibleState?: string;
  /**
   * The fully qualified domain name of a server.
   */
  fullyQualifiedDomainName?: string;
  /**
   * Earliest restore point creation time (ISO8601 format)
   */
  earliestRestoreDate?: Date;
  /**
   * Storage profile of a server.
   */
  storageProfile?: StorageProfile;
  /**
   * The replication role of the server.
   */
  replicationRole?: string;
  /**
   * The master server id of a replica server.
   */
  masterServerId?: string;
  /**
   * The maximum number of replicas that a master server can have.
   */
  replicaCapacity?: number;
}

/**
 * Represents a server to be created.
 */
export interface ServerForCreate {
  /**
   * The SKU (pricing tier) of the server.
   */
  sku?: Sku;
  /**
   * Properties of the server.
   */
  properties: ServerPropertiesForCreate;
  /**
   * The location the resource resides in.
   */
  location: string;
  /**
   * Application-specific metadata in the form of key-value pairs.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * Parameters allowed to update for a server.
 */
export interface ServerUpdateParameters {
  /**
   * The SKU (pricing tier) of the server.
   */
  sku?: Sku;
  /**
   * Storage profile of a server.
   */
  storageProfile?: StorageProfile;
  /**
   * The password of the administrator login.
   */
  administratorLoginPassword?: string;
  /**
   * The version of a server. Possible values include: '9.5', '9.6', '10', '10.0', '10.2'
   */
  version?: string;
  /**
   * Enable ssl enforcement or not when connect to server. Possible values include: 'Enabled',
   * 'Disabled'
   */
  sslEnforcement?: string;
  /**
   * The replication role of the server.
   */
  replicationRole?: string;
  /**
   * Application-specific metadata in the form of key-value pairs.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * Represents a server firewall rule.
 */
export interface FirewallRule extends ProxyResource {
  /**
   * The start IP address of the server firewall rule. Must be IPv4 format.
   */
  startIpAddress: string;
  /**
   * The end IP address of the server firewall rule. Must be IPv4 format.
   */
  endIpAddress: string;
}

/**
 * A virtual network rule.
 */
export interface VirtualNetworkRule extends ProxyResource {
  /**
   * The ARM resource id of the virtual network subnet.
   */
  virtualNetworkSubnetId: string;
  /**
   * Create firewall rule before the virtual network has vnet service endpoint enabled.
   */
  ignoreMissingVnetServiceEndpoint?: boolean;
  /**
   * Virtual Network Rule State. Possible values include: 'Initializing', 'InProgress', 'Ready',
   * 'Deleting', 'Unknown'
   */
  readonly state?: string;
}

/**
 * Represents a Database.
 */
export interface Database extends ProxyResource {
  /**
   * The charset of the database.
   */
  charset?: string;
  /**
   * The collation of the database.
   */
  collation?: string;
}

/**
 * Represents a Configuration.
 */
export interface Configuration extends ProxyResource {
  /**
   * Value of the configuration.
   */
  value?: string;
  /**
   * Description of the configuration.
   */
  readonly description?: string;
  /**
   * Default value of the configuration.
   */
  readonly defaultValue?: string;
  /**
   * Data type of the configuration.
   */
  readonly dataType?: string;
  /**
   * Allowed values of the configuration.
   */
  readonly allowedValues?: string;
  /**
   * Source of the configuration.
   */
  source?: string;
}

/**
 * Display metadata associated with the operation.
 */
export interface OperationDisplay {
  /**
   * Operation resource provider name.
   */
  readonly provider?: string;
  /**
   * Resource on which the operation is performed.
   */
  readonly resource?: string;
  /**
   * Localized friendly name for the operation.
   */
  readonly operation?: string;
  /**
   * Operation description.
   */
  readonly description?: string;
}

/**
 * REST API operation definition.
 */
export interface Operation {
  /**
   * The name of the operation being performed on this particular object.
   */
  readonly name?: string;
  /**
   * The localized display information for this particular operation or action.
   */
  readonly display?: OperationDisplay;
  /**
   * The intended executor of the operation. Possible values include: 'NotSpecified', 'user',
   * 'system'
   */
  readonly origin?: string;
  /**
   * Additional descriptions for the operation.
   */
  readonly properties?: { [propertyName: string]: any };
}

/**
 * A list of resource provider operations.
 */
export interface OperationListResult {
  /**
   * The list of resource provider operations.
   */
  value?: Operation[];
}

/**
 * Represents a log file.
 */
export interface LogFile extends ProxyResource {
  /**
   * Size of the log file.
   */
  sizeInKB?: number;
  /**
   * Creation timestamp of the log file.
   */
  readonly createdTime?: Date;
  /**
   * Last modified timestamp of the log file.
   */
  readonly lastModifiedTime?: Date;
  /**
   * Type of the log file.
   */
  logFileType?: string;
  /**
   * The url to download the log file from.
   */
  url?: string;
}

/**
 * Service level objectives for performance tier.
 */
export interface PerformanceTierServiceLevelObjectives {
  /**
   * ID for the service level objective.
   */
  id?: string;
  /**
   * Edition of the performance tier.
   */
  edition?: string;
  /**
   * vCore associated with the service level objective
   */
  vCore?: number;
  /**
   * Hardware generation associated with the service level objective
   */
  hardwareGeneration?: string;
  /**
   * Maximum Backup retention in days for the performance tier edition
   */
  maxBackupRetentionDays?: number;
  /**
   * Minimum Backup retention in days for the performance tier edition
   */
  minBackupRetentionDays?: number;
  /**
   * Max storage allowed for a server.
   */
  maxStorageMB?: number;
  /**
   * Max storage allowed for a server.
   */
  minStorageMB?: number;
}

/**
 * Performance tier properties
 */
export interface PerformanceTierProperties {
  /**
   * ID of the performance tier.
   */
  id?: string;
  /**
   * Service level objectives associated with the performance tier
   */
  serviceLevelObjectives?: PerformanceTierServiceLevelObjectives[];
}

/**
 * Request from client to check resource name availability.
 */
export interface NameAvailabilityRequest {
  /**
   * Resource name to verify.
   */
  name: string;
  /**
   * Resource type used for verification.
   */
  type?: string;
}

/**
 * Represents a resource name availability.
 */
export interface NameAvailability {
  /**
   * Error Message.
   */
  message?: string;
  /**
   * Indicates whether the resource name is available.
   */
  nameAvailable?: boolean;
  /**
   * Reason for name being unavailable.
   */
  reason?: string;
}

/**
 * A server security alert policy.
 */
export interface ServerSecurityAlertPolicy extends ProxyResource {
  /**
   * Specifies the state of the policy, whether it is enabled or disabled. Possible values include:
   * 'Enabled', 'Disabled'
   */
  state: string;
  /**
   * Specifies an array of alerts that are disabled. Allowed values are: Sql_Injection,
   * Sql_Injection_Vulnerability, Access_Anomaly
   */
  disabledAlerts?: string[];
  /**
   * Specifies an array of e-mail addresses to which the alert is sent.
   */
  emailAddresses?: string[];
  /**
   * Specifies that the alert is sent to the account administrators.
   */
  emailAccountAdmins?: boolean;
  /**
   * Specifies the blob storage endpoint (e.g. https://MyAccount.blob.core.windows.net). This blob
   * storage will hold all Threat Detection audit logs.
   */
  storageEndpoint?: string;
  /**
   * Specifies the identifier key of the Threat Detection audit storage account.
   */
  storageAccountAccessKey?: string;
  /**
   * Specifies the number of days to keep in the Threat Detection audit logs.
   */
  retentionDays?: number;
}

/**
 * A list of servers.
 */
export interface ServerListResult extends Array<Server> {
}

/**
 * A list of firewall rules.
 */
export interface FirewallRuleListResult extends Array<FirewallRule> {
}

/**
 * A list of virtual network rules.
 */
export interface VirtualNetworkRuleListResult extends Array<VirtualNetworkRule> {
  /**
   * Link to retrieve next page of results.
   */
  readonly nextLink?: string;
}

/**
 * A List of databases.
 */
export interface DatabaseListResult extends Array<Database> {
}

/**
 * A list of server configurations.
 */
export interface ConfigurationListResult extends Array<Configuration> {
}

/**
 * A list of log files.
 */
export interface LogFileListResult extends Array<LogFile> {
}

/**
 * A list of performance tiers.
 */
export interface PerformanceTierListResult extends Array<PerformanceTierProperties> {
}
