/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * The object that represents the operation.
 *
 * @member {string} [provider] Service provider: Microsoft.PowerBIDedicated.
 * @member {string} [resource] Resource on which the operation is performed:
 * capacity, etc.
 * @member {string} [operation] Operation type: create, update, delete, etc.
 */
export interface OperationDisplay {
  readonly provider?: string;
  readonly resource?: string;
  readonly operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Capacities REST API operation.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}.
 * @member {object} [display] The object that represents the operation.
 * @member {string} [display.provider] Service provider:
 * Microsoft.PowerBIDedicated.
 * @member {string} [display.resource] Resource on which the operation is
 * performed: capacity, etc.
 * @member {string} [display.operation] Operation type: create, update, delete,
 * etc.
 */
export interface Operation {
  readonly name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the ResourceSku class.
 * @constructor
 * Represents the SKU name and Azure pricing tier for PowerBI Dedicated
 * resource.
 *
 * @member {string} name Name of the SKU level.
 * @member {string} [tier] The name of the Azure pricing tier to which the SKU
 * applies. Possible values include: 'PBIE_Azure'
 */
export interface ResourceSku {
  name: string;
  tier?: string;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * Represents an instance of an PowerBI Dedicated resource.
 *
 * @member {string} [id] An identifier that represents the PowerBI Dedicated
 * resource.
 * @member {string} [name] The name of the PowerBI Dedicated resource.
 * @member {string} [type] The type of the PowerBI Dedicated resource.
 * @member {string} location Location of the PowerBI Dedicated resource.
 * @member {object} sku The SKU of the PowerBI Dedicated resource.
 * @member {string} [sku.name] Name of the SKU level.
 * @member {string} [sku.tier] The name of the Azure pricing tier to which the
 * SKU applies. Possible values include: 'PBIE_Azure'
 * @member {object} [tags] Key-value pairs of additional resource provisioning
 * properties.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location: string;
  sku: ResourceSku;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DedicatedCapacity class.
 * @constructor
 * Represents an instance of a Dedicated Capacity resource.
 *
 * @member {object} [administration] A collection of Dedicated capacity
 * administrators
 * @member {array} [administration.members] An array of administrator user
 * identities.
 * @member {string} [state] The current state of PowerBI Dedicated resource.
 * The state is to indicate more states outside of resource provisioning.
 * Possible values include: 'Deleting', 'Succeeded', 'Failed', 'Paused',
 * 'Suspended', 'Provisioning', 'Updating', 'Suspending', 'Pausing',
 * 'Resuming', 'Preparing', 'Scaling'
 * @member {string} [provisioningState] The current deployment state of PowerBI
 * Dedicatedresource. The provisioningState is to indicate states for resource
 * provisioning. Possible values include: 'Deleting', 'Succeeded', 'Failed',
 * 'Paused', 'Suspended', 'Provisioning', 'Updating', 'Suspending', 'Pausing',
 * 'Resuming', 'Preparing', 'Scaling'
 */
export interface DedicatedCapacity extends Resource {
  administration?: DedicatedCapacityAdministrators;
  readonly state?: string;
  readonly provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the DedicatedCapacityAdministrators class.
 * @constructor
 * An array of administrator user identities
 *
 * @member {array} [members] An array of administrator user identities.
 */
export interface DedicatedCapacityAdministrators {
  members?: string[];
}

/**
 * @class
 * Initializes a new instance of the DedicatedCapacityUpdateParameters class.
 * @constructor
 * Provision request specification
 *
 * @member {object} [sku] The SKU of the Dedicated capacity resource.
 * @member {string} [sku.name] Name of the SKU level.
 * @member {string} [sku.tier] The name of the Azure pricing tier to which the
 * SKU applies. Possible values include: 'PBIE_Azure'
 * @member {object} [tags] Key-value pairs of additional provisioning
 * properties.
 * @member {object} [administration] A collection of Dedicated capacity
 * administrators
 * @member {array} [administration.members] An array of administrator user
 * identities.
 */
export interface DedicatedCapacityUpdateParameters {
  sku?: ResourceSku;
  tags?: { [propertyName: string]: string };
  administration?: DedicatedCapacityAdministrators;
}

/**
 * @class
 * Initializes a new instance of the SkuEnumerationForNewResourceResult class.
 * @constructor
 * An object that represents enumerating SKUs for new resources
 *
 * @member {array} [value] The collection of available SKUs for new resources
 */
export interface SkuEnumerationForNewResourceResult {
  value?: ResourceSku[];
}

/**
 * @class
 * Initializes a new instance of the SkuDetailsForExistingResource class.
 * @constructor
 * An object that represents SKU details for existing resources
 *
 * @member {object} [sku] The SKU in SKU details for existing resources.
 * @member {string} [sku.name] Name of the SKU level.
 * @member {string} [sku.tier] The name of the Azure pricing tier to which the
 * SKU applies. Possible values include: 'PBIE_Azure'
 */
export interface SkuDetailsForExistingResource {
  sku?: ResourceSku;
}

/**
 * @class
 * Initializes a new instance of the SkuEnumerationForExistingResourceResult class.
 * @constructor
 * An object that represents enumerating SKUs for existing resources
 *
 * @member {array} [value] The collection of available SKUs for existing
 * resources
 */
export interface SkuEnumerationForExistingResourceResult {
  value?: SkuDetailsForExistingResource[];
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * Describes the format of Error response.
 *
 * @member {string} [code] Error code
 * @member {string} [message] Error message indicating why the operation
 * failed.
 */
export interface ErrorResponse {
  code?: string;
  message?: string;
}


/**
 * @class
 * Initializes a new instance of the DedicatedCapacities class.
 * @constructor
 * An array of Dedicated capacities resources.
 *
 */
export interface DedicatedCapacities extends Array<DedicatedCapacity> {
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result listing capacities. It contains a list of operations and a URL link
 * to get the next set of results.
 *
 * @member {string} [nextLink] URL to get the next set of operation list
 * results if there are any.
 */
export interface OperationListResult extends Array<Operation> {
  readonly nextLink?: string;
}
