/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Information from validate template deployment response.
 *
 */
class DeploymentValidateResult {
  /**
   * Create a DeploymentValidateResult.
   * @property {object} [error] Validation error.
   * @property {string} [error.code] The error code returned when exporting the
   * template.
   * @property {string} [error.message] The error message describing the export
   * error.
   * @property {string} [error.target] The target of the error.
   * @property {array} [error.details] Validation error.
   * @property {object} [properties] The template deployment properties.
   * @property {string} [properties.provisioningState] The state of the
   * provisioning.
   * @property {string} [properties.correlationId] The correlation ID of the
   * deployment.
   * @property {date} [properties.timestamp] The timestamp of the template
   * deployment.
   * @property {object} [properties.outputs] Key/value pairs that represent
   * deployment output.
   * @property {array} [properties.providers] The list of resource providers
   * needed for the deployment.
   * @property {array} [properties.dependencies] The list of deployment
   * dependencies.
   * @property {object} [properties.template] The template content. Use only
   * one of Template or TemplateLink.
   * @property {object} [properties.templateLink] The URI referencing the
   * template. Use only one of Template or TemplateLink.
   * @property {string} [properties.templateLink.uri] The URI of the template
   * to deploy.
   * @property {string} [properties.templateLink.contentVersion] If included,
   * must match the ContentVersion in the template.
   * @property {object} [properties.parameters] Deployment parameters. Use only
   * one of Parameters or ParametersLink.
   * @property {object} [properties.parametersLink] The URI referencing the
   * parameters. Use only one of Parameters or ParametersLink.
   * @property {string} [properties.parametersLink.uri] The URI of the
   * parameters file.
   * @property {string} [properties.parametersLink.contentVersion] If included,
   * must match the ContentVersion in the template.
   * @property {string} [properties.mode] The deployment mode. Possible values
   * are Incremental and Complete. Possible values include: 'Incremental',
   * 'Complete'
   * @property {object} [properties.debugSetting] The debug setting of the
   * deployment.
   * @property {string} [properties.debugSetting.detailLevel] Specifies the
   * type of information to log for debugging. The permitted values are none,
   * requestContent, responseContent, or both requestContent and
   * responseContent separated by a comma. The default is none. When setting
   * this value, carefully consider the type of information you are passing in
   * during deployment. By logging information about the request or response,
   * you could potentially expose sensitive data that is retrieved through the
   * deployment operations.
   * @property {object} [properties.onErrorDeployment] The deployment on error
   * behavior.
   * @property {string} [properties.onErrorDeployment.provisioningState] The
   * state of the provisioning for the on error deployment.
   * @property {string} [properties.onErrorDeployment.type] The deployment on
   * error behavior type. Possible values are LastSuccessful and
   * SpecificDeployment. Possible values include: 'LastSuccessful',
   * 'SpecificDeployment'
   * @property {string} [properties.onErrorDeployment.deploymentName] The
   * deployment to be used on error case.
   */
  constructor() {
  }

  /**
   * Defines the metadata of DeploymentValidateResult
   *
   * @returns {object} metadata of DeploymentValidateResult
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'DeploymentValidateResult',
      type: {
        name: 'Composite',
        className: 'DeploymentValidateResult',
        modelProperties: {
          error: {
            required: false,
            serializedName: 'error',
            type: {
              name: 'Composite',
              className: 'ResourceManagementErrorWithDetails'
            }
          },
          properties: {
            required: false,
            serializedName: 'properties',
            type: {
              name: 'Composite',
              className: 'DeploymentPropertiesExtended'
            }
          }
        }
      }
    };
  }
}

module.exports = DeploymentValidateResult;
