/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * This type describes a volume resource.
 *
 * @extends models['TrackedResource']
 */
class VolumeResourceDescription extends models['TrackedResource'] {
  /**
   * Create a VolumeResourceDescription.
   * @member {string} [provisioningState] State of the resource.
   * @member {string} [description] User readable description of the volume.
   * @member {string} [status] Status of the volume. Possible values include:
   * 'Unknown', 'Ready', 'Upgrading', 'Creating', 'Deleting', 'Failed'
   * @member {string} [statusDetails] Gives additional information about the
   * current status of the volume.
   * @member {object} [azureFileParameters] This type describes a volume
   * provided by an Azure Files file share.
   * @member {string} [azureFileParameters.accountName] Name of the Azure
   * storage account for the File Share.
   * @member {string} [azureFileParameters.accountKey] Access key of the Azure
   * storage account for the File Share.
   * @member {string} [azureFileParameters.shareName] Name of the Azure Files
   * file share that provides storage for the volume.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of VolumeResourceDescription
   *
   * @returns {object} metadata of VolumeResourceDescription
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'VolumeResourceDescription',
      type: {
        name: 'Composite',
        className: 'VolumeResourceDescription',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'properties.description',
            type: {
              name: 'String'
            }
          },
          status: {
            required: false,
            readOnly: true,
            serializedName: 'properties.status',
            type: {
              name: 'String'
            }
          },
          statusDetails: {
            required: false,
            readOnly: true,
            serializedName: 'properties.statusDetails',
            type: {
              name: 'String'
            }
          },
          provider: {
            required: true,
            isConstant: true,
            serializedName: 'properties.provider',
            defaultValue: 'SFAzureFile',
            type: {
              name: 'String'
            }
          },
          azureFileParameters: {
            required: false,
            serializedName: 'properties.azureFileParameters',
            type: {
              name: 'Composite',
              className: 'VolumeProviderParametersAzureFile'
            }
          }
        }
      }
    };
  }
}

module.exports = VolumeResourceDescription;
