/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;

/**
 * @summary Gets the logs from the container.
 *
 * Gets the logs for the container of the specified code package of the service
 * replica.
 *
 * @param {string} resourceGroupName Azure resource group name
 *
 * @param {string} applicationResourceName The identity of the application.
 *
 * @param {string} serviceResourceName The identity of the service.
 *
 * @param {string} replicaName Service Fabric replica name.
 *
 * @param {string} codePackageName The name of code package of the service.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {number} [options.tail] Number of lines to show from the end of the
 * logs. Default is 100.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link ContainerLogs} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _getContainerLogs(resourceGroupName, applicationResourceName, serviceResourceName, replicaName, codePackageName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let tail = (options && options.tail !== undefined) ? options.tail : undefined;
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (applicationResourceName === null || applicationResourceName === undefined || typeof applicationResourceName.valueOf() !== 'string') {
      throw new Error('applicationResourceName cannot be null or undefined and it must be of type string.');
    }
    if (serviceResourceName === null || serviceResourceName === undefined || typeof serviceResourceName.valueOf() !== 'string') {
      throw new Error('serviceResourceName cannot be null or undefined and it must be of type string.');
    }
    if (replicaName === null || replicaName === undefined || typeof replicaName.valueOf() !== 'string') {
      throw new Error('replicaName cannot be null or undefined and it must be of type string.');
    }
    if (codePackageName === null || codePackageName === undefined || typeof codePackageName.valueOf() !== 'string') {
      throw new Error('codePackageName cannot be null or undefined and it must be of type string.');
    }
    if (tail !== null && tail !== undefined && typeof tail !== 'number') {
      throw new Error('tail must be of type number.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ServiceFabricMesh/applications/{applicationResourceName}/services/{serviceResourceName}/replicas/{replicaName}/codePackages/{codePackageName}/logs';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{applicationResourceName}', applicationResourceName);
  requestUrl = requestUrl.replace('{serviceResourceName}', serviceResourceName);
  requestUrl = requestUrl.replace('{replicaName}', replicaName);
  requestUrl = requestUrl.replace('{codePackageName}', encodeURIComponent(codePackageName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (tail !== null && tail !== undefined) {
    queryParameters.push('tail=' + encodeURIComponent(tail.toString()));
  }
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          let internalError = null;
          if (parsedErrorResponse.error) internalError = parsedErrorResponse.error;
          error.code = internalError ? internalError.code : parsedErrorResponse.code;
          error.message = internalError ? internalError.message : parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['ErrorModel']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['ContainerLogs']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a CodePackage. */
class CodePackage {
  /**
   * Create a CodePackage.
   * @param {ServiceFabricMeshManagementClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._getContainerLogs = _getContainerLogs;
  }

  /**
   * @summary Gets the logs from the container.
   *
   * Gets the logs for the container of the specified code package of the service
   * replica.
   *
   * @param {string} resourceGroupName Azure resource group name
   *
   * @param {string} applicationResourceName The identity of the application.
   *
   * @param {string} serviceResourceName The identity of the service.
   *
   * @param {string} replicaName Service Fabric replica name.
   *
   * @param {string} codePackageName The name of code package of the service.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {number} [options.tail] Number of lines to show from the end of the
   * logs. Default is 100.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ContainerLogs>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getContainerLogsWithHttpOperationResponse(resourceGroupName, applicationResourceName, serviceResourceName, replicaName, codePackageName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._getContainerLogs(resourceGroupName, applicationResourceName, serviceResourceName, replicaName, codePackageName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * @summary Gets the logs from the container.
   *
   * Gets the logs for the container of the specified code package of the service
   * replica.
   *
   * @param {string} resourceGroupName Azure resource group name
   *
   * @param {string} applicationResourceName The identity of the application.
   *
   * @param {string} serviceResourceName The identity of the service.
   *
   * @param {string} replicaName Service Fabric replica name.
   *
   * @param {string} codePackageName The name of code package of the service.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {number} [options.tail] Number of lines to show from the end of the
   * logs. Default is 100.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {ContainerLogs} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ContainerLogs} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  getContainerLogs(resourceGroupName, applicationResourceName, serviceResourceName, replicaName, codePackageName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._getContainerLogs(resourceGroupName, applicationResourceName, serviceResourceName, replicaName, codePackageName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._getContainerLogs(resourceGroupName, applicationResourceName, serviceResourceName, replicaName, codePackageName, options, optionalCallback);
    }
  }

}

module.exports = CodePackage;
