/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Parameters supplied to the Update Environment operation.
 *
 */
class EnvironmentUpdateParameters {
  /**
   * Create a EnvironmentUpdateParameters.
   * @member {object} [sku] The sku of the environment.
   * @member {string} [sku.name] The name of this SKU. Possible values include:
   * 'S1', 'S2'
   * @member {number} [sku.capacity] The capacity of the sku. This value can be
   * changed to support scale out of environments after they have been created.
   * @member {object} [tags] Key-value pairs of additional properties for the
   * environment.
   * @member {moment.duration} [dataRetentionTime] ISO8601 timespan specifying
   * the minimum number of days the environment's events will be available for
   * query.
   * @member {string} [storageLimitExceededBehavior] The behavior the Time
   * Series Insights service should take when the environment's capacity has
   * been exceeded. If "PauseIngress" is specified, new events will not be read
   * from the event source. If "PurgeOldData" is specified, new events will
   * continue to be read and old events will be deleted from the environment.
   * The default behavior is PurgeOldData. Possible values include:
   * 'PurgeOldData', 'PauseIngress'
   * @member {array} [partitionKeyProperties] The list of event properties
   * which will be used to partition data in the environment.
   */
  constructor() {
  }

  /**
   * Defines the metadata of EnvironmentUpdateParameters
   *
   * @returns {object} metadata of EnvironmentUpdateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'EnvironmentUpdateParameters',
      type: {
        name: 'Composite',
        className: 'EnvironmentUpdateParameters',
        modelProperties: {
          sku: {
            required: false,
            serializedName: 'sku',
            type: {
              name: 'Composite',
              className: 'Sku'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          dataRetentionTime: {
            required: false,
            serializedName: 'properties.dataRetentionTime',
            type: {
              name: 'TimeSpan'
            }
          },
          storageLimitExceededBehavior: {
            required: false,
            serializedName: 'properties.storageLimitExceededBehavior',
            type: {
              name: 'Enum',
              allowedValues: [ 'PurgeOldData', 'PauseIngress' ]
            }
          },
          partitionKeyProperties: {
            required: false,
            serializedName: 'properties.partitionKeyProperties',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'PartitionKeyPropertyElementType',
                  type: {
                    name: 'Composite',
                    className: 'PartitionKeyProperty'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = EnvironmentUpdateParameters;
