/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * Azure Data Factory top-level resource.
 *
 * @member {string} [id] The resource identifier.
 * @member {string} [name] The resource name.
 * @member {string} [type] The resource type.
 * @member {string} [location] The resource location.
 * @member {object} [tags] The resource tags.
 * @member {string} [eTag] Etag identifies change in the resource.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
  readonly eTag?: string;
}

/**
 * @class
 * Initializes a new instance of the SubResource class.
 * @constructor
 * Azure Data Factory nested resource, which belongs to a factory.
 *
 * @member {string} [id] The resource identifier.
 * @member {string} [name] The resource name.
 * @member {string} [type] The resource type.
 * @member {string} [etag] Etag identifies change in the resource.
 */
export interface SubResource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  readonly etag?: string;
}

/**
 * @class
 * Initializes a new instance of the Expression class.
 * @constructor
 * Azure Data Factory expression definition.
 *
 * @member {string} value Expression value.
 */
export interface Expression {
  value: string;
}

/**
 * @class
 * Initializes a new instance of the SecretBase class.
 * @constructor
 * The base definition of a secret type.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface SecretBase {
  type: string;
}

/**
 * @class
 * Initializes a new instance of the SecureString class.
 * @constructor
 * Azure Data Factory secure string definition. The string value will be masked
 * with asterisks '*' during Get or List API calls.
 *
 * @member {string} value Value of secure string.
 */
export interface SecureString extends SecretBase {
  value: string;
}

/**
 * @class
 * Initializes a new instance of the LinkedServiceReference class.
 * @constructor
 * Linked service reference type.
 *
 * @member {string} referenceName Reference LinkedService name.
 * @member {object} [parameters] Arguments for LinkedService.
 */
export interface LinkedServiceReference {
  referenceName: string;
  parameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the AzureKeyVaultSecretReference class.
 * @constructor
 * Azure Key Vault secret reference.
 *
 * @member {object} store The Azure Key Vault linked service reference.
 * @member {string} [store.referenceName] Reference LinkedService name.
 * @member {object} [store.parameters] Arguments for LinkedService.
 * @member {object} secretName The name of the secret in Azure Key Vault. Type:
 * string (or Expression with resultType string).
 * @member {object} [secretVersion] The version of the secret in Azure Key
 * Vault. The default value is the latest version of the secret. Type: string
 * (or Expression with resultType string).
 */
export interface AzureKeyVaultSecretReference extends SecretBase {
  store: LinkedServiceReference;
  secretName: any;
  secretVersion?: any;
}

/**
 * @class
 * Initializes a new instance of the FactoryIdentity class.
 * @constructor
 * Identity properties of the factory resource.
 *
 * @member {uuid} [principalId] The principal id of the identity.
 * @member {uuid} [tenantId] The client tenant id of the identity.
 */
export interface FactoryIdentity {
  readonly principalId?: string;
  readonly tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the FactoryRepoConfiguration class.
 * @constructor
 * Factory's git repo information.
 *
 * @member {string} accountName Account name.
 * @member {string} repositoryName Rrepository name.
 * @member {string} collaborationBranch Collaboration branch.
 * @member {string} rootFolder Root folder.
 * @member {string} [lastCommitId] Last commit id.
 * @member {string} type Polymorphic Discriminator
 */
export interface FactoryRepoConfiguration {
  accountName: string;
  repositoryName: string;
  collaborationBranch: string;
  rootFolder: string;
  lastCommitId?: string;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the Factory class.
 * @constructor
 * Factory resource type.
 *
 * @member {object} [identity] Managed service identity of the factory.
 * @member {uuid} [identity.principalId] The principal id of the identity.
 * @member {uuid} [identity.tenantId] The client tenant id of the identity.
 * @member {string} [provisioningState] Factory provisioning state, example
 * Succeeded.
 * @member {date} [createTime] Time the factory was created in ISO8601 format.
 * @member {string} [version] Version of the factory.
 * @member {object} [repoConfiguration] Git repo information of the factory.
 * @member {string} [repoConfiguration.accountName] Account name.
 * @member {string} [repoConfiguration.repositoryName] Rrepository name.
 * @member {string} [repoConfiguration.collaborationBranch] Collaboration
 * branch.
 * @member {string} [repoConfiguration.rootFolder] Root folder.
 * @member {string} [repoConfiguration.lastCommitId] Last commit id.
 * @member {string} [repoConfiguration.type] Polymorphic Discriminator
 */
export interface Factory extends Resource {
  identity?: FactoryIdentity;
  readonly provisioningState?: string;
  readonly createTime?: Date;
  readonly version?: string;
  repoConfiguration?: FactoryRepoConfiguration;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntime class.
 * @constructor
 * Azure Data Factory nested object which serves as a compute resource for
 * activities.
 *
 * @member {string} [description] Integration runtime description.
 * @member {string} type Polymorphic Discriminator
 */
export interface IntegrationRuntime {
  description?: string;
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeResource class.
 * @constructor
 * Integration runtime resource type.
 *
 * @member {object} properties Integration runtime properties.
 * @member {string} [properties.description] Integration runtime description.
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface IntegrationRuntimeResource extends SubResource {
  properties: IntegrationRuntime;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeReference class.
 * @constructor
 * Integration runtime reference type.
 *
 * @member {string} referenceName Reference integration runtime name.
 * @member {object} [parameters] Arguments for integration runtime.
 */
export interface IntegrationRuntimeReference {
  referenceName: string;
  parameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeStatus class.
 * @constructor
 * Integration runtime status.
 *
 * @member {string} [dataFactoryName] The data factory name which the
 * integration runtime belong to.
 * @member {string} [state] The state of integration runtime. Possible values
 * include: 'Initial', 'Stopped', 'Started', 'Starting', 'Stopping',
 * 'NeedRegistration', 'Online', 'Limited', 'Offline', 'AccessDenied'
 * @member {string} type Polymorphic Discriminator
 */
export interface IntegrationRuntimeStatus {
  readonly dataFactoryName?: string;
  readonly state?: string;
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeStatusResponse class.
 * @constructor
 * Integration runtime status response.
 *
 * @member {string} [name] The integration runtime name.
 * @member {object} properties Integration runtime properties.
 * @member {string} [properties.dataFactoryName] The data factory name which
 * the integration runtime belong to.
 * @member {string} [properties.state] The state of integration runtime.
 * Possible values include: 'Initial', 'Stopped', 'Started', 'Starting',
 * 'Stopping', 'NeedRegistration', 'Online', 'Limited', 'Offline',
 * 'AccessDenied'
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface IntegrationRuntimeStatusResponse {
  readonly name?: string;
  properties: IntegrationRuntimeStatus;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeStatusListResponse class.
 * @constructor
 * A list of integration runtime status.
 *
 * @member {array} value List of integration runtime status.
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface IntegrationRuntimeStatusListResponse {
  value: IntegrationRuntimeStatusResponse[];
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the UpdateIntegrationRuntimeRequest class.
 * @constructor
 * Update integration runtime request.
 *
 * @member {string} [autoUpdate] Enables or disables the auto-update feature of
 * the self-hosted integration runtime. See
 * https://go.microsoft.com/fwlink/?linkid=854189. Possible values include:
 * 'On', 'Off'
 * @member {string} [updateDelayOffset] The time offset (in hours) in the day,
 * e.g., PT03H is 3 hours. The integration runtime auto update will happen on
 * that time.
 */
export interface UpdateIntegrationRuntimeRequest {
  autoUpdate?: string;
  updateDelayOffset?: string;
}

/**
 * @class
 * Initializes a new instance of the UpdateIntegrationRuntimeNodeRequest class.
 * @constructor
 * Update integration runtime node request.
 *
 * @member {number} [concurrentJobsLimit] The number of concurrent jobs
 * permitted to run on the integration runtime node. Values between 1 and
 * maxConcurrentJobs(inclusive) are allowed.
 */
export interface UpdateIntegrationRuntimeNodeRequest {
  concurrentJobsLimit?: number;
}

/**
 * @class
 * Initializes a new instance of the LinkedIntegrationRuntimeRequest class.
 * @constructor
 * Data factory name for linked integration runtime request.
 *
 * @member {string} linkedFactoryName The data factory name for linked
 * integration runtime.
 */
export interface LinkedIntegrationRuntimeRequest {
  linkedFactoryName: string;
}

/**
 * @class
 * Initializes a new instance of the CreateLinkedIntegrationRuntimeRequest class.
 * @constructor
 * The linked integration runtime information.
 *
 * @member {string} [name] The name of the linked integration runtime.
 * @member {string} [subscriptionId] The ID of the subscription that the linked
 * integration runtime belongs to.
 * @member {string} [dataFactoryName] The name of the data factory that the
 * linked integration runtime belongs to.
 * @member {string} [dataFactoryLocation] The location of the data factory that
 * the linked integration runtime belongs to.
 */
export interface CreateLinkedIntegrationRuntimeRequest {
  name?: string;
  subscriptionId?: string;
  dataFactoryName?: string;
  dataFactoryLocation?: string;
}

/**
 * @class
 * Initializes a new instance of the ParameterSpecification class.
 * @constructor
 * Definition of a single parameter for an entity.
 *
 * @member {string} type Parameter type. Possible values include: 'Object',
 * 'String', 'Int', 'Float', 'Bool', 'Array', 'SecureString'
 * @member {object} [defaultValue] Default value of parameter.
 */
export interface ParameterSpecification {
  type: string;
  defaultValue?: any;
}

/**
 * @class
 * Initializes a new instance of the LinkedService class.
 * @constructor
 * The Azure Data Factory nested object which contains the information and
 * credential which can be used to connect with related store or compute
 * resource.
 *
 * @member {object} [connectVia] The integration runtime reference.
 * @member {string} [connectVia.referenceName] Reference integration runtime
 * name.
 * @member {object} [connectVia.parameters] Arguments for integration runtime.
 * @member {string} [description] Linked service description.
 * @member {object} [parameters] Parameters for linked service.
 * @member {array} [annotations] List of tags that can be used for describing
 * the Dataset.
 * @member {string} type Polymorphic Discriminator
 */
export interface LinkedService {
  connectVia?: IntegrationRuntimeReference;
  description?: string;
  parameters?: { [propertyName: string]: ParameterSpecification };
  annotations?: any[];
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the LinkedServiceResource class.
 * @constructor
 * Linked service resource type.
 *
 * @member {object} properties Properties of linked service.
 * @member {object} [properties.connectVia] The integration runtime reference.
 * @member {string} [properties.connectVia.referenceName] Reference integration
 * runtime name.
 * @member {object} [properties.connectVia.parameters] Arguments for
 * integration runtime.
 * @member {string} [properties.description] Linked service description.
 * @member {object} [properties.parameters] Parameters for linked service.
 * @member {array} [properties.annotations] List of tags that can be used for
 * describing the Dataset.
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface LinkedServiceResource extends SubResource {
  properties: LinkedService;
}

/**
 * @class
 * Initializes a new instance of the DatasetFolder class.
 * @constructor
 * The folder that this Dataset is in. If not specified, Dataset will appear at
 * the root level.
 *
 * @member {string} [name] The name of the folder that this Dataset is in.
 */
export interface DatasetFolder {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the Dataset class.
 * @constructor
 * The Azure Data Factory nested object which identifies data within different
 * data stores, such as tables, files, folders, and documents.
 *
 * @member {string} [description] Dataset description.
 * @member {object} [structure] Columns that define the structure of the
 * dataset. Type: array (or Expression with resultType array), itemType:
 * DatasetDataElement.
 * @member {object} linkedServiceName Linked service reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [linkedServiceName.parameters] Arguments for LinkedService.
 * @member {object} [parameters] Parameters for dataset.
 * @member {array} [annotations] List of tags that can be used for describing
 * the Dataset.
 * @member {object} [folder] The folder that this Dataset is in. If not
 * specified, Dataset will appear at the root level.
 * @member {string} [folder.name] The name of the folder that this Dataset is
 * in.
 * @member {string} type Polymorphic Discriminator
 */
export interface Dataset {
  description?: string;
  structure?: any;
  linkedServiceName: LinkedServiceReference;
  parameters?: { [propertyName: string]: ParameterSpecification };
  annotations?: any[];
  folder?: DatasetFolder;
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the DatasetResource class.
 * @constructor
 * Dataset resource type.
 *
 * @member {object} properties Dataset properties.
 * @member {string} [properties.description] Dataset description.
 * @member {object} [properties.structure] Columns that define the structure of
 * the dataset. Type: array (or Expression with resultType array), itemType:
 * DatasetDataElement.
 * @member {object} [properties.linkedServiceName] Linked service reference.
 * @member {string} [properties.linkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [properties.linkedServiceName.parameters] Arguments for
 * LinkedService.
 * @member {object} [properties.parameters] Parameters for dataset.
 * @member {array} [properties.annotations] List of tags that can be used for
 * describing the Dataset.
 * @member {object} [properties.folder] The folder that this Dataset is in. If
 * not specified, Dataset will appear at the root level.
 * @member {string} [properties.folder.name] The name of the folder that this
 * Dataset is in.
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface DatasetResource extends SubResource {
  properties: Dataset;
}

/**
 * @class
 * Initializes a new instance of the ActivityDependency class.
 * @constructor
 * Activity dependency information.
 *
 * @member {string} activity Activity name.
 * @member {array} dependencyConditions Match-Condition for the dependency.
 */
export interface ActivityDependency {
  activity: string;
  dependencyConditions: string[];
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the UserProperty class.
 * @constructor
 * User property.
 *
 * @member {string} name User proprety name.
 * @member {object} value User proprety value. Type: string (or Expression with
 * resultType string).
 */
export interface UserProperty {
  name: string;
  value: any;
}

/**
 * @class
 * Initializes a new instance of the Activity class.
 * @constructor
 * A pipeline activity.
 *
 * @member {string} name Activity name.
 * @member {string} [description] Activity description.
 * @member {array} [dependsOn] Activity depends on condition.
 * @member {array} [userProperties] Activity user properties.
 * @member {string} type Polymorphic Discriminator
 */
export interface Activity {
  name: string;
  description?: string;
  dependsOn?: ActivityDependency[];
  userProperties?: UserProperty[];
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the VariableSpecification class.
 * @constructor
 * Definition of a single variable for a Pipeline.
 *
 * @member {string} type Variable type. Possible values include: 'String',
 * 'Bool', 'Array'
 * @member {object} [defaultValue] Default value of variable.
 */
export interface VariableSpecification {
  type: string;
  defaultValue?: any;
}

/**
 * @class
 * Initializes a new instance of the PipelineFolder class.
 * @constructor
 * The folder that this Pipeline is in. If not specified, Pipeline will appear
 * at the root level.
 *
 * @member {string} [name] The name of the folder that this Pipeline is in.
 */
export interface PipelineFolder {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineResource class.
 * @constructor
 * Pipeline resource type.
 *
 * @member {string} [description] The description of the pipeline.
 * @member {array} [activities] List of activities in pipeline.
 * @member {object} [parameters] List of parameters for pipeline.
 * @member {object} [variables] List of variables for pipeline.
 * @member {number} [concurrency] The max number of concurrent runs for the
 * pipeline.
 * @member {array} [annotations] List of tags that can be used for describing
 * the Pipeline.
 * @member {object} [folder] The folder that this Pipeline is in. If not
 * specified, Pipeline will appear at the root level.
 * @member {string} [folder.name] The name of the folder that this Pipeline is
 * in.
 */
export interface PipelineResource extends SubResource {
  description?: string;
  activities?: Activity[];
  parameters?: { [propertyName: string]: ParameterSpecification };
  variables?: { [propertyName: string]: VariableSpecification };
  concurrency?: number;
  annotations?: any[];
  folder?: PipelineFolder;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the Trigger class.
 * @constructor
 * Azure data factory nested object which contains information about creating
 * pipeline run
 *
 * @member {string} [description] Trigger description.
 * @member {string} [runtimeState] Indicates if trigger is running or not.
 * Updated when Start/Stop APIs are called on the Trigger. Possible values
 * include: 'Started', 'Stopped', 'Disabled'
 * @member {string} type Polymorphic Discriminator
 */
export interface Trigger {
  description?: string;
  readonly runtimeState?: string;
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the TriggerResource class.
 * @constructor
 * Trigger resource type.
 *
 * @member {object} properties Properties of the trigger.
 * @member {string} [properties.description] Trigger description.
 * @member {string} [properties.runtimeState] Indicates if trigger is running
 * or not. Updated when Start/Stop APIs are called on the Trigger. Possible
 * values include: 'Started', 'Stopped', 'Disabled'
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface TriggerResource extends SubResource {
  properties: Trigger;
}

/**
 * @class
 * Initializes a new instance of the CreateRunResponse class.
 * @constructor
 * Response body with a run identifier.
 *
 * @member {string} runId Identifier of a run.
 */
export interface CreateRunResponse {
  runId: string;
}

/**
 * @class
 * Initializes a new instance of the FactoryVSTSConfiguration class.
 * @constructor
 * Factory's VSTS repo information.
 *
 * @member {string} projectName VSTS project name.
 * @member {string} [tenantId] VSTS tenant id.
 */
export interface FactoryVSTSConfiguration extends FactoryRepoConfiguration {
  projectName: string;
  tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the FactoryGitHubConfiguration class.
 * @constructor
 * Factory's GitHub repo information.
 *
 * @member {string} [hostName] GitHub Enterprise host name. For example:
 * https://github.mydomain.com
 */
export interface FactoryGitHubConfiguration extends FactoryRepoConfiguration {
  hostName?: string;
}

/**
 * @class
 * Initializes a new instance of the FactoryRepoUpdate class.
 * @constructor
 * Factory's git repo information.
 *
 * @member {string} [factoryResourceId] The factory resource id.
 * @member {object} [repoConfiguration] Git repo information of the factory.
 * @member {string} [repoConfiguration.accountName] Account name.
 * @member {string} [repoConfiguration.repositoryName] Rrepository name.
 * @member {string} [repoConfiguration.collaborationBranch] Collaboration
 * branch.
 * @member {string} [repoConfiguration.rootFolder] Root folder.
 * @member {string} [repoConfiguration.lastCommitId] Last commit id.
 * @member {string} [repoConfiguration.type] Polymorphic Discriminator
 */
export interface FactoryRepoUpdate {
  factoryResourceId?: string;
  repoConfiguration?: FactoryRepoConfiguration;
}

/**
 * @class
 * Initializes a new instance of the GitHubAccessTokenRequest class.
 * @constructor
 * Get GitHub access token request definition.
 *
 * @member {string} gitHubAccessCode GitHub access code.
 * @member {string} [gitHubClientId] GitHub application client ID.
 * @member {string} gitHubAccessTokenBaseUrl GitHub access token base URL.
 */
export interface GitHubAccessTokenRequest {
  gitHubAccessCode: string;
  gitHubClientId?: string;
  gitHubAccessTokenBaseUrl: string;
}

/**
 * @class
 * Initializes a new instance of the GitHubAccessTokenResponse class.
 * @constructor
 * Get GitHub access token response definition.
 *
 * @member {string} [gitHubAccessToken] GitHub access token.
 */
export interface GitHubAccessTokenResponse {
  gitHubAccessToken?: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineReference class.
 * @constructor
 * Pipeline reference type.
 *
 * @member {string} referenceName Reference pipeline name.
 * @member {string} [name] Reference name.
 */
export interface PipelineReference {
  referenceName: string;
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerPipelineReference class.
 * @constructor
 * Pipeline that needs to be triggered with the given parameters.
 *
 * @member {object} [pipelineReference] Pipeline reference.
 * @member {string} [pipelineReference.referenceName] Reference pipeline name.
 * @member {string} [pipelineReference.name] Reference name.
 * @member {object} [parameters] Pipeline parameters.
 */
export interface TriggerPipelineReference {
  pipelineReference?: PipelineReference;
  parameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the FactoryUpdateParameters class.
 * @constructor
 * Parameters for updating a factory resource.
 *
 * @member {object} [tags] The resource tags.
 * @member {object} [identity] Managed service identity of the factory.
 * @member {uuid} [identity.principalId] The principal id of the identity.
 * @member {uuid} [identity.tenantId] The client tenant id of the identity.
 */
export interface FactoryUpdateParameters {
  tags?: { [propertyName: string]: string };
  identity?: FactoryIdentity;
}

/**
 * @class
 * Initializes a new instance of the DatasetReference class.
 * @constructor
 * Dataset reference type.
 *
 * @member {string} referenceName Reference dataset name.
 * @member {object} [parameters] Arguments for dataset.
 */
export interface DatasetReference {
  referenceName: string;
  parameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the RunQueryFilter class.
 * @constructor
 * Query filter option for listing runs.
 *
 * @member {string} operand Parameter name to be used for filter. The allowed
 * operands to query pipeline runs are PipelineName, RunStart, RunEnd and
 * Status; to query activity runs are ActivityName, ActivityRunStart,
 * ActivityRunEnd, ActivityType and Status, and to query trigger runs are
 * TriggerName, TriggerRunTimestamp and Status. Possible values include:
 * 'PipelineName', 'Status', 'RunStart', 'RunEnd', 'ActivityName',
 * 'ActivityRunStart', 'ActivityRunEnd', 'ActivityType', 'TriggerName',
 * 'TriggerRunTimestamp'
 * @member {string} operator Operator to be used for filter. Possible values
 * include: 'Equals', 'NotEquals', 'In', 'NotIn'
 * @member {array} values List of filter values.
 */
export interface RunQueryFilter {
  operand: string;
  operator: string;
  values: string[];
}

/**
 * @class
 * Initializes a new instance of the RunQueryOrderBy class.
 * @constructor
 * An object to provide order by options for listing runs.
 *
 * @member {string} orderBy Parameter name to be used for order by. The allowed
 * parameters to order by for pipeline runs are PipelineName, RunStart, RunEnd
 * and Status; for activity runs are ActivityName, ActivityRunStart,
 * ActivityRunEnd and Status; for trigger runs are TriggerName,
 * TriggerRunTimestamp and Status. Possible values include: 'RunStart',
 * 'RunEnd', 'PipelineName', 'Status', 'ActivityName', 'ActivityRunStart',
 * 'ActivityRunEnd', 'TriggerName', 'TriggerRunTimestamp'
 * @member {string} order Sorting order of the parameter. Possible values
 * include: 'ASC', 'DESC'
 */
export interface RunQueryOrderBy {
  orderBy: string;
  order: string;
}

/**
 * @class
 * Initializes a new instance of the RunFilterParameters class.
 * @constructor
 * Query parameters for listing runs.
 *
 * @member {string} [continuationToken] The continuation token for getting the
 * next page of results. Null for first page.
 * @member {date} lastUpdatedAfter The time at or after which the run event was
 * updated in 'ISO 8601' format.
 * @member {date} lastUpdatedBefore The time at or before which the run event
 * was updated in 'ISO 8601' format.
 * @member {array} [filters] List of filters.
 * @member {array} [orderBy] List of OrderBy option.
 */
export interface RunFilterParameters {
  continuationToken?: string;
  lastUpdatedAfter: Date;
  lastUpdatedBefore: Date;
  filters?: RunQueryFilter[];
  orderBy?: RunQueryOrderBy[];
}

/**
 * @class
 * Initializes a new instance of the PipelineRunInvokedBy class.
 * @constructor
 * Provides entity name and id that started the pipeline run.
 *
 * @member {string} [name] Name of the entity that started the pipeline run.
 * @member {string} [id] The ID of the entity that started the run.
 * @member {string} [invokedByType] The type of the entity that started the
 * run.
 */
export interface PipelineRunInvokedBy {
  readonly name?: string;
  readonly id?: string;
  readonly invokedByType?: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineRun class.
 * @constructor
 * Information about a pipeline run.
 *
 * @member {string} [runId] Identifier of a run.
 * @member {string} [pipelineName] The pipeline name.
 * @member {object} [parameters] The full or partial list of parameter name,
 * value pair used in the pipeline run.
 * @member {object} [invokedBy] Entity that started the pipeline run.
 * @member {string} [invokedBy.name] Name of the entity that started the
 * pipeline run.
 * @member {string} [invokedBy.id] The ID of the entity that started the run.
 * @member {string} [invokedBy.invokedByType] The type of the entity that
 * started the run.
 * @member {date} [lastUpdated] The last updated timestamp for the pipeline run
 * event in ISO8601 format.
 * @member {date} [runStart] The start time of a pipeline run in ISO8601
 * format.
 * @member {date} [runEnd] The end time of a pipeline run in ISO8601 format.
 * @member {number} [durationInMs] The duration of a pipeline run.
 * @member {string} [status] The status of a pipeline run.
 * @member {string} [message] The message from a pipeline run.
 */
export interface PipelineRun {
  readonly runId?: string;
  readonly pipelineName?: string;
  readonly parameters?: { [propertyName: string]: string };
  readonly invokedBy?: PipelineRunInvokedBy;
  readonly lastUpdated?: Date;
  readonly runStart?: Date;
  readonly runEnd?: Date;
  readonly durationInMs?: number;
  readonly status?: string;
  readonly message?: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the PipelineRunsQueryResponse class.
 * @constructor
 * A list pipeline runs.
 *
 * @member {array} value List of pipeline runs.
 * @member {string} [continuationToken] The continuation token for getting the
 * next page of results, if any remaining results exist, null otherwise.
 */
export interface PipelineRunsQueryResponse {
  value: PipelineRun[];
  continuationToken?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityRun class.
 * @constructor
 * Information about an activity run in a pipeline.
 *
 * @member {string} [pipelineName] The name of the pipeline.
 * @member {string} [pipelineRunId] The id of the pipeline run.
 * @member {string} [activityName] The name of the activity.
 * @member {string} [activityType] The type of the activity.
 * @member {string} [activityRunId] The id of the activity run.
 * @member {string} [linkedServiceName] The name of the compute linked service.
 * @member {string} [status] The status of the activity run.
 * @member {date} [activityRunStart] The start time of the activity run in 'ISO
 * 8601' format.
 * @member {date} [activityRunEnd] The end time of the activity run in 'ISO
 * 8601' format.
 * @member {number} [durationInMs] The duration of the activity run.
 * @member {object} [input] The input for the activity.
 * @member {object} [output] The output for the activity.
 * @member {object} [error] The error if any from the activity run.
 */
export interface ActivityRun {
  readonly pipelineName?: string;
  readonly pipelineRunId?: string;
  readonly activityName?: string;
  readonly activityType?: string;
  readonly activityRunId?: string;
  readonly linkedServiceName?: string;
  readonly status?: string;
  readonly activityRunStart?: Date;
  readonly activityRunEnd?: Date;
  readonly durationInMs?: number;
  readonly input?: any;
  readonly output?: any;
  readonly error?: any;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ActivityRunsQueryResponse class.
 * @constructor
 * A list activity runs.
 *
 * @member {array} value List of activity runs.
 * @member {string} [continuationToken] The continuation token for getting the
 * next page of results, if any remaining results exist, null otherwise.
 */
export interface ActivityRunsQueryResponse {
  value: ActivityRun[];
  continuationToken?: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerRun class.
 * @constructor
 * Trigger runs.
 *
 * @member {string} [triggerRunId] Trigger run id.
 * @member {string} [triggerName] Trigger name.
 * @member {string} [triggerType] Trigger type.
 * @member {date} [triggerRunTimestamp] Trigger run start time.
 * @member {string} [status] Trigger run status. Possible values include:
 * 'Succeeded', 'Failed', 'Inprogress'
 * @member {string} [message] Trigger error message.
 * @member {object} [properties] List of property name and value related to
 * trigger run. Name, value pair depends on type of trigger.
 * @member {object} [triggeredPipelines] List of pipeline name and run Id
 * triggered by the trigger run.
 */
export interface TriggerRun {
  readonly triggerRunId?: string;
  readonly triggerName?: string;
  readonly triggerType?: string;
  readonly triggerRunTimestamp?: Date;
  readonly status?: string;
  readonly message?: string;
  readonly properties?: { [propertyName: string]: string };
  readonly triggeredPipelines?: { [propertyName: string]: string };
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the TriggerRunsQueryResponse class.
 * @constructor
 * A list of trigger runs.
 *
 * @member {array} value List of trigger runs.
 * @member {string} [continuationToken] The continuation token for getting the
 * next page of results, if any remaining results exist, null otherwise.
 */
export interface TriggerRunsQueryResponse {
  value: TriggerRun[];
  continuationToken?: string;
}

/**
 * @class
 * Initializes a new instance of the RerunTumblingWindowTriggerActionParameters class.
 * @constructor
 * Rerun tumbling window trigger Parameters.
 *
 * @member {date} startTime The start time for the time period for which
 * restatement is initiated. Only UTC time is currently supported.
 * @member {date} endTime The end time for the time period for which
 * restatement is initiated. Only UTC time is currently supported.
 * @member {number} maxConcurrency The max number of parallel time windows
 * (ready for execution) for which a rerun is triggered.
 */
export interface RerunTumblingWindowTriggerActionParameters {
  startTime: Date;
  endTime: Date;
  maxConcurrency: number;
}

/**
 * @class
 * Initializes a new instance of the RerunTumblingWindowTrigger class.
 * @constructor
 * Trigger that schedules pipeline reruns for all fixed time interval windows
 * from a requested start time to requested end time.
 *
 * @member {object} [parentTrigger] The parent trigger reference.
 * @member {date} requestedStartTime The start time for the time period for
 * which restatement is initiated. Only UTC time is currently supported.
 * @member {date} requestedEndTime The end time for the time period for which
 * restatement is initiated. Only UTC time is currently supported.
 * @member {number} maxConcurrency The max number of parallel time windows
 * (ready for execution) for which a rerun is triggered.
 */
export interface RerunTumblingWindowTrigger extends Trigger {
  parentTrigger?: any;
  requestedStartTime: Date;
  requestedEndTime: Date;
  maxConcurrency: number;
}

/**
 * @class
 * Initializes a new instance of the RerunTriggerResource class.
 * @constructor
 * RerunTrigger resource type.
 *
 * @member {object} properties Properties of the rerun trigger.
 * @member {object} [properties.parentTrigger] The parent trigger reference.
 * @member {date} [properties.requestedStartTime] The start time for the time
 * period for which restatement is initiated. Only UTC time is currently
 * supported.
 * @member {date} [properties.requestedEndTime] The end time for the time
 * period for which restatement is initiated. Only UTC time is currently
 * supported.
 * @member {number} [properties.maxConcurrency] The max number of parallel time
 * windows (ready for execution) for which a rerun is triggered.
 */
export interface RerunTriggerResource extends SubResource {
  properties: RerunTumblingWindowTrigger;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Metadata associated with the operation.
 *
 * @member {string} [description] The description of the operation.
 * @member {string} [provider] The name of the provider.
 * @member {string} [resource] The name of the resource type on which the
 * operation is performed.
 * @member {string} [operation] The type of operation: get, read, delete, etc.
 */
export interface OperationDisplay {
  description?: string;
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationLogSpecification class.
 * @constructor
 * Details about an operation related to logs.
 *
 * @member {string} [name] The name of the log category.
 * @member {string} [displayName] Localized display name.
 * @member {string} [blobDuration] Blobs created in the customer storage
 * account, per hour.
 */
export interface OperationLogSpecification {
  name?: string;
  displayName?: string;
  blobDuration?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationMetricAvailability class.
 * @constructor
 * Defines how often data for a metric becomes available.
 *
 * @member {string} [timeGrain] The granularity for the metric.
 * @member {string} [blobDuration] Blob created in the customer storage
 * account, per hour.
 */
export interface OperationMetricAvailability {
  timeGrain?: string;
  blobDuration?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationMetricDimension class.
 * @constructor
 * Defines the metric dimension.
 *
 * @member {string} [name] The name of the dimension for the metric.
 * @member {string} [displayName] The display name of the metric dimension.
 * @member {boolean} [toBeExportedForShoebox] Whether the dimension should be
 * exported to Azure Monitor.
 */
export interface OperationMetricDimension {
  name?: string;
  displayName?: string;
  toBeExportedForShoebox?: boolean;
}

/**
 * @class
 * Initializes a new instance of the OperationMetricSpecification class.
 * @constructor
 * Details about an operation related to metrics.
 *
 * @member {string} [name] The name of the metric.
 * @member {string} [displayName] Localized display name of the metric.
 * @member {string} [displayDescription] The description of the metric.
 * @member {string} [unit] The unit that the metric is measured in.
 * @member {string} [aggregationType] The type of metric aggregation.
 * @member {string} [enableRegionalMdmAccount] Whether or not the service is
 * using regional MDM accounts.
 * @member {string} [sourceMdmAccount] The name of the MDM account.
 * @member {string} [sourceMdmNamespace] The name of the MDM namespace.
 * @member {array} [availabilities] Defines how often data for metrics becomes
 * available.
 * @member {array} [dimensions] Defines the metric dimension.
 */
export interface OperationMetricSpecification {
  name?: string;
  displayName?: string;
  displayDescription?: string;
  unit?: string;
  aggregationType?: string;
  enableRegionalMdmAccount?: string;
  sourceMdmAccount?: string;
  sourceMdmNamespace?: string;
  availabilities?: OperationMetricAvailability[];
  dimensions?: OperationMetricDimension[];
}

/**
 * @class
 * Initializes a new instance of the OperationServiceSpecification class.
 * @constructor
 * Details about a service operation.
 *
 * @member {array} [logSpecifications] Details about operations related to
 * logs.
 * @member {array} [metricSpecifications] Details about operations related to
 * metrics.
 */
export interface OperationServiceSpecification {
  logSpecifications?: OperationLogSpecification[];
  metricSpecifications?: OperationMetricSpecification[];
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Azure Data Factory API operation definition.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {string} [origin] The intended executor of the operation.
 * @member {object} [display] Metadata associated with the operation.
 * @member {string} [display.description] The description of the operation.
 * @member {string} [display.provider] The name of the provider.
 * @member {string} [display.resource] The name of the resource type on which
 * the operation is performed.
 * @member {string} [display.operation] The type of operation: get, read,
 * delete, etc.
 * @member {object} [serviceSpecification] Details about a service operation.
 * @member {array} [serviceSpecification.logSpecifications] Details about
 * operations related to logs.
 * @member {array} [serviceSpecification.metricSpecifications] Details about
 * operations related to metrics.
 */
export interface Operation {
  name?: string;
  origin?: string;
  display?: OperationDisplay;
  serviceSpecification?: OperationServiceSpecification;
}

/**
 * @class
 * Initializes a new instance of the DependencyReference class.
 * @constructor
 * Referenced dependency.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface DependencyReference {
  type: string;
}

/**
 * @class
 * Initializes a new instance of the SelfDependencyTumblingWindowTriggerReference class.
 * @constructor
 * Self referenced tumbling window trigger dependency.
 *
 * @member {string} offset Timespan applied to the start time of a tumbling
 * window when evaluating dependency.
 * @member {string} [size] The size of the window when evaluating the
 * dependency. If undefined the frequency of the tumbling window will be used.
 */
export interface SelfDependencyTumblingWindowTriggerReference extends DependencyReference {
  offset: string;
  size?: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerReference class.
 * @constructor
 * Trigger reference type.
 *
 * @member {string} referenceName Reference trigger name.
 */
export interface TriggerReference {
  referenceName: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerDependencyReference class.
 * @constructor
 * Trigger referenced dependency.
 *
 * @member {object} referenceTrigger Referenced trigger.
 * @member {string} [referenceTrigger.referenceName] Reference trigger name.
 */
export interface TriggerDependencyReference extends DependencyReference {
  referenceTrigger: TriggerReference;
}

/**
 * @class
 * Initializes a new instance of the TumblingWindowTriggerDependencyReference class.
 * @constructor
 * Referenced tumbling window trigger dependency.
 *
 * @member {string} [offset] Timespan applied to the start time of a tumbling
 * window when evaluating dependency.
 * @member {string} [size] The size of the window when evaluating the
 * dependency. If undefined the frequency of the tumbling window will be used.
 */
export interface TumblingWindowTriggerDependencyReference extends TriggerDependencyReference {
  offset?: string;
  size?: string;
}

/**
 * @class
 * Initializes a new instance of the RetryPolicy class.
 * @constructor
 * Execution policy for an activity.
 *
 * @member {object} [count] Maximum ordinary retry attempts. Default is 0.
 * Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {number} [intervalInSeconds] Interval between retries in seconds.
 * Default is 30.
 */
export interface RetryPolicy {
  count?: any;
  intervalInSeconds?: number;
}

/**
 * @class
 * Initializes a new instance of the TumblingWindowTrigger class.
 * @constructor
 * Trigger that schedules pipeline runs for all fixed time interval windows
 * from a start time without gaps and also supports backfill scenarios (when
 * start time is in the past).
 *
 * @member {object} pipelineProperty Pipeline for which runs are created when
 * an event is fired for trigger window that is ready.
 * @member {object} [pipelineProperty.pipelineReference] Pipeline reference.
 * @member {string} [pipelineProperty.pipelineReference.referenceName]
 * Reference pipeline name.
 * @member {string} [pipelineProperty.pipelineReference.name] Reference name.
 * @member {object} [pipelineProperty.parameters] Pipeline parameters.
 * @member {string} frequency The frequency of the time windows. Possible
 * values include: 'Minute', 'Hour'
 * @member {number} interval The interval of the time windows. The minimum
 * interval allowed is 15 Minutes.
 * @member {date} startTime The start time for the time period for the trigger
 * during which events are fired for windows that are ready. Only UTC time is
 * currently supported.
 * @member {date} [endTime] The end time for the time period for the trigger
 * during which events are fired for windows that are ready. Only UTC time is
 * currently supported.
 * @member {object} [delay] Specifies how long the trigger waits past due time
 * before triggering new run. It doesn't alter window start and end time. The
 * default is 0. Type: string (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {number} maxConcurrency The max number of parallel time windows
 * (ready for execution) for which a new run is triggered.
 * @member {object} [retryPolicy] Retry policy that will be applied for failed
 * pipeline runs.
 * @member {object} [retryPolicy.count] Maximum ordinary retry attempts.
 * Default is 0. Type: integer (or Expression with resultType integer),
 * minimum: 0.
 * @member {number} [retryPolicy.intervalInSeconds] Interval between retries in
 * seconds. Default is 30.
 * @member {array} [dependsOn] Triggers that this trigger depends on. Only
 * tumbling window triggers are supported.
 */
export interface TumblingWindowTrigger extends Trigger {
  pipelineProperty: TriggerPipelineReference;
  frequency: string;
  interval: number;
  startTime: Date;
  endTime?: Date;
  delay?: any;
  maxConcurrency: number;
  retryPolicy?: RetryPolicy;
  dependsOn?: DependencyReference[];
}

/**
 * @class
 * Initializes a new instance of the MultiplePipelineTrigger class.
 * @constructor
 * Base class for all triggers that support one to many model for trigger to
 * pipeline.
 *
 * @member {array} [pipelines] Pipelines that need to be started.
 */
export interface MultiplePipelineTrigger extends Trigger {
  pipelines?: TriggerPipelineReference[];
}

/**
 * @class
 * Initializes a new instance of the BlobEventsTrigger class.
 * @constructor
 * Trigger that runs everytime a Blob event occurs.
 *
 * @member {string} [blobPathBeginsWith] The blob path must begin with the
 * pattern provided for trigger to fire. For example,
 * '/records/blobs/december/' will only fire the trigger for blobs in the
 * december folder under the records container. At least one of these must be
 * provided: blobPathBeginsWith, blobPathEndsWith.
 * @member {string} [blobPathEndsWith] The blob path must end with the pattern
 * provided for trigger to fire. For example, 'december/boxes.csv' will only
 * fire the trigger for blobs named boxes in a december folder. At least one of
 * these must be provided: blobPathBeginsWith, blobPathEndsWith.
 * @member {array} events The type of events that cause this trigger to fire.
 * @member {string} scope The ARM resource ID of the Storage Account.
 */
export interface BlobEventsTrigger extends MultiplePipelineTrigger {
  blobPathBeginsWith?: string;
  blobPathEndsWith?: string;
  events: string[];
  scope: string;
}

/**
 * @class
 * Initializes a new instance of the BlobTrigger class.
 * @constructor
 * Trigger that runs everytime the selected Blob container changes.
 *
 * @member {string} folderPath The path of the container/folder that will
 * trigger the pipeline.
 * @member {number} maxConcurrency The max number of parallel files to handle
 * when it is triggered.
 * @member {object} linkedService The Azure Storage linked service reference.
 * @member {string} [linkedService.referenceName] Reference LinkedService name.
 * @member {object} [linkedService.parameters] Arguments for LinkedService.
 */
export interface BlobTrigger extends MultiplePipelineTrigger {
  folderPath: string;
  maxConcurrency: number;
  linkedService: LinkedServiceReference;
}

/**
 * @class
 * Initializes a new instance of the RecurrenceScheduleOccurrence class.
 * @constructor
 * The recurrence schedule occurence.
 *
 * @member {string} [day] The day of the week. Possible values include:
 * 'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'
 * @member {number} [occurrence] The occurrence.
 */
export interface RecurrenceScheduleOccurrence {
  day?: string;
  occurrence?: number;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the RecurrenceSchedule class.
 * @constructor
 * The recurrence schedule.
 *
 * @member {array} [minutes] The minutes.
 * @member {array} [hours] The hours.
 * @member {array} [weekDays] The days of the week.
 * @member {array} [monthDays] The month days.
 * @member {array} [monthlyOccurrences] The monthly occurrences.
 */
export interface RecurrenceSchedule {
  minutes?: number[];
  hours?: number[];
  weekDays?: string[];
  monthDays?: number[];
  monthlyOccurrences?: RecurrenceScheduleOccurrence[];
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ScheduleTriggerRecurrence class.
 * @constructor
 * The workflow trigger recurrence.
 *
 * @member {string} [frequency] The frequency. Possible values include:
 * 'NotSpecified', 'Minute', 'Hour', 'Day', 'Week', 'Month', 'Year'
 * @member {number} [interval] The interval.
 * @member {date} [startTime] The start time.
 * @member {date} [endTime] The end time.
 * @member {string} [timeZone] The time zone.
 * @member {object} [schedule] The recurrence schedule.
 * @member {array} [schedule.minutes] The minutes.
 * @member {array} [schedule.hours] The hours.
 * @member {array} [schedule.weekDays] The days of the week.
 * @member {array} [schedule.monthDays] The month days.
 * @member {array} [schedule.monthlyOccurrences] The monthly occurrences.
 */
export interface ScheduleTriggerRecurrence {
  frequency?: string;
  interval?: number;
  startTime?: Date;
  endTime?: Date;
  timeZone?: string;
  schedule?: RecurrenceSchedule;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ScheduleTrigger class.
 * @constructor
 * Trigger that creates pipeline runs periodically, on schedule.
 *
 * @member {object} recurrence Recurrence schedule configuration.
 * @member {string} [recurrence.frequency] The frequency. Possible values
 * include: 'NotSpecified', 'Minute', 'Hour', 'Day', 'Week', 'Month', 'Year'
 * @member {number} [recurrence.interval] The interval.
 * @member {date} [recurrence.startTime] The start time.
 * @member {date} [recurrence.endTime] The end time.
 * @member {string} [recurrence.timeZone] The time zone.
 * @member {object} [recurrence.schedule] The recurrence schedule.
 * @member {array} [recurrence.schedule.minutes] The minutes.
 * @member {array} [recurrence.schedule.hours] The hours.
 * @member {array} [recurrence.schedule.weekDays] The days of the week.
 * @member {array} [recurrence.schedule.monthDays] The month days.
 * @member {array} [recurrence.schedule.monthlyOccurrences] The monthly
 * occurrences.
 */
export interface ScheduleTrigger extends MultiplePipelineTrigger {
  recurrence: ScheduleTriggerRecurrence;
}

/**
 * @class
 * Initializes a new instance of the ResponsysLinkedService class.
 * @constructor
 * Responsys linked service.
 *
 * @member {object} endpoint The endpoint of the Responsys server.
 * @member {object} clientId The client ID associated with the Responsys
 * application. Type: string (or Expression with resultType string).
 * @member {object} [clientSecret] The client secret associated with the
 * Responsys application. Type: string (or Expression with resultType string).
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true. Type:
 * boolean (or Expression with resultType boolean).
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true. Type: boolean (or Expression
 * with resultType boolean).
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * Type: boolean (or Expression with resultType boolean).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ResponsysLinkedService extends LinkedService {
  endpoint: any;
  clientId: any;
  clientSecret?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureDatabricksLinkedService class.
 * @constructor
 * Azure Databricks linked service.
 *
 * @member {object} domain <REGION>.azuredatabricks.net, domain name of your
 * Databricks deployment. Type: string (or Expression with resultType string).
 * @member {object} accessToken Access token for databricks REST API. Refer to
 * https://docs.azuredatabricks.net/api/latest/authentication.html. Type:
 * string (or Expression with resultType string).
 * @member {string} [accessToken.type] Polymorphic Discriminator
 * @member {object} [existingClusterId] The id of an existing cluster that will
 * be used for all runs of this job. Type: string (or Expression with
 * resultType string).
 * @member {object} [newClusterVersion] The Spark version of new cluster. Type:
 * string (or Expression with resultType string).
 * @member {object} [newClusterNumOfWorker] Number of worker nodes that new
 * cluster should have. A string formatted Int32, like '1' means numOfWorker is
 * 1 or '1:10' means auto-scale from 1 as min and 10 as max. Type: string (or
 * Expression with resultType string).
 * @member {object} [newClusterNodeType] The node types of new cluster. Type:
 * string (or Expression with resultType string).
 * @member {object} [newClusterSparkConf] A set of optional, user-specified
 * Spark configuration key-value pairs.
 * @member {object} [newClusterSparkEnvVars] A set of optional, user-specified
 * Spark environment variables key-value pairs.
 * @member {object} [newClusterCustomTags] Additional tags for cluster
 * resources.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureDatabricksLinkedService extends LinkedService {
  domain: any;
  accessToken: SecretBase;
  existingClusterId?: any;
  newClusterVersion?: any;
  newClusterNumOfWorker?: any;
  newClusterNodeType?: any;
  newClusterSparkConf?: { [propertyName: string]: any };
  newClusterSparkEnvVars?: { [propertyName: string]: any };
  newClusterCustomTags?: { [propertyName: string]: any };
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeAnalyticsLinkedService class.
 * @constructor
 * Azure Data Lake Analytics linked service.
 *
 * @member {object} accountName The Azure Data Lake Analytics account name.
 * Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The ID of the application used to
 * authenticate against the Azure Data Lake Analytics account. Type: string (or
 * Expression with resultType string).
 * @member {object} [servicePrincipalKey] The Key of the application used to
 * authenticate against the Azure Data Lake Analytics account.
 * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
 * @member {object} tenant The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [subscriptionId] Data Lake Analytics account subscription
 * ID (if different from Data Factory account). Type: string (or Expression
 * with resultType string).
 * @member {object} [resourceGroupName] Data Lake Analytics account resource
 * group name (if different from Data Factory account). Type: string (or
 * Expression with resultType string).
 * @member {object} [dataLakeAnalyticsUri] Azure Data Lake Analytics URI Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureDataLakeAnalyticsLinkedService extends LinkedService {
  accountName: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecretBase;
  tenant: any;
  subscriptionId?: any;
  resourceGroupName?: any;
  dataLakeAnalyticsUri?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the ScriptAction class.
 * @constructor
 * Custom script action to run on HDI ondemand cluster once it's up.
 *
 * @member {string} name The user provided name of the script action.
 * @member {string} uri The URI for the script action.
 * @member {object} roles The node types on which the script action should be
 * executed.
 * @member {string} [parameters] The parameters for the script action.
 */
export interface ScriptAction {
  name: string;
  uri: string;
  roles: any;
  parameters?: string;
}

/**
 * @class
 * Initializes a new instance of the HDInsightOnDemandLinkedService class.
 * @constructor
 * HDInsight ondemand linked service.
 *
 * @member {object} clusterSize Number of worker/data nodes in the cluster.
 * Suggestion value: 4. Type: string (or Expression with resultType string).
 * @member {object} timeToLive The allowed idle time for the on-demand
 * HDInsight cluster. Specifies how long the on-demand HDInsight cluster stays
 * alive after completion of an activity run if there are no other active jobs
 * in the cluster. The minimum value is 5 mins. Type: string (or Expression
 * with resultType string).
 * @member {object} version Version of the HDInsight cluster.  Type: string (or
 * Expression with resultType string).
 * @member {object} linkedServiceName Azure Storage linked service to be used
 * by the on-demand cluster for storing and processing data.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [linkedServiceName.parameters] Arguments for LinkedService.
 * @member {object} hostSubscriptionId The customer’s subscription to host the
 * cluster. Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The service principal id for the
 * hostSubscriptionId. Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalKey] The key for the service principal id.
 * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
 * @member {object} tenant The Tenant id/name to which the service principal
 * belongs. Type: string (or Expression with resultType string).
 * @member {object} clusterResourceGroup The resource group where the cluster
 * belongs. Type: string (or Expression with resultType string).
 * @member {object} [clusterNamePrefix] The prefix of cluster name, postfix
 * will be distinct with timestamp. Type: string (or Expression with resultType
 * string).
 * @member {object} [clusterUserName] The username to access the cluster. Type:
 * string (or Expression with resultType string).
 * @member {object} [clusterPassword] The password to access the cluster.
 * @member {string} [clusterPassword.type] Polymorphic Discriminator
 * @member {object} [clusterSshUserName] The username to SSH remotely connect
 * to cluster’s node (for Linux). Type: string (or Expression with resultType
 * string).
 * @member {object} [clusterSshPassword] The password to SSH remotely connect
 * cluster’s node (for Linux).
 * @member {string} [clusterSshPassword.type] Polymorphic Discriminator
 * @member {array} [additionalLinkedServiceNames] Specifies additional storage
 * accounts for the HDInsight linked service so that the Data Factory service
 * can register them on your behalf.
 * @member {object} [hcatalogLinkedServiceName] The name of Azure SQL linked
 * service that point to the HCatalog database. The on-demand HDInsight cluster
 * is created by using the Azure SQL database as the metastore.
 * @member {string} [hcatalogLinkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [hcatalogLinkedServiceName.parameters] Arguments for
 * LinkedService.
 * @member {object} [clusterType] The cluster type. Type: string (or Expression
 * with resultType string).
 * @member {object} [sparkVersion] The version of spark if the cluster type is
 * 'spark'. Type: string (or Expression with resultType string).
 * @member {object} [coreConfiguration] Specifies the core configuration
 * parameters (as in core-site.xml) for the HDInsight cluster to be created.
 * @member {object} [hBaseConfiguration] Specifies the HBase configuration
 * parameters (hbase-site.xml) for the HDInsight cluster.
 * @member {object} [hdfsConfiguration] Specifies the HDFS configuration
 * parameters (hdfs-site.xml) for the HDInsight cluster.
 * @member {object} [hiveConfiguration] Specifies the hive configuration
 * parameters (hive-site.xml) for the HDInsight cluster.
 * @member {object} [mapReduceConfiguration] Specifies the MapReduce
 * configuration parameters (mapred-site.xml) for the HDInsight cluster.
 * @member {object} [oozieConfiguration] Specifies the Oozie configuration
 * parameters (oozie-site.xml) for the HDInsight cluster.
 * @member {object} [stormConfiguration] Specifies the Storm configuration
 * parameters (storm-site.xml) for the HDInsight cluster.
 * @member {object} [yarnConfiguration] Specifies the Yarn configuration
 * parameters (yarn-site.xml) for the HDInsight cluster.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [headNodeSize] Specifies the size of the head node for the
 * HDInsight cluster.
 * @member {object} [dataNodeSize] Specifies the size of the data node for the
 * HDInsight cluster.
 * @member {object} [zookeeperNodeSize] Specifies the size of the Zoo Keeper
 * node for the HDInsight cluster.
 * @member {array} [scriptActions] Custom script actions to run on HDI ondemand
 * cluster once it's up. Please refer to
 * https://docs.microsoft.com/en-us/azure/hdinsight/hdinsight-hadoop-customize-cluster-linux?toc=%2Fen-us%2Fazure%2Fhdinsight%2Fr-server%2FTOC.json&bc=%2Fen-us%2Fazure%2Fbread%2Ftoc.json#understanding-script-actions.
 */
export interface HDInsightOnDemandLinkedService extends LinkedService {
  clusterSize: any;
  timeToLive: any;
  version: any;
  linkedServiceName: LinkedServiceReference;
  hostSubscriptionId: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecretBase;
  tenant: any;
  clusterResourceGroup: any;
  clusterNamePrefix?: any;
  clusterUserName?: any;
  clusterPassword?: SecretBase;
  clusterSshUserName?: any;
  clusterSshPassword?: SecretBase;
  additionalLinkedServiceNames?: LinkedServiceReference[];
  hcatalogLinkedServiceName?: LinkedServiceReference;
  clusterType?: any;
  sparkVersion?: any;
  coreConfiguration?: any;
  hBaseConfiguration?: any;
  hdfsConfiguration?: any;
  hiveConfiguration?: any;
  mapReduceConfiguration?: any;
  oozieConfiguration?: any;
  stormConfiguration?: any;
  yarnConfiguration?: any;
  encryptedCredential?: any;
  headNodeSize?: any;
  dataNodeSize?: any;
  zookeeperNodeSize?: any;
  scriptActions?: ScriptAction[];
}

/**
 * @class
 * Initializes a new instance of the SalesforceMarketingCloudLinkedService class.
 * @constructor
 * Salesforce Marketing Cloud linked service.
 *
 * @member {object} clientId The client ID associated with the Salesforce
 * Marketing Cloud application. Type: string (or Expression with resultType
 * string).
 * @member {object} [clientSecret] The client secret associated with the
 * Salesforce Marketing Cloud application. Type: string (or Expression with
 * resultType string).
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true. Type:
 * boolean (or Expression with resultType boolean).
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true. Type: boolean (or Expression
 * with resultType boolean).
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * Type: boolean (or Expression with resultType boolean).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SalesforceMarketingCloudLinkedService extends LinkedService {
  clientId: any;
  clientSecret?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the NetezzaLinkedService class.
 * @constructor
 * Netezza linked service.
 *
 * @member {object} [connectionString] An ODBC connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [pwd] The Azure key vault secret reference of password in
 * connection string.
 * @member {object} [pwd.store] The Azure Key Vault linked service reference.
 * @member {string} [pwd.store.referenceName] Reference LinkedService name.
 * @member {object} [pwd.store.parameters] Arguments for LinkedService.
 * @member {object} [pwd.secretName] The name of the secret in Azure Key Vault.
 * Type: string (or Expression with resultType string).
 * @member {object} [pwd.secretVersion] The version of the secret in Azure Key
 * Vault. The default value is the latest version of the secret. Type: string
 * (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface NetezzaLinkedService extends LinkedService {
  connectionString?: any;
  pwd?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the VerticaLinkedService class.
 * @constructor
 * Vertica linked service.
 *
 * @member {object} [connectionString] An ODBC connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [pwd] The Azure key vault secret reference of password in
 * connection string.
 * @member {object} [pwd.store] The Azure Key Vault linked service reference.
 * @member {string} [pwd.store.referenceName] Reference LinkedService name.
 * @member {object} [pwd.store.parameters] Arguments for LinkedService.
 * @member {object} [pwd.secretName] The name of the secret in Azure Key Vault.
 * Type: string (or Expression with resultType string).
 * @member {object} [pwd.secretVersion] The version of the secret in Azure Key
 * Vault. The default value is the latest version of the secret. Type: string
 * (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface VerticaLinkedService extends LinkedService {
  connectionString?: any;
  pwd?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the ZohoLinkedService class.
 * @constructor
 * Zoho server linked service.
 *
 * @member {object} endpoint The endpoint of the Zoho server. (i.e.
 * crm.zoho.com/crm/private)
 * @member {object} [accessToken] The access token for Zoho authentication.
 * @member {string} [accessToken.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ZohoLinkedService extends LinkedService {
  endpoint: any;
  accessToken?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the XeroLinkedService class.
 * @constructor
 * Xero Serivce linked service.
 *
 * @member {object} host The endpoint of the Xero server. (i.e. api.xero.com)
 * @member {object} [consumerKey] The consumer key associated with the Xero
 * application.
 * @member {string} [consumerKey.type] Polymorphic Discriminator
 * @member {object} [privateKey] The private key from the .pem file that was
 * generated for your Xero private application. You must include all the text
 * from the .pem file, including the Unix line endings(
 * ).
 * @member {string} [privateKey.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface XeroLinkedService extends LinkedService {
  host: any;
  consumerKey?: SecretBase;
  privateKey?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SquareLinkedService class.
 * @constructor
 * Square Serivce linked service.
 *
 * @member {object} host The URL of the Square instance. (i.e.
 * mystore.mysquare.com)
 * @member {object} clientId The client ID associated with your Square
 * application.
 * @member {object} [clientSecret] The client secret associated with your
 * Square application.
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} redirectUri The redirect URL assigned in the Square
 * application dashboard. (i.e. http://localhost:2500)
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SquareLinkedService extends LinkedService {
  host: any;
  clientId: any;
  clientSecret?: SecretBase;
  redirectUri: any;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SparkLinkedService class.
 * @constructor
 * Spark Server linked service.
 *
 * @member {object} host IP address or host name of the Spark server
 * @member {object} port The TCP port that the Spark server uses to listen for
 * client connections.
 * @member {string} [serverType] The type of Spark server. Possible values
 * include: 'SharkServer', 'SharkServer2', 'SparkThriftServer'
 * @member {string} [thriftTransportProtocol] The transport protocol to use in
 * the Thrift layer. Possible values include: 'Binary', 'SASL', 'HTTP '
 * @member {string} authenticationType The authentication method used to access
 * the Spark server. Possible values include: 'Anonymous', 'Username',
 * 'UsernameAndPassword', 'WindowsAzureHDInsightService'
 * @member {object} [username] The user name that you use to access Spark
 * Server.
 * @member {object} [password] The password corresponding to the user name that
 * you provided in the Username field
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [httpPath] The partial URL corresponding to the Spark
 * server.
 * @member {object} [enableSsl] Specifies whether the connections to the server
 * are encrypted using SSL. The default value is false.
 * @member {object} [trustedCertPath] The full path of the .pem file containing
 * trusted CA certificates for verifying the server when connecting over SSL.
 * This property can only be set when using SSL on self-hosted IR. The default
 * value is the cacerts.pem file installed with the IR.
 * @member {object} [useSystemTrustStore] Specifies whether to use a CA
 * certificate from the system trust store or from a specified PEM file. The
 * default value is false.
 * @member {object} [allowHostNameCNMismatch] Specifies whether to require a
 * CA-issued SSL certificate name to match the host name of the server when
 * connecting over SSL. The default value is false.
 * @member {object} [allowSelfSignedServerCert] Specifies whether to allow
 * self-signed certificates from the server. The default value is false.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SparkLinkedService extends LinkedService {
  host: any;
  port: any;
  serverType?: string;
  thriftTransportProtocol?: string;
  authenticationType: string;
  username?: any;
  password?: SecretBase;
  httpPath?: any;
  enableSsl?: any;
  trustedCertPath?: any;
  useSystemTrustStore?: any;
  allowHostNameCNMismatch?: any;
  allowSelfSignedServerCert?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the ShopifyLinkedService class.
 * @constructor
 * Shopify Serivce linked service.
 *
 * @member {object} host The endpoint of the Shopify server. (i.e.
 * mystore.myshopify.com)
 * @member {object} [accessToken] The API access token that can be used to
 * access Shopify’s data. The token won't expire if it is offline mode.
 * @member {string} [accessToken.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ShopifyLinkedService extends LinkedService {
  host: any;
  accessToken?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the ServiceNowLinkedService class.
 * @constructor
 * ServiceNow server linked service.
 *
 * @member {object} endpoint The endpoint of the ServiceNow server. (i.e.
 * <instance>.service-now.com)
 * @member {string} authenticationType The authentication type to use. Possible
 * values include: 'Basic', 'OAuth2'
 * @member {object} [username] The user name used to connect to the ServiceNow
 * server for Basic and OAuth2 authentication.
 * @member {object} [password] The password corresponding to the user name for
 * Basic and OAuth2 authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [clientId] The client id for OAuth2 authentication.
 * @member {object} [clientSecret] The client secret for OAuth2 authentication.
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ServiceNowLinkedService extends LinkedService {
  endpoint: any;
  authenticationType: string;
  username?: any;
  password?: SecretBase;
  clientId?: any;
  clientSecret?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the QuickBooksLinkedService class.
 * @constructor
 * QuickBooks server linked service.
 *
 * @member {object} endpoint The endpoint of the QuickBooks server. (i.e.
 * quickbooks.api.intuit.com)
 * @member {object} companyId The company ID of the QuickBooks company to
 * authorize.
 * @member {object} consumerKey The consumer key for OAuth 1.0 authentication.
 * @member {object} consumerSecret The consumer secret for OAuth 1.0
 * authentication.
 * @member {string} [consumerSecret.type] Polymorphic Discriminator
 * @member {object} accessToken The access token for OAuth 1.0 authentication.
 * @member {string} [accessToken.type] Polymorphic Discriminator
 * @member {object} accessTokenSecret The access token secret for OAuth 1.0
 * authentication.
 * @member {string} [accessTokenSecret.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface QuickBooksLinkedService extends LinkedService {
  endpoint: any;
  companyId: any;
  consumerKey: any;
  consumerSecret: SecretBase;
  accessToken: SecretBase;
  accessTokenSecret: SecretBase;
  useEncryptedEndpoints?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the PrestoLinkedService class.
 * @constructor
 * Presto server linked service.
 *
 * @member {object} host The IP address or host name of the Presto server.
 * (i.e. 192.168.222.160)
 * @member {object} serverVersion The version of the Presto server. (i.e.
 * 0.148-t)
 * @member {object} catalog The catalog context for all request against the
 * server.
 * @member {object} [port] The TCP port that the Presto server uses to listen
 * for client connections. The default value is 8080.
 * @member {string} authenticationType The authentication mechanism used to
 * connect to the Presto server. Possible values include: 'Anonymous', 'LDAP'
 * @member {object} [username] The user name used to connect to the Presto
 * server.
 * @member {object} [password] The password corresponding to the user name.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [enableSsl] Specifies whether the connections to the server
 * are encrypted using SSL. The default value is false.
 * @member {object} [trustedCertPath] The full path of the .pem file containing
 * trusted CA certificates for verifying the server when connecting over SSL.
 * This property can only be set when using SSL on self-hosted IR. The default
 * value is the cacerts.pem file installed with the IR.
 * @member {object} [useSystemTrustStore] Specifies whether to use a CA
 * certificate from the system trust store or from a specified PEM file. The
 * default value is false.
 * @member {object} [allowHostNameCNMismatch] Specifies whether to require a
 * CA-issued SSL certificate name to match the host name of the server when
 * connecting over SSL. The default value is false.
 * @member {object} [allowSelfSignedServerCert] Specifies whether to allow
 * self-signed certificates from the server. The default value is false.
 * @member {object} [timeZoneID] The local time zone used by the connection.
 * Valid values for this option are specified in the IANA Time Zone Database.
 * The default value is the system time zone.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface PrestoLinkedService extends LinkedService {
  host: any;
  serverVersion: any;
  catalog: any;
  port?: any;
  authenticationType: string;
  username?: any;
  password?: SecretBase;
  enableSsl?: any;
  trustedCertPath?: any;
  useSystemTrustStore?: any;
  allowHostNameCNMismatch?: any;
  allowSelfSignedServerCert?: any;
  timeZoneID?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the PhoenixLinkedService class.
 * @constructor
 * Phoenix server linked service.
 *
 * @member {object} host The IP address or host name of the Phoenix server.
 * (i.e. 192.168.222.160)
 * @member {object} [port] The TCP port that the Phoenix server uses to listen
 * for client connections. The default value is 8765.
 * @member {object} [httpPath] The partial URL corresponding to the Phoenix
 * server. (i.e. /gateway/sandbox/phoenix/version). The default value is
 * hbasephoenix if using WindowsAzureHDInsightService.
 * @member {string} authenticationType The authentication mechanism used to
 * connect to the Phoenix server. Possible values include: 'Anonymous',
 * 'UsernameAndPassword', 'WindowsAzureHDInsightService'
 * @member {object} [username] The user name used to connect to the Phoenix
 * server.
 * @member {object} [password] The password corresponding to the user name.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [enableSsl] Specifies whether the connections to the server
 * are encrypted using SSL. The default value is false.
 * @member {object} [trustedCertPath] The full path of the .pem file containing
 * trusted CA certificates for verifying the server when connecting over SSL.
 * This property can only be set when using SSL on self-hosted IR. The default
 * value is the cacerts.pem file installed with the IR.
 * @member {object} [useSystemTrustStore] Specifies whether to use a CA
 * certificate from the system trust store or from a specified PEM file. The
 * default value is false.
 * @member {object} [allowHostNameCNMismatch] Specifies whether to require a
 * CA-issued SSL certificate name to match the host name of the server when
 * connecting over SSL. The default value is false.
 * @member {object} [allowSelfSignedServerCert] Specifies whether to allow
 * self-signed certificates from the server. The default value is false.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface PhoenixLinkedService extends LinkedService {
  host: any;
  port?: any;
  httpPath?: any;
  authenticationType: string;
  username?: any;
  password?: SecretBase;
  enableSsl?: any;
  trustedCertPath?: any;
  useSystemTrustStore?: any;
  allowHostNameCNMismatch?: any;
  allowSelfSignedServerCert?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the PaypalLinkedService class.
 * @constructor
 * Paypal Serivce linked service.
 *
 * @member {object} host The URL of the PayPal instance. (i.e.
 * api.sandbox.paypal.com)
 * @member {object} clientId The client ID associated with your PayPal
 * application.
 * @member {object} [clientSecret] The client secret associated with your
 * PayPal application.
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface PaypalLinkedService extends LinkedService {
  host: any;
  clientId: any;
  clientSecret?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the MarketoLinkedService class.
 * @constructor
 * Marketo server linked service.
 *
 * @member {object} endpoint The endpoint of the Marketo server. (i.e.
 * 123-ABC-321.mktorest.com)
 * @member {object} clientId The client Id of your Marketo service.
 * @member {object} [clientSecret] The client secret of your Marketo service.
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface MarketoLinkedService extends LinkedService {
  endpoint: any;
  clientId: any;
  clientSecret?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the MariaDBLinkedService class.
 * @constructor
 * MariaDB server linked service.
 *
 * @member {object} [connectionString] An ODBC connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [pwd] The Azure key vault secret reference of password in
 * connection string.
 * @member {object} [pwd.store] The Azure Key Vault linked service reference.
 * @member {string} [pwd.store.referenceName] Reference LinkedService name.
 * @member {object} [pwd.store.parameters] Arguments for LinkedService.
 * @member {object} [pwd.secretName] The name of the secret in Azure Key Vault.
 * Type: string (or Expression with resultType string).
 * @member {object} [pwd.secretVersion] The version of the secret in Azure Key
 * Vault. The default value is the latest version of the secret. Type: string
 * (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface MariaDBLinkedService extends LinkedService {
  connectionString?: any;
  pwd?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the MagentoLinkedService class.
 * @constructor
 * Magento server linked service.
 *
 * @member {object} host The URL of the Magento instance. (i.e.
 * 192.168.222.110/magento3)
 * @member {object} [accessToken] The access token from Magento.
 * @member {string} [accessToken.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface MagentoLinkedService extends LinkedService {
  host: any;
  accessToken?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the JiraLinkedService class.
 * @constructor
 * Jira Serivce linked service.
 *
 * @member {object} host The IP address or host name of the Jira service. (e.g.
 * jira.example.com)
 * @member {object} [port] The TCP port that the Jira server uses to listen for
 * client connections. The default value is 443 if connecting through HTTPS, or
 * 8080 if connecting through HTTP.
 * @member {object} username The user name that you use to access Jira Service.
 * @member {object} [password] The password corresponding to the user name that
 * you provided in the username field.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface JiraLinkedService extends LinkedService {
  host: any;
  port?: any;
  username: any;
  password?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the ImpalaLinkedService class.
 * @constructor
 * Impala server linked service.
 *
 * @member {object} host The IP address or host name of the Impala server.
 * (i.e. 192.168.222.160)
 * @member {object} [port] The TCP port that the Impala server uses to listen
 * for client connections. The default value is 21050.
 * @member {string} authenticationType The authentication type to use. Possible
 * values include: 'Anonymous', 'SASLUsername', 'UsernameAndPassword'
 * @member {object} [username] The user name used to access the Impala server.
 * The default value is anonymous when using SASLUsername.
 * @member {object} [password] The password corresponding to the user name when
 * using UsernameAndPassword.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [enableSsl] Specifies whether the connections to the server
 * are encrypted using SSL. The default value is false.
 * @member {object} [trustedCertPath] The full path of the .pem file containing
 * trusted CA certificates for verifying the server when connecting over SSL.
 * This property can only be set when using SSL on self-hosted IR. The default
 * value is the cacerts.pem file installed with the IR.
 * @member {object} [useSystemTrustStore] Specifies whether to use a CA
 * certificate from the system trust store or from a specified PEM file. The
 * default value is false.
 * @member {object} [allowHostNameCNMismatch] Specifies whether to require a
 * CA-issued SSL certificate name to match the host name of the server when
 * connecting over SSL. The default value is false.
 * @member {object} [allowSelfSignedServerCert] Specifies whether to allow
 * self-signed certificates from the server. The default value is false.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ImpalaLinkedService extends LinkedService {
  host: any;
  port?: any;
  authenticationType: string;
  username?: any;
  password?: SecretBase;
  enableSsl?: any;
  trustedCertPath?: any;
  useSystemTrustStore?: any;
  allowHostNameCNMismatch?: any;
  allowSelfSignedServerCert?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HubspotLinkedService class.
 * @constructor
 * Hubspot Serivce linked service.
 *
 * @member {object} clientId The client ID associated with your Hubspot
 * application.
 * @member {object} [clientSecret] The client secret associated with your
 * Hubspot application.
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} [accessToken] The access token obtained when initially
 * authenticating your OAuth integration.
 * @member {string} [accessToken.type] Polymorphic Discriminator
 * @member {object} [refreshToken] The refresh token obtained when initially
 * authenticating your OAuth integration.
 * @member {string} [refreshToken.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface HubspotLinkedService extends LinkedService {
  clientId: any;
  clientSecret?: SecretBase;
  accessToken?: SecretBase;
  refreshToken?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HiveLinkedService class.
 * @constructor
 * Hive Server linked service.
 *
 * @member {object} host IP address or host name of the Hive server, separated
 * by ';' for multiple hosts (only when serviceDiscoveryMode is enable).
 * @member {object} [port] The TCP port that the Hive server uses to listen for
 * client connections.
 * @member {string} [serverType] The type of Hive server. Possible values
 * include: 'HiveServer1', 'HiveServer2', 'HiveThriftServer'
 * @member {string} [thriftTransportProtocol] The transport protocol to use in
 * the Thrift layer. Possible values include: 'Binary', 'SASL', 'HTTP '
 * @member {string} authenticationType The authentication method used to access
 * the Hive server. Possible values include: 'Anonymous', 'Username',
 * 'UsernameAndPassword', 'WindowsAzureHDInsightService'
 * @member {object} [serviceDiscoveryMode] true to indicate using the ZooKeeper
 * service, false not.
 * @member {object} [zooKeeperNameSpace] The namespace on ZooKeeper under which
 * Hive Server 2 nodes are added.
 * @member {object} [useNativeQuery] Specifies whether the driver uses native
 * HiveQL queries,or converts them into an equivalent form in HiveQL.
 * @member {object} [username] The user name that you use to access Hive
 * Server.
 * @member {object} [password] The password corresponding to the user name that
 * you provided in the Username field
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [httpPath] The partial URL corresponding to the Hive
 * server.
 * @member {object} [enableSsl] Specifies whether the connections to the server
 * are encrypted using SSL. The default value is false.
 * @member {object} [trustedCertPath] The full path of the .pem file containing
 * trusted CA certificates for verifying the server when connecting over SSL.
 * This property can only be set when using SSL on self-hosted IR. The default
 * value is the cacerts.pem file installed with the IR.
 * @member {object} [useSystemTrustStore] Specifies whether to use a CA
 * certificate from the system trust store or from a specified PEM file. The
 * default value is false.
 * @member {object} [allowHostNameCNMismatch] Specifies whether to require a
 * CA-issued SSL certificate name to match the host name of the server when
 * connecting over SSL. The default value is false.
 * @member {object} [allowSelfSignedServerCert] Specifies whether to allow
 * self-signed certificates from the server. The default value is false.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface HiveLinkedService extends LinkedService {
  host: any;
  port?: any;
  serverType?: string;
  thriftTransportProtocol?: string;
  authenticationType: string;
  serviceDiscoveryMode?: any;
  zooKeeperNameSpace?: any;
  useNativeQuery?: any;
  username?: any;
  password?: SecretBase;
  httpPath?: any;
  enableSsl?: any;
  trustedCertPath?: any;
  useSystemTrustStore?: any;
  allowHostNameCNMismatch?: any;
  allowSelfSignedServerCert?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HBaseLinkedService class.
 * @constructor
 * HBase server linked service.
 *
 * @member {object} host The IP address or host name of the HBase server. (i.e.
 * 192.168.222.160)
 * @member {object} [port] The TCP port that the HBase instance uses to listen
 * for client connections. The default value is 9090.
 * @member {object} [httpPath] The partial URL corresponding to the HBase
 * server. (i.e. /gateway/sandbox/hbase/version)
 * @member {string} authenticationType The authentication mechanism to use to
 * connect to the HBase server. Possible values include: 'Anonymous', 'Basic'
 * @member {object} [username] The user name used to connect to the HBase
 * instance.
 * @member {object} [password] The password corresponding to the user name.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [enableSsl] Specifies whether the connections to the server
 * are encrypted using SSL. The default value is false.
 * @member {object} [trustedCertPath] The full path of the .pem file containing
 * trusted CA certificates for verifying the server when connecting over SSL.
 * This property can only be set when using SSL on self-hosted IR. The default
 * value is the cacerts.pem file installed with the IR.
 * @member {object} [allowHostNameCNMismatch] Specifies whether to require a
 * CA-issued SSL certificate name to match the host name of the server when
 * connecting over SSL. The default value is false.
 * @member {object} [allowSelfSignedServerCert] Specifies whether to allow
 * self-signed certificates from the server. The default value is false.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface HBaseLinkedService extends LinkedService {
  host: any;
  port?: any;
  httpPath?: any;
  authenticationType: string;
  username?: any;
  password?: SecretBase;
  enableSsl?: any;
  trustedCertPath?: any;
  allowHostNameCNMismatch?: any;
  allowSelfSignedServerCert?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the GreenplumLinkedService class.
 * @constructor
 * Greenplum Database linked service.
 *
 * @member {object} [connectionString] An ODBC connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [pwd] The Azure key vault secret reference of password in
 * connection string.
 * @member {object} [pwd.store] The Azure Key Vault linked service reference.
 * @member {string} [pwd.store.referenceName] Reference LinkedService name.
 * @member {object} [pwd.store.parameters] Arguments for LinkedService.
 * @member {object} [pwd.secretName] The name of the secret in Azure Key Vault.
 * Type: string (or Expression with resultType string).
 * @member {object} [pwd.secretVersion] The version of the secret in Azure Key
 * Vault. The default value is the latest version of the secret. Type: string
 * (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface GreenplumLinkedService extends LinkedService {
  connectionString?: any;
  pwd?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the GoogleBigQueryLinkedService class.
 * @constructor
 * Google BigQuery service linked service.
 *
 * @member {object} project The default BigQuery project to query against.
 * @member {object} [additionalProjects] A comma-separated list of public
 * BigQuery projects to access.
 * @member {object} [requestGoogleDriveScope] Whether to request access to
 * Google Drive. Allowing Google Drive access enables support for federated
 * tables that combine BigQuery data with data from Google Drive. The default
 * value is false.
 * @member {string} authenticationType The OAuth 2.0 authentication mechanism
 * used for authentication. ServiceAuthentication can only be used on
 * self-hosted IR. Possible values include: 'ServiceAuthentication',
 * 'UserAuthentication'
 * @member {object} [refreshToken] The refresh token obtained from Google for
 * authorizing access to BigQuery for UserAuthentication.
 * @member {string} [refreshToken.type] Polymorphic Discriminator
 * @member {object} [clientId] The client id of the google application used to
 * acquire the refresh token.
 * @member {string} [clientId.type] Polymorphic Discriminator
 * @member {object} [clientSecret] The client secret of the google application
 * used to acquire the refresh token.
 * @member {string} [clientSecret.type] Polymorphic Discriminator
 * @member {object} [email] The service account email ID that is used for
 * ServiceAuthentication and can only be used on self-hosted IR.
 * @member {object} [keyFilePath] The full path to the .p12 key file that is
 * used to authenticate the service account email address and can only be used
 * on self-hosted IR.
 * @member {object} [trustedCertPath] The full path of the .pem file containing
 * trusted CA certificates for verifying the server when connecting over SSL.
 * This property can only be set when using SSL on self-hosted IR. The default
 * value is the cacerts.pem file installed with the IR.
 * @member {object} [useSystemTrustStore] Specifies whether to use a CA
 * certificate from the system trust store or from a specified PEM file. The
 * default value is false.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface GoogleBigQueryLinkedService extends LinkedService {
  project: any;
  additionalProjects?: any;
  requestGoogleDriveScope?: any;
  authenticationType: string;
  refreshToken?: SecretBase;
  clientId?: SecretBase;
  clientSecret?: SecretBase;
  email?: any;
  keyFilePath?: any;
  trustedCertPath?: any;
  useSystemTrustStore?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the EloquaLinkedService class.
 * @constructor
 * Eloqua server linked service.
 *
 * @member {object} endpoint The endpoint of the Eloqua server. (i.e.
 * eloqua.example.com)
 * @member {object} username The site name and user name of your Eloqua account
 * in the form: sitename/username. (i.e. Eloqua/Alice)
 * @member {object} [password] The password corresponding to the user name.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface EloquaLinkedService extends LinkedService {
  endpoint: any;
  username: any;
  password?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the DrillLinkedService class.
 * @constructor
 * Drill server linked service.
 *
 * @member {object} [connectionString] An ODBC connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [pwd] The Azure key vault secret reference of password in
 * connection string.
 * @member {object} [pwd.store] The Azure Key Vault linked service reference.
 * @member {string} [pwd.store.referenceName] Reference LinkedService name.
 * @member {object} [pwd.store.parameters] Arguments for LinkedService.
 * @member {object} [pwd.secretName] The name of the secret in Azure Key Vault.
 * Type: string (or Expression with resultType string).
 * @member {object} [pwd.secretVersion] The version of the secret in Azure Key
 * Vault. The default value is the latest version of the secret. Type: string
 * (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface DrillLinkedService extends LinkedService {
  connectionString?: any;
  pwd?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the CouchbaseLinkedService class.
 * @constructor
 * Couchbase server linked service.
 *
 * @member {object} [connectionString] An ODBC connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [credString] The Azure key vault secret reference of
 * credString in connection string.
 * @member {object} [credString.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [credString.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [credString.store.parameters] Arguments for LinkedService.
 * @member {object} [credString.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [credString.secretVersion] The version of the secret in
 * Azure Key Vault. The default value is the latest version of the secret.
 * Type: string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface CouchbaseLinkedService extends LinkedService {
  connectionString?: any;
  credString?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the ConcurLinkedService class.
 * @constructor
 * Concur Serivce linked service.
 *
 * @member {object} clientId Application client_id supplied by Concur App
 * Management.
 * @member {object} username The user name that you use to access Concur
 * Service.
 * @member {object} [password] The password corresponding to the user name that
 * you provided in the username field.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ConcurLinkedService extends LinkedService {
  clientId: any;
  username: any;
  password?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzurePostgreSqlLinkedService class.
 * @constructor
 * Azure PostgreSQL linked service.
 *
 * @member {object} [connectionString] An ODBC connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [password] The Azure key vault secret reference of password
 * in connection string.
 * @member {object} [password.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [password.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [password.store.parameters] Arguments for LinkedService.
 * @member {object} [password.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzurePostgreSqlLinkedService extends LinkedService {
  connectionString?: any;
  password?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonMWSLinkedService class.
 * @constructor
 * Amazon Marketplace Web Service linked service.
 *
 * @member {object} endpoint The endpoint of the Amazon MWS server, (i.e.
 * mws.amazonservices.com)
 * @member {object} marketplaceID The Amazon Marketplace ID you want to
 * retrieve data from. To retrive data from multiple Marketplace IDs, seperate
 * them with a comma (,). (i.e. A2EUQ1WTGCTBG2)
 * @member {object} sellerID The Amazon seller ID.
 * @member {object} [mwsAuthToken] The Amazon MWS authentication token.
 * @member {string} [mwsAuthToken.type] Polymorphic Discriminator
 * @member {object} accessKeyId The access key id used to access data.
 * @member {object} [secretKey] The secret key used to access data.
 * @member {string} [secretKey.type] Polymorphic Discriminator
 * @member {object} [useEncryptedEndpoints] Specifies whether the data source
 * endpoints are encrypted using HTTPS. The default value is true.
 * @member {object} [useHostVerification] Specifies whether to require the host
 * name in the server's certificate to match the host name of the server when
 * connecting over SSL. The default value is true.
 * @member {object} [usePeerVerification] Specifies whether to verify the
 * identity of the server when connecting over SSL. The default value is true.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AmazonMWSLinkedService extends LinkedService {
  endpoint: any;
  marketplaceID: any;
  sellerID: any;
  mwsAuthToken?: SecretBase;
  accessKeyId: any;
  secretKey?: SecretBase;
  useEncryptedEndpoints?: any;
  useHostVerification?: any;
  usePeerVerification?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SapHanaLinkedService class.
 * @constructor
 * SAP HANA Linked Service.
 *
 * @member {object} server Host name of the SAP HANA server. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the SAP HANA server. Possible values include: 'Basic', 'Windows'
 * @member {object} [userName] Username to access the SAP HANA server. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] Password to access the SAP HANA server.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SapHanaLinkedService extends LinkedService {
  server: any;
  authenticationType?: string;
  userName?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SapBWLinkedService class.
 * @constructor
 * SAP Business Warehouse Linked Service.
 *
 * @member {object} server Host name of the SAP BW instance. Type: string (or
 * Expression with resultType string).
 * @member {object} systemNumber System number of the BW system. (Usually a
 * two-digit decimal number represented as a string.) Type: string (or
 * Expression with resultType string).
 * @member {object} clientId Client ID of the client on the BW system. (Usually
 * a three-digit decimal number represented as a string) Type: string (or
 * Expression with resultType string).
 * @member {object} [userName] Username to access the SAP BW server. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] Password to access the SAP BW server.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SapBWLinkedService extends LinkedService {
  server: any;
  systemNumber: any;
  clientId: any;
  userName?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SftpServerLinkedService class.
 * @constructor
 * A linked service for an SSH File Transfer Protocol (SFTP) server.
 *
 * @member {object} host The SFTP server host name. Type: string (or Expression
 * with resultType string).
 * @member {object} [port] The TCP port number that the SFTP server uses to
 * listen for client connections. Default value is 22. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the FTP server. Possible values include: 'Basic', 'SshPublicKey'
 * @member {object} [userName] The username used to log on to the SFTP server.
 * Type: string (or Expression with resultType string).
 * @member {object} [password] Password to logon the SFTP server for Basic
 * authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [privateKeyPath] The SSH private key file path for
 * SshPublicKey authentication. Only valid for on-premises copy. For
 * on-premises copy with SshPublicKey authentication, either PrivateKeyPath or
 * PrivateKeyContent should be specified. SSH private key should be OpenSSH
 * format. Type: string (or Expression with resultType string).
 * @member {object} [privateKeyContent] Base64 encoded SSH private key content
 * for SshPublicKey authentication. For on-premises copy with SshPublicKey
 * authentication, either PrivateKeyPath or PrivateKeyContent should be
 * specified. SSH private key should be OpenSSH format.
 * @member {string} [privateKeyContent.type] Polymorphic Discriminator
 * @member {object} [passPhrase] The password to decrypt the SSH private key if
 * the SSH private key is encrypted.
 * @member {string} [passPhrase.type] Polymorphic Discriminator
 * @member {object} [skipHostKeyValidation] If true, skip the SSH host key
 * validation. Default value is false. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [hostKeyFingerprint] The host key finger-print of the SFTP
 * server. When SkipHostKeyValidation is false, HostKeyFingerprint should be
 * specified. Type: string (or Expression with resultType string).
 */
export interface SftpServerLinkedService extends LinkedService {
  host: any;
  port?: any;
  authenticationType?: string;
  userName?: any;
  password?: SecretBase;
  encryptedCredential?: any;
  privateKeyPath?: any;
  privateKeyContent?: SecretBase;
  passPhrase?: SecretBase;
  skipHostKeyValidation?: any;
  hostKeyFingerprint?: any;
}

/**
 * @class
 * Initializes a new instance of the FtpServerLinkedService class.
 * @constructor
 * A FTP server Linked Service.
 *
 * @member {object} host Host name of the FTP server. Type: string (or
 * Expression with resultType string).
 * @member {object} [port] The TCP port number that the FTP server uses to
 * listen for client connections. Default value is 21. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the FTP server. Possible values include: 'Basic', 'Anonymous'
 * @member {object} [userName] Username to logon the FTP server. Type: string
 * (or Expression with resultType string).
 * @member {object} [password] Password to logon the FTP server.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [enableSsl] If true, connect to the FTP server over SSL/TLS
 * channel. Default value is true. Type: boolean (or Expression with resultType
 * boolean).
 * @member {object} [enableServerCertificateValidation] If true, validate the
 * FTP server SSL certificate when connect over SSL/TLS channel. Default value
 * is true. Type: boolean (or Expression with resultType boolean).
 */
export interface FtpServerLinkedService extends LinkedService {
  host: any;
  port?: any;
  authenticationType?: string;
  userName?: any;
  password?: SecretBase;
  encryptedCredential?: any;
  enableSsl?: any;
  enableServerCertificateValidation?: any;
}

/**
 * @class
 * Initializes a new instance of the HttpLinkedService class.
 * @constructor
 * Linked service for an HTTP source.
 *
 * @member {object} url The base URL of the HTTP endpoint, e.g.
 * http://www.microsoft.com. Type: string (or Expression with resultType
 * string).
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the HTTP server. Possible values include: 'Basic', 'Anonymous',
 * 'Digest', 'Windows', 'ClientCertificate'
 * @member {object} [userName] User name for Basic, Digest, or Windows
 * authentication. Type: string (or Expression with resultType string).
 * @member {object} [password] Password for Basic, Digest, Windows, or
 * ClientCertificate with EmbeddedCertData authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [embeddedCertData] Base64 encoded certificate data for
 * ClientCertificate authentication. For on-premises copy with
 * ClientCertificate authentication, either CertThumbprint or
 * EmbeddedCertData/Password should be specified. Type: string (or Expression
 * with resultType string).
 * @member {object} [certThumbprint] Thumbprint of certificate for
 * ClientCertificate authentication. Only valid for on-premises copy. For
 * on-premises copy with ClientCertificate authentication, either
 * CertThumbprint or EmbeddedCertData/Password should be specified. Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [enableServerCertificateValidation] If true, validate the
 * HTTPS server SSL certificate. Default value is true. Type: boolean (or
 * Expression with resultType boolean).
 */
export interface HttpLinkedService extends LinkedService {
  url: any;
  authenticationType?: string;
  userName?: any;
  password?: SecretBase;
  embeddedCertData?: any;
  certThumbprint?: any;
  encryptedCredential?: any;
  enableServerCertificateValidation?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSearchLinkedService class.
 * @constructor
 * Linked service for Windows Azure Search Service.
 *
 * @member {object} url URL for Azure Search service. Type: string (or
 * Expression with resultType string).
 * @member {object} [key] Admin Key for Azure Search service
 * @member {string} [key.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureSearchLinkedService extends LinkedService {
  url: any;
  key?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the CustomDataSourceLinkedService class.
 * @constructor
 * Custom linked service.
 *
 * @member {object} typeProperties Custom linked service properties.
 */
export interface CustomDataSourceLinkedService extends LinkedService {
  typeProperties: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonRedshiftLinkedService class.
 * @constructor
 * Linked service for Amazon Redshift.
 *
 * @member {object} server The name of the Amazon Redshift server. Type: string
 * (or Expression with resultType string).
 * @member {object} [username] The username of the Amazon Redshift source.
 * Type: string (or Expression with resultType string).
 * @member {object} [password] The password of the Amazon Redshift source.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} database The database name of the Amazon Redshift source.
 * Type: string (or Expression with resultType string).
 * @member {object} [port] The TCP port number that the Amazon Redshift server
 * uses to listen for client connections. The default value is 5439. Type:
 * integer (or Expression with resultType integer).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AmazonRedshiftLinkedService extends LinkedService {
  server: any;
  username?: any;
  password?: SecretBase;
  database: any;
  port?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonS3LinkedService class.
 * @constructor
 * Linked service for Amazon S3.
 *
 * @member {object} [accessKeyId] The access key identifier of the Amazon S3
 * Identity and Access Management (IAM) user. Type: string (or Expression with
 * resultType string).
 * @member {object} [secretAccessKey] The secret access key of the Amazon S3
 * Identity and Access Management (IAM) user.
 * @member {string} [secretAccessKey.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AmazonS3LinkedService extends LinkedService {
  accessKeyId?: any;
  secretAccessKey?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SapEccLinkedService class.
 * @constructor
 * Linked service for SAP ERP Central Component(SAP ECC).
 *
 * @member {string} url The URL of SAP ECC OData API. For example,
 * '[https://hostname:port/sap/opu/odata/sap/servicename/]'. Type: string (or
 * Expression with resultType string).
 * @member {string} [username] The username for Basic authentication. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] The password for Basic authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {string} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Either encryptedCredential or username/password must be
 * provided. Type: string (or Expression with resultType string).
 */
export interface SapEccLinkedService extends LinkedService {
  url: string;
  username?: string;
  password?: SecretBase;
  encryptedCredential?: string;
}

/**
 * @class
 * Initializes a new instance of the SapCloudForCustomerLinkedService class.
 * @constructor
 * Linked service for SAP Cloud for Customer.
 *
 * @member {object} url The URL of SAP Cloud for Customer OData API. For
 * example, '[https://[tenantname].crm.ondemand.com/sap/c4c/odata/v1]'. Type:
 * string (or Expression with resultType string).
 * @member {object} [username] The username for Basic authentication. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] The password for Basic authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Either encryptedCredential or username/password must be
 * provided. Type: string (or Expression with resultType string).
 */
export interface SapCloudForCustomerLinkedService extends LinkedService {
  url: any;
  username?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SalesforceLinkedService class.
 * @constructor
 * Linked service for Salesforce.
 *
 * @member {object} [environmentUrl] The URL of Salesforce instance. Default is
 * 'https://login.salesforce.com'. To copy data from sandbox, specify
 * 'https://test.salesforce.com'. To copy data from custom domain, specify, for
 * example, 'https://[domain].my.salesforce.com'. Type: string (or Expression
 * with resultType string).
 * @member {object} [username] The username for Basic authentication of the
 * Salesforce instance. Type: string (or Expression with resultType string).
 * @member {object} [password] The password for Basic authentication of the
 * Salesforce instance.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [securityToken] The security token is required to remotely
 * access Salesforce instance.
 * @member {string} [securityToken.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SalesforceLinkedService extends LinkedService {
  environmentUrl?: any;
  username?: any;
  password?: SecretBase;
  securityToken?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreLinkedService class.
 * @constructor
 * Azure Data Lake Store linked service.
 *
 * @member {object} dataLakeStoreUri Data Lake Store service URI. Type: string
 * (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The ID of the application used to
 * authenticate against the Azure Data Lake Store account. Type: string (or
 * Expression with resultType string).
 * @member {object} [servicePrincipalKey] The Key of the application used to
 * authenticate against the Azure Data Lake Store account.
 * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
 * @member {object} [tenant] The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [accountName] Data Lake Store account name. Type: string
 * (or Expression with resultType string).
 * @member {object} [subscriptionId] Data Lake Store account subscription ID
 * (if different from Data Factory account). Type: string (or Expression with
 * resultType string).
 * @member {object} [resourceGroupName] Data Lake Store account resource group
 * name (if different from Data Factory account). Type: string (or Expression
 * with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureDataLakeStoreLinkedService extends LinkedService {
  dataLakeStoreUri: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecretBase;
  tenant?: any;
  accountName?: any;
  subscriptionId?: any;
  resourceGroupName?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the MongoDbLinkedService class.
 * @constructor
 * Linked service for MongoDb data source.
 *
 * @member {object} server The IP address or server name of the MongoDB server.
 * Type: string (or Expression with resultType string).
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the MongoDB database. Possible values include: 'Basic',
 * 'Anonymous'
 * @member {object} databaseName The name of the MongoDB database that you want
 * to access. Type: string (or Expression with resultType string).
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [authSource] Database to verify the username and password.
 * Type: string (or Expression with resultType string).
 * @member {object} [port] The TCP port number that the MongoDB server uses to
 * listen for client connections. The default value is 27017. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {object} [enableSsl] Specifies whether the connections to the server
 * are encrypted using SSL. The default value is false. Type: boolean (or
 * Expression with resultType boolean).
 * @member {object} [allowSelfSignedServerCert] Specifies whether to allow
 * self-signed certificates from the server. The default value is false. Type:
 * boolean (or Expression with resultType boolean).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface MongoDbLinkedService extends LinkedService {
  server: any;
  authenticationType?: string;
  databaseName: any;
  username?: any;
  password?: SecretBase;
  authSource?: any;
  port?: any;
  enableSsl?: any;
  allowSelfSignedServerCert?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the CassandraLinkedService class.
 * @constructor
 * Linked service for Cassandra data source.
 *
 * @member {object} host Host name for connection. Type: string (or Expression
 * with resultType string).
 * @member {object} [authenticationType] AuthenticationType to be used for
 * connection. Type: string (or Expression with resultType string).
 * @member {object} [port] The port for the connection. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface CassandraLinkedService extends LinkedService {
  host: any;
  authenticationType?: any;
  port?: any;
  username?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the WebLinkedServiceTypeProperties class.
 * @constructor
 * Base definition of WebLinkedServiceTypeProperties, this typeProperties is
 * polymorphic based on authenticationType, so not flattened in SDK models.
 *
 * @member {object} url The URL of the web service endpoint, e.g.
 * http://www.microsoft.com . Type: string (or Expression with resultType
 * string).
 * @member {string} authenticationType Polymorphic Discriminator
 */
export interface WebLinkedServiceTypeProperties {
  url: any;
  authenticationType: string;
}

/**
 * @class
 * Initializes a new instance of the WebClientCertificateAuthentication class.
 * @constructor
 * A WebLinkedService that uses client certificate based authentication to
 * communicate with an HTTP endpoint. This scheme follows mutual
 * authentication; the server must also provide valid credentials to the
 * client.
 *
 * @member {object} pfx Base64-encoded contents of a PFX file.
 * @member {string} [pfx.type] Polymorphic Discriminator
 * @member {object} password Password for the PFX file.
 * @member {string} [password.type] Polymorphic Discriminator
 */
export interface WebClientCertificateAuthentication extends WebLinkedServiceTypeProperties {
  pfx: SecretBase;
  password: SecretBase;
}

/**
 * @class
 * Initializes a new instance of the WebBasicAuthentication class.
 * @constructor
 * A WebLinkedService that uses basic authentication to communicate with an
 * HTTP endpoint.
 *
 * @member {object} username User name for Basic authentication. Type: string
 * (or Expression with resultType string).
 * @member {object} password The password for Basic authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 */
export interface WebBasicAuthentication extends WebLinkedServiceTypeProperties {
  username: any;
  password: SecretBase;
}

/**
 * @class
 * Initializes a new instance of the WebAnonymousAuthentication class.
 * @constructor
 * A WebLinkedService that uses anonymous authentication to communicate with an
 * HTTP endpoint.
 *
 */
export interface WebAnonymousAuthentication extends WebLinkedServiceTypeProperties {
}

/**
 * @class
 * Initializes a new instance of the WebLinkedService class.
 * @constructor
 * Web linked service.
 *
 * @member {object} typeProperties Web linked service properties.
 * @member {object} [typeProperties.url] The URL of the web service endpoint,
 * e.g. http://www.microsoft.com . Type: string (or Expression with resultType
 * string).
 * @member {string} [typeProperties.authenticationType] Polymorphic
 * Discriminator
 */
export interface WebLinkedService extends LinkedService {
  typeProperties: WebLinkedServiceTypeProperties;
}

/**
 * @class
 * Initializes a new instance of the ODataLinkedService class.
 * @constructor
 * Open Data Protocol (OData) linked service.
 *
 * @member {object} url The URL of the OData service endpoint. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] Type of authentication used to connect
 * to the OData service. Possible values include: 'Basic', 'Anonymous'
 * @member {object} [userName] User name of the OData service. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password of the OData service.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ODataLinkedService extends LinkedService {
  url: any;
  authenticationType?: string;
  userName?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HdfsLinkedService class.
 * @constructor
 * Hadoop Distributed File System (HDFS) linked service.
 *
 * @member {object} url The URL of the HDFS service endpoint, e.g.
 * http://myhostname:50070/webhdfs/v1 . Type: string (or Expression with
 * resultType string).
 * @member {object} [authenticationType] Type of authentication used to connect
 * to the HDFS. Possible values are: Anonymous and Windows. Type: string (or
 * Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [userName] User name for Windows authentication. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] Password for Windows authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 */
export interface HdfsLinkedService extends LinkedService {
  url: any;
  authenticationType?: any;
  encryptedCredential?: any;
  userName?: any;
  password?: SecretBase;
}

/**
 * @class
 * Initializes a new instance of the OdbcLinkedService class.
 * @constructor
 * Open Database Connectivity (ODBC) linked service.
 *
 * @member {object} connectionString The non-access credential portion of the
 * connection string as well as an optional encrypted credential. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [authenticationType] Type of authentication used to connect
 * to the ODBC data store. Possible values are: Anonymous and Basic. Type:
 * string (or Expression with resultType string).
 * @member {object} [credential] The access credential portion of the
 * connection string specified in driver-specific property-value format.
 * @member {string} [credential.type] Polymorphic Discriminator
 * @member {object} [userName] User name for Basic authentication. Type: string
 * (or Expression with resultType string).
 * @member {object} [password] Password for Basic authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface OdbcLinkedService extends LinkedService {
  connectionString: any;
  authenticationType?: any;
  credential?: SecretBase;
  userName?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMLLinkedService class.
 * @constructor
 * Azure ML Web Service linked service.
 *
 * @member {object} mlEndpoint The Batch Execution REST URL for an Azure ML Web
 * Service endpoint. Type: string (or Expression with resultType string).
 * @member {object} apiKey The API key for accessing the Azure ML model
 * endpoint.
 * @member {string} [apiKey.type] Polymorphic Discriminator
 * @member {object} [updateResourceEndpoint] The Update Resource REST URL for
 * an Azure ML Web Service endpoint. Type: string (or Expression with
 * resultType string).
 * @member {object} [servicePrincipalId] The ID of the service principal used
 * to authenticate against the ARM-based updateResourceEndpoint of an Azure ML
 * web service. Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalKey] The key of the service principal used
 * to authenticate against the ARM-based updateResourceEndpoint of an Azure ML
 * web service.
 * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
 * @member {object} [tenant] The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureMLLinkedService extends LinkedService {
  mlEndpoint: any;
  apiKey: SecretBase;
  updateResourceEndpoint?: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecretBase;
  tenant?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the TeradataLinkedService class.
 * @constructor
 * Linked service for Teradata data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] AuthenticationType to be used for
 * connection. Possible values include: 'Basic', 'Windows'
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface TeradataLinkedService extends LinkedService {
  server: any;
  authenticationType?: string;
  username?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the Db2LinkedService class.
 * @constructor
 * Linked service for DB2 data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} database Database name for connection. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] AuthenticationType to be used for
 * connection. Possible values include: 'Basic'
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface Db2LinkedService extends LinkedService {
  server: any;
  database: any;
  authenticationType?: string;
  username?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SybaseLinkedService class.
 * @constructor
 * Linked service for Sybase data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} database Database name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [schema] Schema name for connection. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] AuthenticationType to be used for
 * connection. Possible values include: 'Basic', 'Windows'
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SybaseLinkedService extends LinkedService {
  server: any;
  database: any;
  schema?: any;
  authenticationType?: string;
  username?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the PostgreSqlLinkedService class.
 * @constructor
 * Linked service for PostgreSQL data source.
 *
 * @member {object} connectionString The connection string.
 * @member {string} [connectionString.type] Polymorphic Discriminator
 * @member {object} [password] The Azure key vault secret reference of password
 * in connection string.
 * @member {object} [password.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [password.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [password.store.parameters] Arguments for LinkedService.
 * @member {object} [password.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface PostgreSqlLinkedService extends LinkedService {
  connectionString: SecretBase;
  password?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the MySqlLinkedService class.
 * @constructor
 * Linked service for MySQL data source.
 *
 * @member {object} connectionString The connection string.
 * @member {string} [connectionString.type] Polymorphic Discriminator
 * @member {object} [password] The Azure key vault secret reference of password
 * in connection string.
 * @member {object} [password.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [password.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [password.store.parameters] Arguments for LinkedService.
 * @member {object} [password.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface MySqlLinkedService extends LinkedService {
  connectionString: SecretBase;
  password?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMySqlLinkedService class.
 * @constructor
 * Azure MySQL database linked service.
 *
 * @member {object} connectionString The connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [password] The Azure key vault secret reference of password
 * in connection string.
 * @member {object} [password.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [password.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [password.store.parameters] Arguments for LinkedService.
 * @member {object} [password.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureMySqlLinkedService extends LinkedService {
  connectionString: any;
  password?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the OracleLinkedService class.
 * @constructor
 * Oracle database.
 *
 * @member {object} connectionString The connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [password] The Azure key vault secret reference of password
 * in connection string.
 * @member {object} [password.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [password.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [password.store.parameters] Arguments for LinkedService.
 * @member {object} [password.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface OracleLinkedService extends LinkedService {
  connectionString: any;
  password?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the FileServerLinkedService class.
 * @constructor
 * File system linked service.
 *
 * @member {object} host Host name of the server. Type: string (or Expression
 * with resultType string).
 * @member {object} [userId] User ID to logon the server. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password to logon the server.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface FileServerLinkedService extends LinkedService {
  host: any;
  userId?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HDInsightLinkedService class.
 * @constructor
 * HDInsight linked service.
 *
 * @member {object} clusterUri HDInsight cluster URI. Type: string (or
 * Expression with resultType string).
 * @member {object} [userName] HDInsight cluster user name. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] HDInsight cluster password.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [linkedServiceName] The Azure Storage linked service
 * reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [linkedServiceName.parameters] Arguments for LinkedService.
 * @member {object} [hcatalogLinkedServiceName] A reference to the Azure SQL
 * linked service that points to the HCatalog database.
 * @member {string} [hcatalogLinkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [hcatalogLinkedServiceName.parameters] Arguments for
 * LinkedService.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface HDInsightLinkedService extends LinkedService {
  clusterUri: any;
  userName?: any;
  password?: SecretBase;
  linkedServiceName?: LinkedServiceReference;
  hcatalogLinkedServiceName?: LinkedServiceReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the DynamicsLinkedService class.
 * @constructor
 * Dynamics linked service.
 *
 * @member {object} deploymentType The deployment type of the Dynamics
 * instance. 'Online' for Dynamics Online and 'OnPremisesWithIfd' for Dynamics
 * on-premises with Ifd. Type: string (or Expression with resultType string).
 * @member {object} [hostName] The host name of the on-premises Dynamics
 * server. The property is required for on-prem and not allowed for online.
 * Type: string (or Expression with resultType string).
 * @member {object} [port] The port of on-premises Dynamics server. The
 * property is required for on-prem and not allowed for online. Default is 443.
 * Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {object} [serviceUri] The URL to the Microsoft Dynamics server. The
 * property is required for on-line and not allowed for on-prem. Type: string
 * (or Expression with resultType string).
 * @member {object} [organizationName] The organization name of the Dynamics
 * instance. The property is required for on-prem and required for online when
 * there are more than one Dynamics instances associated with the user. Type:
 * string (or Expression with resultType string).
 * @member {object} authenticationType The authentication type to connect to
 * Dynamics server. 'Office365' for online scenario, 'Ifd' for on-premises with
 * Ifd scenario. Type: string (or Expression with resultType string).
 * @member {object} username User name to access the Dynamics instance. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] Password to access the Dynamics instance.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface DynamicsLinkedService extends LinkedService {
  deploymentType: any;
  hostName?: any;
  port?: any;
  serviceUri?: any;
  organizationName?: any;
  authenticationType: any;
  username: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the CosmosDbLinkedService class.
 * @constructor
 * Microsoft Azure Cosmos Database (CosmosDB) linked service.
 *
 * @member {object} connectionString The connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [accountKey] The Azure key vault secret reference of
 * accountKey in connection string.
 * @member {object} [accountKey.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [accountKey.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [accountKey.store.parameters] Arguments for LinkedService.
 * @member {object} [accountKey.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [accountKey.secretVersion] The version of the secret in
 * Azure Key Vault. The default value is the latest version of the secret.
 * Type: string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface CosmosDbLinkedService extends LinkedService {
  connectionString: any;
  accountKey?: AzureKeyVaultSecretReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureKeyVaultLinkedService class.
 * @constructor
 * Azure Key Vault linked service.
 *
 * @member {object} baseUrl The base URL of the Azure Key Vault. e.g.
 * https://myakv.vault.azure.net Type: string (or Expression with resultType
 * string).
 */
export interface AzureKeyVaultLinkedService extends LinkedService {
  baseUrl: any;
}

/**
 * @class
 * Initializes a new instance of the AzureBatchLinkedService class.
 * @constructor
 * Azure Batch linked service.
 *
 * @member {object} accountName The Azure Batch account name. Type: string (or
 * Expression with resultType string).
 * @member {object} [accessKey] The Azure Batch account access key.
 * @member {string} [accessKey.type] Polymorphic Discriminator
 * @member {object} batchUri The Azure Batch URI. Type: string (or Expression
 * with resultType string).
 * @member {object} poolName The Azure Batch pool name. Type: string (or
 * Expression with resultType string).
 * @member {object} linkedServiceName The Azure Storage linked service
 * reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [linkedServiceName.parameters] Arguments for LinkedService.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureBatchLinkedService extends LinkedService {
  accountName: any;
  accessKey?: SecretBase;
  batchUri: any;
  poolName: any;
  linkedServiceName: LinkedServiceReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlDatabaseLinkedService class.
 * @constructor
 * Microsoft Azure SQL Database linked service.
 *
 * @member {object} connectionString The connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [password] The Azure key vault secret reference of password
 * in connection string.
 * @member {object} [password.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [password.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [password.store.parameters] Arguments for LinkedService.
 * @member {object} [password.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The ID of the service principal used
 * to authenticate against Azure SQL Database. Type: string (or Expression with
 * resultType string).
 * @member {object} [servicePrincipalKey] The key of the service principal used
 * to authenticate against Azure SQL Database.
 * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
 * @member {object} [tenant] The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureSqlDatabaseLinkedService extends LinkedService {
  connectionString: any;
  password?: AzureKeyVaultSecretReference;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecretBase;
  tenant?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlServerLinkedService class.
 * @constructor
 * SQL Server linked service.
 *
 * @member {object} connectionString The connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [userName] The on-premises Windows authentication user
 * name. Type: string (or Expression with resultType string).
 * @member {object} [password] The on-premises Windows authentication password.
 * @member {string} [password.type] Polymorphic Discriminator
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SqlServerLinkedService extends LinkedService {
  connectionString: any;
  userName?: any;
  password?: SecretBase;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlDWLinkedService class.
 * @constructor
 * Azure SQL Data Warehouse linked service.
 *
 * @member {object} connectionString The connection string. Type: string,
 * SecureString or AzureKeyVaultSecretReference. Type: string, SecureString or
 * AzureKeyVaultSecretReference.
 * @member {object} [password] The Azure key vault secret reference of password
 * in connection string.
 * @member {object} [password.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [password.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [password.store.parameters] Arguments for LinkedService.
 * @member {object} [password.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The ID of the service principal used
 * to authenticate against Azure SQL Data Warehouse. Type: string (or
 * Expression with resultType string).
 * @member {object} [servicePrincipalKey] The key of the service principal used
 * to authenticate against Azure SQL Data Warehouse.
 * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
 * @member {object} [tenant] The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureSqlDWLinkedService extends LinkedService {
  connectionString: any;
  password?: AzureKeyVaultSecretReference;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecretBase;
  tenant?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureTableStorageLinkedService class.
 * @constructor
 * The azure table storage linked service.
 *
 * @member {object} [connectionString] The connection string. It is mutually
 * exclusive with sasUri property. Type: string, SecureString or
 * AzureKeyVaultSecretReference.
 * @member {object} [accountKey] The Azure key vault secret reference of
 * accountKey in connection string.
 * @member {object} [accountKey.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [accountKey.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [accountKey.store.parameters] Arguments for LinkedService.
 * @member {object} [accountKey.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [accountKey.secretVersion] The version of the secret in
 * Azure Key Vault. The default value is the latest version of the secret.
 * Type: string (or Expression with resultType string).
 * @member {object} [sasUri] SAS URI of the Azure Storage resource. It is
 * mutually exclusive with connectionString property. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [sasToken] The Azure key vault secret reference of sasToken
 * in sas uri.
 * @member {object} [sasToken.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [sasToken.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [sasToken.store.parameters] Arguments for LinkedService.
 * @member {object} [sasToken.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [sasToken.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {string} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureTableStorageLinkedService extends LinkedService {
  connectionString?: any;
  accountKey?: AzureKeyVaultSecretReference;
  sasUri?: any;
  sasToken?: AzureKeyVaultSecretReference;
  encryptedCredential?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureBlobStorageLinkedService class.
 * @constructor
 * The azure blob storage linked service.
 *
 * @member {object} [connectionString] The connection string. It is mutually
 * exclusive with sasUri, serviceEndpoint property. Type: string, SecureString
 * or AzureKeyVaultSecretReference.
 * @member {object} [accountKey] The Azure key vault secret reference of
 * accountKey in connection string.
 * @member {object} [accountKey.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [accountKey.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [accountKey.store.parameters] Arguments for LinkedService.
 * @member {object} [accountKey.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [accountKey.secretVersion] The version of the secret in
 * Azure Key Vault. The default value is the latest version of the secret.
 * Type: string (or Expression with resultType string).
 * @member {object} [sasUri] SAS URI of the Azure Blob Storage resource. It is
 * mutually exclusive with connectionString, serviceEndpoint property. Type:
 * string, SecureString or AzureKeyVaultSecretReference.
 * @member {object} [sasToken] The Azure key vault secret reference of sasToken
 * in sas uri.
 * @member {object} [sasToken.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [sasToken.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [sasToken.store.parameters] Arguments for LinkedService.
 * @member {object} [sasToken.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [sasToken.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {string} [serviceEndpoint] Blob service endpoint of the Azure Blob
 * Storage resource. It is mutually exclusive with connectionString, sasUri
 * property.
 * @member {object} [servicePrincipalId] The ID of the service principal used
 * to authenticate against Azure SQL Data Warehouse. Type: string (or
 * Expression with resultType string).
 * @member {object} [servicePrincipalKey] The key of the service principal used
 * to authenticate against Azure SQL Data Warehouse.
 * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
 * @member {object} [tenant] The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {string} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureBlobStorageLinkedService extends LinkedService {
  connectionString?: any;
  accountKey?: AzureKeyVaultSecretReference;
  sasUri?: any;
  sasToken?: AzureKeyVaultSecretReference;
  serviceEndpoint?: string;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecretBase;
  tenant?: any;
  encryptedCredential?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureStorageLinkedService class.
 * @constructor
 * The storage account linked service.
 *
 * @member {object} [connectionString] The connection string. It is mutually
 * exclusive with sasUri property. Type: string, SecureString or
 * AzureKeyVaultSecretReference.
 * @member {object} [accountKey] The Azure key vault secret reference of
 * accountKey in connection string.
 * @member {object} [accountKey.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [accountKey.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [accountKey.store.parameters] Arguments for LinkedService.
 * @member {object} [accountKey.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [accountKey.secretVersion] The version of the secret in
 * Azure Key Vault. The default value is the latest version of the secret.
 * Type: string (or Expression with resultType string).
 * @member {object} [sasUri] SAS URI of the Azure Storage resource. It is
 * mutually exclusive with connectionString property. Type: string,
 * SecureString or AzureKeyVaultSecretReference.
 * @member {object} [sasToken] The Azure key vault secret reference of sasToken
 * in sas uri.
 * @member {object} [sasToken.store] The Azure Key Vault linked service
 * reference.
 * @member {string} [sasToken.store.referenceName] Reference LinkedService
 * name.
 * @member {object} [sasToken.store.parameters] Arguments for LinkedService.
 * @member {object} [sasToken.secretName] The name of the secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [sasToken.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 * @member {string} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureStorageLinkedService extends LinkedService {
  connectionString?: any;
  accountKey?: AzureKeyVaultSecretReference;
  sasUri?: any;
  sasToken?: AzureKeyVaultSecretReference;
  encryptedCredential?: string;
}

/**
 * @class
 * Initializes a new instance of the ResponsysObjectDataset class.
 * @constructor
 * Responsys dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface ResponsysObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the SalesforceMarketingCloudObjectDataset class.
 * @constructor
 * Salesforce Marketing Cloud dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface SalesforceMarketingCloudObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the VerticaTableDataset class.
 * @constructor
 * Vertica dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface VerticaTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the NetezzaTableDataset class.
 * @constructor
 * Netezza dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface NetezzaTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the ZohoObjectDataset class.
 * @constructor
 * Zoho server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface ZohoObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the XeroObjectDataset class.
 * @constructor
 * Xero Serivce dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface XeroObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the SquareObjectDataset class.
 * @constructor
 * Square Serivce dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface SquareObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the SparkObjectDataset class.
 * @constructor
 * Spark Server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface SparkObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the ShopifyObjectDataset class.
 * @constructor
 * Shopify Serivce dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface ShopifyObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the ServiceNowObjectDataset class.
 * @constructor
 * ServiceNow server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface ServiceNowObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the QuickBooksObjectDataset class.
 * @constructor
 * QuickBooks server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface QuickBooksObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the PrestoObjectDataset class.
 * @constructor
 * Presto server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface PrestoObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the PhoenixObjectDataset class.
 * @constructor
 * Phoenix server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface PhoenixObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the PaypalObjectDataset class.
 * @constructor
 * Paypal Serivce dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface PaypalObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the MarketoObjectDataset class.
 * @constructor
 * Marketo server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface MarketoObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the MariaDBTableDataset class.
 * @constructor
 * MariaDB server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface MariaDBTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the MagentoObjectDataset class.
 * @constructor
 * Magento server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface MagentoObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the JiraObjectDataset class.
 * @constructor
 * Jira Serivce dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface JiraObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the ImpalaObjectDataset class.
 * @constructor
 * Impala server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface ImpalaObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the HubspotObjectDataset class.
 * @constructor
 * Hubspot Serivce dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface HubspotObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the HiveObjectDataset class.
 * @constructor
 * Hive Server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface HiveObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the HBaseObjectDataset class.
 * @constructor
 * HBase server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface HBaseObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the GreenplumTableDataset class.
 * @constructor
 * Greenplum Database dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface GreenplumTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the GoogleBigQueryObjectDataset class.
 * @constructor
 * Google BigQuery service dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface GoogleBigQueryObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the EloquaObjectDataset class.
 * @constructor
 * Eloqua server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface EloquaObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the DrillTableDataset class.
 * @constructor
 * Drill server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface DrillTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the CouchbaseTableDataset class.
 * @constructor
 * Couchbase server dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface CouchbaseTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the ConcurObjectDataset class.
 * @constructor
 * Concur Serivce dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface ConcurObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the AzurePostgreSqlTableDataset class.
 * @constructor
 * Azure PostgreSQL dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface AzurePostgreSqlTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonMWSObjectDataset class.
 * @constructor
 * Amazon Marketplace Web Service dataset.
 *
 * @member {object} [tableName] The table name. Type: string (or Expression
 * with resultType string).
 */
export interface AmazonMWSObjectDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the DatasetCompression class.
 * @constructor
 * The compression method used on a dataset.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface DatasetCompression {
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the DatasetZipDeflateCompression class.
 * @constructor
 * The ZipDeflate compression method used on a dataset.
 *
 * @member {string} [level] The ZipDeflate compression level. Possible values
 * include: 'Optimal', 'Fastest'
 */
export interface DatasetZipDeflateCompression extends DatasetCompression {
  level?: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetDeflateCompression class.
 * @constructor
 * The Deflate compression method used on a dataset.
 *
 * @member {string} [level] The Deflate compression level. Possible values
 * include: 'Optimal', 'Fastest'
 */
export interface DatasetDeflateCompression extends DatasetCompression {
  level?: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetGZipCompression class.
 * @constructor
 * The GZip compression method used on a dataset.
 *
 * @member {string} [level] The GZip compression level. Possible values
 * include: 'Optimal', 'Fastest'
 */
export interface DatasetGZipCompression extends DatasetCompression {
  level?: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetBZip2Compression class.
 * @constructor
 * The BZip2 compression method used on a dataset.
 *
 */
export interface DatasetBZip2Compression extends DatasetCompression {
}

/**
 * @class
 * Initializes a new instance of the DatasetStorageFormat class.
 * @constructor
 * The format definition of a storage.
 *
 * @member {object} [serializer] Serializer. Type: string (or Expression with
 * resultType string).
 * @member {object} [deserializer] Deserializer. Type: string (or Expression
 * with resultType string).
 * @member {string} type Polymorphic Discriminator
 */
export interface DatasetStorageFormat {
  serializer?: any;
  deserializer?: any;
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ParquetFormat class.
 * @constructor
 * The data stored in Parquet format.
 *
 */
export interface ParquetFormat extends DatasetStorageFormat {
}

/**
 * @class
 * Initializes a new instance of the OrcFormat class.
 * @constructor
 * The data stored in Optimized Row Columnar (ORC) format.
 *
 */
export interface OrcFormat extends DatasetStorageFormat {
}

/**
 * @class
 * Initializes a new instance of the AvroFormat class.
 * @constructor
 * The data stored in Avro format.
 *
 */
export interface AvroFormat extends DatasetStorageFormat {
}

/**
 * @class
 * Initializes a new instance of the JsonFormat class.
 * @constructor
 * The data stored in JSON format.
 *
 * @member {string} [filePattern] File pattern of JSON. To be more specific,
 * the way of separating a collection of JSON objects. The default value is
 * 'setOfObjects'. It is case-sensitive. Possible values include:
 * 'setOfObjects', 'arrayOfObjects'
 * @member {object} [nestingSeparator] The character used to separate nesting
 * levels. Default value is '.' (dot). Type: string (or Expression with
 * resultType string).
 * @member {object} [encodingName] The code page name of the preferred
 * encoding. If not provided, the default value is 'utf-8', unless the byte
 * order mark (BOM) denotes another Unicode encoding. The full list of
 * supported values can be found in the 'Name' column of the table of encodings
 * in the following reference: https://go.microsoft.com/fwlink/?linkid=861078.
 * Type: string (or Expression with resultType string).
 * @member {object} [jsonNodeReference] The JSONPath of the JSON array element
 * to be flattened. Example: "$.ArrayPath". Type: string (or Expression with
 * resultType string).
 * @member {object} [jsonPathDefinition] The JSONPath definition for each
 * column mapping with a customized column name to extract data from JSON file.
 * For fields under root object, start with "$"; for fields inside the array
 * chosen by jsonNodeReference property, start from the array element. Example:
 * {"Column1": "$.Column1Path", "Column2": "Column2PathInArray"}. Type: object
 * (or Expression with resultType object).
 */
export interface JsonFormat extends DatasetStorageFormat {
  filePattern?: string;
  nestingSeparator?: any;
  encodingName?: any;
  jsonNodeReference?: any;
  jsonPathDefinition?: any;
}

/**
 * @class
 * Initializes a new instance of the TextFormat class.
 * @constructor
 * The data stored in text format.
 *
 * @member {object} [columnDelimiter] The column delimiter. Type: string (or
 * Expression with resultType string).
 * @member {object} [rowDelimiter] The row delimiter. Type: string (or
 * Expression with resultType string).
 * @member {object} [escapeChar] The escape character. Type: string (or
 * Expression with resultType string).
 * @member {object} [quoteChar] The quote character. Type: string (or
 * Expression with resultType string).
 * @member {object} [nullValue] The null value string. Type: string (or
 * Expression with resultType string).
 * @member {object} [encodingName] The code page name of the preferred
 * encoding. If miss, the default value is ΓÇ£utf-8ΓÇ¥, unless BOM denotes
 * another Unicode encoding. Refer to the ΓÇ£NameΓÇ¥ column of the table in the
 * following link to set supported values:
 * https://msdn.microsoft.com/library/system.text.encoding.aspx. Type: string
 * (or Expression with resultType string).
 * @member {object} [treatEmptyAsNull] Treat empty column values in the text
 * file as null. The default value is true. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [skipLineCount] The number of lines/rows to be skipped when
 * parsing text files. The default value is 0. Type: integer (or Expression
 * with resultType integer).
 * @member {object} [firstRowAsHeader] When used as input, treat the first row
 * of data as headers. When used as output,write the headers into the output as
 * the first row of data. The default value is false. Type: boolean (or
 * Expression with resultType boolean).
 */
export interface TextFormat extends DatasetStorageFormat {
  columnDelimiter?: any;
  rowDelimiter?: any;
  escapeChar?: any;
  quoteChar?: any;
  nullValue?: any;
  encodingName?: any;
  treatEmptyAsNull?: any;
  skipLineCount?: any;
  firstRowAsHeader?: any;
}

/**
 * @class
 * Initializes a new instance of the HttpDataset class.
 * @constructor
 * A file in an HTTP web server.
 *
 * @member {object} [relativeUrl] The relative URL based on the URL in the
 * HttpLinkedService refers to an HTTP file Type: string (or Expression with
 * resultType string).
 * @member {object} [requestMethod] The HTTP method for the HTTP request. Type:
 * string (or Expression with resultType string).
 * @member {object} [requestBody] The body for the HTTP request. Type: string
 * (or Expression with resultType string).
 * @member {object} [additionalHeaders] The headers for the HTTP Request. e.g.
 * request-header-name-1:request-header-value-1
 * ...
 * request-header-name-n:request-header-value-n Type: string (or Expression
 * with resultType string).
 * @member {object} [format] The format of files.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used on files.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface HttpDataset extends Dataset {
  relativeUrl?: any;
  requestMethod?: any;
  requestBody?: any;
  additionalHeaders?: any;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the AzureSearchIndexDataset class.
 * @constructor
 * The Azure Search Index.
 *
 * @member {object} indexName The name of the Azure Search Index. Type: string
 * (or Expression with resultType string).
 */
export interface AzureSearchIndexDataset extends Dataset {
  indexName: any;
}

/**
 * @class
 * Initializes a new instance of the WebTableDataset class.
 * @constructor
 * The dataset points to a HTML table in the web page.
 *
 * @member {object} index The zero-based index of the table in the web page.
 * Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {object} [path] The relative URL to the web page from the linked
 * service URL. Type: string (or Expression with resultType string).
 */
export interface WebTableDataset extends Dataset {
  index: any;
  path?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlServerTableDataset class.
 * @constructor
 * The on-premises SQL Server dataset.
 *
 * @member {object} tableName The table name of the SQL Server dataset. Type:
 * string (or Expression with resultType string).
 */
export interface SqlServerTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the SapEccResourceDataset class.
 * @constructor
 * The path of the SAP ECC OData entity.
 *
 * @member {string} path The path of the SAP ECC OData entity. Type: string (or
 * Expression with resultType string).
 */
export interface SapEccResourceDataset extends Dataset {
  path: string;
}

/**
 * @class
 * Initializes a new instance of the SapCloudForCustomerResourceDataset class.
 * @constructor
 * The path of the SAP Cloud for Customer OData entity.
 *
 * @member {object} path The path of the SAP Cloud for Customer OData entity.
 * Type: string (or Expression with resultType string).
 */
export interface SapCloudForCustomerResourceDataset extends Dataset {
  path: any;
}

/**
 * @class
 * Initializes a new instance of the SalesforceObjectDataset class.
 * @constructor
 * The Salesforce object dataset.
 *
 * @member {object} [objectApiName] The Salesforce object API name. Type:
 * string (or Expression with resultType string).
 */
export interface SalesforceObjectDataset extends Dataset {
  objectApiName?: any;
}

/**
 * @class
 * Initializes a new instance of the RelationalTableDataset class.
 * @constructor
 * The relational table dataset.
 *
 * @member {object} [tableName] The relational table name. Type: string (or
 * Expression with resultType string).
 */
export interface RelationalTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMySqlTableDataset class.
 * @constructor
 * The Azure MySQL database dataset.
 *
 * @member {object} [tableName] The Azure MySQL database table name. Type:
 * string (or Expression with resultType string).
 */
export interface AzureMySqlTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the OracleTableDataset class.
 * @constructor
 * The on-premises Oracle database dataset.
 *
 * @member {object} tableName The table name of the on-premises Oracle
 * database. Type: string (or Expression with resultType string).
 */
export interface OracleTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the ODataResourceDataset class.
 * @constructor
 * The Open Data Protocol (OData) resource dataset.
 *
 * @member {object} [path] The OData resource path. Type: string (or Expression
 * with resultType string).
 */
export interface ODataResourceDataset extends Dataset {
  path?: any;
}

/**
 * @class
 * Initializes a new instance of the MongoDbCollectionDataset class.
 * @constructor
 * The MongoDB database dataset.
 *
 * @member {object} collectionName The table name of the MongoDB database.
 * Type: string (or Expression with resultType string).
 */
export interface MongoDbCollectionDataset extends Dataset {
  collectionName: any;
}

/**
 * @class
 * Initializes a new instance of the FileShareDataset class.
 * @constructor
 * An on-premises file system dataset.
 *
 * @member {object} [folderPath] The path of the on-premises file system. Type:
 * string (or Expression with resultType string).
 * @member {object} [fileName] The name of the on-premises file system. Type:
 * string (or Expression with resultType string).
 * @member {object} [format] The format of the files.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [fileFilter] Specify a filter to be used to select a subset
 * of files in the folderPath rather than all files. Type: string (or
 * Expression with resultType string).
 * @member {object} [compression] The data compression method used for the file
 * system.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface FileShareDataset extends Dataset {
  folderPath?: any;
  fileName?: any;
  format?: DatasetStorageFormat;
  fileFilter?: any;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreDataset class.
 * @constructor
 * Azure Data Lake Store dataset.
 *
 * @member {object} folderPath Path to the folder in the Azure Data Lake Store.
 * Type: string (or Expression with resultType string).
 * @member {object} [fileName] The name of the file in the Azure Data Lake
 * Store. Type: string (or Expression with resultType string).
 * @member {object} [format] The format of the Data Lake Store.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used for the
 * item(s) in the Azure Data Lake Store.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface AzureDataLakeStoreDataset extends Dataset {
  folderPath: any;
  fileName?: any;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the DynamicsEntityDataset class.
 * @constructor
 * The Dynamics entity dataset.
 *
 * @member {object} [entityName] The logical name of the entity. Type: string
 * (or Expression with resultType string).
 */
export interface DynamicsEntityDataset extends Dataset {
  entityName?: any;
}

/**
 * @class
 * Initializes a new instance of the DocumentDbCollectionDataset class.
 * @constructor
 * Microsoft Azure Document Database Collection dataset.
 *
 * @member {object} collectionName Document Database collection name. Type:
 * string (or Expression with resultType string).
 */
export interface DocumentDbCollectionDataset extends Dataset {
  collectionName: any;
}

/**
 * @class
 * Initializes a new instance of the CustomDataset class.
 * @constructor
 * The custom dataset.
 *
 * @member {object} typeProperties Custom dataset properties.
 */
export interface CustomDataset extends Dataset {
  typeProperties: any;
}

/**
 * @class
 * Initializes a new instance of the CassandraTableDataset class.
 * @constructor
 * The Cassandra database dataset.
 *
 * @member {object} [tableName] The table name of the Cassandra database. Type:
 * string (or Expression with resultType string).
 * @member {object} [keyspace] The keyspace of the Cassandra database. Type:
 * string (or Expression with resultType string).
 */
export interface CassandraTableDataset extends Dataset {
  tableName?: any;
  keyspace?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlDWTableDataset class.
 * @constructor
 * The Azure SQL Data Warehouse dataset.
 *
 * @member {object} tableName The table name of the Azure SQL Data Warehouse.
 * Type: string (or Expression with resultType string).
 */
export interface AzureSqlDWTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlTableDataset class.
 * @constructor
 * The Azure SQL Server database dataset.
 *
 * @member {object} tableName The table name of the Azure SQL database. Type:
 * string (or Expression with resultType string).
 */
export interface AzureSqlTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the AzureTableDataset class.
 * @constructor
 * The Azure Table storage dataset.
 *
 * @member {object} tableName The table name of the Azure Table storage. Type:
 * string (or Expression with resultType string).
 */
export interface AzureTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the AzureBlobDataset class.
 * @constructor
 * The Azure Blob storage.
 *
 * @member {object} [folderPath] The path of the Azure Blob storage. Type:
 * string (or Expression with resultType string).
 * @member {object} [tableRootLocation] The root of blob path. Type: string (or
 * Expression with resultType string).
 * @member {object} [fileName] The name of the Azure Blob. Type: string (or
 * Expression with resultType string).
 * @member {object} [format] The format of the Azure Blob storage.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used for the blob
 * storage.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface AzureBlobDataset extends Dataset {
  folderPath?: any;
  tableRootLocation?: any;
  fileName?: any;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the AmazonS3Dataset class.
 * @constructor
 * A single Amazon Simple Storage Service (S3) object or a set of S3 objects.
 *
 * @member {object} bucketName The name of the Amazon S3 bucket. Type: string
 * (or Expression with resultType string).
 * @member {object} [key] The key of the Amazon S3 object. Type: string (or
 * Expression with resultType string).
 * @member {object} [prefix] The prefix filter for the S3 object name. Type:
 * string (or Expression with resultType string).
 * @member {object} [version] The version for the S3 object. Type: string (or
 * Expression with resultType string).
 * @member {object} [format] The format of files.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used for the
 * Amazon S3 object.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface AmazonS3Dataset extends Dataset {
  bucketName: any;
  key?: any;
  prefix?: any;
  version?: any;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the ActivityPolicy class.
 * @constructor
 * Execution policy for an activity.
 *
 * @member {object} [timeout] Specifies the timeout for the activity to run.
 * The default timeout is 7 days. Type: string (or Expression with resultType
 * string), pattern: ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [retry] Maximum ordinary retry attempts. Default is 0.
 * Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {number} [retryIntervalInSeconds] Interval between each retry
 * attempt (in seconds). The default is 30 sec.
 * @member {boolean} [secureInput] When set to true, Input from activity is
 * considered as secure and will not be logged to monitoring.
 * @member {boolean} [secureOutput] When set to true, Output from activity is
 * considered as secure and will not be logged to monitoring.
 */
export interface ActivityPolicy {
  timeout?: any;
  retry?: any;
  retryIntervalInSeconds?: number;
  secureInput?: boolean;
  secureOutput?: boolean;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ExecutionActivity class.
 * @constructor
 * Base class for all execution activities.
 *
 * @member {object} [linkedServiceName] Linked service reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [linkedServiceName.parameters] Arguments for LinkedService.
 * @member {object} [policy] Activity policy.
 * @member {object} [policy.timeout] Specifies the timeout for the activity to
 * run. The default timeout is 7 days. Type: string (or Expression with
 * resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [policy.retry] Maximum ordinary retry attempts. Default is
 * 0. Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {number} [policy.retryIntervalInSeconds] Interval between each retry
 * attempt (in seconds). The default is 30 sec.
 * @member {boolean} [policy.secureInput] When set to true, Input from activity
 * is considered as secure and will not be logged to monitoring.
 * @member {boolean} [policy.secureOutput] When set to true, Output from
 * activity is considered as secure and will not be logged to monitoring.
 */
export interface ExecutionActivity extends Activity {
  linkedServiceName?: LinkedServiceReference;
  policy?: ActivityPolicy;
}

/**
 * @class
 * Initializes a new instance of the DatabricksSparkPythonActivity class.
 * @constructor
 * DatabricksSparkPython activity.
 *
 * @member {object} pythonFile The URI of the Python file to be executed. DBFS
 * paths are supported. Type: string (or Expression with resultType string).
 * @member {array} [parameters] Command line parameters that will be passed to
 * the Python file.
 * @member {array} [libraries] A list of libraries to be installed on the
 * cluster that will execute the job.
 */
export interface DatabricksSparkPythonActivity extends ExecutionActivity {
  pythonFile: any;
  parameters?: any[];
  libraries?: { [propertyName: string]: any }[];
}

/**
 * @class
 * Initializes a new instance of the DatabricksSparkJarActivity class.
 * @constructor
 * DatabricksSparkJar activity.
 *
 * @member {object} mainClassName The full name of the class containing the
 * main method to be executed. This class must be contained in a JAR provided
 * as a library. Type: string (or Expression with resultType string).
 * @member {array} [parameters] Parameters that will be passed to the main
 * method.
 * @member {array} [libraries] A list of libraries to be installed on the
 * cluster that will execute the job.
 */
export interface DatabricksSparkJarActivity extends ExecutionActivity {
  mainClassName: any;
  parameters?: any[];
  libraries?: { [propertyName: string]: any }[];
}

/**
 * @class
 * Initializes a new instance of the DatabricksNotebookActivity class.
 * @constructor
 * DatabricksNotebook activity.
 *
 * @member {object} notebookPath The absolute path of the notebook to be run in
 * the Databricks Workspace. This path must begin with a slash. Type: string
 * (or Expression with resultType string).
 * @member {object} [baseParameters] Base parameters to be used for each run of
 * this job.If the notebook takes a parameter that is not specified, the
 * default value from the notebook will be used.
 * @member {array} [libraries] A list of libraries to be installed on the
 * cluster that will execute the job.
 */
export interface DatabricksNotebookActivity extends ExecutionActivity {
  notebookPath: any;
  baseParameters?: { [propertyName: string]: any };
  libraries?: { [propertyName: string]: any }[];
}

/**
 * @class
 * Initializes a new instance of the DataLakeAnalyticsUSQLActivity class.
 * @constructor
 * Data Lake Analytics U-SQL activity.
 *
 * @member {object} scriptPath Case-sensitive path to folder that contains the
 * U-SQL script. Type: string (or Expression with resultType string).
 * @member {object} scriptLinkedService Script linked service reference.
 * @member {string} [scriptLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [scriptLinkedService.parameters] Arguments for
 * LinkedService.
 * @member {object} [degreeOfParallelism] The maximum number of nodes
 * simultaneously used to run the job. Default value is 1. Type: integer (or
 * Expression with resultType integer), minimum: 1.
 * @member {object} [priority] Determines which jobs out of all that are queued
 * should be selected to run first. The lower the number, the higher the
 * priority. Default value is 1000. Type: integer (or Expression with
 * resultType integer), minimum: 1.
 * @member {object} [parameters] Parameters for U-SQL job request.
 * @member {object} [runtimeVersion] Runtime version of the U-SQL engine to
 * use. Type: string (or Expression with resultType string).
 * @member {object} [compilationMode] Compilation mode of U-SQL. Must be one of
 * these values : Semantic, Full and SingleBox. Type: string (or Expression
 * with resultType string).
 */
export interface DataLakeAnalyticsUSQLActivity extends ExecutionActivity {
  scriptPath: any;
  scriptLinkedService: LinkedServiceReference;
  degreeOfParallelism?: any;
  priority?: any;
  parameters?: { [propertyName: string]: any };
  runtimeVersion?: any;
  compilationMode?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMLUpdateResourceActivity class.
 * @constructor
 * Azure ML Update Resource management activity.
 *
 * @member {object} trainedModelName Name of the Trained Model module in the
 * Web Service experiment to be updated. Type: string (or Expression with
 * resultType string).
 * @member {object} trainedModelLinkedServiceName Name of Azure Storage linked
 * service holding the .ilearner file that will be uploaded by the update
 * operation.
 * @member {string} [trainedModelLinkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [trainedModelLinkedServiceName.parameters] Arguments for
 * LinkedService.
 * @member {object} trainedModelFilePath The relative file path in
 * trainedModelLinkedService to represent the .ilearner file that will be
 * uploaded by the update operation.  Type: string (or Expression with
 * resultType string).
 */
export interface AzureMLUpdateResourceActivity extends ExecutionActivity {
  trainedModelName: any;
  trainedModelLinkedServiceName: LinkedServiceReference;
  trainedModelFilePath: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMLWebServiceFile class.
 * @constructor
 * Azure ML WebService Input/Output file
 *
 * @member {object} filePath The relative file path, including container name,
 * in the Azure Blob Storage specified by the LinkedService. Type: string (or
 * Expression with resultType string).
 * @member {object} linkedServiceName Reference to an Azure Storage
 * LinkedService, where Azure ML WebService Input/Output file located.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [linkedServiceName.parameters] Arguments for LinkedService.
 */
export interface AzureMLWebServiceFile {
  filePath: any;
  linkedServiceName: LinkedServiceReference;
}

/**
 * @class
 * Initializes a new instance of the AzureMLBatchExecutionActivity class.
 * @constructor
 * Azure ML Batch Execution activity.
 *
 * @member {object} [globalParameters] Key,Value pairs to be passed to the
 * Azure ML Batch Execution Service endpoint. Keys must match the names of web
 * service parameters defined in the published Azure ML web service. Values
 * will be passed in the GlobalParameters property of the Azure ML batch
 * execution request.
 * @member {object} [webServiceOutputs] Key,Value pairs, mapping the names of
 * Azure ML endpoint's Web Service Outputs to AzureMLWebServiceFile objects
 * specifying the output Blob locations. This information will be passed in the
 * WebServiceOutputs property of the Azure ML batch execution request.
 * @member {object} [webServiceInputs] Key,Value pairs, mapping the names of
 * Azure ML endpoint's Web Service Inputs to AzureMLWebServiceFile objects
 * specifying the input Blob locations.. This information will be passed in the
 * WebServiceInputs property of the Azure ML batch execution request.
 */
export interface AzureMLBatchExecutionActivity extends ExecutionActivity {
  globalParameters?: { [propertyName: string]: any };
  webServiceOutputs?: { [propertyName: string]: AzureMLWebServiceFile };
  webServiceInputs?: { [propertyName: string]: AzureMLWebServiceFile };
}

/**
 * @class
 * Initializes a new instance of the GetMetadataActivity class.
 * @constructor
 * Activity to get metadata of dataset
 *
 * @member {object} dataset GetMetadata activity dataset reference.
 * @member {string} [dataset.referenceName] Reference dataset name.
 * @member {object} [dataset.parameters] Arguments for dataset.
 * @member {array} [fieldList] Fields of metadata to get from dataset.
 */
export interface GetMetadataActivity extends ExecutionActivity {
  dataset: DatasetReference;
  fieldList?: any[];
}

/**
 * @class
 * Initializes a new instance of the WebActivityAuthentication class.
 * @constructor
 * Web activity authentication properties.
 *
 * @member {string} type Web activity authentication
 * (Basic/ClientCertificate/MSI)
 * @member {object} [pfx] Base64-encoded contents of a PFX file.
 * @member {string} [pfx.value] Value of secure string.
 * @member {string} [username] Web activity authentication user name for basic
 * authentication.
 * @member {object} [password] Password for the PFX file or basic
 * authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {string} [resource] Resource for which Azure Auth token will be
 * requested when using MSI Authentication.
 */
export interface WebActivityAuthentication {
  type: string;
  pfx?: SecureString;
  username?: string;
  password?: SecureString;
  resource?: string;
}

/**
 * @class
 * Initializes a new instance of the WebActivity class.
 * @constructor
 * Web activity.
 *
 * @member {string} method Rest API method for target endpoint. Possible values
 * include: 'GET', 'POST', 'PUT', 'DELETE'
 * @member {object} url Web activity target endpoint and path. Type: string (or
 * Expression with resultType string).
 * @member {object} [headers] Represents the headers that will be sent to the
 * request. For example, to set the language and type on a request: "headers" :
 * { "Accept-Language": "en-us", "Content-Type": "application/json" }. Type:
 * string (or Expression with resultType string).
 * @member {object} [body] Represents the payload that will be sent to the
 * endpoint. Required for POST/PUT method, not allowed for GET method Type:
 * string (or Expression with resultType string).
 * @member {object} [authentication] Authentication method used for calling the
 * endpoint.
 * @member {string} [authentication.type] Web activity authentication
 * (Basic/ClientCertificate/MSI)
 * @member {object} [authentication.pfx] Base64-encoded contents of a PFX file.
 * @member {string} [authentication.pfx.value] Value of secure string.
 * @member {string} [authentication.username] Web activity authentication user
 * name for basic authentication.
 * @member {object} [authentication.password] Password for the PFX file or
 * basic authentication.
 * @member {string} [authentication.password.value] Value of secure string.
 * @member {string} [authentication.resource] Resource for which Azure Auth
 * token will be requested when using MSI Authentication.
 * @member {array} [datasets] List of datasets passed to web endpoint.
 * @member {array} [linkedServices] List of linked services passed to web
 * endpoint.
 */
export interface WebActivity extends ExecutionActivity {
  method: string;
  url: any;
  headers?: any;
  body?: any;
  authentication?: WebActivityAuthentication;
  datasets?: DatasetReference[];
  linkedServices?: LinkedServiceReference[];
}

/**
 * @class
 * Initializes a new instance of the RedshiftUnloadSettings class.
 * @constructor
 * The Amazon S3 settings needed for the interim Amazon S3 when copying from
 * Amazon Redshift with unload. With this, data from Amazon Redshift source
 * will be unloaded into S3 first and then copied into the targeted sink from
 * the interim S3.
 *
 * @member {object} s3LinkedServiceName The name of the Amazon S3 linked
 * service which will be used for the unload operation when copying from the
 * Amazon Redshift source.
 * @member {string} [s3LinkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [s3LinkedServiceName.parameters] Arguments for
 * LinkedService.
 * @member {object} bucketName The bucket of the interim Amazon S3 which will
 * be used to store the unloaded data from Amazon Redshift source. The bucket
 * must be in the same region as the Amazon Redshift source. Type: string (or
 * Expression with resultType string).
 */
export interface RedshiftUnloadSettings {
  s3LinkedServiceName: LinkedServiceReference;
  bucketName: any;
}

/**
 * @class
 * Initializes a new instance of the CopySource class.
 * @constructor
 * A copy activity source.
 *
 * @member {object} [sourceRetryCount] Source retry count. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [sourceRetryWait] Source retry wait. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} type Polymorphic Discriminator
 */
export interface CopySource {
  sourceRetryCount?: any;
  sourceRetryWait?: any;
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonRedshiftSource class.
 * @constructor
 * A copy activity source for Amazon Redshift Source.
 *
 * @member {object} [query] Database query. Type: string (or Expression with
 * resultType string).
 * @member {object} [redshiftUnloadSettings] The Amazon S3 settings needed for
 * the interim Amazon S3 when copying from Amazon Redshift with unload. With
 * this, data from Amazon Redshift source will be unloaded into S3 first and
 * then copied into the targeted sink from the interim S3.
 * @member {object} [redshiftUnloadSettings.s3LinkedServiceName] The name of
 * the Amazon S3 linked service which will be used for the unload operation
 * when copying from the Amazon Redshift source.
 * @member {string} [redshiftUnloadSettings.s3LinkedServiceName.referenceName]
 * Reference LinkedService name.
 * @member {object} [redshiftUnloadSettings.s3LinkedServiceName.parameters]
 * Arguments for LinkedService.
 * @member {object} [redshiftUnloadSettings.bucketName] The bucket of the
 * interim Amazon S3 which will be used to store the unloaded data from Amazon
 * Redshift source. The bucket must be in the same region as the Amazon
 * Redshift source. Type: string (or Expression with resultType string).
 */
export interface AmazonRedshiftSource extends CopySource {
  query?: any;
  redshiftUnloadSettings?: RedshiftUnloadSettings;
}

/**
 * @class
 * Initializes a new instance of the ResponsysSource class.
 * @constructor
 * A copy activity Responsys source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface ResponsysSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the SalesforceMarketingCloudSource class.
 * @constructor
 * A copy activity Salesforce Marketing Cloud source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface SalesforceMarketingCloudSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the VerticaSource class.
 * @constructor
 * A copy activity Vertica source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface VerticaSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the NetezzaSource class.
 * @constructor
 * A copy activity Netezza source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface NetezzaSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the ZohoSource class.
 * @constructor
 * A copy activity Zoho server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface ZohoSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the XeroSource class.
 * @constructor
 * A copy activity Xero Serivce source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface XeroSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the SquareSource class.
 * @constructor
 * A copy activity Square Serivce source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface SquareSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the SparkSource class.
 * @constructor
 * A copy activity Spark Server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface SparkSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the ShopifySource class.
 * @constructor
 * A copy activity Shopify Serivce source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface ShopifySource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the ServiceNowSource class.
 * @constructor
 * A copy activity ServiceNow server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface ServiceNowSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the QuickBooksSource class.
 * @constructor
 * A copy activity QuickBooks server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface QuickBooksSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the PrestoSource class.
 * @constructor
 * A copy activity Presto server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface PrestoSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the PhoenixSource class.
 * @constructor
 * A copy activity Phoenix server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface PhoenixSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the PaypalSource class.
 * @constructor
 * A copy activity Paypal Serivce source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface PaypalSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the MarketoSource class.
 * @constructor
 * A copy activity Marketo server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface MarketoSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the MariaDBSource class.
 * @constructor
 * A copy activity MariaDB server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface MariaDBSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the MagentoSource class.
 * @constructor
 * A copy activity Magento server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface MagentoSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the JiraSource class.
 * @constructor
 * A copy activity Jira Serivce source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface JiraSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the ImpalaSource class.
 * @constructor
 * A copy activity Impala server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface ImpalaSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the HubspotSource class.
 * @constructor
 * A copy activity Hubspot Serivce source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface HubspotSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the HiveSource class.
 * @constructor
 * A copy activity Hive Server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface HiveSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the HBaseSource class.
 * @constructor
 * A copy activity HBase server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface HBaseSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the GreenplumSource class.
 * @constructor
 * A copy activity Greenplum Database source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface GreenplumSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the GoogleBigQuerySource class.
 * @constructor
 * A copy activity Google BigQuery service source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface GoogleBigQuerySource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the EloquaSource class.
 * @constructor
 * A copy activity Eloqua server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface EloquaSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the DrillSource class.
 * @constructor
 * A copy activity Drill server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface DrillSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the CouchbaseSource class.
 * @constructor
 * A copy activity Couchbase server source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface CouchbaseSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the ConcurSource class.
 * @constructor
 * A copy activity Concur Serivce source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface ConcurSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the AzurePostgreSqlSource class.
 * @constructor
 * A copy activity Azure PostgreSQL source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface AzurePostgreSqlSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonMWSSource class.
 * @constructor
 * A copy activity Amazon Marketplace Web Service source.
 *
 * @member {object} [query] A query to retrieve data from source. Type: string
 * (or Expression with resultType string).
 */
export interface AmazonMWSSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the HttpSource class.
 * @constructor
 * A copy activity source for an HTTP file.
 *
 * @member {object} [httpRequestTimeout] Specifies the timeout for a HTTP
 * client to get HTTP response from HTTP server. The default value is
 * equivalent to System.Net.HttpWebRequest.Timeout. Type: string (or Expression
 * with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 */
export interface HttpSource extends CopySource {
  httpRequestTimeout?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreSource class.
 * @constructor
 * A copy activity Azure Data Lake source.
 *
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 */
export interface AzureDataLakeStoreSource extends CopySource {
  recursive?: any;
}

/**
 * @class
 * Initializes a new instance of the MongoDbSource class.
 * @constructor
 * A copy activity source for a MongoDB database.
 *
 * @member {object} [query] Database query. Should be a SQL-92 query
 * expression. Type: string (or Expression with resultType string).
 */
export interface MongoDbSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the CassandraSource class.
 * @constructor
 * A copy activity source for a Cassandra database.
 *
 * @member {object} [query] Database query. Should be a SQL-92 query expression
 * or Cassandra Query Language (CQL) command. Type: string (or Expression with
 * resultType string).
 * @member {string} [consistencyLevel] The consistency level specifies how many
 * Cassandra servers must respond to a read request before returning data to
 * the client application. Cassandra checks the specified number of Cassandra
 * servers for data to satisfy the read request. Must be one of
 * cassandraSourceReadConsistencyLevels. The default value is 'ONE'. It is
 * case-insensitive. Possible values include: 'ALL', 'EACH_QUORUM', 'QUORUM',
 * 'LOCAL_QUORUM', 'ONE', 'TWO', 'THREE', 'LOCAL_ONE', 'SERIAL', 'LOCAL_SERIAL'
 */
export interface CassandraSource extends CopySource {
  query?: any;
  consistencyLevel?: string;
}

/**
 * @class
 * Initializes a new instance of the WebSource class.
 * @constructor
 * A copy activity source for web page table.
 *
 */
export interface WebSource extends CopySource {
}

/**
 * @class
 * Initializes a new instance of the OracleSource class.
 * @constructor
 * A copy activity Oracle source.
 *
 * @member {object} [oracleReaderQuery] Oracle reader query. Type: string (or
 * Expression with resultType string).
 * @member {object} [queryTimeout] Query timeout. Type: string (or Expression
 * with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 */
export interface OracleSource extends CopySource {
  oracleReaderQuery?: any;
  queryTimeout?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMySqlSource class.
 * @constructor
 * A copy activity Azure MySQL source.
 *
 * @member {object} [query] Database query. Type: string (or Expression with
 * resultType string).
 */
export interface AzureMySqlSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the DistcpSettings class.
 * @constructor
 * Distcp settings.
 *
 * @member {object} resourceManagerEndpoint Specifies the Yarn ResourceManager
 * endpoint. Type: string (or Expression with resultType string).
 * @member {object} tempScriptPath Specifies an existing folder path which will
 * be used to store temp Distcp command script. The script file is generated by
 * ADF and will be removed after Copy job finished. Type: string (or Expression
 * with resultType string).
 * @member {object} [distcpOptions] Specifies the Distcp options. Type: string
 * (or Expression with resultType string).
 */
export interface DistcpSettings {
  resourceManagerEndpoint: any;
  tempScriptPath: any;
  distcpOptions?: any;
}

/**
 * @class
 * Initializes a new instance of the HdfsSource class.
 * @constructor
 * A copy activity HDFS source.
 *
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [distcpSettings] Specifies Distcp-related settings.
 * @member {object} [distcpSettings.resourceManagerEndpoint] Specifies the Yarn
 * ResourceManager endpoint. Type: string (or Expression with resultType
 * string).
 * @member {object} [distcpSettings.tempScriptPath] Specifies an existing
 * folder path which will be used to store temp Distcp command script. The
 * script file is generated by ADF and will be removed after Copy job finished.
 * Type: string (or Expression with resultType string).
 * @member {object} [distcpSettings.distcpOptions] Specifies the Distcp
 * options. Type: string (or Expression with resultType string).
 */
export interface HdfsSource extends CopySource {
  recursive?: any;
  distcpSettings?: DistcpSettings;
}

/**
 * @class
 * Initializes a new instance of the FileSystemSource class.
 * @constructor
 * A copy activity file system source.
 *
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 */
export interface FileSystemSource extends CopySource {
  recursive?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlDWSource class.
 * @constructor
 * A copy activity SQL Data Warehouse source.
 *
 * @member {object} [sqlReaderQuery] SQL Data Warehouse reader query. Type:
 * string (or Expression with resultType string).
 * @member {object} [sqlReaderStoredProcedureName] Name of the stored procedure
 * for a SQL Data Warehouse source. This cannot be used at the same time as
 * SqlReaderQuery. Type: string (or Expression with resultType string).
 * @member {object} [storedProcedureParameters] Value and type setting for
 * stored procedure parameters. Example: "{Parameter1: {value: "1", type:
 * "int"}}". Type: object (or Expression with resultType object), itemType:
 * StoredProcedureParameter.
 */
export interface SqlDWSource extends CopySource {
  sqlReaderQuery?: any;
  sqlReaderStoredProcedureName?: any;
  storedProcedureParameters?: any;
}

/**
 * @class
 * Initializes a new instance of the StoredProcedureParameter class.
 * @constructor
 * SQL stored procedure parameter.
 *
 * @member {object} [value] Stored procedure parameter value. Type: string (or
 * Expression with resultType string).
 * @member {string} [type] Stored procedure parameter type. Possible values
 * include: 'String', 'Int', 'Decimal', 'Guid', 'Boolean', 'Date'
 */
export interface StoredProcedureParameter {
  value?: any;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the SqlSource class.
 * @constructor
 * A copy activity SQL source.
 *
 * @member {object} [sqlReaderQuery] SQL reader query. Type: string (or
 * Expression with resultType string).
 * @member {object} [sqlReaderStoredProcedureName] Name of the stored procedure
 * for a SQL Database source. This cannot be used at the same time as
 * SqlReaderQuery. Type: string (or Expression with resultType string).
 * @member {object} [storedProcedureParameters] Value and type setting for
 * stored procedure parameters. Example: "{Parameter1: {value: "1", type:
 * "int"}}".
 */
export interface SqlSource extends CopySource {
  sqlReaderQuery?: any;
  sqlReaderStoredProcedureName?: any;
  storedProcedureParameters?: { [propertyName: string]: StoredProcedureParameter };
}

/**
 * @class
 * Initializes a new instance of the SapEccSource class.
 * @constructor
 * A copy activity source for SAP ECC source.
 *
 * @member {string} [query] SAP ECC OData query. For example, "$top=1". Type:
 * string (or Expression with resultType string).
 */
export interface SapEccSource extends CopySource {
  query?: string;
}

/**
 * @class
 * Initializes a new instance of the SapCloudForCustomerSource class.
 * @constructor
 * A copy activity source for SAP Cloud for Customer source.
 *
 * @member {object} [query] SAP Cloud for Customer OData query. For example,
 * "$top=1". Type: string (or Expression with resultType string).
 */
export interface SapCloudForCustomerSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the SalesforceSource class.
 * @constructor
 * A copy activity Salesforce source.
 *
 * @member {object} [query] Database query. Type: string (or Expression with
 * resultType string).
 * @member {string} [readBehavior] The read behavior for the operation. Default
 * is Query. Possible values include: 'Query', 'QueryAll'
 */
export interface SalesforceSource extends CopySource {
  query?: any;
  readBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the RelationalSource class.
 * @constructor
 * A copy activity source for various relational databases.
 *
 * @member {object} [query] Database query. Type: string (or Expression with
 * resultType string).
 */
export interface RelationalSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the DynamicsSource class.
 * @constructor
 * A copy activity Dynamics source.
 *
 * @member {object} [query] FetchXML is a proprietary query language that is
 * used in Microsoft Dynamics (online & on-premises). Type: string (or
 * Expression with resultType string).
 */
export interface DynamicsSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the DocumentDbCollectionSource class.
 * @constructor
 * A copy activity Document Database Collection source.
 *
 * @member {object} [query] Documents query. Type: string (or Expression with
 * resultType string).
 * @member {object} [nestingSeparator] Nested properties separator. Type:
 * string (or Expression with resultType string).
 */
export interface DocumentDbCollectionSource extends CopySource {
  query?: any;
  nestingSeparator?: any;
}

/**
 * @class
 * Initializes a new instance of the BlobSource class.
 * @constructor
 * A copy activity Azure Blob source.
 *
 * @member {object} [treatEmptyAsNull] Treat empty as null. Type: boolean (or
 * Expression with resultType boolean).
 * @member {object} [skipHeaderLineCount] Number of header lines to skip from
 * each blob. Type: integer (or Expression with resultType integer).
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 */
export interface BlobSource extends CopySource {
  treatEmptyAsNull?: any;
  skipHeaderLineCount?: any;
  recursive?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureTableSource class.
 * @constructor
 * A copy activity Azure Table source.
 *
 * @member {object} [azureTableSourceQuery] Azure Table source query. Type:
 * string (or Expression with resultType string).
 * @member {object} [azureTableSourceIgnoreTableNotFound] Azure Table source
 * ignore table not found. Type: boolean (or Expression with resultType
 * boolean).
 */
export interface AzureTableSource extends CopySource {
  azureTableSourceQuery?: any;
  azureTableSourceIgnoreTableNotFound?: any;
}

/**
 * @class
 * Initializes a new instance of the LookupActivity class.
 * @constructor
 * Lookup activity.
 *
 * @member {object} source Dataset-specific source properties, same as copy
 * activity source.
 * @member {object} [source.sourceRetryCount] Source retry count. Type: integer
 * (or Expression with resultType integer).
 * @member {object} [source.sourceRetryWait] Source retry wait. Type: string
 * (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} [source.type] Polymorphic Discriminator
 * @member {object} dataset Lookup activity dataset reference.
 * @member {string} [dataset.referenceName] Reference dataset name.
 * @member {object} [dataset.parameters] Arguments for dataset.
 * @member {object} [firstRowOnly] Whether to return first row or all rows.
 * Default value is true. Type: boolean (or Expression with resultType
 * boolean).
 */
export interface LookupActivity extends ExecutionActivity {
  source: CopySource;
  dataset: DatasetReference;
  firstRowOnly?: any;
}

/**
 * @class
 * Initializes a new instance of the DeleteActivity class.
 * @constructor
 * Delete activity.
 *
 * @member {object} [recursive] If true, files under the folder path will be
 * deleted recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} dataset Delete activity dataset reference.
 * @member {string} [dataset.referenceName] Reference dataset name.
 * @member {object} [dataset.parameters] Arguments for dataset.
 */
export interface DeleteActivity extends ExecutionActivity {
  recursive?: any;
  dataset: DatasetReference;
}

/**
 * @class
 * Initializes a new instance of the SqlServerStoredProcedureActivity class.
 * @constructor
 * SQL stored procedure activity type.
 *
 * @member {object} storedProcedureName Stored procedure name. Type: string (or
 * Expression with resultType string).
 * @member {object} [storedProcedureParameters] Value and type setting for
 * stored procedure parameters. Example: "{Parameter1: {value: "1", type:
 * "int"}}".
 */
export interface SqlServerStoredProcedureActivity extends ExecutionActivity {
  storedProcedureName: any;
  storedProcedureParameters?: { [propertyName: string]: StoredProcedureParameter };
}

/**
 * @class
 * Initializes a new instance of the CustomActivityReferenceObject class.
 * @constructor
 * Reference objects for custom activity
 *
 * @member {array} [linkedServices] Linked service references.
 * @member {array} [datasets] Dataset references.
 */
export interface CustomActivityReferenceObject {
  linkedServices?: LinkedServiceReference[];
  datasets?: DatasetReference[];
}

/**
 * @class
 * Initializes a new instance of the CustomActivity class.
 * @constructor
 * Custom activity type.
 *
 * @member {object} command Command for custom activity Type: string (or
 * Expression with resultType string).
 * @member {object} [resourceLinkedService] Resource linked service reference.
 * @member {string} [resourceLinkedService.referenceName] Reference
 * LinkedService name.
 * @member {object} [resourceLinkedService.parameters] Arguments for
 * LinkedService.
 * @member {object} [folderPath] Folder path for resource files Type: string
 * (or Expression with resultType string).
 * @member {object} [referenceObjects] Reference objects
 * @member {array} [referenceObjects.linkedServices] Linked service references.
 * @member {array} [referenceObjects.datasets] Dataset references.
 * @member {object} [extendedProperties] User defined property bag. There is no
 * restriction on the keys or values that can be used. The user specified
 * custom activity has the full responsibility to consume and interpret the
 * content defined.
 */
export interface CustomActivity extends ExecutionActivity {
  command: any;
  resourceLinkedService?: LinkedServiceReference;
  folderPath?: any;
  referenceObjects?: CustomActivityReferenceObject;
  extendedProperties?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the SSISPropertyOverride class.
 * @constructor
 * SSIS property override.
 *
 * @member {object} value SSIS package property override value. Type: string
 * (or Expression with resultType string).
 * @member {boolean} [isSensitive] Whether SSIS package property override value
 * is sensitive data. Value will be encrypted in SSISDB if it is true
 */
export interface SSISPropertyOverride {
  value: any;
  isSensitive?: boolean;
}

/**
 * @class
 * Initializes a new instance of the SSISExecutionParameter class.
 * @constructor
 * SSIS execution parameter.
 *
 * @member {object} value SSIS package execution parameter value. Type: string
 * (or Expression with resultType string).
 */
export interface SSISExecutionParameter {
  value: any;
}

/**
 * @class
 * Initializes a new instance of the SSISPackageLocation class.
 * @constructor
 * SSIS package location.
 *
 * @member {string} packagePath The SSIS package path.
 */
export interface SSISPackageLocation {
  packagePath: string;
}

/**
 * @class
 * Initializes a new instance of the ExecuteSSISPackageActivity class.
 * @constructor
 * Execute SSIS package activity.
 *
 * @member {object} packageLocation SSIS package location.
 * @member {string} [packageLocation.packagePath] The SSIS package path.
 * @member {string} [runtime] Specifies the runtime to execute SSIS package.
 * Possible values include: 'x64', 'x86'
 * @member {string} [loggingLevel] The logging level of SSIS package execution.
 * @member {string} [environmentPath] The environment path to execute the SSIS
 * package.
 * @member {object} connectVia The integration runtime reference.
 * @member {string} [connectVia.referenceName] Reference integration runtime
 * name.
 * @member {object} [connectVia.parameters] Arguments for integration runtime.
 * @member {object} [projectParameters] The project level parameters to execute
 * the SSIS package.
 * @member {object} [packageParameters] The package level parameters to execute
 * the SSIS package.
 * @member {object} [projectConnectionManagers] The project level connection
 * managers to execute the SSIS package.
 * @member {object} [packageConnectionManagers] The package level connection
 * managers to execute the SSIS package.
 * @member {object} [propertyOverrides] The property overrides to execute the
 * SSIS package.
 */
export interface ExecuteSSISPackageActivity extends ExecutionActivity {
  packageLocation: SSISPackageLocation;
  runtime?: string;
  loggingLevel?: string;
  environmentPath?: string;
  connectVia: IntegrationRuntimeReference;
  projectParameters?: { [propertyName: string]: SSISExecutionParameter };
  packageParameters?: { [propertyName: string]: SSISExecutionParameter };
  projectConnectionManagers?: { [propertyName: string]: { [propertyName: string]: SSISExecutionParameter } };
  packageConnectionManagers?: { [propertyName: string]: { [propertyName: string]: SSISExecutionParameter } };
  propertyOverrides?: { [propertyName: string]: SSISPropertyOverride };
}

/**
 * @class
 * Initializes a new instance of the HDInsightSparkActivity class.
 * @constructor
 * HDInsight Spark activity.
 *
 * @member {object} rootPath The root path in 'sparkJobLinkedService' for all
 * the job’s files. Type: string (or Expression with resultType string).
 * @member {object} entryFilePath The relative path to the root folder of the
 * code/package to be executed. Type: string (or Expression with resultType
 * string).
 * @member {array} [argumentsProperty] The user-specified arguments to
 * HDInsightSparkActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} [sparkJobLinkedService] The storage linked service for
 * uploading the entry file and dependencies, and for receiving logs.
 * @member {string} [sparkJobLinkedService.referenceName] Reference
 * LinkedService name.
 * @member {object} [sparkJobLinkedService.parameters] Arguments for
 * LinkedService.
 * @member {string} [className] The application's Java/Spark main class.
 * @member {object} [proxyUser] The user to impersonate that will execute the
 * job. Type: string (or Expression with resultType string).
 * @member {object} [sparkConfig] Spark configuration property.
 */
export interface HDInsightSparkActivity extends ExecutionActivity {
  rootPath: any;
  entryFilePath: any;
  argumentsProperty?: any[];
  getDebugInfo?: string;
  sparkJobLinkedService?: LinkedServiceReference;
  className?: string;
  proxyUser?: any;
  sparkConfig?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightStreamingActivity class.
 * @constructor
 * HDInsight streaming activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [argumentsProperty] User specified arguments to
 * HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} mapper Mapper executable name. Type: string (or Expression
 * with resultType string).
 * @member {object} reducer Reducer executable name. Type: string (or
 * Expression with resultType string).
 * @member {object} input Input blob path. Type: string (or Expression with
 * resultType string).
 * @member {object} output Output blob path. Type: string (or Expression with
 * resultType string).
 * @member {array} filePaths Paths to streaming job files. Can be directories.
 * @member {object} [fileLinkedService] Linked service reference where the
 * files are located.
 * @member {string} [fileLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [fileLinkedService.parameters] Arguments for LinkedService.
 * @member {object} [combiner] Combiner executable name. Type: string (or
 * Expression with resultType string).
 * @member {array} [commandEnvironment] Command line environment values.
 * @member {object} [defines] Allows user to specify defines for streaming job
 * request.
 */
export interface HDInsightStreamingActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  argumentsProperty?: any[];
  getDebugInfo?: string;
  mapper: any;
  reducer: any;
  input: any;
  output: any;
  filePaths: any[];
  fileLinkedService?: LinkedServiceReference;
  combiner?: any;
  commandEnvironment?: any[];
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightMapReduceActivity class.
 * @constructor
 * HDInsight MapReduce activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [argumentsProperty] User specified arguments to
 * HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} className Class name. Type: string (or Expression with
 * resultType string).
 * @member {object} jarFilePath Jar path. Type: string (or Expression with
 * resultType string).
 * @member {object} [jarLinkedService] Jar linked service reference.
 * @member {string} [jarLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [jarLinkedService.parameters] Arguments for LinkedService.
 * @member {array} [jarLibs] Jar libs.
 * @member {object} [defines] Allows user to specify defines for the MapReduce
 * job request.
 */
export interface HDInsightMapReduceActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  argumentsProperty?: any[];
  getDebugInfo?: string;
  className: any;
  jarFilePath: any;
  jarLinkedService?: LinkedServiceReference;
  jarLibs?: any[];
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightPigActivity class.
 * @constructor
 * HDInsight Pig activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [argumentsProperty] User specified arguments to
 * HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} [scriptPath] Script path. Type: string (or Expression with
 * resultType string).
 * @member {object} [scriptLinkedService] Script linked service reference.
 * @member {string} [scriptLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [scriptLinkedService.parameters] Arguments for
 * LinkedService.
 * @member {object} [defines] Allows user to specify defines for Pig job
 * request.
 */
export interface HDInsightPigActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  argumentsProperty?: any[];
  getDebugInfo?: string;
  scriptPath?: any;
  scriptLinkedService?: LinkedServiceReference;
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightHiveActivity class.
 * @constructor
 * HDInsight Hive activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [argumentsProperty] User specified arguments to
 * HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} [scriptPath] Script path. Type: string (or Expression with
 * resultType string).
 * @member {object} [scriptLinkedService] Script linked service reference.
 * @member {string} [scriptLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [scriptLinkedService.parameters] Arguments for
 * LinkedService.
 * @member {object} [defines] Allows user to specify defines for Hive job
 * request.
 */
export interface HDInsightHiveActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  argumentsProperty?: any[];
  getDebugInfo?: string;
  scriptPath?: any;
  scriptLinkedService?: LinkedServiceReference;
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the RedirectIncompatibleRowSettings class.
 * @constructor
 * Redirect incompatible row settings
 *
 * @member {object} linkedServiceName Name of the Azure Storage, Storage SAS,
 * or Azure Data Lake Store linked service used for redirecting incompatible
 * row. Must be specified if redirectIncompatibleRowSettings is specified.
 * Type: string (or Expression with resultType string).
 * @member {object} [path] The path for storing the redirect incompatible row
 * data. Type: string (or Expression with resultType string).
 */
export interface RedirectIncompatibleRowSettings {
  linkedServiceName: any;
  path?: any;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the StagingSettings class.
 * @constructor
 * Staging settings.
 *
 * @member {object} linkedServiceName Staging linked service reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [linkedServiceName.parameters] Arguments for LinkedService.
 * @member {object} [path] The path to storage for storing the interim data.
 * Type: string (or Expression with resultType string).
 * @member {object} [enableCompression] Specifies whether to use compression
 * when copying data via an interim staging. Default value is false. Type:
 * boolean (or Expression with resultType boolean).
 */
export interface StagingSettings {
  linkedServiceName: LinkedServiceReference;
  path?: any;
  enableCompression?: any;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the CopyTranslator class.
 * @constructor
 * A copy activity translator.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface CopyTranslator {
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the TabularTranslator class.
 * @constructor
 * A copy activity tabular translator.
 *
 * @member {object} [columnMappings] Column mappings. Example: "UserId:
 * MyUserId, Group: MyGroup, Name: MyName" Type: string (or Expression with
 * resultType string).
 * @member {object} [schemaMapping] The schema mapping to map between tabular
 * data and hierarchical data. Example: {"Column1": "$.Column1", "Column2":
 * "$.Column2.Property1", "Column3": "$.Column2.Property2"}. Type: object (or
 * Expression with resultType object).
 */
export interface TabularTranslator extends CopyTranslator {
  columnMappings?: any;
  schemaMapping?: any;
}

/**
 * @class
 * Initializes a new instance of the CopySink class.
 * @constructor
 * A copy activity sink.
 *
 * @member {object} [writeBatchSize] Write batch size. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {object} [writeBatchTimeout] Write batch timeout. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [sinkRetryCount] Sink retry count. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [sinkRetryWait] Sink retry wait. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} type Polymorphic Discriminator
 */
export interface CopySink {
  writeBatchSize?: any;
  writeBatchTimeout?: any;
  sinkRetryCount?: any;
  sinkRetryWait?: any;
  type: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the SalesforceSink class.
 * @constructor
 * A copy activity Salesforce sink.
 *
 * @member {string} [writeBehavior] The write behavior for the operation.
 * Default is Insert. Possible values include: 'Insert', 'Upsert'
 * @member {object} [externalIdFieldName] The name of the external ID field for
 * upsert operation. Default value is 'Id' column. Type: string (or Expression
 * with resultType string).
 * @member {object} [ignoreNullValues] The flag indicating whether or not to
 * ignore null values from input dataset (except key fields) during write
 * operation. Default value is false. If set it to true, it means ADF will
 * leave the data in the destination object unchanged when doing upsert/update
 * operation and insert defined default value when doing insert operation,
 * versus ADF will update the data in the destination object to NULL when doing
 * upsert/update operation and insert NULL value when doing insert operation.
 * Type: boolean (or Expression with resultType boolean).
 */
export interface SalesforceSink extends CopySink {
  writeBehavior?: string;
  externalIdFieldName?: any;
  ignoreNullValues?: any;
}

/**
 * @class
 * Initializes a new instance of the DynamicsSink class.
 * @constructor
 * A copy activity Dynamics sink.
 *
 * @member {object} [ignoreNullValues] The flag indicating whether ignore null
 * values from input dataset (except key fields) during write operation.
 * Default is false. Type: boolean (or Expression with resultType boolean).
 */
export interface DynamicsSink extends CopySink {
  ignoreNullValues?: any;
}

/**
 * @class
 * Initializes a new instance of the OdbcSink class.
 * @constructor
 * A copy activity ODBC sink.
 *
 * @member {object} [preCopyScript] A query to execute before starting the
 * copy. Type: string (or Expression with resultType string).
 */
export interface OdbcSink extends CopySink {
  preCopyScript?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSearchIndexSink class.
 * @constructor
 * A copy activity Azure Search Index sink.
 *
 * @member {string} [writeBehavior] Specify the write behavior when upserting
 * documents into Azure Search Index. Possible values include: 'Merge',
 * 'Upload'
 */
export interface AzureSearchIndexSink extends CopySink {
  writeBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreSink class.
 * @constructor
 * A copy activity Azure Data Lake Store sink.
 *
 * @member {string} [copyBehavior] The type of copy behavior for copy sink.
 * Possible values include: 'PreserveHierarchy', 'FlattenHierarchy',
 * 'MergeFiles'
 */
export interface AzureDataLakeStoreSink extends CopySink {
  copyBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the OracleSink class.
 * @constructor
 * A copy activity Oracle sink.
 *
 * @member {object} [preCopyScript] SQL pre-copy script. Type: string (or
 * Expression with resultType string).
 */
export interface OracleSink extends CopySink {
  preCopyScript?: any;
}

/**
 * @class
 * Initializes a new instance of the PolybaseSettings class.
 * @constructor
 * PolyBase settings.
 *
 * @member {string} [rejectType] Reject type. Possible values include: 'value',
 * 'percentage'
 * @member {object} [rejectValue] Specifies the value or the percentage of rows
 * that can be rejected before the query fails. Type: number (or Expression
 * with resultType number), minimum: 0.
 * @member {object} [rejectSampleValue] Determines the number of rows to
 * attempt to retrieve before the PolyBase recalculates the percentage of
 * rejected rows. Type: integer (or Expression with resultType integer),
 * minimum: 0.
 * @member {object} [useTypeDefault] Specifies how to handle missing values in
 * delimited text files when PolyBase retrieves data from the text file. Type:
 * boolean (or Expression with resultType boolean).
 */
export interface PolybaseSettings {
  rejectType?: string;
  rejectValue?: any;
  rejectSampleValue?: any;
  useTypeDefault?: any;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the SqlDWSink class.
 * @constructor
 * A copy activity SQL Data Warehouse sink.
 *
 * @member {object} [preCopyScript] SQL pre-copy script. Type: string (or
 * Expression with resultType string).
 * @member {object} [allowPolyBase] Indicates to use PolyBase to copy data into
 * SQL Data Warehouse when applicable. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [polyBaseSettings] Specifies PolyBase-related settings when
 * allowPolyBase is true.
 * @member {string} [polyBaseSettings.rejectType] Reject type. Possible values
 * include: 'value', 'percentage'
 * @member {object} [polyBaseSettings.rejectValue] Specifies the value or the
 * percentage of rows that can be rejected before the query fails. Type: number
 * (or Expression with resultType number), minimum: 0.
 * @member {object} [polyBaseSettings.rejectSampleValue] Determines the number
 * of rows to attempt to retrieve before the PolyBase recalculates the
 * percentage of rejected rows. Type: integer (or Expression with resultType
 * integer), minimum: 0.
 * @member {object} [polyBaseSettings.useTypeDefault] Specifies how to handle
 * missing values in delimited text files when PolyBase retrieves data from the
 * text file. Type: boolean (or Expression with resultType boolean).
 */
export interface SqlDWSink extends CopySink {
  preCopyScript?: any;
  allowPolyBase?: any;
  polyBaseSettings?: PolybaseSettings;
}

/**
 * @class
 * Initializes a new instance of the SqlSink class.
 * @constructor
 * A copy activity SQL sink.
 *
 * @member {object} [sqlWriterStoredProcedureName] SQL writer stored procedure
 * name. Type: string (or Expression with resultType string).
 * @member {object} [sqlWriterTableType] SQL writer table type. Type: string
 * (or Expression with resultType string).
 * @member {object} [preCopyScript] SQL pre-copy script. Type: string (or
 * Expression with resultType string).
 * @member {object} [storedProcedureParameters] SQL stored procedure
 * parameters.
 */
export interface SqlSink extends CopySink {
  sqlWriterStoredProcedureName?: any;
  sqlWriterTableType?: any;
  preCopyScript?: any;
  storedProcedureParameters?: { [propertyName: string]: StoredProcedureParameter };
}

/**
 * @class
 * Initializes a new instance of the DocumentDbCollectionSink class.
 * @constructor
 * A copy activity Document Database Collection sink.
 *
 * @member {object} [nestingSeparator] Nested properties separator. Default is
 * . (dot). Type: string (or Expression with resultType string).
 */
export interface DocumentDbCollectionSink extends CopySink {
  nestingSeparator?: any;
}

/**
 * @class
 * Initializes a new instance of the FileSystemSink class.
 * @constructor
 * A copy activity file system sink.
 *
 * @member {string} [copyBehavior] The type of copy behavior for copy sink.
 * Possible values include: 'PreserveHierarchy', 'FlattenHierarchy',
 * 'MergeFiles'
 */
export interface FileSystemSink extends CopySink {
  copyBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the BlobSink class.
 * @constructor
 * A copy activity Azure Blob sink.
 *
 * @member {object} [blobWriterOverwriteFiles] Blob writer overwrite files.
 * Type: boolean (or Expression with resultType boolean).
 * @member {object} [blobWriterDateTimeFormat] Blob writer date time format.
 * Type: string (or Expression with resultType string).
 * @member {object} [blobWriterAddHeader] Blob writer add header. Type: boolean
 * (or Expression with resultType boolean).
 * @member {string} [copyBehavior] The type of copy behavior for copy sink.
 * Possible values include: 'PreserveHierarchy', 'FlattenHierarchy',
 * 'MergeFiles'
 */
export interface BlobSink extends CopySink {
  blobWriterOverwriteFiles?: any;
  blobWriterDateTimeFormat?: any;
  blobWriterAddHeader?: any;
  copyBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureTableSink class.
 * @constructor
 * A copy activity Azure Table sink.
 *
 * @member {object} [azureTableDefaultPartitionKeyValue] Azure Table default
 * partition key value. Type: string (or Expression with resultType string).
 * @member {object} [azureTablePartitionKeyName] Azure Table partition key
 * name. Type: string (or Expression with resultType string).
 * @member {object} [azureTableRowKeyName] Azure Table row key name. Type:
 * string (or Expression with resultType string).
 * @member {object} [azureTableInsertType] Azure Table insert type. Type:
 * string (or Expression with resultType string).
 */
export interface AzureTableSink extends CopySink {
  azureTableDefaultPartitionKeyValue?: any;
  azureTablePartitionKeyName?: any;
  azureTableRowKeyName?: any;
  azureTableInsertType?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureQueueSink class.
 * @constructor
 * A copy activity Azure Queue sink.
 *
 */
export interface AzureQueueSink extends CopySink {
}

/**
 * @class
 * Initializes a new instance of the SapCloudForCustomerSink class.
 * @constructor
 * A copy activity SAP Cloud for Customer sink.
 *
 * @member {string} [writeBehavior] The write behavior for the operation.
 * Default is 'Insert'. Possible values include: 'Insert', 'Update'
 */
export interface SapCloudForCustomerSink extends CopySink {
  writeBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the CopyActivity class.
 * @constructor
 * Copy activity.
 *
 * @member {object} source Copy activity source.
 * @member {object} [source.sourceRetryCount] Source retry count. Type: integer
 * (or Expression with resultType integer).
 * @member {object} [source.sourceRetryWait] Source retry wait. Type: string
 * (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} [source.type] Polymorphic Discriminator
 * @member {object} sink Copy activity sink.
 * @member {object} [sink.writeBatchSize] Write batch size. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {object} [sink.writeBatchTimeout] Write batch timeout. Type: string
 * (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [sink.sinkRetryCount] Sink retry count. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [sink.sinkRetryWait] Sink retry wait. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} [sink.type] Polymorphic Discriminator
 * @member {object} [translator] Copy activity translator. If not specificed,
 * tabular translator is used.
 * @member {string} [translator.type] Polymorphic Discriminator
 * @member {object} [enableStaging] Specifies whether to copy data via an
 * interim staging. Default value is false. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [stagingSettings] Specifies interim staging settings when
 * EnableStaging is true.
 * @member {object} [stagingSettings.linkedServiceName] Staging linked service
 * reference.
 * @member {string} [stagingSettings.linkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [stagingSettings.linkedServiceName.parameters] Arguments
 * for LinkedService.
 * @member {object} [stagingSettings.path] The path to storage for storing the
 * interim data. Type: string (or Expression with resultType string).
 * @member {object} [stagingSettings.enableCompression] Specifies whether to
 * use compression when copying data via an interim staging. Default value is
 * false. Type: boolean (or Expression with resultType boolean).
 * @member {object} [parallelCopies] Maximum number of concurrent sessions
 * opened on the source or sink to avoid overloading the data store. Type:
 * integer (or Expression with resultType integer), minimum: 0.
 * @member {object} [dataIntegrationUnits] Maximum number of data integration
 * units that can be used to perform this data movement. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {object} [enableSkipIncompatibleRow] Whether to skip incompatible
 * row. Default value is false. Type: boolean (or Expression with resultType
 * boolean).
 * @member {object} [redirectIncompatibleRowSettings] Redirect incompatible row
 * settings when EnableSkipIncompatibleRow is true.
 * @member {object} [redirectIncompatibleRowSettings.linkedServiceName] Name of
 * the Azure Storage, Storage SAS, or Azure Data Lake Store linked service used
 * for redirecting incompatible row. Must be specified if
 * redirectIncompatibleRowSettings is specified. Type: string (or Expression
 * with resultType string).
 * @member {object} [redirectIncompatibleRowSettings.path] The path for storing
 * the redirect incompatible row data. Type: string (or Expression with
 * resultType string).
 * @member {array} [inputs] List of inputs for the activity.
 * @member {array} [outputs] List of outputs for the activity.
 */
export interface CopyActivity extends ExecutionActivity {
  source: CopySource;
  sink: CopySink;
  translator?: CopyTranslator;
  enableStaging?: any;
  stagingSettings?: StagingSettings;
  parallelCopies?: any;
  dataIntegrationUnits?: any;
  enableSkipIncompatibleRow?: any;
  redirectIncompatibleRowSettings?: RedirectIncompatibleRowSettings;
  inputs?: DatasetReference[];
  outputs?: DatasetReference[];
}

/**
 * @class
 * Initializes a new instance of the ControlActivity class.
 * @constructor
 * Base class for all control activities like IfCondition, ForEach , Until.
 *
 */
export interface ControlActivity extends Activity {
}

/**
 * @class
 * Initializes a new instance of the AppendVariableActivity class.
 * @constructor
 * Append value for a Variable of type Array.
 *
 * @member {string} [variableName] Name of the variable whose value needs to be
 * appended to.
 * @member {object} [value] Value to be appended. Could be a static value or
 * Expression
 */
export interface AppendVariableActivity extends ControlActivity {
  variableName?: string;
  value?: any;
}

/**
 * @class
 * Initializes a new instance of the SetVariableActivity class.
 * @constructor
 * Set value for a Variable.
 *
 * @member {string} [variableName] Name of the variable whose value needs to be
 * set.
 * @member {object} [value] Value to be set. Could be a static value or
 * Expression
 */
export interface SetVariableActivity extends ControlActivity {
  variableName?: string;
  value?: any;
}

/**
 * @class
 * Initializes a new instance of the FilterActivity class.
 * @constructor
 * Filter and return results from input array based on the conditions.
 *
 * @member {object} items Input array on which filter should be applied.
 * @member {string} [items.value] Expression value.
 * @member {object} condition Condition to be used for filtering the input.
 * @member {string} [condition.value] Expression value.
 */
export interface FilterActivity extends ControlActivity {
  items: Expression;
  condition: Expression;
}

/**
 * @class
 * Initializes a new instance of the UntilActivity class.
 * @constructor
 * This activity executes inner activities until the specified boolean
 * expression results to true or timeout is reached, whichever is earlier.
 *
 * @member {object} expression An expression that would evaluate to Boolean.
 * The loop will continue until this expression evaluates to true
 * @member {string} [expression.value] Expression value.
 * @member {object} [timeout] Specifies the timeout for the activity to run. If
 * there is no value specified, it takes the value of TimeSpan.FromDays(7)
 * which is 1 week as default. Type: string (or Expression with resultType
 * string), pattern: ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * Type: string (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {array} activities List of activities to execute.
 */
export interface UntilActivity extends ControlActivity {
  expression: Expression;
  timeout?: any;
  activities: Activity[];
}

/**
 * @class
 * Initializes a new instance of the WaitActivity class.
 * @constructor
 * This activity suspends pipeline execution for the specified interval.
 *
 * @member {number} waitTimeInSeconds Duration in seconds.
 */
export interface WaitActivity extends ControlActivity {
  waitTimeInSeconds: number;
}

/**
 * @class
 * Initializes a new instance of the ForEachActivity class.
 * @constructor
 * This activity is used for iterating over a collection and execute given
 * activities.
 *
 * @member {boolean} [isSequential] Should the loop be executed in sequence or
 * in parallel (max 50)
 * @member {number} [batchCount] Batch count to be used for controlling the
 * number of parallel execution (when isSequential is set to false).
 * @member {object} items Collection to iterate.
 * @member {string} [items.value] Expression value.
 * @member {array} activities List of activities to execute .
 */
export interface ForEachActivity extends ControlActivity {
  isSequential?: boolean;
  batchCount?: number;
  items: Expression;
  activities: Activity[];
}

/**
 * @class
 * Initializes a new instance of the IfConditionActivity class.
 * @constructor
 * This activity evaluates a boolean expression and executes either the
 * activities under the ifTrueActivities property or the ifFalseActivities
 * property depending on the result of the expression.
 *
 * @member {object} expression An expression that would evaluate to Boolean.
 * This is used to determine the block of activities (ifTrueActivities or
 * ifFalseActivities) that will be executed.
 * @member {string} [expression.value] Expression value.
 * @member {array} [ifTrueActivities] List of activities to execute if
 * expression is evaluated to true. This is an optional property and if not
 * provided, the activity will exit without any action.
 * @member {array} [ifFalseActivities] List of activities to execute if
 * expression is evaluated to false. This is an optional property and if not
 * provided, the activity will exit without any action.
 */
export interface IfConditionActivity extends ControlActivity {
  expression: Expression;
  ifTrueActivities?: Activity[];
  ifFalseActivities?: Activity[];
}

/**
 * @class
 * Initializes a new instance of the ExecutePipelineActivity class.
 * @constructor
 * Execute pipeline activity.
 *
 * @member {object} pipelineProperty Pipeline reference.
 * @member {string} [pipelineProperty.referenceName] Reference pipeline name.
 * @member {string} [pipelineProperty.name] Reference name.
 * @member {object} [parameters] Pipeline parameters.
 * @member {boolean} [waitOnCompletion] Defines whether activity execution will
 * wait for the dependent pipeline execution to finish. Default is false.
 */
export interface ExecutePipelineActivity extends ControlActivity {
  pipelineProperty: PipelineReference;
  parameters?: { [propertyName: string]: any };
  waitOnCompletion?: boolean;
}

/**
 * @class
 * Initializes a new instance of the LinkedIntegrationRuntime class.
 * @constructor
 * The linked integration runtime information.
 *
 * @member {string} [name] The name of the linked integration runtime.
 * @member {string} [subscriptionId] The subscription ID for which the linked
 * integration runtime belong to.
 * @member {string} [dataFactoryName] The name of the data factory for which
 * the linked integration runtime belong to.
 * @member {string} [dataFactoryLocation] The location of the data factory for
 * which the linked integration runtime belong to.
 * @member {date} [createTime] The creating time of the linked integration
 * runtime.
 */
export interface LinkedIntegrationRuntime {
  readonly name?: string;
  readonly subscriptionId?: string;
  readonly dataFactoryName?: string;
  readonly dataFactoryLocation?: string;
  readonly createTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the SelfHostedIntegrationRuntimeNode class.
 * @constructor
 * Properties of Self-hosted integration runtime node.
 *
 * @member {string} [nodeName] Name of the integration runtime node.
 * @member {string} [machineName] Machine name of the integration runtime node.
 * @member {string} [hostServiceUri] URI for the host machine of the
 * integration runtime.
 * @member {string} [status] Status of the integration runtime node. Possible
 * values include: 'NeedRegistration', 'Online', 'Limited', 'Offline',
 * 'Upgrading', 'Initializing', 'InitializeFailed'
 * @member {object} [capabilities] The integration runtime capabilities
 * dictionary
 * @member {string} [versionStatus] Status of the integration runtime node
 * version.
 * @member {string} [version] Version of the integration runtime node.
 * @member {date} [registerTime] The time at which the integration runtime node
 * was registered in ISO8601 format.
 * @member {date} [lastConnectTime] The most recent time at which the
 * integration runtime was connected in ISO8601 format.
 * @member {date} [expiryTime] The time at which the integration runtime will
 * expire in ISO8601 format.
 * @member {date} [lastStartTime] The time the node last started up.
 * @member {date} [lastStopTime] The integration runtime node last stop time.
 * @member {string} [lastUpdateResult] The result of the last integration
 * runtime node update. Possible values include: 'None', 'Succeed', 'Fail'
 * @member {date} [lastStartUpdateTime] The last time for the integration
 * runtime node update start.
 * @member {date} [lastEndUpdateTime] The last time for the integration runtime
 * node update end.
 * @member {boolean} [isActiveDispatcher] Indicates whether this node is the
 * active dispatcher for integration runtime requests.
 * @member {number} [concurrentJobsLimit] Maximum concurrent jobs on the
 * integration runtime node.
 * @member {number} [maxConcurrentJobs] The maximum concurrent jobs in this
 * integration runtime.
 */
export interface SelfHostedIntegrationRuntimeNode {
  readonly nodeName?: string;
  readonly machineName?: string;
  readonly hostServiceUri?: string;
  readonly status?: string;
  readonly capabilities?: { [propertyName: string]: string };
  readonly versionStatus?: string;
  readonly version?: string;
  readonly registerTime?: Date;
  readonly lastConnectTime?: Date;
  readonly expiryTime?: Date;
  readonly lastStartTime?: Date;
  readonly lastStopTime?: Date;
  readonly lastUpdateResult?: string;
  readonly lastStartUpdateTime?: Date;
  readonly lastEndUpdateTime?: Date;
  readonly isActiveDispatcher?: boolean;
  readonly concurrentJobsLimit?: number;
  readonly maxConcurrentJobs?: number;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the SelfHostedIntegrationRuntimeStatus class.
 * @constructor
 * Self-hosted integration runtime status.
 *
 * @member {date} [createTime] The time at which the integration runtime was
 * created, in ISO8601 format.
 * @member {string} [taskQueueId] The task queue id of the integration runtime.
 * @member {string} [internalChannelEncryption] It is used to set the
 * encryption mode for node-node communication channel (when more than 2
 * self-hosted integration runtime nodes exist). Possible values include:
 * 'NotSet', 'SslEncrypted', 'NotEncrypted'
 * @member {string} [version] Version of the integration runtime.
 * @member {array} [nodes] The list of nodes for this integration runtime.
 * @member {date} [scheduledUpdateDate] The date at which the integration
 * runtime will be scheduled to update, in ISO8601 format.
 * @member {string} [updateDelayOffset] The time in the date scheduled by
 * service to update the integration runtime, e.g., PT03H is 3 hours
 * @member {string} [localTimeZoneOffset] The local time zone offset in hours.
 * @member {object} [capabilities] Object with additional information about
 * integration runtime capabilities.
 * @member {array} [serviceUrls] The URLs for the services used in integration
 * runtime backend service.
 * @member {string} [autoUpdate] Whether Self-hosted integration runtime auto
 * update has been turned on. Possible values include: 'On', 'Off'
 * @member {string} [versionStatus] Status of the integration runtime version.
 * @member {array} [links] The list of linked integration runtimes that are
 * created to share with this integration runtime.
 * @member {string} [pushedVersion] The version that the integration runtime is
 * going to update to.
 * @member {string} [latestVersion] The latest version on download center.
 * @member {date} [autoUpdateETA] The estimated time when the self-hosted
 * integration runtime will be updated.
 */
export interface SelfHostedIntegrationRuntimeStatus extends IntegrationRuntimeStatus {
  readonly createTime?: Date;
  readonly taskQueueId?: string;
  readonly internalChannelEncryption?: string;
  readonly version?: string;
  nodes?: SelfHostedIntegrationRuntimeNode[];
  readonly scheduledUpdateDate?: Date;
  readonly updateDelayOffset?: string;
  readonly localTimeZoneOffset?: string;
  readonly capabilities?: { [propertyName: string]: string };
  readonly serviceUrls?: string[];
  readonly autoUpdate?: string;
  readonly versionStatus?: string;
  links?: LinkedIntegrationRuntime[];
  readonly pushedVersion?: string;
  readonly latestVersion?: string;
  readonly autoUpdateETA?: Date;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeOperationResult class.
 * @constructor
 * Properties of managed integration runtime operation result.
 *
 * @member {string} [type] The operation type. Could be start or stop.
 * @member {date} [startTime] The start time of the operation.
 * @member {string} [result] The operation result.
 * @member {string} [errorCode] The error code.
 * @member {array} [parameters] Managed integration runtime error parameters.
 * @member {string} [activityId] The activity id for the operation request.
 */
export interface ManagedIntegrationRuntimeOperationResult {
  readonly type?: string;
  readonly startTime?: Date;
  readonly result?: string;
  readonly errorCode?: string;
  readonly parameters?: string[];
  readonly activityId?: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeError class.
 * @constructor
 * Error definition for managed integration runtime.
 *
 * @member {date} [time] The time when the error occurred.
 * @member {string} [code] Error code.
 * @member {array} [parameters] Managed integration runtime error parameters.
 * @member {string} [message] Error message.
 */
export interface ManagedIntegrationRuntimeError {
  readonly time?: Date;
  readonly code?: string;
  readonly parameters?: string[];
  readonly message?: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeNode class.
 * @constructor
 * Properties of integration runtime node.
 *
 * @member {string} [nodeId] The managed integration runtime node id.
 * @member {string} [status] The managed integration runtime node status.
 * Possible values include: 'Starting', 'Available', 'Recycling', 'Unavailable'
 * @member {array} [errors] The errors that occurred on this integration
 * runtime node.
 */
export interface ManagedIntegrationRuntimeNode {
  readonly nodeId?: string;
  readonly status?: string;
  errors?: ManagedIntegrationRuntimeError[];
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeStatus class.
 * @constructor
 * Managed integration runtime status.
 *
 * @member {date} [createTime] The time at which the integration runtime was
 * created, in ISO8601 format.
 * @member {array} [nodes] The list of nodes for managed integration runtime.
 * @member {array} [otherErrors] The errors that occurred on this integration
 * runtime.
 * @member {object} [lastOperation] The last operation result that occurred on
 * this integration runtime.
 * @member {string} [lastOperation.type] The operation type. Could be start or
 * stop.
 * @member {date} [lastOperation.startTime] The start time of the operation.
 * @member {string} [lastOperation.result] The operation result.
 * @member {string} [lastOperation.errorCode] The error code.
 * @member {array} [lastOperation.parameters] Managed integration runtime error
 * parameters.
 * @member {string} [lastOperation.activityId] The activity id for the
 * operation request.
 */
export interface ManagedIntegrationRuntimeStatus extends IntegrationRuntimeStatus {
  readonly createTime?: Date;
  readonly nodes?: ManagedIntegrationRuntimeNode[];
  readonly otherErrors?: ManagedIntegrationRuntimeError[];
  readonly lastOperation?: ManagedIntegrationRuntimeOperationResult;
}

/**
 * @class
 * Initializes a new instance of the LinkedIntegrationRuntimeType class.
 * @constructor
 * The base definition of a linked integration runtime.
 *
 * @member {string} authorizationType Polymorphic Discriminator
 */
export interface LinkedIntegrationRuntimeType {
  authorizationType: string;
}

/**
 * @class
 * Initializes a new instance of the LinkedIntegrationRuntimeRbacAuthorization class.
 * @constructor
 * The role based access control (RBAC) authorization type integration runtime.
 *
 * @member {string} resourceId The resource identifier of the integration
 * runtime to be shared.
 */
export interface LinkedIntegrationRuntimeRbacAuthorization extends LinkedIntegrationRuntimeType {
  resourceId: string;
}

/**
 * @class
 * Initializes a new instance of the LinkedIntegrationRuntimeKeyAuthorization class.
 * @constructor
 * The key authorization type integration runtime.
 *
 * @member {object} key The key used for authorization.
 * @member {string} [key.value] Value of secure string.
 */
export interface LinkedIntegrationRuntimeKeyAuthorization extends LinkedIntegrationRuntimeType {
  key: SecureString;
}

/**
 * @class
 * Initializes a new instance of the SelfHostedIntegrationRuntime class.
 * @constructor
 * Self-hosted integration runtime.
 *
 * @member {object} [linkedInfo]
 * @member {string} [linkedInfo.authorizationType] Polymorphic Discriminator
 */
export interface SelfHostedIntegrationRuntime extends IntegrationRuntime {
  linkedInfo?: LinkedIntegrationRuntimeType;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeCustomSetupScriptProperties class.
 * @constructor
 * Custom setup script properties for a managed dedicated integration runtime.
 *
 * @member {string} [blobContainerUri] The URI of the Azure blob container that
 * contains the custom setup script.
 * @member {object} [sasToken] The SAS token of the Azure blob container.
 * @member {string} [sasToken.value] Value of secure string.
 */
export interface IntegrationRuntimeCustomSetupScriptProperties {
  blobContainerUri?: string;
  sasToken?: SecureString;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeSsisCatalogInfo class.
 * @constructor
 * Catalog information for managed dedicated integration runtime.
 *
 * @member {string} [catalogServerEndpoint] The catalog database server URL.
 * @member {string} [catalogAdminUserName] The administrator user name of
 * catalog database.
 * @member {object} [catalogAdminPassword] The password of the administrator
 * user account of the catalog database.
 * @member {string} [catalogAdminPassword.value] Value of secure string.
 * @member {string} [catalogPricingTier] The pricing tier for the catalog
 * database. The valid values could be found in
 * https://azure.microsoft.com/en-us/pricing/details/sql-database/. Possible
 * values include: 'Basic', 'Standard', 'Premium', 'PremiumRS'
 */
export interface IntegrationRuntimeSsisCatalogInfo {
  catalogServerEndpoint?: string;
  catalogAdminUserName?: string;
  catalogAdminPassword?: SecureString;
  catalogPricingTier?: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeSsisProperties class.
 * @constructor
 * SSIS properties for managed integration runtime.
 *
 * @member {object} [catalogInfo] Catalog information for managed dedicated
 * integration runtime.
 * @member {string} [catalogInfo.catalogServerEndpoint] The catalog database
 * server URL.
 * @member {string} [catalogInfo.catalogAdminUserName] The administrator user
 * name of catalog database.
 * @member {object} [catalogInfo.catalogAdminPassword] The password of the
 * administrator user account of the catalog database.
 * @member {string} [catalogInfo.catalogAdminPassword.value] Value of secure
 * string.
 * @member {string} [catalogInfo.catalogPricingTier] The pricing tier for the
 * catalog database. The valid values could be found in
 * https://azure.microsoft.com/en-us/pricing/details/sql-database/. Possible
 * values include: 'Basic', 'Standard', 'Premium', 'PremiumRS'
 * @member {string} [licenseType] License type for bringing your own license
 * scenario. Possible values include: 'BasePrice', 'LicenseIncluded'
 * @member {object} [customSetupScriptProperties] Custom setup script
 * properties for a managed dedicated integration runtime.
 * @member {string} [customSetupScriptProperties.blobContainerUri] The URI of
 * the Azure blob container that contains the custom setup script.
 * @member {object} [customSetupScriptProperties.sasToken] The SAS token of the
 * Azure blob container.
 * @member {string} [customSetupScriptProperties.sasToken.value] Value of
 * secure string.
 * @member {string} [edition] The edition for the SSIS Integration Runtime.
 * Possible values include: 'Standard', 'Enterprise'
 */
export interface IntegrationRuntimeSsisProperties {
  catalogInfo?: IntegrationRuntimeSsisCatalogInfo;
  licenseType?: string;
  customSetupScriptProperties?: IntegrationRuntimeCustomSetupScriptProperties;
  edition?: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeVNetProperties class.
 * @constructor
 * VNet properties for managed integration runtime.
 *
 * @member {string} [vNetId] The ID of the VNet that this integration runtime
 * will join.
 * @member {string} [subnet] The name of the subnet this integration runtime
 * will join.
 */
export interface IntegrationRuntimeVNetProperties {
  vNetId?: string;
  subnet?: string;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeComputeProperties class.
 * @constructor
 * The compute resource properties for managed integration runtime.
 *
 * @member {string} [location] The location for managed integration runtime.
 * The supported regions could be found on
 * https://docs.microsoft.com/en-us/azure/data-factory/data-factory-data-movement-activities
 * @member {string} [nodeSize] The node size requirement to managed integration
 * runtime.
 * @member {number} [numberOfNodes] The required number of nodes for managed
 * integration runtime.
 * @member {number} [maxParallelExecutionsPerNode] Maximum parallel executions
 * count per node for managed integration runtime.
 * @member {object} [vNetProperties] VNet properties for managed integration
 * runtime.
 * @member {string} [vNetProperties.vNetId] The ID of the VNet that this
 * integration runtime will join.
 * @member {string} [vNetProperties.subnet] The name of the subnet this
 * integration runtime will join.
 */
export interface IntegrationRuntimeComputeProperties {
  location?: string;
  nodeSize?: string;
  numberOfNodes?: number;
  maxParallelExecutionsPerNode?: number;
  vNetProperties?: IntegrationRuntimeVNetProperties;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntime class.
 * @constructor
 * Managed integration runtime, including managed elastic and managed dedicated
 * integration runtimes.
 *
 * @member {string} [state] Integration runtime state, only valid for managed
 * dedicated integration runtime. Possible values include: 'Initial',
 * 'Stopped', 'Started', 'Starting', 'Stopping', 'NeedRegistration', 'Online',
 * 'Limited', 'Offline', 'AccessDenied'
 * @member {object} [computeProperties] The compute resource for managed
 * integration runtime.
 * @member {string} [computeProperties.location] The location for managed
 * integration runtime. The supported regions could be found on
 * https://docs.microsoft.com/en-us/azure/data-factory/data-factory-data-movement-activities
 * @member {string} [computeProperties.nodeSize] The node size requirement to
 * managed integration runtime.
 * @member {number} [computeProperties.numberOfNodes] The required number of
 * nodes for managed integration runtime.
 * @member {number} [computeProperties.maxParallelExecutionsPerNode] Maximum
 * parallel executions count per node for managed integration runtime.
 * @member {object} [computeProperties.vNetProperties] VNet properties for
 * managed integration runtime.
 * @member {string} [computeProperties.vNetProperties.vNetId] The ID of the
 * VNet that this integration runtime will join.
 * @member {string} [computeProperties.vNetProperties.subnet] The name of the
 * subnet this integration runtime will join.
 * @member {object} [ssisProperties] SSIS properties for managed integration
 * runtime.
 * @member {object} [ssisProperties.catalogInfo] Catalog information for
 * managed dedicated integration runtime.
 * @member {string} [ssisProperties.catalogInfo.catalogServerEndpoint] The
 * catalog database server URL.
 * @member {string} [ssisProperties.catalogInfo.catalogAdminUserName] The
 * administrator user name of catalog database.
 * @member {object} [ssisProperties.catalogInfo.catalogAdminPassword] The
 * password of the administrator user account of the catalog database.
 * @member {string} [ssisProperties.catalogInfo.catalogAdminPassword.value]
 * Value of secure string.
 * @member {string} [ssisProperties.catalogInfo.catalogPricingTier] The pricing
 * tier for the catalog database. The valid values could be found in
 * https://azure.microsoft.com/en-us/pricing/details/sql-database/. Possible
 * values include: 'Basic', 'Standard', 'Premium', 'PremiumRS'
 * @member {string} [ssisProperties.licenseType] License type for bringing your
 * own license scenario. Possible values include: 'BasePrice',
 * 'LicenseIncluded'
 * @member {object} [ssisProperties.customSetupScriptProperties] Custom setup
 * script properties for a managed dedicated integration runtime.
 * @member {string}
 * [ssisProperties.customSetupScriptProperties.blobContainerUri] The URI of the
 * Azure blob container that contains the custom setup script.
 * @member {object} [ssisProperties.customSetupScriptProperties.sasToken] The
 * SAS token of the Azure blob container.
 * @member {string} [ssisProperties.customSetupScriptProperties.sasToken.value]
 * Value of secure string.
 * @member {string} [ssisProperties.edition] The edition for the SSIS
 * Integration Runtime. Possible values include: 'Standard', 'Enterprise'
 */
export interface ManagedIntegrationRuntime extends IntegrationRuntime {
  readonly state?: string;
  computeProperties?: IntegrationRuntimeComputeProperties;
  ssisProperties?: IntegrationRuntimeSsisProperties;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeNodeIpAddress class.
 * @constructor
 * The IP address of self-hosted integration runtime node.
 *
 * @member {string} [ipAddress] The IP address of self-hosted integration
 * runtime node.
 */
export interface IntegrationRuntimeNodeIpAddress {
  readonly ipAddress?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeNodeMonitoringData class.
 * @constructor
 * Monitoring data for integration runtime node.
 *
 * @member {string} [nodeName] Name of the integration runtime node.
 * @member {number} [availableMemoryInMB] Available memory (MB) on the
 * integration runtime node.
 * @member {number} [cpuUtilization] CPU percentage on the integration runtime
 * node.
 * @member {number} [concurrentJobsLimit] Maximum concurrent jobs on the
 * integration runtime node.
 * @member {number} [concurrentJobsRunning] The number of jobs currently
 * running on the integration runtime node.
 * @member {number} [maxConcurrentJobs] The maximum concurrent jobs in this
 * integration runtime.
 * @member {number} [sentBytes] Sent bytes on the integration runtime node.
 * @member {number} [receivedBytes] Received bytes on the integration runtime
 * node.
 */
export interface IntegrationRuntimeNodeMonitoringData {
  readonly nodeName?: string;
  readonly availableMemoryInMB?: number;
  readonly cpuUtilization?: number;
  readonly concurrentJobsLimit?: number;
  readonly concurrentJobsRunning?: number;
  readonly maxConcurrentJobs?: number;
  readonly sentBytes?: number;
  readonly receivedBytes?: number;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeMonitoringData class.
 * @constructor
 * Get monitoring data response.
 *
 * @member {string} [name] Integration runtime name.
 * @member {array} [nodes] Integration runtime node monitoring data.
 */
export interface IntegrationRuntimeMonitoringData {
  name?: string;
  nodes?: IntegrationRuntimeNodeMonitoringData[];
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeAuthKeys class.
 * @constructor
 * The integration runtime authentication keys.
 *
 * @member {string} [authKey1] The primary integration runtime authentication
 * key.
 * @member {string} [authKey2] The secondary integration runtime authentication
 * key.
 */
export interface IntegrationRuntimeAuthKeys {
  authKey1?: string;
  authKey2?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeRegenerateKeyParameters class.
 * @constructor
 * Parameters to regenerate the authentication key.
 *
 * @member {string} [keyName] The name of the authentication key to regenerate.
 * Possible values include: 'authKey1', 'authKey2'
 */
export interface IntegrationRuntimeRegenerateKeyParameters {
  keyName?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeConnectionInfo class.
 * @constructor
 * Connection information for encrypting the on-premises data source
 * credentials.
 *
 * @member {string} [serviceToken] The token generated in service. Callers use
 * this token to authenticate to integration runtime.
 * @member {string} [identityCertThumbprint] The integration runtime SSL
 * certificate thumbprint. Click-Once application uses it to do server
 * validation.
 * @member {string} [hostServiceUri] The on-premises integration runtime host
 * URL.
 * @member {string} [version] The integration runtime version.
 * @member {string} [publicKey] The public key for encrypting a credential when
 * transferring the credential to the integration runtime.
 * @member {boolean} [isIdentityCertExprired] Whether the identity certificate
 * is expired.
 */
export interface IntegrationRuntimeConnectionInfo {
  readonly serviceToken?: string;
  readonly identityCertThumbprint?: string;
  readonly hostServiceUri?: string;
  readonly version?: string;
  readonly publicKey?: string;
  readonly isIdentityCertExprired?: boolean;
  /**
   * @property Describes unknown properties. The value of an unknown property
   * can be of "any" type.
   */
  [property: string]: any;
}


/**
 * @class
 * Initializes a new instance of the OperationListResponse class.
 * @constructor
 * A list of operations that can be performed by the Data Factory service.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface OperationListResponse extends Array<Operation> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the FactoryListResponse class.
 * @constructor
 * A list of factory resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface FactoryListResponse extends Array<Factory> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeListResponse class.
 * @constructor
 * A list of integration runtime resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface IntegrationRuntimeListResponse extends Array<IntegrationRuntimeResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the LinkedServiceListResponse class.
 * @constructor
 * A list of linked service resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface LinkedServiceListResponse extends Array<LinkedServiceResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetListResponse class.
 * @constructor
 * A list of dataset resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface DatasetListResponse extends Array<DatasetResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineListResponse class.
 * @constructor
 * A list of pipeline resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface PipelineListResponse extends Array<PipelineResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerListResponse class.
 * @constructor
 * A list of trigger resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface TriggerListResponse extends Array<TriggerResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RerunTriggerListResponse class.
 * @constructor
 * A list of rerun triggers.
 *
 * @member {string} [nextLink] The continuation token for getting the next page
 * of results, if any remaining results exist, null otherwise.
 */
export interface RerunTriggerListResponse extends Array<RerunTriggerResource> {
  readonly nextLink?: string;
}
